@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -22,6 +22,7 @@
 #include "CheckListWidget.hxx"
 #include "DialogContainer.hxx"
 #include "Dialog.hxx"
+#include "Font.hxx"
 #include "InputTextDialog.hxx"
 #include "OSystem.hxx"
 #include "Props.hxx"
@@ -57,13 +58,13 @@ CheatCodeDialog::CheatCodeDialog(OSystem& osystem, DialogContainer& parent,
   xpos += myCheatList->getWidth() + 5;  ypos = 15;
 
   b = new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
-                       "Add", kAddCheatCmd);
+                       "Add" + ELLIPSIS, kAddCheatCmd);
   wid.push_back(b);
   ypos += lineHeight + 10;
 
   myEditButton =
     new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
-                     "Edit", kEditCheatCmd);
+                     "Edit" + ELLIPSIS, kEditCheatCmd);
   wid.push_back(myEditButton);
   ypos += lineHeight + 10;
 
@@ -74,7 +75,7 @@ CheatCodeDialog::CheatCodeDialog(OSystem& osystem, DialogContainer& parent,
   ypos += lineHeight + 10;
 
   b = new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
-                       "One shot", kAddOneShotCmd);
+                       "One shot" + ELLIPSIS, kAddOneShotCmd);
   wid.push_back(b);
 
   // Inputbox which will pop up when adding/editing a cheat
@@ -103,6 +104,11 @@ CheatCodeDialog::CheatCodeDialog(OSystem& osystem, DialogContainer& parent,
   addBGroupToFocusList(wid);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+CheatCodeDialog::~CheatCodeDialog()
+{
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CheatCodeDialog::loadConfig()
 {
@@ -196,12 +202,12 @@ void CheatCodeDialog::handleCommand(CommandSender* sender, int cmd,
 {
   switch(cmd)
   {
-    case kOKCmd:
+    case GuiObject::kOKCmd:
       saveConfig();
       close();
       break;
 
-    case kCloseCmd:
+    case GuiObject::kCloseCmd:
       close();
       break;
 
