@@ -8,16 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <fstream>
-#include <sstream>
-
 #include "OSystem.hxx"
 #include "Console.hxx"
 #include "Cheat.hxx"
@@ -140,6 +137,7 @@ shared_ptr<Cheat> CheatManager::createCheat(const string& name, const string& co
   {
     case 4:  return make_shared<RamCheat>(myOSystem, name, code);
     case 6:  return make_shared<CheetahCheat>(myOSystem, name, code);
+    case 7:  return make_shared<BankRomCheat>(myOSystem, name, code);
     case 8:  return make_shared<BankRomCheat>(myOSystem, name, code);
     default: return nullptr;
   }
@@ -332,5 +330,5 @@ bool CheatManager::isValidCode(const string& code) const
       return false;
 
   uInt32 length = uInt32(code.length());
-  return (length == 4 || length == 6 || length == 8);
+  return (length == 4 || length == 6 || length == 7 || length == 8);
 }
