@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -60,22 +60,21 @@ void EventHandlerSDL2::pollEvent()
       case SDL_MOUSEMOTION:
       {
         handleMouseMotionEvent(myEvent.motion.x, myEvent.motion.y,
-                               myEvent.motion.xrel, myEvent.motion.yrel, 0);
+                               myEvent.motion.xrel, myEvent.motion.yrel);
         break;
       }
 
       case SDL_MOUSEBUTTONDOWN:
       case SDL_MOUSEBUTTONUP:
       {
-        bool pressed = myEvent.button.type == SDL_MOUSEBUTTONDOWN;
         switch(myEvent.button.button)
         {
           case SDL_BUTTON_LEFT:
-            handleMouseButtonEvent(pressed ? EVENT_LBUTTONDOWN : EVENT_LBUTTONUP,
+            handleMouseButtonEvent(MouseButton::LEFT, myEvent.button.type == SDL_MOUSEBUTTONDOWN,
                                    myEvent.button.x, myEvent.button.y);
             break;
           case SDL_BUTTON_RIGHT:
-            handleMouseButtonEvent(pressed ? EVENT_RBUTTONDOWN : EVENT_RBUTTONUP,
+            handleMouseButtonEvent(MouseButton::RIGHT, myEvent.button.type == SDL_MOUSEBUTTONDOWN,
                                    myEvent.button.x, myEvent.button.y);
             break;
         }
@@ -87,9 +86,9 @@ void EventHandlerSDL2::pollEvent()
         int x, y;
         SDL_GetMouseState(&x, &y);  // we need mouse position too
         if(myEvent.wheel.y < 0)
-          handleMouseButtonEvent(EVENT_WHEELDOWN, x, y);
+          handleMouseButtonEvent(MouseButton::WHEELDOWN, true, x, y);
         else if(myEvent.wheel.y > 0)
-          handleMouseButtonEvent(EVENT_WHEELUP, x, y);
+          handleMouseButtonEvent(MouseButton::WHEELUP, true, x, y);
         break;
       }
 
@@ -112,11 +111,15 @@ void EventHandlerSDL2::pollEvent()
       case SDL_JOYHATMOTION:
       {
         int v = myEvent.jhat.value, value = 0;
-        if(v & SDL_HAT_UP)        value |= EVENT_HATUP_M;
-        if(v & SDL_HAT_DOWN)      value |= EVENT_HATDOWN_M;
-        if(v & SDL_HAT_LEFT)      value |= EVENT_HATLEFT_M;
-        if(v & SDL_HAT_RIGHT)     value |= EVENT_HATRIGHT_M;
-        if(v == SDL_HAT_CENTERED) value  = EVENT_HATCENTER_M;
+        if(v & SDL_HAT_CENTERED)
+          value  = EVENT_HATCENTER_M;
+        else
+        {
+          if(v & SDL_HAT_UP)    value |= EVENT_HATUP_M;
+          if(v & SDL_HAT_DOWN)  value |= EVENT_HATDOWN_M;
+          if(v & SDL_HAT_LEFT)  value |= EVENT_HATLEFT_M;
+          if(v & SDL_HAT_RIGHT) value |= EVENT_HATRIGHT_M;
+        }
 
         handleJoyHatEvent(myEvent.jhat.which, myEvent.jhat.hat, value);
         break;  // SDL_JOYHATMOTION
@@ -144,42 +147,42 @@ void EventHandlerSDL2::pollEvent()
         switch(myEvent.window.event)
         {
           case SDL_WINDOWEVENT_SHOWN:
-            handleSystemEvent(EVENT_WINDOW_SHOWN);
+            handleSystemEvent(SystemEvent::WINDOW_SHOWN);
             break;
           case SDL_WINDOWEVENT_HIDDEN:
-            handleSystemEvent(EVENT_WINDOW_HIDDEN);
+            handleSystemEvent(SystemEvent::WINDOW_HIDDEN);
             break;
           case SDL_WINDOWEVENT_EXPOSED:
-            handleSystemEvent(EVENT_WINDOW_EXPOSED);
+            handleSystemEvent(SystemEvent::WINDOW_EXPOSED);
             break;
           case SDL_WINDOWEVENT_MOVED:
-            handleSystemEvent(EVENT_WINDOW_MOVED,
+            handleSystemEvent(SystemEvent::WINDOW_MOVED,
                               myEvent.window.data1, myEvent.window.data1);
             break;
           case SDL_WINDOWEVENT_RESIZED:
-            handleSystemEvent(EVENT_WINDOW_RESIZED,
+            handleSystemEvent(SystemEvent::WINDOW_RESIZED,
                               myEvent.window.data1, myEvent.window.data1);
             break;
           case SDL_WINDOWEVENT_MINIMIZED:
-            handleSystemEvent(EVENT_WINDOW_MINIMIZED);
+            handleSystemEvent(SystemEvent::WINDOW_MINIMIZED);
             break;
           case SDL_WINDOWEVENT_MAXIMIZED:
-            handleSystemEvent(EVENT_WINDOW_MAXIMIZED);
+            handleSystemEvent(SystemEvent::WINDOW_MAXIMIZED);
             break;
           case SDL_WINDOWEVENT_RESTORED:
-            handleSystemEvent(EVENT_WINDOW_RESTORED);
+            handleSystemEvent(SystemEvent::WINDOW_RESTORED);
             break;
           case SDL_WINDOWEVENT_ENTER:
-            handleSystemEvent(EVENT_WINDOW_ENTER);
+            handleSystemEvent(SystemEvent::WINDOW_ENTER);
             break;
           case SDL_WINDOWEVENT_LEAVE:
-            handleSystemEvent(EVENT_WINDOW_LEAVE);
+            handleSystemEvent(SystemEvent::WINDOW_LEAVE);
             break;
           case SDL_WINDOWEVENT_FOCUS_GAINED:
-            handleSystemEvent(EVENT_WINDOW_FOCUS_GAINED);
+            handleSystemEvent(SystemEvent::WINDOW_FOCUS_GAINED);
             break;
           case SDL_WINDOWEVENT_FOCUS_LOST:
-            handleSystemEvent(EVENT_WINDOW_FOCUS_LOST);
+            handleSystemEvent(SystemEvent::WINDOW_FOCUS_LOST);
             break;
         }
         break;  // SDL_WINDOWEVENT
