@@ -8,18 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <fstream>
 #include <cmath>
 
 #include "bspf.hxx"
 #include "FrameBuffer.hxx"
+#include "FBSurface.hxx"
 #include "Props.hxx"
 #include "PNGLibrary.hxx"
 
@@ -108,7 +108,7 @@ void PNGLibrary::loadImage(const string& filename, FBSurface& surface)
   // Cleanup
 done:
   if(png_ptr)
-    png_destroy_read_struct(&png_ptr, info_ptr ? &info_ptr : 0, 0);
+    png_destroy_read_struct(&png_ptr, info_ptr ? &info_ptr : nullptr, nullptr);
 
   if(err_message)
     throw runtime_error(err_message);
