@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -23,7 +23,7 @@
 
 #include "SDL_lib.hxx"
 #include "TIASnd.hxx"
-#include "TIATypes.hxx"
+#include "TIAConstants.hxx"
 #include "FrameBuffer.hxx"
 #include "Settings.hxx"
 #include "System.hxx"
@@ -226,12 +226,6 @@ void SoundSDL2::adjustVolume(Int8 direction)
   myOSystem.frameBuffer().showMessage(message);
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void SoundSDL2::adjustCycleCounter(Int32 amount)
-{
-  myLastRegisterSetCycle += amount;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void SoundSDL2::setChannels(uInt32 channels)
 {
@@ -249,7 +243,7 @@ void SoundSDL2::setFrameRate(float framerate)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void SoundSDL2::set(uInt16 addr, uInt8 value, Int32 cycle)
+void SoundSDL2::set(uInt16 addr, uInt8 value, uInt64 cycle)
 {
   SDL_LockAudio();
 
@@ -386,7 +380,7 @@ bool SoundSDL2::save(Serializer& out) const
       for(int i = 0; i < 6; ++i)
         out.putByte(0);
 
-    out.putInt(myLastRegisterSetCycle);
+    out.putLong(myLastRegisterSetCycle);
   }
   catch(...)
   {
@@ -423,7 +417,7 @@ bool SoundSDL2::load(Serializer& in)
       for(int i = 0; i < 6; ++i)
         in.getByte();
 
-    myLastRegisterSetCycle = in.getInt();
+    myLastRegisterSetCycle = in.getLong();
   }
   catch(...)
   {
