@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -38,18 +38,6 @@
   #include "CheatManager.hxx"
 #endif
 
-// Pointer to the main parent osystem object or the null pointer
-unique_ptr<OSystem> theOSystem;
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-// Does general Cleanup in case any operation failed (or at end of program)
-int Cleanup()
-{
-  theOSystem->logMessage("Cleanup from main", 2);
-  theOSystem->saveConfig();
-
-  return 0;
-}
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #if defined(BSPF_MAC_OSX)
@@ -61,10 +49,17 @@ int main(int argc, char* argv[])
   std::ios_base::sync_with_stdio(false);
 
   // Create the parent OSystem object
-  theOSystem = MediaFactory::createOSystem();
+  unique_ptr<OSystem> theOSystem = MediaFactory::createOSystem();
   theOSystem->loadConfig();
   theOSystem->logMessage("Loading config options ...", 2);
 
+  auto Cleanup = [&theOSystem]() {
+    theOSystem->logMessage("Cleanup from main", 2);
+    theOSystem->saveConfig();
+
+    return 0;
+  };
+
   // Take care of commandline arguments
   theOSystem->logMessage("Loading commandline arguments ...", 2);
   string romfile = theOSystem->settings().loadCommandLine(argc, argv);
