@@ -8,27 +8,32 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <time.h>
+#include <ctime>
 
 #include "bspf.hxx"
 #include "System.hxx"
+#include "M6502.hxx"
 #include "FSNode.hxx"
 #include "DiStella.hxx"
 #include "Debugger.hxx"
+#include "DebuggerParser.hxx"
 #include "CpuDebug.hxx"
 #include "OSystem.hxx"
 #include "Settings.hxx"
 #include "Version.hxx"
+#include "Cart.hxx"
 #include "CartDebug.hxx"
 #include "CartDebugWidget.hxx"
 #include "CartRamWidget.hxx"
+#include "RomWidget.hxx"
+#include "Base.hxx"
 using Common::Base;
 using std::hex;
 using std::dec;
@@ -58,7 +63,7 @@ CartDebug::CartDebug(Debugger& dbg, Console& console, const OSystem& osystem)
   mySystemAddresses = LabelToAddr(sysCmp);
 
   // Add Zero-page RAM addresses
-  for(uInt32 i = 0x80; i <= 0xFF; ++i)
+  for(uInt16 i = 0x80; i <= 0xFF; ++i)
   {
     myState.rport.push_back(i);
     myState.wport.push_back(i);
@@ -70,20 +75,21 @@ CartDebug::CartDebug(Debugger& dbg, Console& console, const OSystem& osystem)
   // Banksizes greater than 4096 indicate multi-bank ROMs, but we handle only
   // 4K pieces at a time
   // Banksizes less than 4K use the actual value
-  int banksize = 0;
+  uInt32 banksize = 0;
   myConsole.cartridge().getImage(banksize);
 
   BankInfo info;
-  info.size = std::min(banksize, 4096);
-  for(int i = 0; i < myConsole.cartridge().bankCount(); ++i)
+  info.size = std::min(banksize, 4096u);
+  for(uInt32 i = 0; i < myConsole.cartridge().bankCount(); ++i)
     myBankInfo.push_back(info);
 
   info.size = 128;  // ZP RAM
   myBankInfo.push_back(info);
 
   // We know the address for the startup bank right now
-  myBankInfo[myConsole.cartridge().startBank()].addressList.push_back(myDebugger.dpeek(0xfffc));
-  addLabel("START", myDebugger.dpeek(0xfffc));
+  myBankInfo[myConsole.cartridge().startBank()].addressList.push_front(
+    myDebugger.dpeek(0xfffc));
+  addLabel("Start", myDebugger.dpeek(0xfffc, DATA));
 
   // Add system equates
   for(uInt16 addr = 0x00; addr <= 0x0F; ++addr)
@@ -114,16 +120,17 @@ CartDebug::CartDebug(Debugger& dbg, Console& console, const OSystem& osystem)
   myDisassembly.list.reserve(2048);
 
   // Add settings for Distella
-  DiStella::settings.gfx_format =
+  DiStella::settings.gfxFormat =
     myOSystem.settings().getInt("dis.gfxformat") == 16 ? Base::F_16 : Base::F_2;
-  DiStella::settings.resolve_code =
+  DiStella::settings.resolveCode =
     myOSystem.settings().getBool("dis.resolve");
-  DiStella::settings.show_addresses =
+  DiStella::settings.showAddresses =
     myOSystem.settings().getBool("dis.showaddr");
-  DiStella::settings.aflag = false; // Not currently configurable
-  DiStella::settings.fflag = true;  // Not currently configurable
-  DiStella::settings.rflag = myOSystem.settings().getBool("dis.relocate");
-  DiStella::settings.bwidth = 9;  // TODO - configure based on window size
+  DiStella::settings.aFlag = false; // Not currently configurable
+  DiStella::settings.fFlag = true;  // Not currently configurable
+  DiStella::settings.rFlag = myOSystem.settings().getBool("dis.relocate");
+  DiStella::settings.bFlag = true;  // Not currently configurable
+  DiStella::settings.bytesWidth = 8+1;  // TODO - configure based on window size
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -131,7 +138,7 @@ const DebuggerState& CartDebug::getState()
 {
   myState.ram.clear();
   for(uInt32 i = 0; i < myState.rport.size(); ++i)
-    myState.ram.push_back(peek(myState.rport[i]));
+    myState.ram.push_back(myDebugger.peek(myState.rport[i]));
 
   if(myDebugWidget)
     myState.bank = myDebugWidget->bankState();
@@ -144,7 +151,7 @@ void CartDebug::saveOldState()
 {
   myOldState.ram.clear();
   for(uInt32 i = 0; i < myOldState.rport.size(); ++i)
-    myOldState.ram.push_back(peek(myOldState.rport[i]));
+    myOldState.ram.push_back(myDebugger.peek(myOldState.rport[i]));
 
   if(myDebugWidget)
   {
@@ -177,6 +184,18 @@ int CartDebug::readFromWritePort()
     return 0;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartDebug::lastReadBaseAddress()
+{
+  return mySystem.m6502().lastReadBaseAddress();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartDebug::lastWriteBaseAddress()
+{
+  return mySystem.m6502().lastWriteBaseAddress();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string CartDebug::toString()
 {
@@ -263,34 +282,28 @@ bool CartDebug::disassemble(bool force)
 
     // Only add addresses when absolutely necessary, to cut down on the
     // work that Distella has to do
-    // Distella expects the addresses to be unique and in sorted order
     if(bankChanged || !pcfound)
     {
       AddressList::const_iterator i;
       for(i = addresses.cbegin(); i != addresses.cend(); ++i)
       {
-        if(PC < *i)
-        {
-          addresses.insert(i, PC);
-          break;
-        }
-        else if(PC == *i)  // already present
+        if (PC == *i)  // already present
           break;
       }
       // Otherwise, add the item at the end
-      if(i == addresses.end())
+      if (i == addresses.end())
         addresses.push_back(PC);
     }
 
     // Always attempt to resolve code sections unless it's been
     // specifically disabled
     bool found = fillDisassemblyList(info, PC);
-    if(!found && DiStella::settings.resolve_code)
+    if(!found && DiStella::settings.resolveCode)
     {
       // Temporarily turn off code resolution
-      DiStella::settings.resolve_code = false;
+      DiStella::settings.resolveCode = false;
       fillDisassemblyList(info, PC);
-      DiStella::settings.resolve_code = true;
+      DiStella::settings.resolveCode = true;
     }
   }
 
@@ -305,7 +318,7 @@ bool CartDebug::fillDisassemblyList(BankInfo& info, uInt16 search)
     return false;
 
   myDisassembly.list.clear();
-  myDisassembly.fieldwidth = 14 + myLabelLength;
+  myDisassembly.fieldwidth = 24 + myLabelLength;
   DiStella distella(*this, myDisassembly.list, info, DiStella::settings,
                     myDisLabels, myDisDirectives, myReserved);
 
@@ -517,6 +530,18 @@ bool CartDebug::addDirective(CartDebug::DisasmType type,
   return true;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartDebug::getBank()
+{
+  return myConsole.cartridge().getBank();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartDebug::bankCount() const
+{
+  return myConsole.cartridge().bankCount();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartDebug::addLabel(const string& label, uInt16 address)
 {
@@ -704,8 +729,6 @@ string CartDebug::loadListFile()
   if(!in.is_open())
     return DebuggerParser::red("list file '" + node.getShortPath() + "' not readable");
 
-  myUserCLabels.clear();
-
   while(!in.eof())
   {
     string line, addr_s;
@@ -725,7 +748,7 @@ string CartDebug::loadListFile()
       if(addr_s.length() == 0)
         continue;
       const char* p = addr_s[0] == 'U' ? addr_s.c_str() + 1 : addr_s.c_str();
-      addr = int(strtoul(p, NULL, 16));
+      addr = int(strtoul(p, nullptr, 16));
 
       // For now, completely ignore ROM addresses
       if(!(addr & 0x1000))
@@ -736,7 +759,8 @@ string CartDebug::loadListFile()
         char eq = '\0';
         buf >> hex >> xx >> hex >> yy >> line >> eq;
         if(xx >= 0 && yy >= 0 && eq == '=')
-          myUserCLabels.emplace(xx*256+yy, line);
+          //myUserCLabels.emplace(xx*256+yy, line);
+          addLabel(line, xx * 256 + yy);
       }
     }
   }
@@ -781,14 +805,21 @@ string CartDebug::loadSymbolFile()
     {
       // Make sure the value doesn't represent a constant
       // For now, we simply ignore constants completely
-      const auto& iter = myUserCLabels.find(value);
-      if(iter == myUserCLabels.end() || !BSPF::equalsIgnoreCase(label, iter->second))
+      //const auto& iter = myUserCLabels.find(value);
+      //if(iter == myUserCLabels.end() || !BSPF::equalsIgnoreCase(label, iter->second))
+      const auto& iter = myUserLabels.find(value);
+      if (iter == myUserLabels.end() || !BSPF::equalsIgnoreCase(label, iter->second))
       {
         // Check for period, and strip leading number
-        if(string::size_type pos = label.find_first_of(".", 0) != string::npos)
+        string::size_type pos = label.find_first_of(".", 0);
+        if(pos != string::npos)
           addLabel(label.substr(pos), value);
         else
-          addLabel(label, value);
+        {
+          pos = label.find_last_of("$");
+          if (pos == string::npos || pos != label.length() - 1)
+            addLabel(label, value);
+        }
       }
     }
   }
@@ -800,9 +831,6 @@ string CartDebug::loadSymbolFile()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string CartDebug::loadConfigFile()
 {
-  if(myConsole.cartridge().bankCount() > 1)
-    return DebuggerParser::red("config file for multi-bank ROM not yet supported");
-
   // There are two possible locations for loading config files
   //   (in order of decreasing relevance):
   // 1) ROM dir based on properties entry name
@@ -903,15 +931,17 @@ string CartDebug::loadConfigFile()
   }
   myDebugger.rom().invalidate();
 
-  return "loaded " + node.getShortPath() + " OK";
+  stringstream retVal;
+  if(myConsole.cartridge().bankCount() > 1)
+    retVal << DebuggerParser::red("config file for multi-bank ROM not fully supported\n");
+  retVal << "loaded " << node.getShortPath() << " OK";
+  return retVal.str();
+
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string CartDebug::saveConfigFile()
 {
-  if(myConsole.cartridge().bankCount() > 1)
-    return DebuggerParser::red("config file for multi-bank ROM not yet supported");
-
   // While there are two possible locations for loading config files,
   // the main 'config' directory is used whenever possible when saving,
   // unless the rom-specific file already exists
@@ -946,29 +976,22 @@ string CartDebug::saveConfigFile()
     getBankDirectives(out, myBankInfo[b]);
   }
 
-  return "saved " + node.getShortPath() + " OK";
+  stringstream retVal;
+  if(myConsole.cartridge().bankCount() > 1)
+    retVal << DebuggerParser::red("config file for multi-bank ROM not fully supported\n");
+  retVal << "saved " << node.getShortPath() << " OK";
+  return retVal.str();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string CartDebug::saveDisassembly()
 {
-  if(myConsole.cartridge().bankCount() > 1)
-    return DebuggerParser::red("disassembly for multi-bank ROM not yet supported");
-
-  // Currently, the default naming/location for disassembly files is:
-  // 1) ROM dir based on properties entry name
-
   if(myDisasmFile == "")
   {
     const string& propsname =
       myConsole.properties().get(Cartridge_Name) + ".asm";
 
-    FilesystemNode case0(myOSystem.romFile().getParent().getPath() + propsname);
-    if(case0.getParent().isWritable())
-      myDisasmFile = case0.getPath();
-    else
-      return DebuggerParser::red("disassembly file not writable:\n  " +
-          case0.getShortPath());
+    myDisasmFile = FilesystemNode(myOSystem.defaultSaveDir() + propsname).getPath();
   }
 
   FilesystemNode node(myDisasmFile);
@@ -981,20 +1004,23 @@ string CartDebug::saveDisassembly()
   // We can't print the header to the disassembly until it's actually
   // been processed; therefore buffer output to a string first
   ostringstream buf;
-  buf << "\n;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\n;\n"
-      << ";      MAIN PROGRAM\n"
-      << ";\n;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\n\n";
+  buf << "\n\n;***********************************************************\n"
+      << ";      Bank " << myConsole.cartridge().getBank();
+  if (myConsole.cartridge().bankCount() > 1)
+    buf << " / 0.." << myConsole.cartridge().bankCount() - 1;
+  buf << "\n;***********************************************************\n\n";
 
   // Use specific settings for disassembly output
   // This will most likely differ from what you see in the debugger
   DiStella::Settings settings;
-  settings.gfx_format = DiStella::settings.gfx_format;
-  settings.resolve_code = true;
-  settings.show_addresses = false;
-  settings.aflag = false; // Otherwise DASM gets confused
-  settings.fflag = DiStella::settings.fflag;
-  settings.rflag = DiStella::settings.rflag;
-  settings.bwidth = 17;  // default from Distella
+  settings.gfxFormat = DiStella::settings.gfxFormat;
+  settings.resolveCode = true;
+  settings.showAddresses = false;
+  settings.aFlag = false; // Otherwise DASM gets confused
+  settings.fFlag = DiStella::settings.fFlag;
+  settings.rFlag = DiStella::settings.rFlag;
+  settings.bytesWidth = 8+1;  // same as Stella debugger
+  settings.bFlag = DiStella::settings.bFlag;; // process break routine (TODO)
 
   Disassembly disasm;
   disasm.list.reserve(2048);
@@ -1010,8 +1036,11 @@ string CartDebug::saveDisassembly()
     DiStella distella(*this, disasm.list, info, settings,
                       myDisLabels, myDisDirectives, myReserved);
 
-    buf << "       SEG CODE\n"
-        << "       ORG $" << Base::HEX4 << info.offset << "\n\n";
+    if (myReserved.breakFound)
+      addLabel("Break", myDebugger.dpeek(0xfffe));
+
+    buf << "    SEG     CODE\n"
+        << "    ORG     $" << Base::HEX4 << info.offset << "\n\n";
 
     // Format in 'distella' style
     for(uInt32 i = 0; i < disasm.list.size(); ++i)
@@ -1020,55 +1049,53 @@ string CartDebug::saveDisassembly()
 
       // Add label (if any)
       if(tag.label != "")
-        buf << ALIGN(7) << (tag.label+":") << endl;
-      buf << "       ";
+        buf << ALIGN(4) << (tag.label) << "\n";
+      buf << "    ";
 
       switch(tag.type)
       {
         case CartDebug::CODE:
         {
-          buf << ALIGN(25) << tag.disasm << tag.ccount << "\n";
-          break;
-        }
-        case CartDebug::NONE:
-        {
-          buf << "\n";
+          buf << ALIGN(32) << tag.disasm << tag.ccount.substr(0, 5) << tag.ctotal << tag.ccount.substr(5, 2);
+          if (tag.disasm.find("WSYNC") != std::string::npos)
+            buf << "\n;---------------------------------------";
           break;
         }
         case CartDebug::ROW:
         {
-          buf << tag.disasm << "\n";
+          buf << ".byte   " << ALIGN(32) << tag.disasm.substr(6, 8*4-1) << "; $" << Base::HEX4 << tag.address << " (*)";
           break;
         }
         case CartDebug::GFX:
         {
-          buf << ".byte " << (settings.gfx_format == Base::F_2 ? "%" : "$")
+          buf << ".byte   " << (settings.gfxFormat == Base::F_2 ? "%" : "$")
               << tag.bytes << " ; |";
           for(int c = 12; c < 20; ++c)
             buf << ((tag.disasm[c] == '\x1e') ? "#" : " ");
-          buf << "| $" << Base::HEX4 << tag.address << " (G)\n";
+          buf << ALIGN(13) << "|" << "$" << Base::HEX4 << tag.address << " (G)";
           break;
         }
         case CartDebug::PGFX:
         {
-          buf << ".byte " << (settings.gfx_format == Base::F_2 ? "%" : "$")
+          buf << ".byte   " << (settings.gfxFormat == Base::F_2 ? "%" : "$")
               << tag.bytes << " ; |";
           for(int c = 12; c < 20; ++c)
             buf << ((tag.disasm[c] == '\x1f') ? "*" : " ");
-          buf << "| $" << Base::HEX4 << tag.address << " (P)\n";
+          buf << ALIGN(13) << "|" << "$" << Base::HEX4 << tag.address << " (P)";
           break;
         }
         case CartDebug::DATA:
         {
-          buf << tag.disasm.substr(0, 9) << " ;  $" << Base::HEX4 << tag.address << " (D)\n";
+          buf << ".byte   " << ALIGN(32) << tag.disasm.substr(6, 8 * 4 - 1) << "; $" << Base::HEX4 << tag.address << " (D)";
           break;
         }
+        case CartDebug::NONE:
         default:
         {
-          buf << "\n";
           break;
         }
-      }
+      } // switch
+      buf << "\n";
     }
   }
 
@@ -1084,88 +1111,135 @@ string CartDebug::saveDisassembly()
       << "; Legend: * = CODE not yet run (tentative code)\n"
       << ";         D = DATA directive (referenced in some way)\n"
       << ";         G = GFX directive, shown as '#' (stored in player, missile, ball)\n"
-      << ";         P = PGFX directive, shown as '*' (stored in playfield)\n\n"
-      << "      processor 6502\n\n";
+      << ";         P = PGFX directive, shown as '*' (stored in playfield)\n"
+      << ";         i = indexed accessed only\n"
+      << ";         c = used by code executed in RAM\n"
+      << ";         s = used by stack\n"
+      << ";         ! = page crossed, 1 cycle penalty\n"
+      << "\n    processor 6502\n\n";
 
   bool addrUsed = false;
   for(uInt16 addr = 0x00; addr <= 0x0F; ++addr)
-    addrUsed = addrUsed || myReserved.TIARead[addr];
+    addrUsed = addrUsed || myReserved.TIARead[addr] || (mySystem.getAccessFlags(addr) & WRITE);
   for(uInt16 addr = 0x00; addr <= 0x3F; ++addr)
-    addrUsed = addrUsed || myReserved.TIAWrite[addr];
+    addrUsed = addrUsed || myReserved.TIAWrite[addr] || (mySystem.getAccessFlags(addr) & DATA);
   for(uInt16 addr = 0x00; addr <= 0x17; ++addr)
     addrUsed = addrUsed || myReserved.IOReadWrite[addr];
   if(addrUsed)
   {
-    out << ";;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\n"
-        << ";      TIA AND IO CONSTANTS\n"
-        << ";;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\n\n";
+    out << "\n;-----------------------------------------------------------\n"
+        << ";      TIA and IO constants accessed\n"
+        << ";-----------------------------------------------------------\n\n";
+
+    // TIA read access
     for(uInt16 addr = 0x00; addr <= 0x0F; ++addr)
       if(myReserved.TIARead[addr] && ourTIAMnemonicR[addr])
-        out << ALIGN(6) << ourTIAMnemonicR[addr] << "  =  $"
-            << Base::HEX2 << right << addr << " ; (R)\n";
+        out << ALIGN(16) << ourTIAMnemonicR[addr] << "= $"
+            << Base::HEX2 << right << addr << "  ; (R)\n";
+      else if (mySystem.getAccessFlags(addr) & DATA)
+        out << ";" << ALIGN(16-1) << ourTIAMnemonicR[addr] << "= $"
+        << Base::HEX2 << right << addr << "  ; (Ri)\n";
+    out << "\n";
+
+    // TIA write access
     for(uInt16 addr = 0x00; addr <= 0x3F; ++addr)
       if(myReserved.TIAWrite[addr] && ourTIAMnemonicW[addr])
-        out << ALIGN(6) << ourTIAMnemonicW[addr] << "  =  $"
-            << Base::HEX2 << right << addr << " ; (W)\n";
+        out << ALIGN(16) << ourTIAMnemonicW[addr] << "= $"
+            << Base::HEX2 << right << addr << "  ; (W)\n";
+      else if (mySystem.getAccessFlags(addr) & WRITE)
+        out << ";" << ALIGN(16-1) << ourTIAMnemonicW[addr] << "= $"
+        << Base::HEX2 << right << addr << "  ; (Wi)\n";
+    out << "\n";
+
+    // RIOT IO access
     for(uInt16 addr = 0x00; addr <= 0x17; ++addr)
       if(myReserved.IOReadWrite[addr] && ourIOMnemonic[addr])
-        out << ALIGN(6) << ourIOMnemonic[addr] << "  =  $"
+        out << ALIGN(16) << ourIOMnemonic[addr] << "= $"
             << Base::HEX4 << right << (addr+0x280) << "\n";
   }
 
   addrUsed = false;
   for(uInt16 addr = 0x80; addr <= 0xFF; ++addr)
-    addrUsed = addrUsed || myReserved.ZPRAM[addr-0x80];
+    addrUsed = addrUsed || myReserved.ZPRAM[addr-0x80]
+      || (mySystem.getAccessFlags(addr) & (DATA | WRITE))
+      || (mySystem.getAccessFlags(addr|0x100) & (DATA | WRITE));
   if(addrUsed)
   {
-    out << "\n;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\n"
-        << ";      RIOT RAM (zero-page)\n"
-        << ";;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\n\n";
-    for(uInt16 addr = 0x80; addr <= 0xFF; ++addr)
-    {
-      if(myReserved.ZPRAM[addr-0x80] &&
-         myUserLabels.find(addr) == myUserLabels.end())
-      {
-        out << ALIGN(6) << ourZPMnemonic[addr-0x80] << "  =  $"
-            << Base::HEX2 << right << (addr) << "\n";
-      }
+    bool addLine = false;
+    out << "\n\n;-----------------------------------------------------------\n"
+        << ";      RIOT RAM (zero-page) labels\n"
+        << ";-----------------------------------------------------------\n\n";
+
+    for (uInt16 addr = 0x80; addr <= 0xFF; ++addr) {
+      bool ramUsed = (mySystem.getAccessFlags(addr) & (DATA | WRITE));
+      bool codeUsed = (mySystem.getAccessFlags(addr) & CODE);
+      bool stackUsed = (mySystem.getAccessFlags(addr|0x100) & (DATA | WRITE));
+
+      if (myReserved.ZPRAM[addr - 0x80] &&
+          myUserLabels.find(addr) == myUserLabels.end()) {
+        if (addLine)
+          out << "\n";
+        out << ALIGN(16) << ourZPMnemonic[addr - 0x80] << "= $"
+          << Base::HEX2 << right << (addr)
+          << (stackUsed|codeUsed ? "; (" : "")
+          << (codeUsed ? "c" : "")
+          << (stackUsed ? "s" : "")
+          << (stackUsed | codeUsed ? ")" : "")
+          << "\n";
+        addLine = false;
+      } else if (ramUsed|codeUsed|stackUsed) {
+        if (addLine)
+          out << "\n";
+        out << ALIGN(18) << ";" << "$"
+          << Base::HEX2 << right << (addr)
+          << "  ("
+          << (ramUsed ? "i" : "")
+          << (codeUsed ? "c" : "")
+          << (stackUsed ? "s" : "")
+          << ")\n";
+        addLine = false;
+      } else
+        addLine = true;
     }
   }
 
   if(myReserved.Label.size() > 0)
   {
-    out << "\n;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\n"
-        << ";      NON LOCATABLE\n"
-        << ";;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\n\n";
+    out << "\n\n;-----------------------------------------------------------\n"
+        << ";      Non Locatable Labels\n"
+        << ";-----------------------------------------------------------\n\n";
     for(const auto& iter: myReserved.Label)
-        out << ALIGN(10) << iter.second << "  =  $" << iter.first << "\n";
+        out << ALIGN(16) << iter.second << "= $" << iter.first << "\n";
   }
 
   if(myUserLabels.size() > 0)
   {
-    out << "\n;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\n"
-        << ";      USER DEFINED\n"
-        << ";;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\n\n";
-    int max_len = 0;
+    out << "\n\n;-----------------------------------------------------------\n"
+        << ";      User Defined Labels\n"
+        << ";-----------------------------------------------------------\n\n";
+    int max_len = 16;
     for(const auto& iter: myUserLabels)
       max_len = std::max(max_len, int(iter.second.size()));
     for(const auto& iter: myUserLabels)
-        out << ALIGN(max_len) << iter.second << "  =  $" << iter.first << "\n";
+      out << ALIGN(max_len) << iter.second << "= $" << iter.first << "\n";
   }
 
   // And finally, output the disassembly
   out << buf.str();
 
-  return "saved " + node.getShortPath() + " OK";
+  stringstream retVal;
+  if(myConsole.cartridge().bankCount() > 1)
+    retVal << DebuggerParser::red("disassembly for multi-bank ROM not fully supported, only currently enabled banks disassembled\n");
+  retVal << "saved " << node.getShortPath() << " OK";
+  return retVal.str();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string CartDebug::saveRom()
 {
-  const string& path = string("~") + BSPF::PATH_SEPARATOR +
-    myConsole.properties().get(Cartridge_Name) + ".a26";
+  const string& rom = myConsole.properties().get(Cartridge_Name) + ".a26";
 
-  FilesystemNode node(path);
+  FilesystemNode node(myOSystem.defaultSaveDir() + rom);
   ofstream out(node.getPath(), std::ios::binary);
   if(out && myConsole.cartridge().saveROM(out))
     return "saved ROM as " + node.getShortPath();
@@ -1238,23 +1312,23 @@ void CartDebug::getCompletions(const char* in, StringList& completions) const
 {
   // First scan system equates
   for(uInt16 addr = 0x00; addr <= 0x0F; ++addr)
-    if(ourTIAMnemonicR[addr] && BSPF::startsWithIgnoreCase(ourTIAMnemonicR[addr], in))
+    if(ourTIAMnemonicR[addr] && BSPF::matches(ourTIAMnemonicR[addr], in))
       completions.push_back(ourTIAMnemonicR[addr]);
   for(uInt16 addr = 0x00; addr <= 0x3F; ++addr)
-    if(ourTIAMnemonicW[addr] && BSPF::startsWithIgnoreCase(ourTIAMnemonicW[addr], in))
+    if(ourTIAMnemonicW[addr] && BSPF::matches(ourTIAMnemonicW[addr], in))
       completions.push_back(ourTIAMnemonicW[addr]);
   for(uInt16 addr = 0; addr <= 0x297-0x280; ++addr)
-    if(ourIOMnemonic[addr] && BSPF::startsWithIgnoreCase(ourIOMnemonic[addr], in))
+    if(ourIOMnemonic[addr] && BSPF::matches(ourIOMnemonic[addr], in))
       completions.push_back(ourIOMnemonic[addr]);
   for(uInt16 addr = 0; addr <= 0x7F; ++addr)
-    if(ourZPMnemonic[addr] && BSPF::startsWithIgnoreCase(ourZPMnemonic[addr], in))
+    if(ourZPMnemonic[addr] && BSPF::matches(ourZPMnemonic[addr], in))
       completions.push_back(ourZPMnemonic[addr]);
 
   // Now scan user-defined labels
   for(const auto& iter: myUserAddresses)
   {
     const char* l = iter.first.c_str();
-    if(BSPF::startsWithIgnoreCase(l, in))
+    if(BSPF::matches(l, in))
       completions.push_back(l);
   }
 }
@@ -1404,7 +1478,7 @@ void CartDebug::disasmTypeAsString(ostream& buf, uInt8 flags) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 const char* const CartDebug::ourTIAMnemonicR[16] = {
   "CXM0P", "CXM1P", "CXP0FB", "CXP1FB", "CXM0FB", "CXM1FB", "CXBLPF", "CXPPMM",
-  "INPT0", "INPT1", "INPT2", "INPT3", "INPT4", "INPT5", 0, 0
+  "INPT0", "INPT1", "INPT2", "INPT3", "INPT4", "INPT5", "$1e", "$1f"
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1414,14 +1488,17 @@ const char* const CartDebug::ourTIAMnemonicW[64] = {
   "RESP0", "RESP1", "RESM0", "RESM1", "RESBL", "AUDC0", "AUDC1", "AUDF0",
   "AUDF1", "AUDV0", "AUDV1", "GRP0", "GRP1", "ENAM0", "ENAM1", "ENABL",
   "HMP0", "HMP1", "HMM0", "HMM1", "HMBL", "VDELP0", "VDELP1", "VDELBL",
-  "RESMP0", "RESMP1", "HMOVE", "HMCLR", "CXCLR", 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+  "RESMP0", "RESMP1", "HMOVE", "HMCLR", "CXCLR", "$2d", "$2e", "$2f",
+  "$30", "$31", "$32", "$33", "$34", "$35", "$36", "$37",
+  "$38", "$39", "$3a", "$3b", "$3c", "$3d", "$3e", "$3f"
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 const char* const CartDebug::ourIOMnemonic[24] = {
-  "SWCHA", "SWACNT", "SWCHB", "SWBCNT", "INTIM", "TIMINT", 0, 0,  0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, "TIM1T", "TIM8T", "TIM64T", "T1024T"
+  "SWCHA", "SWACNT", "SWCHB", "SWBCNT", "INTIM", "TIMINT",
+  "$286", "$287", "$288", "$289", "$28a", "$28b", "$28c",
+  "$28d", "$28e", "$28f", "$290", "$291", "$292", "$293",
+  "TIM1T", "TIM8T", "TIM64T", "T1024T"
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
