@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -18,6 +18,7 @@
 #include <sstream>
 
 #include "M6502.hxx"
+#include "System.hxx"
 #include "Debugger.hxx"
 #include "CartDebug.hxx"
 #include "TIADebug.hxx"
@@ -69,6 +70,84 @@ void CpuDebug::saveOldState()
   Debugger::set_bits(myOldState.PS, myOldState.PSbits);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::pc() const
+{
+  return mySystem.m6502().PC;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::sp() const
+{
+  return mySystem.m6502().SP;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::a() const
+{
+  return mySystem.m6502().A;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::x() const
+{
+  return mySystem.m6502().X;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::y() const
+{
+  return mySystem.m6502().Y;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::n() const
+{
+  return mySystem.m6502().N;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::v() const
+{
+  return mySystem.m6502().V;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::b() const
+{
+  return mySystem.m6502().B;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::d() const
+{
+  return mySystem.m6502().D;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::i() const
+{
+  return mySystem.m6502().I;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::z() const
+{
+  return !mySystem.m6502().notZ;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::c() const
+{
+  return mySystem.m6502().C;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::icycles() const
+{
+  return mySystem.m6502().icycles;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CpuDebug::setPC(int pc)
 {
@@ -147,6 +226,12 @@ void CpuDebug::setC(bool on)
   my6502.C = on;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void CpuDebug::setCycles(int cycles)
+{
+  my6502.icycles = cycles;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CpuDebug::toggleN()
 {
