@@ -8,17 +8,16 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <sstream>
-
 #include "System.hxx"
 #include "TIA.hxx"
+#include "M6532.hxx"
 #include "Debugger.hxx"
 #include "Switches.hxx"
 
@@ -55,6 +54,9 @@ const DebuggerState& RiotDebug::getState()
   myState.INPT4 = inpt(4);
   myState.INPT5 = inpt(5);
 
+  myState.INPTLatch = vblank(6);
+  myState.INPTDump = vblank(7);
+
   // Timer registers
   myState.TIM1T     = tim1T();
   myState.TIM8T     = tim8T();
@@ -93,6 +95,9 @@ void RiotDebug::saveOldState()
   myOldState.INPT4 = inpt(4);
   myOldState.INPT5 = inpt(5);
 
+  myOldState.INPTLatch = vblank(6);
+  myOldState.INPTDump = vblank(7);
+
   // Timer registers
   myOldState.TIM1T     = tim1T();
   myOldState.TIM8T     = tim8T();
@@ -194,6 +199,30 @@ uInt8 RiotDebug::tim1024T(int newVal)
   return mySystem.m6532().myOutTimer[3];
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8 RiotDebug::intim() const
+{
+  return mySystem.m6532().intim();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8 RiotDebug::timint() const
+{
+  return mySystem.m6532().timint();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Int32 RiotDebug::timClocks() const
+{
+  return mySystem.m6532().timerClocks();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Int32 RiotDebug::intimClocks() const
+{
+  return mySystem.m6532().intimClocks();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Controller& RiotDebug::controller(Controller::Jack jack) const
 {
