@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,11 +16,14 @@
 //============================================================================
 
 #include "DataGridWidget.hxx"
-#include "FrameBuffer.hxx"
 #include "GuiObject.hxx"
+#include "Font.hxx"
 #include "OSystem.hxx"
+#include "Debugger.hxx"
 #include "TIADebug.hxx"
 #include "Widget.hxx"
+#include "Base.hxx"
+using Common::Base;
 
 #include "AudioWidget.hxx"
 
@@ -39,52 +42,46 @@ AudioWidget::AudioWidget(GuiObject* boss, const GUI::Font& lfont,
   // AudF registers
   new StaticTextWidget(boss, lfont, xpos, ypos+2,
                        lwidth, fontHeight,
-                       "AUDF", kTextAlignLeft);
+                       "AUDF", TextAlign::Left);
   xpos += lwidth;
   myAudF = new DataGridWidget(boss, nfont, xpos, ypos,
                               2, 1, 2, 5, Common::Base::F_16);
   myAudF->setTarget(this);
   myAudF->setID(kAUDFID);
-  myAudF->setEditable(false);
   addFocusWidget(myAudF);
 
   for(int col = 0; col < 2; ++col)
   {
-    new StaticTextWidget(boss, lfont, xpos + col*myAudF->colWidth() + 7,
+    new StaticTextWidget(boss, lfont, xpos + col * myAudF->colWidth() + int(myAudF->colWidth() / 2.75),
                          ypos - lineHeight, fontWidth, fontHeight,
                          Common::Base::toString(col, Common::Base::F_16_1),
-                         kTextAlignLeft);
+                         TextAlign::Left);
   }
 
   // AudC registers
   xpos = 10;  ypos += lineHeight + 5;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, lwidth, fontHeight,
-                       "AUDC", kTextAlignLeft);
+                       "AUDC", TextAlign::Left);
   xpos += lwidth;
-  myAudC = new DataGridWidget(boss, nfont, xpos, ypos,
-                              2, 1, 2, 4, Common::Base::F_16);
+  myAudC = new DataGridWidget(boss, nfont, xpos + int(myAudF->colWidth() / 2.75), ypos,
+                              2, 1, 1, 4, Common::Base::F_16_1);
   myAudC->setTarget(this);
   myAudC->setID(kAUDCID);
-  myAudC->setEditable(false);
   addFocusWidget(myAudC);
 
   // AudV registers
   xpos = 10;  ypos += lineHeight + 5;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, lwidth, fontHeight,
-                       "AUDV", kTextAlignLeft);
+                       "AUDV", TextAlign::Left);
   xpos += lwidth;
-  myAudV = new DataGridWidget(boss, nfont, xpos, ypos,
-                              2, 1, 2, 4, Common::Base::F_16);
+  myAudV = new DataGridWidget(boss, nfont, xpos + int(myAudF->colWidth() / 2.75), ypos,
+                              2, 1, 1, 4, Common::Base::F_16_1);
   myAudV->setTarget(this);
   myAudV->setID(kAUDVID);
-  myAudV->setEditable(false);
   addFocusWidget(myAudV);
-}
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void AudioWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
-{
-  // TODO - implement this
+  myAudEffV = new StaticTextWidget(boss, lfont, myAudV->getRight() + fontWidth, myAudV->getTop() + 2,
+                                   "100% (eff. volume)");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -128,4 +125,112 @@ void AudioWidget::loadConfig()
     changed.push_back(state.aud[i] != oldstate.aud[i]);
   }
   myAudV->setList(alist, vlist, changed);
+
+  handleVolume();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void AudioWidget::handleVolume()
+{
+  stringstream s;
+
+  s << getEffectiveVolume() << "% (eff. volume)";
+  myAudEffV->setLabel(s.str());
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void AudioWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
+{
+  switch(cmd)
+  {
+    case DataGridWidget::kItemDataChangedCmd:
+      switch(id)
+      {
+        case kAUDFID:
+          changeFrequencyRegs();
+          break;
+
+        case kAUDCID:
+          changeControlRegs();
+          break;
+
+        case kAUDVID:
+          changeVolumeRegs();
+          break;
+
+        default:
+          cerr << "AudioWidget DG changed\n";
+          break;
+      }
+      break;
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void AudioWidget::changeFrequencyRegs()
+{
+  int addr = myAudF->getSelectedAddr();
+  int value = myAudF->getSelectedValue();
+
+  switch(addr)
+  {
+    case kAud0Addr:
+      instance().debugger().tiaDebug().audF0(value);
+      break;
+
+    case kAud1Addr:
+      instance().debugger().tiaDebug().audF1(value);
+      break;
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void AudioWidget::changeControlRegs()
+{
+  int addr = myAudC->getSelectedAddr();
+  int value = myAudC->getSelectedValue();
+
+  switch(addr)
+  {
+    case kAud0Addr:
+      instance().debugger().tiaDebug().audC0(value);
+      break;
+
+    case kAud1Addr:
+      instance().debugger().tiaDebug().audC1(value);
+      break;
+  }
+  handleVolume();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void AudioWidget::changeVolumeRegs()
+{
+  int addr = myAudV->getSelectedAddr();
+  int value = myAudV->getSelectedValue();
+
+  switch(addr)
+  {
+    case kAud0Addr:
+      instance().debugger().tiaDebug().audV0(value);
+      break;
+
+    case kAud1Addr:
+      instance().debugger().tiaDebug().audV1(value);
+      break;
+  }
+  handleVolume();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt32 AudioWidget::getEffectiveVolume()
+{
+  const int EFF_VOL[] = {
+     0,  6, 13, 18, 24, 29, 33, 38,
+    42, 46, 50, 54, 57, 60, 64, 67,
+    70, 72, 75, 78, 80, 82, 85, 87,
+    89, 91, 93, 95, 97, 98,100};
+
+  return EFF_VOL[(instance().debugger().tiaDebug().audC0() ? instance().debugger().tiaDebug().audV0() : 0) +
+    (instance().debugger().tiaDebug().audC1() ? instance().debugger().tiaDebug().audV1() : 0)];
 }
