@@ -8,13 +8,15 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
+#include "Debugger.hxx"
+#include "CartDebug.hxx"
 #include "CartFA2.hxx"
 #include "PopUpWidget.hxx"
 #include "CartFA2Widget.hxx"
@@ -33,7 +35,7 @@ CartridgeFA2Widget::CartridgeFA2Widget(
        << "256 bytes RAM @ $F000 - $F1FF\n"
        << "  $F100 - $F1FF (R), $F000 - $F0FF (W)\n"
        << "RAM can be loaded/saved to Harmony flash by accessing $FFF4\n"
-       << "Startup bank = " << cart.myStartBank << "\n";
+       << "Startup bank = " << cart.myStartBank << " or undetermined\n";
 
   // Eventually, we should query this from the debugger/disassembler
   for(uInt32 i = 0, offset = 0xFFC, spot = 0xFF5; i < cart.bankCount();
@@ -71,7 +73,7 @@ CartridgeFA2Widget::CartridgeFA2Widget(
 
   StaticTextWidget* t = new StaticTextWidget(boss, _font, xpos, ypos,
       _font.getStringWidth("Harmony Flash "),
-      myFontHeight, "Harmony Flash ", kTextAlignLeft);
+      myFontHeight, "Harmony Flash ", TextAlign::Left);
 
   xpos += t->getWidth() + 4;
   myFlashErase =
@@ -104,19 +106,21 @@ void CartridgeFA2Widget::saveOldState()
   {
     myOldState.internalram.push_back(myCart.myRAM[i]);
   }
+
+  myOldState.bank = myCart.getBank();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeFA2Widget::loadConfig()
 {
-  myBank->setSelectedIndex(myCart.myCurrentBank);
+  myBank->setSelectedIndex(myCart.getBank(), myCart.getBank() != myOldState.bank);
 
   CartDebugWidget::loadConfig();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeFA2Widget::handleCommand(CommandSender* sender,
-                                      int cmd, int data, int id)
+                                       int cmd, int data, int id)
 {
   switch(cmd)
   {
@@ -149,8 +153,8 @@ string CartridgeFA2Widget::bankState()
   static const char* const spot[] = {
     "$FFF5", "$FFF6", "$FFF7", "$FFF8", "$FFF9", "$FFFA", "$FFFB"
   };
-  buf << "Bank = " << std::dec << myCart.myCurrentBank
-      << ", hotspot = " << spot[myCart.myCurrentBank];
+  buf << "Bank = " << std::dec << myCart.getBank()
+      << ", hotspot = " << spot[myCart.getBank()];
 
   return buf.str();
 }
