@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -65,7 +65,12 @@ CartridgeSBWidget::CartridgeSBWidget(
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeSBWidget::loadConfig()
 {
-  myBank->setSelectedIndex(myCart.myCurrentBank);
+  Debugger& dbg = instance().debugger();
+  CartDebug& cart = dbg.cartDebug();
+  const CartState& state = static_cast<const CartState&>(cart.getState());
+  const CartState& oldstate = static_cast<const CartState&>(cart.getOldState());
+
+  myBank->setSelectedIndex(myCart.getBank(), state.bank != oldstate.bank);
 
   CartDebugWidget::loadConfig();
 }
@@ -88,8 +93,8 @@ string CartridgeSBWidget::bankState()
 {
   ostringstream& buf = buffer();
 
-  buf << "Bank = " << std::dec << myCart.myCurrentBank
-      << ", hotspot = $" << Common::Base::HEX2 << (myCart.myCurrentBank + 0x800);
+  buf << "Bank = " << std::dec << myCart.getBank()
+      << ", hotspot = $" << Common::Base::HEX2 << (myCart.getBank() + 0x800);
 
   return buf.str();
 }
