@@ -8,13 +8,14 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
+#include "Font.hxx"
 #include "DataGridOpsWidget.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -30,9 +31,9 @@ DataGridOpsWidget::DataGridOpsWidget(GuiObject* boss, const GUI::Font& font,
     _shiftLeftButton(nullptr),
     _shiftRightButton(nullptr)
 {
-  const int bwidth  = _font.getMaxCharWidth() * 4,
+  const int bwidth  = _font.getMaxCharWidth() * 4+2,
             bheight = _font.getFontHeight() + 3,
-            space = 6;
+            space = 4;
   int xpos, ypos;
 
   // Create operations buttons
