@@ -8,22 +8,24 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include "OSystem.hxx"
 #include "Widget.hxx"
 #include "Dialog.hxx"
+#include "Font.hxx"
 #include "Debugger.hxx"
 #include "FrameBuffer.hxx"
+#include "FBSurface.hxx"
 #include "DataGridWidget.hxx"
 #include "DataGridOpsWidget.hxx"
 #include "RamWidget.hxx"
 #include "ScrollBarWidget.hxx"
+#include "StellaKeys.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DataGridWidget::DataGridWidget(GuiObject* boss, const GUI::Font& font,
@@ -249,7 +251,7 @@ void DataGridWidget::setRange(int lower, int upper)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DataGridWidget::handleMouseDown(int x, int y, int button, int clickCount)
+void DataGridWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
 {
   if (!isEnabled())
     return;
@@ -275,7 +277,7 @@ void DataGridWidget::handleMouseDown(int x, int y, int button, int clickCount)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DataGridWidget::handleMouseUp(int x, int y, int button, int clickCount)
+void DataGridWidget::handleMouseUp(int x, int y, MouseButton b, int clickCount)
 {
   // If this was a double click and the mouse is still over the selected item,
   // send the double click command
@@ -330,8 +332,7 @@ bool DataGridWidget::handleText(char text)
 bool DataGridWidget::handleKeyDown(StellaKey key, StellaMod mod)
 {
   // Ignore all mod keys
-  if(instance().eventHandler().kbdControl(mod) ||
-     instance().eventHandler().kbdAlt(mod))
+  if(StellaModTest::isControl(mod) || StellaModTest::isAlt(mod))
     return true;
 
   bool handled = true;
@@ -413,7 +414,7 @@ bool DataGridWidget::handleKeyDown(StellaKey key, StellaMod mod)
         break;
 
       case KBDK_PAGEUP:
-        if(instance().eventHandler().kbdShift(mod) && _scrollBar)
+        if(StellaModTest::isShift(mod) && _scrollBar)
           handleMouseWheel(0, 0, -1);
         else if (_currentRow > 0)
         {
@@ -423,7 +424,7 @@ bool DataGridWidget::handleKeyDown(StellaKey key, StellaMod mod)
         break;
 
       case KBDK_PAGEDOWN:
-        if(instance().eventHandler().kbdShift(mod) && _scrollBar)
+        if(StellaModTest::isShift(mod) && _scrollBar)
           handleMouseWheel(0, 0, +1);
         else if (_currentRow < int(_rows) - 1)
         {
@@ -540,9 +541,9 @@ void DataGridWidget::handleCommand(CommandSender* sender, int cmd,
 {
   switch(cmd)
   {
-    case kSetPositionCmd:
+    case GuiObject::kSetPositionCmd:
       // Chain access; pass to parent
-      sendCommand(kSetPositionCmd, data, _id);
+      sendCommand(GuiObject::kSetPositionCmd, data, _id);
       break;
 
     case kDGZeroCmd:
@@ -583,11 +584,23 @@ void DataGridWidget::drawWidget(bool hilite)
 
   // Draw the internal grid and labels
   int linewidth = _cols * _colWidth;
+#ifndef FLAT_UI
   for (row = 0; row <= _rows; row++)
     s.hLine(_x, _y + (row * _rowHeight), _x + linewidth, kColor);
   int lineheight = _rows * _rowHeight;
   for (col = 0; col <= _cols; col++)
     s.vLine(_x + (col * _colWidth), _y, _y + lineheight, kColor);
+#else
+  s.frameRect(_x, _y, _w, _h, kColor);
+  for(row = 1; row <= _rows-1; row++)
+    s.hLine(_x+1, _y + (row * _rowHeight), _x + linewidth-1, kBGColorLo);
+
+  int lineheight = _rows * _rowHeight;
+  for(col = 1; col <= _cols-1; col++)
+    s.vLine(_x + (col * _colWidth), _y+1, _y + lineheight-1, kBGColorLo);
+#endif
+
+
 
   // Draw the list items
   for (row = 0; row < _rows; row++)
@@ -597,34 +610,37 @@ void DataGridWidget::drawWidget(bool hilite)
       int x = _x + 4 + (col * _colWidth);
       int y = _y + 2 + (row * _rowHeight);
       int pos = row*_cols + col;
+      uInt32 textColor = kTextColor;
 
       // Draw the selected item inverted, on a highlighted background.
       if (_currentRow == row && _currentCol == col &&
           _hasFocus && !_editMode)
+      {
         s.fillRect(x - 4, y - 2, _colWidth+1, _rowHeight+1, kTextColorHi);
+        textColor = kTextColorInv;
+      }
 
       if (_selectedItem == pos && _editMode)
       {
         adjustOffset();
-        s.drawString(_font, editString(), x, y, _colWidth, kTextColor,
-                     kTextAlignLeft, -_editScrollOffset, false);
+        s.drawString(_font, editString(), x, y, _colWidth, textColor,
+                     TextAlign::Left, -_editScrollOffset, false);
       }
       else
       {
-        uInt32 color = kTextColor;
         if(_changedList[pos])
         {
           s.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1, kDbgChangedColor);
 
           if(_hiliteList[pos])
-            color = kDbgColorHi;
+            textColor = kDbgColorHi;
           else
-            color = kDbgChangedTextColor;
+            textColor = kDbgChangedTextColor;
         }
         else if(_hiliteList[pos])
-          color = kDbgColorHi;
+          textColor = kDbgColorHi;
 
-        s.drawString(_font, _valueStringList[pos], x, y, _colWidth, color);
+        s.drawString(_font, _valueStringList[pos], x, y, _colWidth, textColor);
       }
     }
   }
