@@ -8,15 +8,19 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
+#include "Cart.hxx"
 #include "Widget.hxx"
 #include "Dialog.hxx"
+#include "Settings.hxx"
+#include "StellaKeys.hxx"
+#include "EventHandler.hxx"
 #include "TabWidget.hxx"
 #include "TiaInfoWidget.hxx"
 #include "TiaOutputWidget.hxx"
@@ -40,6 +44,7 @@
 #include "ConsoleMediumFont.hxx"
 #include "ConsoleMediumBFont.hxx"
 #include "StellaMediumFont.hxx"
+#include "OptionsDialog.hxx"
 #include "DebuggerDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -59,6 +64,8 @@ DebuggerDialog::DebuggerDialog(OSystem& osystem, DialogContainer& parent,
 
   // Inform the TIA output widget about its associated zoom widget
   myTiaOutput->setZoomWidget(myTiaZoom);
+
+  myOptions = make_unique<OptionsDialog>(osystem, parent, this, w, h, OptionsDialog::debugger);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -78,7 +85,7 @@ void DebuggerDialog::loadConfig()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::handleKeyDown(StellaKey key, StellaMod mod)
 {
-  if(key == KBDK_GRAVE && !instance().eventHandler().kbdShift(mod))
+  if(key == KBDK_GRAVE && !StellaModTest::isShift(mod))
   {
     // Swallow backtick, so we don't see it when exiting the debugger
     instance().eventHandler().enableTextEvents(false);
@@ -86,26 +93,68 @@ void DebuggerDialog::handleKeyDown(StellaKey key, StellaMod mod)
   else if(key == KBDK_F12)
   {
     instance().debugger().parser().run("savesnap");
+    return;
   }
-  else if(instance().eventHandler().kbdControl(mod))
+  else if(StellaModTest::isAlt(mod) && !StellaModTest::isControl(mod))
   {
     switch(key)
     {
-      case KBDK_R:
-        doRewind();
+      case KBDK_LEFT:  // Alt-left(-shift) rewinds 1(10) states
+        if(StellaModTest::isShift(mod))
+          doRewind10();
+        else
+          doRewind();
+        return;
+      case KBDK_RIGHT:  // Alt-right(-shift) unwinds 1(10) states
+        if(StellaModTest::isShift(mod))
+          doUnwind10();
+        else
+          doUnwind();
+        return;
+      case KBDK_DOWN:  // Alt-down rewinds to start of list
+        doRewindAll();
+        return;
+      case KBDK_UP:  // Alt-up rewinds to end of list
+        doUnwindAll();
+        return;
+      default:
         break;
+    }
+  }
+  else if(StellaModTest::isControl(mod))
+  {
+    switch(key)
+    {
+#if 0
+      case KBDK_R:
+        if(StellaModTest::isAlt(mod))
+          doRewindAll();
+        else if(StellaModTest::isShift(mod))
+          doRewind10();
+        else
+          doRewind();
+        return;
+      case KBDK_Y:
+        if(StellaModTest::isAlt(mod))
+          doUnwindAll();
+        else if(StellaModTest::isShift(mod))
+          doUnwind10();
+        else
+          doUnwind();
+        return;
+#endif
       case KBDK_S:
         doStep();
-        break;
+        return;
       case KBDK_T:
         doTrace();
-        break;
+        return;
       case KBDK_L:
         doScanlineAdvance();
-        break;
+        return;
       case KBDK_F:
         doAdvance();
-        break;
+        return;
       default:
         break;
     }
@@ -141,6 +190,10 @@ void DebuggerDialog::handleCommand(CommandSender* sender, int cmd,
       doRewind();
       break;
 
+    case kDDUnwindCmd:
+      doUnwind();
+      break;
+
     case kDDExitCmd:
       doExitDebugger();
       break;
@@ -149,6 +202,11 @@ void DebuggerDialog::handleCommand(CommandSender* sender, int cmd,
       doExitRom();
       break;
 
+    case kDDOptionsCmd:
+      myOptions->open();
+      loadConfig();
+      break;
+
     case RomWidget::kInvalidateListing:
       // Only do a full redraw if the disassembly tab is actually showing
       myRom->invalidate(myRomTab->getActiveTab() == 0);
@@ -189,6 +247,36 @@ void DebuggerDialog::doRewind()
   instance().debugger().parser().run("rewind");
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DebuggerDialog::doUnwind()
+{
+  instance().debugger().parser().run("unwind");
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DebuggerDialog::doRewind10()
+{
+  instance().debugger().parser().run("rewind #10");
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DebuggerDialog::doUnwind10()
+{
+  instance().debugger().parser().run("unwind #10");
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DebuggerDialog::doRewindAll()
+{
+  instance().debugger().parser().run("rewind #1000");
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DebuggerDialog::doUnwindAll()
+{
+  instance().debugger().parser().run("unwind #1000");
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::doExitDebugger()
 {
@@ -204,59 +292,57 @@ void DebuggerDialog::doExitRom()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::createFont()
 {
-  int fontstyle = instance().settings().getInt("dbg.fontstyle");
+  string fontSize = instance().settings().getString("dbg.fontsize");
+  int fontStyle = instance().settings().getInt("dbg.fontstyle");
 
-  // For now, these sizes are hardcoded based on actual font size
-  if(_w >= kLargeFontMinW && _h >= kLargeFontMinH)
+  if(fontSize == "large")
   {
-    // Large font doesn't use fontstyle at all
+    // Large font doesn't use fontStyle at all
     myLFont = make_unique<GUI::Font>(GUI::stellaMediumDesc);
     myNFont = make_unique<GUI::Font>(GUI::stellaMediumDesc);
   }
-  else if(_w >= kMediumFontMinW && _h >= kMediumFontMinH)
-  {
-    if(fontstyle == 1)
+  else if(fontSize == "medium")
     {
-      myLFont = make_unique<GUI::Font>(GUI::consoleMediumBDesc);
-      myNFont = make_unique<GUI::Font>(GUI::consoleMediumDesc);
+      switch(fontStyle)
+      {
+        case 1:
+          myLFont = make_unique<GUI::Font>(GUI::consoleMediumBDesc);
+          myNFont = make_unique<GUI::Font>(GUI::consoleMediumDesc);
+          break;
+        case 2:
+          myLFont = make_unique<GUI::Font>(GUI::consoleMediumDesc);
+          myNFont = make_unique<GUI::Font>(GUI::consoleMediumBDesc);
+          break;
+        case 3:
+          myLFont = make_unique<GUI::Font>(GUI::consoleMediumBDesc);
+          myNFont = make_unique<GUI::Font>(GUI::consoleMediumBDesc);
+          break;
+        default: // default to zero
+          myLFont = make_unique<GUI::Font>(GUI::consoleMediumDesc);
+          myNFont = make_unique<GUI::Font>(GUI::consoleMediumDesc);
+          break;
+      };
     }
-    else if(fontstyle == 2)
-    {
-      myLFont = make_unique<GUI::Font>(GUI::consoleMediumDesc);
-      myNFont = make_unique<GUI::Font>(GUI::consoleMediumBDesc);
-    }
-    else if(fontstyle == 3)
-    {
-      myLFont = make_unique<GUI::Font>(GUI::consoleMediumBDesc);
-      myNFont = make_unique<GUI::Font>(GUI::consoleMediumBDesc);
-    }
-    else  // default to zero
-    {
-      myLFont = make_unique<GUI::Font>(GUI::consoleMediumDesc);
-      myNFont = make_unique<GUI::Font>(GUI::consoleMediumDesc);
-    }
-  }
   else
   {
-    if(fontstyle == 1)
+    switch(fontStyle)
     {
-      myLFont = make_unique<GUI::Font>(GUI::consoleBDesc);
-      myNFont = make_unique<GUI::Font>(GUI::consoleDesc);
-    }
-    else if(fontstyle == 2)
-    {
-      myLFont = make_unique<GUI::Font>(GUI::consoleDesc);
-      myNFont = make_unique<GUI::Font>(GUI::consoleBDesc);
-    }
-    else if(fontstyle == 3)
-    {
-      myLFont = make_unique<GUI::Font>(GUI::consoleBDesc);
-      myNFont = make_unique<GUI::Font>(GUI::consoleBDesc);
-    }
-    else  // default to zero
-    {
-      myLFont = make_unique<GUI::Font>(GUI::consoleDesc);
-      myNFont = make_unique<GUI::Font>(GUI::consoleDesc);
+      case 1:
+        myLFont = make_unique<GUI::Font>(GUI::consoleBDesc);
+        myNFont = make_unique<GUI::Font>(GUI::consoleDesc);
+        break;
+      case 2:
+        myLFont = make_unique<GUI::Font>(GUI::consoleDesc);
+        myNFont = make_unique<GUI::Font>(GUI::consoleBDesc);
+        break;
+      case 3:
+        myLFont = make_unique<GUI::Font>(GUI::consoleBDesc);
+        myNFont = make_unique<GUI::Font>(GUI::consoleBDesc);
+        break;
+      default: // default to zero
+        myLFont = make_unique<GUI::Font>(GUI::consoleDesc);
+        myNFont = make_unique<GUI::Font>(GUI::consoleDesc);
+        break;
     }
   }
 }
@@ -296,9 +382,15 @@ void DebuggerDialog::addTabArea()
   int tabID;
 
   // The Prompt/console tab
+#ifndef FLAT_UI
   tabID = myTab->addTab(" Prompt ");
   myPrompt = new PromptWidget(myTab, *myNFont,
                               2, 2, widWidth, widHeight);
+#else
+  tabID = myTab->addTab("Prompt");
+  myPrompt = new PromptWidget(myTab, *myNFont,
+                              2, 2, widWidth - 4, widHeight);
+#endif
   myTab->setParentWidget(tabID, myPrompt);
   addToFocusList(myPrompt->getFocusList(), myTab, tabID);
 
@@ -353,8 +445,38 @@ void DebuggerDialog::addStatusArea()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::addRomArea()
 {
+  static uInt32 LEFT_ARROW[11] =
+  {
+    0b0000010,
+    0b0000110,
+    0b0001110,
+    0b0011110,
+    0b0111110,
+    0b1111110,
+    0b0111110,
+    0b0011110,
+    0b0001110,
+    0b0000110,
+    0b0000010
+  };
+  static uInt32 RIGHT_ARROW[11] =
+  {
+    0b0100000,
+    0b0110000,
+    0b0111000,
+    0b0111100,
+    0b0111110,
+    0b0111111,
+    0b0111110,
+    0b0111100,
+    0b0111000,
+    0b0110000,
+    0b0100000
+  };
+
   const GUI::Rect& r = getRomBounds();
-  const int vBorder = 4;
+  const int VBORDER = 4;
+  const string ELLIPSIS = "\x1d";
 
   int bwidth  = myLFont->getStringWidth("Frame +1 "),
       bheight = myLFont->getLineHeight() + 2;
@@ -374,16 +496,32 @@ void DebuggerDialog::addRomArea()
   new ButtonWidget(this, *myLFont, buttonX, buttonY,
                    bwidth, bheight, "Exit", kDDExitCmd);
 
-  bwidth = myLFont->getStringWidth("< ") + 4;
-  bheight = bheight * 5 + 4*4;
+  bwidth = bheight; // 7 + 12;
+  bheight = bheight * 3 + 4 * 2;
   buttonX -= (bwidth + 5);
   buttonY = r.top + 5;
+
   myRewindButton =
     new ButtonWidget(this, *myLFont, buttonX, buttonY,
-                     bwidth, bheight, "<", kDDRewindCmd);
+                     bwidth, bheight, LEFT_ARROW, 7, 11, kDDRewindCmd);
+
   myRewindButton->clearFlags(WIDGET_ENABLED);
 
-  int xpos = buttonX - 8*myLFont->getMaxCharWidth() - 20, ypos = 20;
+  buttonY += bheight + 4;
+  bheight = (myLFont->getLineHeight() + 2) * 2 + 4 * 1;
+
+  myUnwindButton =
+    new ButtonWidget(this, *myLFont, buttonX, buttonY,
+                     bwidth, bheight, RIGHT_ARROW, 7, 11, kDDUnwindCmd);
+  myUnwindButton->clearFlags(WIDGET_ENABLED);
+
+  int xpos = buttonX - 8*myLFont->getMaxCharWidth() - 20, ypos = 30;
+
+  bwidth = myLFont->getStringWidth("Options " + ELLIPSIS);
+  bheight = myLFont->getLineHeight() + 2;
+
+  new ButtonWidget(this, *myLFont, xpos, r.top + 5, bwidth, bheight, "Options" + ELLIPSIS, kDDOptionsCmd);
+
   DataGridOpsWidget* ops = new DataGridOpsWidget(this, *myLFont, xpos, ypos);
 
   int max_w = xpos - r.left - 10;
@@ -403,8 +541,8 @@ void DebuggerDialog::addRomArea()
   ////////////////////////////////////////////////////////////////////
   // Disassembly area
 
-  xpos = r.left + vBorder;  ypos += myRam->getHeight() + 5;
-  const int tabWidth  = r.width() - vBorder - 1;
+  xpos = r.left + VBORDER;  ypos += myRam->getHeight() + 5;
+  const int tabWidth  = r.width() - VBORDER - 1;
   const int tabHeight = r.height() - ypos - 1;
   int tabID;
 
