@@ -8,14 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include "OSystem.hxx"
 #include "EventHandler.hxx"
 #include "KeyboardWidget.hxx"
 
@@ -24,7 +23,7 @@ KeyboardWidget::KeyboardWidget(GuiObject* boss, const GUI::Font& font,
                                int x, int y, Controller& controller)
   : ControllerWidget(boss, font, x, y, controller)
 {
-  bool leftport = myController.jack() == Controller::Left;
+  bool leftport = isLeftPort();
   const string& label = leftport ? "Left (Keyboard)" : "Right (Keyboard)";
 
   const int fontHeight = font.getFontHeight();
@@ -32,13 +31,14 @@ KeyboardWidget::KeyboardWidget(GuiObject* boss, const GUI::Font& font,
   StaticTextWidget* t;
 
   t = new StaticTextWidget(boss, font, xpos, ypos+2, lwidth,
-                           fontHeight, label, kTextAlignLeft);
+                           fontHeight, label, TextAlign::Left);
 
   xpos += 30;  ypos += t->getHeight() + 20;
 
   for(int i = 0; i < 12; ++i)
   {
-    myBox[i] = new CheckboxWidget(boss, font, xpos, ypos, "", kCheckActionCmd);
+    myBox[i] = new CheckboxWidget(boss, font, xpos, ypos, "",
+                                  CheckboxWidget::kCheckActionCmd);
     myBox[i]->setID(i);
     myBox[i]->setTarget(this);
     xpos += myBox[i]->getWidth() + 5;
@@ -63,7 +63,7 @@ void KeyboardWidget::loadConfig()
 void KeyboardWidget::handleCommand(
     CommandSender* sender, int cmd, int data, int id)
 {
-  if(cmd == kCheckActionCmd)
+  if(cmd == CheckboxWidget::kCheckActionCmd)
     instance().eventHandler().handleEvent(myEvent[id], myBox[id]->getState());
 }
 
