@@ -8,16 +8,14 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include "OSystem.hxx"
 #include "Paddles.hxx"
-#include "EventHandler.hxx"
 #include "PaddleWidget.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -25,8 +23,8 @@ PaddleWidget::PaddleWidget(GuiObject* boss, const GUI::Font& font,
                            int x, int y, Controller& controller)
   : ControllerWidget(boss, font, x, y, controller)
 {
-  bool leftport = myController.jack() == Controller::Left;
-  const string& label = leftport ? "Left (Paddles)" : "Right (Paddles)";
+  bool leftport = isLeftPort();
+  const string& label = getHeader();
 
   const int fontWidth  = font.getMaxCharWidth(),
             fontHeight = font.getFontHeight(),
@@ -34,7 +32,7 @@ PaddleWidget::PaddleWidget(GuiObject* boss, const GUI::Font& font,
   int xpos = x, ypos = y, lwidth = font.getStringWidth("Right (Paddles)");
 
   new StaticTextWidget(boss, font, xpos, ypos+2, lwidth,
-                       fontHeight, label, kTextAlignLeft);
+                       fontHeight, label, TextAlign::Left);
 
   ypos += lineHeight + 20;
   const string& p0string = leftport ? "P0 pot " : "P2 pot ";
