@@ -8,20 +8,17 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <iostream>
-#include <fstream>
-#include <algorithm>
-
 #include "ScrollBarWidget.hxx"
-#include "FrameBuffer.hxx"
-#include "EventHandler.hxx"
+#include "FBSurface.hxx"
+#include "Font.hxx"
+#include "StellaKeys.hxx"
 #include "Version.hxx"
 #include "Debugger.hxx"
 #include "DebuggerDialog.hxx"
@@ -109,7 +106,7 @@ void PromptWidget::drawWidget(bool hilite)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PromptWidget::handleMouseDown(int x, int y, int button, int clickCount)
+void PromptWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
 {
 //  cerr << "PromptWidget::handleMouseDown\n";
 }
@@ -208,17 +205,19 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
       int lastDelimPos = -1;
       char delimiter = '\0';
 
-      char str[256];
+      char inputStr[256];
       for (int i = 0; i < len; i++)
       {
-        str[i] = buffer(_promptStartPos + i) & 0x7f;
-        if(strchr("{*@<> ", str[i]) != NULL )
+        inputStr[i] = buffer(_promptStartPos + i) & 0x7f;
+        // whitespace characters
+        if(strchr("{*@<> =[]()+-/&|!^~%", inputStr[i]))
         {
           lastDelimPos = i;
-          delimiter = str[i];
+          delimiter = inputStr[i];
         }
       }
-      str[len] = '\0';
+      inputStr[len] = '\0';
+      size_t strLen = len - lastDelimPos - 1;
 
       StringList list;
       string completionList;
@@ -226,9 +225,9 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
 
       if(lastDelimPos < 0)
       {
-        // no delimiters, do command completion:
+        // no delimiters, do only command completion:
         const DebuggerParser& parser = instance().debugger().parser();
-        parser.getCompletions(str, list);
+        parser.getCompletions(inputStr, list);
 
         if(list.size() < 1)
           break;
@@ -237,22 +236,26 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
         completionList = list[0];
         for(uInt32 i = 1; i < list.size(); ++i)
           completionList += " " + list[i];
-        prefix = getCompletionPrefix(list, str);
+        prefix = getCompletionPrefix(list);
       }
       else
       {
         // Special case for 'help' command
-        if(BSPF::startsWithIgnoreCase(str, "help"))
+        if(BSPF::startsWithIgnoreCase(inputStr, "help"))
         {
-          instance().debugger().parser().getCompletions(str + lastDelimPos + 1, list);
+          instance().debugger().parser().getCompletions(inputStr + lastDelimPos + 1, list);
         }
         else
         {
-          // we got a delimiter, so this must be a label or a function
-          const Debugger& dbg = instance().debugger();
+          // do not show ALL labels without any filter as it makes no sense
+          if(strLen > 0)
+          {
+            // we got a delimiter, so this must be a label or a function
+            const Debugger& dbg = instance().debugger();
 
-          dbg.cartDebug().getCompletions(str + lastDelimPos + 1, list);
-          dbg.getCompletions(str + lastDelimPos + 1, list);
+            dbg.cartDebug().getCompletions(inputStr + lastDelimPos + 1, list);
+            dbg.getCompletions(inputStr + lastDelimPos + 1, list);
+          }
         }
 
         if(list.size() < 1)
@@ -262,9 +265,11 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
         completionList = list[0];
         for(uInt32 i = 1; i < list.size(); ++i)
           completionList += " " + list[i];
-        prefix = getCompletionPrefix(list, str + lastDelimPos + 1);
+        prefix = getCompletionPrefix(list);
       }
 
+      // TODO: tab through list
+
       if(list.size() == 1)
       {
         // add to buffer as though user typed it (plus a space)
@@ -289,13 +294,21 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
 
         _promptStartPos = _currentPos;
 
-        for(int i = 0; i < lastDelimPos; i++)
-          putcharIntern(str[i]);
+        if(prefix.length() < strLen)
+        {
+          for(int i = 0; i < len; i++)
+            putcharIntern(inputStr[i]);
+        }
+        else
+        {
+          for(int i = 0; i < lastDelimPos; i++)
+            putcharIntern(inputStr[i]);
 
-        if(lastDelimPos > 0)
-          putcharIntern(delimiter);
+          if(lastDelimPos > 0)
+            putcharIntern(delimiter);
 
-        print(prefix);
+          print(prefix);
+        }
         _promptEndPos = _currentPos;
       }
       dirty = true;
@@ -316,7 +329,7 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
       break;
 
     case KBDK_PAGEUP:
-      if (instance().eventHandler().kbdShift(mod))
+      if (StellaModTest::isShift(mod))
       {
         // Don't scroll up when at top of buffer
         if(_scrollLine < _linesPerPage)
@@ -332,7 +345,7 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
       break;
 
     case KBDK_PAGEDOWN:
-      if (instance().eventHandler().kbdShift(mod))
+      if (StellaModTest::isShift(mod))
       {
         // Don't scroll down when at bottom of buffer
         if(_scrollLine >= _promptEndPos / _lineWidth)
@@ -348,7 +361,7 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
       break;
 
     case KBDK_HOME:
-      if (instance().eventHandler().kbdShift(mod))
+      if (StellaModTest::isShift(mod))
       {
         _scrollLine = _firstLineInBuffer + _linesPerPage - 1;
         updateScrollBuffer();
@@ -360,7 +373,7 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
       break;
 
     case KBDK_END:
-      if (instance().eventHandler().kbdShift(mod))
+      if (StellaModTest::isShift(mod))
       {
         _scrollLine = _promptEndPos / _lineWidth;
         if (_scrollLine < _linesPerPage - 1)
@@ -374,7 +387,7 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
       break;
 
     case KBDK_UP:
-      if (instance().eventHandler().kbdShift(mod))
+      if (StellaModTest::isShift(mod))
       {
         if(_scrollLine <= _firstLineInBuffer + _linesPerPage - 1)
           break;
@@ -389,7 +402,7 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
       break;
 
     case KBDK_DOWN:
-      if (instance().eventHandler().kbdShift(mod))
+      if (StellaModTest::isShift(mod))
       {
         // Don't scroll down when at bottom of buffer
         if(_scrollLine >= _promptEndPos / _lineWidth)
@@ -419,11 +432,11 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
       break;
 
     default:
-      if (instance().eventHandler().kbdControl(mod))
+      if (StellaModTest::isControl(mod))
       {
         specialKeys(key);
       }
-      else if (instance().eventHandler().kbdAlt(mod))
+      else if (StellaModTest::isAlt(mod))
       {
         // Placeholder only - this will never be reached
       }
@@ -475,7 +488,7 @@ void PromptWidget::handleCommand(CommandSender* sender, int cmd,
 {
   switch (cmd)
   {
-    case kSetPositionCmd:
+    case GuiObject::kSetPositionCmd:
       int newPos = int(data) + _linesPerPage - 1 + _firstLineInBuffer;
       if (newPos != _scrollLine)
       {
@@ -495,20 +508,28 @@ void PromptWidget::loadConfig()
   // Show the prompt the first time we draw this widget
   if(_firstTime)
   {
+    _firstTime = false;
+
     // Display greetings & prompt
     string version = string("Stella ") + STELLA_VERSION + "\n";
     print(version);
     print(PROMPT);
 
     // Take care of one-time debugger stuff
-    print(instance().debugger().autoExec());
+    // fill the history from the saved breaks, traps and watches commands
+    StringList history;
+    print(instance().debugger().autoExec(&history));
+    for(uInt32 i = 0; i < history.size(); i++)
+    {
+      addToHistory(history[i].c_str());
+    }
+    history.clear();
     print(instance().debugger().cartDebug().loadConfigFile() + "\n");
     print(instance().debugger().cartDebug().loadListFile() + "\n");
     print(instance().debugger().cartDebug().loadSymbolFile() + "\n");
     print(PROMPT);
 
     _promptStartPos = _promptEndPos = _currentPos;
-    _firstTime = false;
     _exitedEarly = false;
   }
   else if(_exitedEarly)
@@ -697,8 +718,9 @@ void PromptWidget::historyScroll(int direction)
 
   // Advance to the next line in the history
   int line = _historyLine + direction;
-  if ((direction < 0 && line < 0) || (direction > 0 && line > _historySize))
-    return;
+  if(line < 0)
+    line += _historySize + 1;
+  line %= (_historySize + 1);
 
   // If they press arrow-up with anything in the buffer, search backwards
   // in the history.
@@ -876,21 +898,21 @@ void PromptWidget::scrollToCurrent()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool PromptWidget::saveBuffer(string& filename)
+bool PromptWidget::saveBuffer(const FilesystemNode& file)
 {
-  ofstream out(filename);
+  ofstream out(file.getPath());
   if(!out.is_open())
     return false;
 
-  for(int start=0; start<_promptStartPos; start+=_lineWidth)
+  for(int start = 0; start < _promptStartPos; start += _lineWidth)
   {
-    int end = start+_lineWidth-1;
+    int end = start + _lineWidth - 1;
 
-    // look for first non-space, printing char from end of line
+    // Look for first non-space, printing char from end of line
     while( char(_buffer[end] & 0xff) <= ' ' && end >= start)
       end--;
 
-    // spit out the line minus its trailing junk.
+    // Spit out the line minus its trailing junk
     // Strip off any color/inverse bits
     for(int j = start; j <= end; ++j)
       out << char(_buffer[j] & 0xff);
@@ -903,28 +925,26 @@ bool PromptWidget::saveBuffer(string& filename)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string PromptWidget::getCompletionPrefix(const StringList& completions, string prefix)
+string PromptWidget::getCompletionPrefix(const StringList& completions)
 {
-  // Search for prefix in every string, progressively growing it
-  // Once a mismatch is found or length is past one of the strings, we're done
-  // We *could* use the longest common string algorithm, but for the lengths
-  // of the strings we're dealing with, it's probably not worth it
-  for(;;)
+  // Find the number of characters matching for each of the completions provided
+  for(uInt32 len = 1;; len++)
   {
-    for(const auto& s: completions)
+    for(uInt32 i = 0; i < completions.size(); i++)
     {
-      if(s.length() < prefix.length())
-        return prefix;  // current prefix is the best we're going to get
-      else if(!BSPF::startsWithIgnoreCase(s, prefix))
+      string s1 = completions[i];
+      if(s1.length() < len)
       {
-        prefix.erase(prefix.length()-1);
-        return prefix;
+        return s1.substr(0, len - 1);
+      }
+      string find = s1.substr(0, len);
+
+      for(uInt32 j = i + 1; j < completions.size(); j++)
+      {
+        if(!BSPF::startsWithIgnoreCase(completions[j], find))
+          return s1.substr(0, len - 1);
       }
     }
-    if(completions[0].length() > prefix.length())
-      prefix = completions[0].substr(0, prefix.length() + 1);
-    else
-      return prefix;
   }
 }
 
