@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -21,6 +21,7 @@
 #include "FrameBuffer.hxx"
 #include "GuiObject.hxx"
 #include "OSystem.hxx"
+#include "Debugger.hxx"
 #include "RiotDebug.hxx"
 #include "PopUpWidget.hxx"
 #include "ToggleBitWidget.hxx"
@@ -35,6 +36,9 @@
 #include "KeyboardWidget.hxx"
 #include "AtariVoxWidget.hxx"
 #include "SaveKeyWidget.hxx"
+#include "AmigaMouseWidget.hxx"
+#include "AtariMouseWidget.hxx"
+#include "TrakBallWidget.hxx"
 
 #include "RiotWidget.hxx"
 
@@ -63,7 +67,7 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& lfont,
 
 #define CREATE_IO_REGS(desc, bits, bitsID, editable)                     \
   t = new StaticTextWidget(boss, lfont, xpos, ypos+2, lwidth, fontHeight,\
-                           desc, kTextAlignLeft);                        \
+                           desc, TextAlign::Left);                        \
   xpos += t->getWidth() + 5;                                             \
   bits = new ToggleBitWidget(boss, nfont, xpos, ypos, 8, 1);             \
   bits->setTarget(this);                                                 \
@@ -82,7 +86,7 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& lfont,
 
   // SWCHA bits in 'peek' mode
   xpos = 10;  ypos += lineHeight + 5;
-  CREATE_IO_REGS("SWCHA(R)", mySWCHAReadBits, 0, false);
+  CREATE_IO_REGS("SWCHA(R)", mySWCHAReadBits, kSWCHARBitsID, true);
 
   // SWCHB bits in 'poke' mode
   xpos = 10;  ypos += 2 * lineHeight;
@@ -102,7 +106,7 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& lfont,
   for(int row = 0; row < 4; ++row)
   {
     t = new StaticTextWidget(boss, lfont, xpos, ypos + row*lineHeight + 2,
-                             lwidth, fontHeight, writeNames[row], kTextAlignLeft);
+                             lwidth, fontHeight, writeNames[row], TextAlign::Left);
   }
   xpos += t->getWidth() + 5;
   myTimWrite = new DataGridWidget(boss, nfont, xpos, ypos, 1, 4, 2, 8, Common::Base::F_16);
@@ -116,7 +120,7 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& lfont,
   for(int row = 0; row < 4; ++row)
   {
     t = new StaticTextWidget(boss, lfont, xpos, ypos + row*lineHeight + 2,
-                             10*fontWidth, fontHeight, readNames[row], kTextAlignLeft);
+                             10*fontWidth, fontHeight, readNames[row], TextAlign::Left);
   }
   xpos += t->getWidth() + 5;
   myTimRead = new DataGridWidget(boss, nfont, xpos, ypos, 1, 4, 8, 32, Common::Base::F_16);
@@ -138,7 +142,7 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& lfont,
   for(int row = 0; row < 3; ++row)
   {
     new StaticTextWidget(boss, lfont, xpos, ypos + row*lineHeight + 2,
-                         5*fontWidth, fontHeight, contLeftReadNames[row], kTextAlignLeft);
+                         5*fontWidth, fontHeight, contLeftReadNames[row], TextAlign::Left);
   }
   xpos += 5*fontWidth + 5;
   myLeftINPT = new DataGridWidget(boss, nfont, xpos, ypos, 1, 3, 2, 8, Common::Base::F_16);
@@ -151,7 +155,7 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& lfont,
   for(int row = 0; row < 3; ++row)
   {
     new StaticTextWidget(boss, lfont, xpos, ypos + row*lineHeight + 2,
-                         5*fontWidth, fontHeight, contRightReadNames[row], kTextAlignLeft);
+                         5*fontWidth, fontHeight, contRightReadNames[row], TextAlign::Left);
   }
   xpos += 5*fontWidth + 5;
   myRightINPT = new DataGridWidget(boss, nfont, xpos, ypos, 1, 3, 2, 8, Common::Base::F_16);
@@ -196,50 +200,35 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& lfont,
   myTVType->setTarget(this);
   addFocusWidget(myTVType);
 
+  // 2600/7800 mode
+  lwidth = lfont.getStringWidth("Console") + 29;
+  pwidth = lfont.getStringWidth("Atari 2600") + 4;
+  new StaticTextWidget(boss, lfont, 10, ypos+1, "Console");
+  myConsole = new EditTextWidget(boss, lfont, 10 + lwidth, ypos - 1, pwidth, lineHeight);
+  myConsole->setEditable(false, true);
+  addFocusWidget(myConsole);
+
   // Select and Reset
-  xpos += myP0Diff->getWidth() + 20;  ypos = col2_ypos + lineHeight;
+  xpos += myP0Diff->getWidth() + 20;  ypos = col2_ypos + 1;
   mySelect = new CheckboxWidget(boss, lfont, xpos, ypos, "Select",
-                                kCheckActionCmd);
+                                CheckboxWidget::kCheckActionCmd);
   mySelect->setID(kSelectID);
   mySelect->setTarget(this);
   addFocusWidget(mySelect);
-  ypos += mySelect->getHeight() + 5;
+
+  ypos += myP0Diff->getHeight() + 5;
   myReset = new CheckboxWidget(boss, lfont, xpos, ypos, "Reset",
-                               kCheckActionCmd);
+                               CheckboxWidget::kCheckActionCmd);
   myReset->setID(kResetID);
   myReset->setTarget(this);
   addFocusWidget(myReset);
 
-  // Randomize items
-  xpos = 10;  ypos += 3*lineHeight;
-  new StaticTextWidget(boss, lfont, xpos, ypos,
-      lfont.getStringWidth("When loading a ROM:"), fontHeight,
-      "When loading a ROM:", kTextAlignLeft);
-
-  // Randomize RAM
-  xpos += 30;  ypos += lineHeight + 4;
-  myRandomizeRAM = new CheckboxWidget(boss, lfont, xpos, ypos+1,
-      "Randomize zero-page and extended RAM", kCheckActionCmd);
-  myRandomizeRAM->setID(kRandRAMID);
-  myRandomizeRAM->setTarget(this);
-  addFocusWidget(myRandomizeRAM);
-
-  // Randomize CPU
-  ypos += lineHeight + 8;
-  lwidth = lfont.getStringWidth("Randomize CPU ");
-  new StaticTextWidget(boss, lfont, xpos, ypos+1,
-      lwidth, fontHeight, "Randomize CPU ", kTextAlignLeft);
-  xpos += lwidth + 10;
-  const char* const cpuregs[] = { "SP", "A", "X", "Y", "PS" };
-  for(int i = 0; i < 5; ++i)
-  {
-    myRandomizeCPU[i] = new CheckboxWidget(boss, lfont, xpos, ypos+1,
-      cpuregs[i], kCheckActionCmd);
-    myRandomizeCPU[i]->setID(kRandCPUID);
-    myRandomizeCPU[i]->setTarget(this);
-    addFocusWidget(myRandomizeCPU[i]);
-    xpos += CheckboxWidget::boxSize() + lfont.getStringWidth("XX") + 20;
-  }
+  ypos += myP0Diff->getHeight() + 5;
+  myPause = new CheckboxWidget(boss, lfont, xpos, ypos, "Pause",
+                               CheckboxWidget::kCheckActionCmd);
+  myPause->setID(kPauseID);
+  myPause->setTarget(this);
+  addFocusWidget(myPause);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -299,8 +288,8 @@ void RiotWidget::loadConfig()
   myRightINPT->setList(alist, vlist, changed);
 
   // Update TIA VBLANK bits
-  myINPTLatch->setState(riot.vblank(6));
-  myINPTDump->setState(riot.vblank(7));
+  myINPTLatch->setState(riot.vblank(6), state.INPTLatch != oldstate.INPTLatch);
+  myINPTDump->setState(riot.vblank(7), state.INPTDump != oldstate.INPTDump);
 
   // Update timer write registers
   alist.clear();  vlist.clear();  changed.clear();
@@ -328,21 +317,22 @@ void RiotWidget::loadConfig()
 
   // Console switches (inverted, since 'selected' in the UI
   // means 'grounded' in the system)
-  myP0Diff->setSelectedIndex(riot.diffP0());
-  myP1Diff->setSelectedIndex(riot.diffP1());
-  myTVType->setSelectedIndex(riot.tvType());
-  mySelect->setState(!riot.select());
-  myReset->setState(!riot.reset());
+  myP0Diff->setSelectedIndex(riot.diffP0(), state.swchbReadBits[1] != oldstate.swchbReadBits[1]);
+  myP1Diff->setSelectedIndex(riot.diffP1(), state.swchbReadBits[0] != oldstate.swchbReadBits[0]);
+
+  bool devSettings = instance().settings().getBool("dev.settings");
+  myConsole->setText(instance().settings().getString(devSettings ? "dev.console" : "plr.console") == "7800" ? "Atari 7800" : "Atari 2600");
+  myConsole->setEditable(false, true);
+
+  myTVType->setSelectedIndex(riot.tvType(), state.swchbReadBits[4] != oldstate.swchbReadBits[4]);
+  myPause->setState(!riot.tvType(), state.swchbReadBits[4] != oldstate.swchbReadBits[4]);
+  mySelect->setState(!riot.select(), state.swchbReadBits[6] != oldstate.swchbReadBits[6]);
+  myReset->setState(!riot.reset(), state.swchbReadBits[7] != oldstate.swchbReadBits[7]);
 
   myLeftControl->loadConfig();
   myRightControl->loadConfig();
 
-  myRandomizeRAM->setState(instance().settings().getBool("ramrandom"));
-
-  const string& cpurandom = instance().settings().getString("cpurandom");
-  const char* const cpuregs[] = { "S", "A", "X", "Y", "P" };
-  for(int i = 0; i < 5; ++i)
-    myRandomizeCPU[i]->setState(BSPF::containsIgnoreCase(cpurandom, cpuregs[i]));
+  handleConsole();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -395,10 +385,24 @@ void RiotWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
           value = Debugger::get_bits(mySWBCNTBits->getState());
           riot.swbcnt(value & 0xff);
           break;
+        case kSWCHARBitsID:
+        {
+          // TODO: Check if there is a nicer way to do this
+          value = Debugger::get_bits(mySWCHAReadBits->getState());
+          riot.controller(Controller::Left).set(Controller::One, value & 0b00010000);
+          riot.controller(Controller::Left).set(Controller::Two, value & 0b00100000);
+          riot.controller(Controller::Left).set(Controller::Three, value & 0b01000000);
+          riot.controller(Controller::Left).set(Controller::Four, value & 0b10000000);
+          riot.controller(Controller::Right).set(Controller::One, value & 0b00000001);
+          riot.controller(Controller::Right).set(Controller::Two, value & 0b00000010);
+          riot.controller(Controller::Right).set(Controller::Three, value & 0b00000100);
+          riot.controller(Controller::Right).set(Controller::Four, value & 0b00001000);
+          break;
+        }
       }
       break;
 
-    case kCheckActionCmd:
+    case CheckboxWidget::kCheckActionCmd:
       switch(id)
       {
         case kSelectID:
@@ -407,11 +411,8 @@ void RiotWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
         case kResetID:
           riot.reset(!myReset->getState());
           break;
-        case kRandRAMID:
-          instance().settings().setValue("ramrandom", myRandomizeRAM->getState());
-          break;
-        case kRandCPUID:
-          handleRandomCPU();
+        case kPauseID:
+          handleConsole();
           break;
       }
       break;
@@ -425,7 +426,7 @@ void RiotWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       break;
 
     case kTVTypeChanged:
-      riot.tvType(myTVType->getSelected());
+      handleConsole();
       break;
   }
 }
@@ -436,16 +437,14 @@ ControllerWidget* RiotWidget::addControlWidget(GuiObject* boss, const GUI::Font&
 {
   switch(controller.type())
   {
-    case Controller::AmigaMouse:  // TODO - implement this
-      return new NullControlWidget(boss, font, x, y, controller);
-    case Controller::AtariMouse:  // TODO - implement this
-      return new NullControlWidget(boss, font, x, y, controller);
+    case Controller::AmigaMouse:
+      return new AmigaMouseWidget(boss, font, x, y, controller);
+    case Controller::AtariMouse:
+      return new AtariMouseWidget(boss, font, x, y, controller);
     case Controller::AtariVox:
       return new AtariVoxWidget(boss, font, x, y, controller);
     case Controller::BoosterGrip:
       return new BoosterWidget(boss, font, x, y, controller);
-    case Controller::CompuMate:   // TODO - implement this
-      return new NullControlWidget(boss, font, x, y, controller);
     case Controller::Driving:
       return new DrivingWidget(boss, font, x, y, controller);
     case Controller::Genesis:
@@ -454,29 +453,31 @@ ControllerWidget* RiotWidget::addControlWidget(GuiObject* boss, const GUI::Font&
       return new JoystickWidget(boss, font, x, y, controller);
     case Controller::Keyboard:
       return new KeyboardWidget(boss, font, x, y, controller);
-    case Controller::KidVid:      // TODO - implement this
-      return new NullControlWidget(boss, font, x, y, controller);
-    case Controller::MindLink:    // TODO - implement this
-      return new NullControlWidget(boss, font, x, y, controller);
+//    case Controller::KidVid:      // TODO - implement this
+//    case Controller::MindLink:    // TODO - implement this
     case Controller::Paddles:
       return new PaddleWidget(boss, font, x, y, controller);
     case Controller::SaveKey:
       return new SaveKeyWidget(boss, font, x, y, controller);
-    case Controller::TrakBall:    // TODO - implement this
-      return new NullControlWidget(boss, font, x, y, controller);
+    case Controller::TrakBall:
+      return new TrakBallWidget(boss, font, x, y, controller);
     default:
-      return nullptr;  // make compiler happy
+      return new NullControlWidget(boss, font, x, y, controller);
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void RiotWidget::handleRandomCPU()
+void RiotWidget::handleConsole()
 {
-  string cpurandom;
-  const char* const cpuregs[] = { "S", "A", "X", "Y", "P" };
-  for(int i = 0; i < 5; ++i)
-    if(myRandomizeCPU[i]->getState())
-      cpurandom += cpuregs[i];
+  RiotDebug& riot = instance().debugger().riotDebug();
+  bool devSettings = instance().settings().getBool("dev.settings");
+  bool is7800 = instance().settings().getString(devSettings ? "dev.console" : "plr.console") == "7800";
 
-  instance().settings().setValue("cpurandom", cpurandom);
+  myTVType->setEnabled(!is7800);
+  myPause->setEnabled(is7800);
+  if(is7800)
+    myTVType->setSelectedIndex(myPause->getState() ? 0 : 1);
+  else
+    myPause->setState(myTVType->getSelected() == 0);
+  riot.tvType(myTVType->getSelected());
 }
