@@ -8,20 +8,20 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <sstream>
-
+#include "Settings.hxx"
 #include "Debugger.hxx"
 #include "CartDebug.hxx"
 #include "DiStella.hxx"
 #include "CpuDebug.hxx"
 #include "GuiObject.hxx"
+#include "Font.hxx"
 #include "DataGridWidget.hxx"
 #include "EditTextWidget.hxx"
 #include "PopUpWidget.hxx"
@@ -45,11 +45,12 @@ RomWidget::RomWidget(GuiObject* boss, const GUI::Font& lfont, const GUI::Font& n
   t = new StaticTextWidget(boss, lfont, xpos, ypos,
                            lfont.getStringWidth("Bank"),
                            lfont.getFontHeight(),
-                           "Bank", kTextAlignLeft);
+                           "Bank", TextAlign::Left);
 
   xpos += t->getWidth() + 5;
   myBank = new EditTextWidget(boss, nfont, xpos, ypos-2,
                               _w - 2 - xpos, nfont.getLineHeight());
+  myBank->setEditable(false);
 
   // Create rom listing
   xpos = x;  ypos += myBank->getHeight() + 4;
@@ -122,18 +123,18 @@ void RomWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
     case RomListWidget::kTentativeCodeCmd:
     {
       // 'data' is the boolean value
-      DiStella::settings.resolve_code = data;
+      DiStella::settings.resolveCode = data;
       instance().settings().setValue("dis.resolve",
-          DiStella::settings.resolve_code);
+          DiStella::settings.resolveCode);
       invalidate();
       break;
     }
 
     case RomListWidget::kPCAddressesCmd:
       // 'data' is the boolean value
-      DiStella::settings.show_addresses = data;
+      DiStella::settings.showAddresses = data;
       instance().settings().setValue("dis.showaddr",
-          DiStella::settings.show_addresses);
+          DiStella::settings.showAddresses);
       invalidate();
       break;
 
@@ -141,12 +142,12 @@ void RomWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       // 'data' is the boolean value
       if(data)
       {
-        DiStella::settings.gfx_format = Common::Base::F_2;
+        DiStella::settings.gfxFormat = Common::Base::F_2;
         instance().settings().setValue("dis.gfxformat", "2");
       }
       else
       {
-        DiStella::settings.gfx_format = Common::Base::F_16;
+        DiStella::settings.gfxFormat = Common::Base::F_16;
         instance().settings().setValue("dis.gfxformat", "16");
       }
       invalidate();
@@ -154,9 +155,9 @@ void RomWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
 
     case RomListWidget::kAddrRelocationCmd:
       // 'data' is the boolean value
-      DiStella::settings.rflag = data;
+      DiStella::settings.rFlag = data;
       instance().settings().setValue("dis.relocate",
-          DiStella::settings.rflag);
+          DiStella::settings.rFlag);
       invalidate();
       break;
   }
@@ -227,3 +228,9 @@ void RomWidget::patchROM(int disasm_line, const string& bytes,
     Common::Base::setFormat(oldbase);
   }
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void RomWidget::scrollTo(int line)
+{
+  myRomList->setSelected(line);
+}
