@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -19,12 +19,14 @@
 
 #include "OSystem.hxx"
 #include "FrameBuffer.hxx"
+#include "FBSurface.hxx"
 #include "Widget.hxx"
 #include "GuiObject.hxx"
 #include "ContextMenu.hxx"
 #include "TiaZoomWidget.hxx"
 #include "Debugger.hxx"
 #include "DebuggerParser.hxx"
+#include "PNGLibrary.hxx"
 #include "TIADebug.hxx"
 #include "TIASurface.hxx"
 #include "TIA.hxx"
@@ -43,7 +45,7 @@ TiaOutputWidget::TiaOutputWidget(GuiObject* boss, const GUI::Font& font,
   // Create context menu for commands
   VariantList l;
   VarList::push_back(l, "Fill to scanline", "scanline");
-  VarList::push_back(l, "Set breakpoint", "bp");
+  VarList::push_back(l, "Toggle breakpoint", "bp");
   VarList::push_back(l, "Set zoom position", "zoom");
   VarList::push_back(l, "Save snapshot", "snap");
   myMenu = make_unique<ContextMenu>(this, font, l);
@@ -56,13 +58,19 @@ void TiaOutputWidget::loadConfig()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void TiaOutputWidget::saveSnapshot()
+void TiaOutputWidget::saveSnapshot(int execDepth, const string& execPrefix)
 {
-  int number = int(instance().getTicks() / 1000);
+  if (execDepth > 0) {
+    drawWidget(false);
+  }
   ostringstream sspath;
   sspath << instance().snapshotSaveDir()
-         << instance().console().properties().get(Cartridge_Name)
-         << "_dbg_" << std::hex << std::setw(8) << std::setfill('0') << number << ".png";
+         << instance().console().properties().get(Cartridge_Name);
+  sspath << "_dbg_";
+  if (execDepth > 0 && !execPrefix.empty()) {
+    sspath << execPrefix << "_";
+  }
+  sspath << std::hex << std::setw(8) << std::setfill('0') << uInt32(instance().getTicks()/1000) << ".png";
 
   const uInt32 width  = instance().console().tia().width(),
                height = instance().console().tia().height();
@@ -78,14 +86,16 @@ void TiaOutputWidget::saveSnapshot()
   {
     message = e.what();
   }
-  instance().frameBuffer().showMessage(message);
+  if (execDepth == 0) {
+    instance().frameBuffer().showMessage(message);
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void TiaOutputWidget::handleMouseDown(int x, int y, int button, int clickCount)
+void TiaOutputWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
 {
   // Grab right mouse button for command context menu
-  if(button == 2)
+  if(b == MouseButton::RIGHT)
   {
     myClickX = x;
     myClickY = y;
@@ -115,7 +125,8 @@ void TiaOutputWidget::handleCommand(CommandSender* sender, int cmd, int data, in
         if(lines > 0)
         {
           command << "scanline #" << lines;
-          instance().debugger().parser().run(command.str());
+          string message = instance().debugger().parser().run(command.str());
+          instance().frameBuffer().showMessage(message);
         }
       }
       else if(rmb == "bp")
@@ -123,7 +134,8 @@ void TiaOutputWidget::handleCommand(CommandSender* sender, int cmd, int data, in
         ostringstream command;
         int scanline = myClickY + ystart;
         command << "breakif _scan==#" << scanline;
-        instance().debugger().parser().run(command.str());
+        string message = instance().debugger().parser().run(command.str());
+        instance().frameBuffer().showMessage(message);
       }
       else if(rmb == "zoom")
       {
@@ -147,6 +159,9 @@ void TiaOutputWidget::drawWidget(bool hilite)
                height = instance().console().tia().height();
   FBSurface& s = dialog().surface();
 
+  s.vLine(_x + _w + 1, _y, height, kColor);
+  s.hLine(_x, _y + height + 1, _x +_w + 1, kColor);
+
   // Get current scanline position
   // This determines where the frame greying should start, and where a
   // scanline 'pointer' should be drawn
@@ -164,10 +179,10 @@ void TiaOutputWidget::drawWidget(bool hilite)
       *line_ptr++ = pixel;
       *line_ptr++ = pixel;
     }
-    s.drawPixels(myLineBuffer, _x, _y+y, width << 1);
+    s.drawPixels(myLineBuffer, _x + 1, _y + 1 + y, width << 1);
   }
 
   // Show electron beam position
   if(visible && scanx < width && scany+2u < height)
-    s.fillRect(_x+(scanx<<1), _y+scany, 3, 3, kBtnTextColor);
+    s.fillRect(_x + 1 + (scanx<<1), _y + 1 + scany, 3, 3, kColorInfo);
 }
