@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -19,8 +19,10 @@
 #include "DataGridWidget.hxx"
 #include "EditTextWidget.hxx"
 #include "FrameBuffer.hxx"
+#include "Font.hxx"
 #include "GuiObject.hxx"
 #include "OSystem.hxx"
+#include "Debugger.hxx"
 #include "CartDebug.hxx"
 #include "TIA.hxx"
 #include "TIADebug.hxx"
@@ -50,7 +52,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   for(int row = 0; row < 4; ++row)
   {
     new StaticTextWidget(boss, lfont, xpos, ypos + row*lineHeight + 2,
-                         6*fontWidth, fontHeight, regNames[row], kTextAlignLeft);
+                         6*fontWidth, fontHeight, regNames[row], TextAlign::Left);
   }
   xpos += 6*fontWidth + 8;
   myColorRegs = new DataGridWidget(boss, nfont, xpos, ypos,
@@ -90,7 +92,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   {
     ypos += lineHeight;
     t = new StaticTextWidget(boss, lfont, xpos, ypos + 2, 2*fontWidth, fontHeight,
-                             dbgLabels[row], kTextAlignLeft);
+                             dbgLabels[row], TextAlign::Left);
     myFixedColors[row] = new ColorWidget(boss, nfont, xpos + 2 + t->getWidth() + 4,
                                ypos + 2, uInt32(1.5*lineHeight), lineHeight - 4);
     myFixedColors[row]->setTarget(this);
@@ -101,7 +103,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   {
     ypos += lineHeight;
     t = new StaticTextWidget(boss, lfont, xpos, ypos + 2, 2*fontWidth, fontHeight,
-                             dbgLabels[row], kTextAlignLeft);
+                             dbgLabels[row], TextAlign::Left);
     myFixedColors[row] = new ColorWidget(boss, nfont, xpos + 2 + t->getWidth() + 4,
                                ypos + 2, uInt32(1.5*lineHeight), lineHeight - 4);
     myFixedColors[row]->setTarget(this);
@@ -123,14 +125,13 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
     // Add vertical label
     new StaticTextWidget(boss, lfont, xpos, ypos + row*(lineHeight+3),
                          2*fontWidth, fontHeight,
-                         rowLabel[row], kTextAlignLeft);
+                         rowLabel[row], TextAlign::Left);
 
     for(uInt32 col = 0; col < 5 - row; ++col)
     {
-      myCollision[idx] = new CheckboxWidget(boss, lfont, collX, collY, "");
+      myCollision[idx] = new CheckboxWidget(boss, lfont, collX, collY, "", CheckboxWidget::kCheckActionCmd);
       myCollision[idx]->setTarget(this);
       myCollision[idx]->setID(idx);
-      myCollision[idx]->setEditable(false);  // TODO - enable this?
 
       // We need to know where the PF_BL register is, to properly position
       // the CXCLR button
@@ -145,7 +146,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
         labelx += (myCollision[idx]->getWidth() - lwidth) / 2;
 
       new StaticTextWidget(boss, lfont, labelx, ypos-lineHeight, lwidth, fontHeight,
-                           colLabel[col], kTextAlignLeft);
+                           colLabel[col], TextAlign::Left);
 
       collX += myCollision[idx]->getWidth() + 10;
       idx++;
@@ -168,7 +169,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // grP0 (new)
   xpos = 10;  ypos = collY + 4;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 2*fontWidth, fontHeight,
-                       "P0", kTextAlignLeft);
+                       "P0", TextAlign::Left);
   xpos += 2*fontWidth + 5;
   myGRP0 = new TogglePixelWidget(boss, nfont, xpos, ypos+1, 8, 1);
   myGRP0->setTarget(this);
@@ -179,7 +180,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // posP0
   xpos += myGRP0->getWidth() + 12;
   t = new StaticTextWidget(boss, lfont, xpos, ypos+2, 4*fontWidth, fontHeight,
-                           "Pos#", kTextAlignLeft);
+                           "Pos#", TextAlign::Left);
   xpos += t->getWidth() + 2;
   myPosP0 = new DataGridWidget(boss, nfont, xpos, ypos,
                                1, 1, 3, 8, Common::Base::F_10);
@@ -191,7 +192,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // hmP0
   xpos += myPosP0->getWidth() + fontWidth + 12;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 2*fontWidth, fontHeight,
-                       "HM", kTextAlignLeft);
+                       "HM", TextAlign::Left);
   xpos += 2*fontWidth + 5;
   myHMP0 = new DataGridWidget(boss, nfont, xpos, ypos,
                               1, 1, 1, 4, Common::Base::F_16_1);
@@ -202,7 +203,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // P0 reflect
   xpos += myHMP0->getWidth() + 15;
   myRefP0 = new CheckboxWidget(boss, lfont, xpos, ypos+1,
-                               "Reflect", kCheckActionCmd);
+                               "Reflect", CheckboxWidget::kCheckActionCmd);
   myRefP0->setTarget(this);
   myRefP0->setID(kRefP0ID);
   addFocusWidget(myRefP0);
@@ -226,7 +227,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // P0 delay
   xpos += myGRP0Old->getWidth() + 12;
   myDelP0 = new CheckboxWidget(boss, lfont, xpos, ypos+1,
-                               "VDel", kCheckActionCmd);
+                               "VDel", CheckboxWidget::kCheckActionCmd);
   myDelP0->setTarget(this);
   myDelP0->setID(kDelP0ID);
   addFocusWidget(myDelP0);
@@ -234,7 +235,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // NUSIZ0 (player portion)
   xpos += myDelP0->getWidth() + 12;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 5*fontWidth, fontHeight,
-                       "NuSiz", kTextAlignLeft);
+                       "NuSiz", TextAlign::Left);
   xpos += 5*fontWidth + 5;
   myNusizP0 = new DataGridWidget(boss, nfont, xpos, ypos,
                                  1, 1, 1, 3, Common::Base::F_16_1);
@@ -253,7 +254,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // grP1 (new)
   xpos = 10;  ypos += lineHeight + 12;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 2*fontWidth, fontHeight,
-                       "P1", kTextAlignLeft);
+                       "P1", TextAlign::Left);
   xpos += 2*fontWidth + 5;
   myGRP1 = new TogglePixelWidget(boss, nfont, xpos, ypos+1, 8, 1);
   myGRP1->setTarget(this);
@@ -264,7 +265,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // posP1
   xpos += myGRP1->getWidth() + 12;
   t = new StaticTextWidget(boss, lfont, xpos, ypos+2, 4*fontWidth, fontHeight,
-                           "Pos#", kTextAlignLeft);
+                           "Pos#", TextAlign::Left);
   xpos += t->getWidth() + 2;
   myPosP1 = new DataGridWidget(boss, nfont, xpos, ypos,
                                1, 1, 3, 8, Common::Base::F_10);
@@ -276,7 +277,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // hmP1
   xpos += myPosP1->getWidth() + fontWidth + 12;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 2*fontWidth, fontHeight,
-                       "HM", kTextAlignLeft);
+                       "HM", TextAlign::Left);
   xpos += 2*fontWidth + 5;
   myHMP1 = new DataGridWidget(boss, nfont, xpos, ypos,
                               1, 1, 1, 4, Common::Base::F_16_1);
@@ -287,7 +288,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // P1 reflect
   xpos += myHMP1->getWidth() + 15;
   myRefP1 = new CheckboxWidget(boss, lfont, xpos, ypos+1,
-                               "Reflect", kCheckActionCmd);
+                               "Reflect", CheckboxWidget::kCheckActionCmd);
   myRefP1->setTarget(this);
   myRefP1->setID(kRefP1ID);
   addFocusWidget(myRefP1);
@@ -310,7 +311,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // P1 delay
   xpos += myGRP1Old->getWidth() + 12;
   myDelP1 = new CheckboxWidget(boss, lfont, xpos, ypos+1,
-                               "VDel", kCheckActionCmd);
+                               "VDel", CheckboxWidget::kCheckActionCmd);
   myDelP1->setTarget(this);
   myDelP1->setID(kDelP1ID);
   addFocusWidget(myDelP1);
@@ -318,7 +319,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // NUSIZ1 (player portion)
   xpos += myDelP1->getWidth() + 12;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 5*fontWidth, fontHeight,
-                       "NuSiz", kTextAlignLeft);
+                       "NuSiz", TextAlign::Left);
   xpos += 5*fontWidth + 5;
   myNusizP1 = new DataGridWidget(boss, nfont, xpos, ypos,
                                  1, 1, 1, 3, Common::Base::F_16_1);
@@ -337,7 +338,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // enaM0
   xpos = 10;  ypos += lineHeight + 12;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 2*fontWidth, fontHeight,
-                       "M0", kTextAlignLeft);
+                       "M0", TextAlign::Left);
   xpos += 2*fontWidth + 5;
   myEnaM0 = new TogglePixelWidget(boss, nfont, xpos, ypos+1, 1, 1);
   myEnaM0->setTarget(this);
@@ -348,7 +349,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // posM0
   xpos += myEnaM0->getWidth() + 12;
   t = new StaticTextWidget(boss, lfont, xpos, ypos+2, 4*fontWidth, fontHeight,
-                           "Pos#", kTextAlignLeft);
+                           "Pos#", TextAlign::Left);
   xpos += t->getWidth() + 2;
   myPosM0 = new DataGridWidget(boss, nfont, xpos, ypos,
                                1, 1, 3, 8, Common::Base::F_10);
@@ -360,7 +361,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // hmM0
   xpos += myPosM0->getWidth() + 12;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 2*fontWidth, fontHeight,
-                       "HM", kTextAlignLeft);
+                       "HM", TextAlign::Left);
   xpos += 2*fontWidth + 5;
   myHMM0 = new DataGridWidget(boss, nfont, xpos, ypos,
                               1, 1, 1, 4, Common::Base::F_16_1);
@@ -371,7 +372,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // NUSIZ0 (missile portion)
   xpos += myHMM0->getWidth() + 12;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 4*fontWidth, fontHeight,
-                       "Size", kTextAlignLeft);
+                       "Size", TextAlign::Left);
   xpos += 4*fontWidth + 5;
   myNusizM0 = new DataGridWidget(boss, nfont, xpos, ypos,
                                  1, 1, 1, 2, Common::Base::F_16_1);
@@ -382,7 +383,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // M0 reset to player 0
   xpos += myNusizM0->getWidth() + 15;
   myResMP0 = new CheckboxWidget(boss, lfont, xpos, ypos+1,
-                                "Reset to P0", kCheckActionCmd);
+                                "Reset to P0", CheckboxWidget::kCheckActionCmd);
   myResMP0->setTarget(this);
   myResMP0->setID(kResMP0ID);
   addFocusWidget(myResMP0);
@@ -400,7 +401,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // enaM1
   xpos = 10;  ypos += lineHeight + 4;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 2*fontWidth, fontHeight,
-                       "M1", kTextAlignLeft);
+                       "M1", TextAlign::Left);
   xpos += 2*fontWidth + 5;
   myEnaM1 = new TogglePixelWidget(boss, nfont, xpos, ypos+1, 1, 1);
   myEnaM1->setTarget(this);
@@ -411,7 +412,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // posM0
   xpos += myEnaM1->getWidth() + 12;
   t = new StaticTextWidget(boss, lfont, xpos, ypos+2, 4*fontWidth, fontHeight,
-                           "Pos#", kTextAlignLeft);
+                           "Pos#", TextAlign::Left);
   xpos += t->getWidth() + 2;
   myPosM1 = new DataGridWidget(boss, nfont, xpos, ypos,
                                1, 1, 3, 8, Common::Base::F_10);
@@ -423,7 +424,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // hmM0
   xpos += myPosM1->getWidth() + 12;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 2*fontWidth, fontHeight,
-                       "HM", kTextAlignLeft);
+                       "HM", TextAlign::Left);
   xpos += 2*fontWidth + 5;
   myHMM1 = new DataGridWidget(boss, nfont, xpos, ypos,
                               1, 1, 1, 4, Common::Base::F_16_1);
@@ -434,7 +435,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // NUSIZ1 (missile portion)
   xpos += myHMM1->getWidth() + 12;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 4*fontWidth, fontHeight,
-                       "Size", kTextAlignLeft);
+                       "Size", TextAlign::Left);
   xpos += 4*fontWidth + 5;
   myNusizM1 = new DataGridWidget(boss, nfont, xpos, ypos,
                                  1, 1, 1, 2, Common::Base::F_16_1);
@@ -445,7 +446,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // M1 reset to player 0
   xpos += myNusizM1->getWidth() + 15;
   myResMP1 = new CheckboxWidget(boss, lfont, xpos, ypos+1,
-                                "Reset to P1", kCheckActionCmd);
+                                "Reset to P1", CheckboxWidget::kCheckActionCmd);
   myResMP1->setTarget(this);
   myResMP1->setID(kResMP1ID);
   addFocusWidget(myResMP1);
@@ -463,7 +464,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // enaBL
   xpos = 10;  ypos += lineHeight + 4;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 2*fontWidth, fontHeight,
-                       "BL", kTextAlignLeft);
+                       "BL", TextAlign::Left);
   xpos += 2*fontWidth + 5;
   myEnaBL = new TogglePixelWidget(boss, nfont, xpos, ypos+1, 1, 1);
   myEnaBL->setTarget(this);
@@ -474,7 +475,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // posBL
   xpos += myEnaBL->getWidth() + 12;
   t = new StaticTextWidget(boss, lfont, xpos, ypos+2, 4*fontWidth, fontHeight,
-                           "Pos#", kTextAlignLeft);
+                           "Pos#", TextAlign::Left);
   xpos += t->getWidth() + 2;
   myPosBL = new DataGridWidget(boss, nfont, xpos, ypos,
                                1, 1, 3, 8, Common::Base::F_10);
@@ -486,7 +487,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // hmBL
   xpos += myPosBL->getWidth() + 12;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 2*fontWidth, fontHeight,
-                       "HM", kTextAlignLeft);
+                       "HM", TextAlign::Left);
   xpos += 2*fontWidth + 5;
   myHMBL = new DataGridWidget(boss, nfont, xpos, ypos,
                               1, 1, 1, 4, Common::Base::F_16_1);
@@ -497,7 +498,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // CTRLPF (size portion)
   xpos += myHMBL->getWidth() + 12;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 4*fontWidth, fontHeight,
-                       "Size", kTextAlignLeft);
+                       "Size", TextAlign::Left);
   xpos += 4*fontWidth + 5;
   mySizeBL = new DataGridWidget(boss, nfont, xpos, ypos,
                                 1, 1, 1, 2, Common::Base::F_16_1);
@@ -523,7 +524,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // Ball delay
   xpos += myEnaBLOld->getWidth() + 12;
   myDelBL = new CheckboxWidget(boss, lfont, xpos, ypos+1,
-                               "VDel", kCheckActionCmd);
+                               "VDel", CheckboxWidget::kCheckActionCmd);
   myDelBL->setTarget(this);
   myDelBL->setID(kDelBLID);
   addFocusWidget(myDelBL);
@@ -539,7 +540,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   // PF0
   xpos = 10;  ypos += lineHeight + sfHeight + 6;
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 2*fontWidth, fontHeight,
-                       "PF", kTextAlignLeft);
+                       "PF", TextAlign::Left);
   xpos += 2*fontWidth + 5;
   myPF[0] = new TogglePixelWidget(boss, nfont, xpos, ypos+1, 4, 1);
   myPF[0]->setTarget(this);
@@ -561,21 +562,21 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   addFocusWidget(myPF[2]);
 
   // PFx bit labels
-  auto start = [&](int w) { return (w - sfWidth) / 2; };
+  auto start = [&](int sw) { return (sw - sfWidth) / 2; };
   int colw = myPF[0]->getWidth() / 4;
   xpos = 10 + 2*fontWidth + 5 + start(colw);
   int _ypos = ypos - sfHeight;
   for(int i = 4; i <= 7; ++i)
   {
     new StaticTextWidget(boss, sf, xpos, _ypos, sfWidth, sfHeight,
-                         bitNames[i], kTextAlignLeft);
+                         bitNames[i], TextAlign::Left);
     xpos += colw;
   }
   xpos = 10 + 2*fontWidth + 5 + myPF[0]->getWidth() + 2 + start(colw);
   for(int i = 7; i >= 0; --i)
   {
     new StaticTextWidget(boss, sf, xpos, _ypos, sfWidth, sfHeight,
-                         bitNames[i], kTextAlignLeft);
+                         bitNames[i], TextAlign::Left);
     xpos += colw;
   }
   xpos = 10 + 2*fontWidth + 5 + myPF[0]->getWidth() + 2 +
@@ -583,28 +584,28 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   for(int i = 0; i <= 7; ++i)
   {
     new StaticTextWidget(boss, sf, xpos, _ypos, sfWidth, sfHeight,
-                         bitNames[i], kTextAlignLeft);
+                         bitNames[i], TextAlign::Left);
     xpos += colw;
   }
 
   // PF reflect, score, priority
   xpos = 10 + 4*fontWidth;  ypos += lineHeight + 6;
   myRefPF = new CheckboxWidget(boss, lfont, xpos, ypos+1,
-                               "Reflect", kCheckActionCmd);
+                               "Reflect", CheckboxWidget::kCheckActionCmd);
   myRefPF->setTarget(this);
   myRefPF->setID(kRefPFID);
   addFocusWidget(myRefPF);
 
   xpos += myRefPF->getWidth() + 15;
   myScorePF = new CheckboxWidget(boss, lfont, xpos, ypos+1,
-                                 "Score", kCheckActionCmd);
+                                 "Score", CheckboxWidget::kCheckActionCmd);
   myScorePF->setTarget(this);
   myScorePF->setID(kScorePFID);
   addFocusWidget(myScorePF);
 
   xpos += myScorePF->getWidth() + 15;
   myPriorityPF = new CheckboxWidget(boss, lfont, xpos, ypos+1,
-                                    "Priority", kCheckActionCmd);
+                                    "Priority", CheckboxWidget::kCheckActionCmd);
   myPriorityPF->setTarget(this);
   myPriorityPF->setID(kPriorityPFID);
   addFocusWidget(myPriorityPF);
@@ -612,7 +613,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   xpos = 10;
   ypos += 2 * lineHeight;
   t = new StaticTextWidget(boss, lfont, xpos, ypos, 13*fontWidth, fontHeight,
-    "Queued Writes", kTextAlignLeft);
+    "Queued Writes", TextAlign::Left);
 
   xpos += t->getWidth() + 10;
   myDelayQueueWidget = new DelayQueueWidget(boss, lfont, xpos, ypos);
@@ -644,13 +645,6 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
                        "HMCLR", kHmclrCmd);
   b->setTarget(this);
   addFocusWidget(b);
-
-  // How to handle undriven pins
-  xpos = 10;  ypos += (myDelayQueueWidget->getHeight() + lineHeight);
-  myUndrivenPins = new CheckboxWidget(boss, lfont, xpos, ypos+1,
-      "Drive unused TIA pins randomly on a read/peek", kPPinCmd);
-  myUndrivenPins->setTarget(this);
-  addFocusWidget(myUndrivenPins);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -705,10 +699,6 @@ void TiaWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       myFixedEnabled->setState(tia.tia().toggleFixedColors());
       break;
 
-    case kPPinCmd:
-      tia.tia().driveUnusedPinsRandom(myUndrivenPins->getState());
-      break;
-
     case DataGridWidget::kItemDataChangedCmd:
       switch(id)
       {
@@ -833,9 +823,67 @@ void TiaWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       }
       break;
 
-    case kCheckActionCmd:
+    case CheckboxWidget::kCheckActionCmd:
       switch(id)
       {
+        case kP0_PFID:
+          tia.collision(CollisionBit::P0PF, true);
+          break;
+
+        case kP0_BLID:
+          tia.collision(CollisionBit::P0BL, true);
+          break;
+
+        case kP0_M1ID:
+          tia.collision(CollisionBit::M1P0, true);
+          break;
+
+        case kP0_M0ID:
+          tia.collision(CollisionBit::M0P0, true);
+          break;
+
+        case kP0_P1ID:
+          tia.collision(CollisionBit::P0P1, true);
+          break;
+
+        case kP1_PFID:
+          tia.collision(CollisionBit::P1PF, true);
+          break;
+        case kP1_BLID:
+          tia.collision(CollisionBit::P1BL, true);
+          break;
+
+        case kP1_M1ID:
+          tia.collision(CollisionBit::M1P1, true);
+          break;
+        case kP1_M0ID:
+          tia.collision(CollisionBit::M0P1, true);
+          break;
+
+        case kM0_PFID:
+          tia.collision(CollisionBit::M0PF, true);
+          break;
+
+        case kM0_BLID:
+          tia.collision(CollisionBit::M0BL, true);
+          break;
+
+        case kM0_M1ID:
+          tia.collision(CollisionBit::M0M1, true);
+          break;
+
+        case kM1_PFID:
+          tia.collision(CollisionBit::M1PF, true);
+          break;
+
+        case kM1_BLID:
+          tia.collision(CollisionBit::M1BL, true);
+          break;
+
+        case kBL_PFID:
+          tia.collision(CollisionBit::BLPF, true);
+          break;
+
         case kRefP0ID:
           tia.refP0(myRefP0->getState() ? 1 : 0);
           break;
@@ -924,21 +972,21 @@ void TiaWidget::loadConfig()
   ////////////////////////////
   // Collision register bits
   ////////////////////////////
-  myCollision[kP0_PFID]->setState(tia.collP0_PF());
-  myCollision[kP0_BLID]->setState(tia.collP0_BL());
-  myCollision[kP0_M1ID]->setState(tia.collM1_P0());
-  myCollision[kP0_M0ID]->setState(tia.collM0_P0());
-  myCollision[kP0_P1ID]->setState(tia.collP0_P1());
-  myCollision[kP1_PFID]->setState(tia.collP1_PF());
-  myCollision[kP1_BLID]->setState(tia.collP1_BL());
-  myCollision[kP1_M1ID]->setState(tia.collM1_P1());
-  myCollision[kP1_M0ID]->setState(tia.collM0_P1());
-  myCollision[kM0_PFID]->setState(tia.collM0_PF());
-  myCollision[kM0_BLID]->setState(tia.collM0_BL());
-  myCollision[kM0_M1ID]->setState(tia.collM0_M1());
-  myCollision[kM1_PFID]->setState(tia.collM1_PF());
-  myCollision[kM1_BLID]->setState(tia.collM1_BL());
-  myCollision[kBL_PFID]->setState(tia.collBL_PF());
+  myCollision[kP0_PFID]->setState(tia.collP0_PF(), state.cx[0] != oldstate.cx[0]);
+  myCollision[kP0_BLID]->setState(tia.collP0_BL(), state.cx[1] != oldstate.cx[1]);
+  myCollision[kP0_M1ID]->setState(tia.collM1_P0(), state.cx[2] != oldstate.cx[2]);
+  myCollision[kP0_M0ID]->setState(tia.collM0_P0(), state.cx[3] != oldstate.cx[3]);
+  myCollision[kP0_P1ID]->setState(tia.collP0_P1(), state.cx[4] != oldstate.cx[4]);
+  myCollision[kP1_PFID]->setState(tia.collP1_PF(), state.cx[5] != oldstate.cx[5]);
+  myCollision[kP1_BLID]->setState(tia.collP1_BL(), state.cx[6] != oldstate.cx[6]);
+  myCollision[kP1_M1ID]->setState(tia.collM1_P1(), state.cx[7] != oldstate.cx[7]);
+  myCollision[kP1_M0ID]->setState(tia.collM0_P1(), state.cx[8] != oldstate.cx[8]);
+  myCollision[kM0_PFID]->setState(tia.collM0_PF(), state.cx[9] != oldstate.cx[9]);
+  myCollision[kM0_BLID]->setState(tia.collM0_BL(), state.cx[10] != oldstate.cx[10]);
+  myCollision[kM0_M1ID]->setState(tia.collM0_M1(), state.cx[11] != oldstate.cx[11]);
+  myCollision[kM1_PFID]->setState(tia.collM1_PF(), state.cx[12] != oldstate.cx[12]);
+  myCollision[kM1_BLID]->setState(tia.collM1_BL(), state.cx[13] != oldstate.cx[13]);
+  myCollision[kBL_PFID]->setState(tia.collBL_PF(), state.cx[14] != oldstate.cx[14]);
 
   ////////////////////////////
   // P0 register info
@@ -966,8 +1014,8 @@ void TiaWidget::loadConfig()
   myHMP0->setList(0, state.hm[P0], state.hm[P0] != oldstate.hm[P0]);
 
   // refP0 & vdelP0
-  myRefP0->setState(tia.refP0());
-  myDelP0->setState(tia.vdelP0());
+  myRefP0->setState(tia.refP0(), state.ref[P0] != oldstate.ref[P0]);
+  myDelP0->setState(tia.vdelP0(), state.vdel[P0] != oldstate.vdel[P0]);
 
   // NUSIZ0 (player portion)
   bool nusiz0changed = state.size[P0] != oldstate.size[P0];
@@ -1000,8 +1048,8 @@ void TiaWidget::loadConfig()
   myHMP1->setList(0, state.hm[P1], state.hm[P1] != oldstate.hm[P1]);
 
   // refP1 & vdelP1
-  myRefP1->setState(tia.refP1());
-  myDelP1->setState(tia.vdelP1());
+  myRefP1->setState(tia.refP1(), state.ref[P1] != oldstate.ref[P1]);
+  myDelP1->setState(tia.vdelP1(), state.vdel[P1] != oldstate.vdel[P1]);
 
   // NUSIZ1 (player portion)
   bool nusiz1changed = state.size[P1] != oldstate.size[P1];
@@ -1013,7 +1061,7 @@ void TiaWidget::loadConfig()
   ////////////////////////////
   // enaM0
   myEnaM0->setColor(state.coluRegs[0]);
-  myEnaM0->setIntState(tia.enaM0() ? 1: 0, false);
+  myEnaM0->setIntState(tia.enaM0() ? 1 : 0, false);
 
   // posM0
   myPosM0->setList(0, state.pos[M0], state.pos[M0] != oldstate.pos[M0]);
@@ -1025,14 +1073,14 @@ void TiaWidget::loadConfig()
   myNusizM0->setList(0, state.size[M0], state.size[M0] != oldstate.size[M0]);
 
   // resMP0
-  myResMP0->setState(tia.resMP0());
+  myResMP0->setState(tia.resMP0(), state.res[P0] != oldstate.res[P0]);
 
   ////////////////////////////
   // M1 register info
   ////////////////////////////
   // enaM1
   myEnaM1->setColor(state.coluRegs[1]);
-  myEnaM1->setIntState(tia.enaM1() ? 1: 0, false);
+  myEnaM1->setIntState(tia.enaM1() ? 1 : 0, false);
 
   // posM1
   myPosM1->setList(0, state.pos[M1], state.pos[M1] != oldstate.pos[M1]);
@@ -1044,7 +1092,7 @@ void TiaWidget::loadConfig()
   myNusizM1->setList(0, state.size[M1], state.size[M1] != oldstate.size[M1]);
 
   // resMP1
-  myResMP1->setState(tia.resMP1());
+  myResMP1->setState(tia.resMP1(),state.res[P1] != oldstate.res[P1]);
 
   ////////////////////////////
   // BL register info
@@ -1075,7 +1123,7 @@ void TiaWidget::loadConfig()
   mySizeBL->setList(0, state.size[BL], state.size[BL] != oldstate.size[BL]);
 
   // vdelBL
-  myDelBL->setState(tia.vdelBL());
+  myDelBL->setState(tia.vdelBL(), state.vdel[2] != oldstate.vdel[2]);
 
   ////////////////////////////
   // PF register info
@@ -1093,16 +1141,13 @@ void TiaWidget::loadConfig()
   myPF[2]->setIntState(state.pf[2], true);  // reverse bit order
 
   // Reflect
-  myRefPF->setState(tia.refPF());
+  myRefPF->setState(tia.refPF(), state.pf[3] != oldstate.pf[3]);
 
   // Score
-  myScorePF->setState(tia.scorePF());
+  myScorePF->setState(tia.scorePF(), state.pf[4] != oldstate.pf[4]);
 
   // Priority
-  myPriorityPF->setState(tia.priorityPF());
-
-  // Undriven pins
-  myUndrivenPins->setState(tia.tia().driveUnusedPinsRandom());
+  myPriorityPF->setState(tia.priorityPF(), state.pf[5] != oldstate.pf[5]);
 
   myDelayQueueWidget->loadConfig();
 }
