@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -22,7 +22,7 @@
 Cartridge0840::Cartridge0840(const BytePtr& image, uInt32 size,
                              const Settings& settings)
   : Cartridge(settings),
-    myCurrentBank(0)
+    myBankOffset(0)
 {
   // Copy the ROM image into my buffer
   memcpy(myImage, image.get(), std::min(8192u, size));
@@ -46,19 +46,19 @@ void Cartridge0840::install(System& system)
 
   // Get the page accessing methods for the hot spots since they overlap
   // areas within the TIA we'll need to forward requests to the TIA
-  myHotSpotPageAccess[0] = mySystem->getPageAccess(0x0800 >> System::PAGE_SHIFT);
-  myHotSpotPageAccess[1] = mySystem->getPageAccess(0x0900 >> System::PAGE_SHIFT);
-  myHotSpotPageAccess[2] = mySystem->getPageAccess(0x0A00 >> System::PAGE_SHIFT);
-  myHotSpotPageAccess[3] = mySystem->getPageAccess(0x0B00 >> System::PAGE_SHIFT);
-  myHotSpotPageAccess[4] = mySystem->getPageAccess(0x0C00 >> System::PAGE_SHIFT);
-  myHotSpotPageAccess[5] = mySystem->getPageAccess(0x0D00 >> System::PAGE_SHIFT);
-  myHotSpotPageAccess[6] = mySystem->getPageAccess(0x0E00 >> System::PAGE_SHIFT);
-  myHotSpotPageAccess[7] = mySystem->getPageAccess(0x0F00 >> System::PAGE_SHIFT);
+  myHotSpotPageAccess[0] = mySystem->getPageAccess(0x0800);
+  myHotSpotPageAccess[1] = mySystem->getPageAccess(0x0900);
+  myHotSpotPageAccess[2] = mySystem->getPageAccess(0x0A00);
+  myHotSpotPageAccess[3] = mySystem->getPageAccess(0x0B00);
+  myHotSpotPageAccess[4] = mySystem->getPageAccess(0x0C00);
+  myHotSpotPageAccess[5] = mySystem->getPageAccess(0x0D00);
+  myHotSpotPageAccess[6] = mySystem->getPageAccess(0x0E00);
+  myHotSpotPageAccess[7] = mySystem->getPageAccess(0x0F00);
 
   // Set the page accessing methods for the hot spots
   System::PageAccess access(this, System::PA_READ);
-  for(uInt32 i = 0x0800; i < 0x0FFF; i += (1 << System::PAGE_SHIFT))
-    mySystem->setPageAccess(i >> System::PAGE_SHIFT, access);
+  for(uInt16 addr = 0x0800; addr < 0x0FFF; addr += System::PAGE_SIZE)
+    mySystem->setPageAccess(addr, access);
 
   // Install pages for bank 0
   bank(myStartBank);
@@ -86,15 +86,10 @@ uInt8 Cartridge0840::peek(uInt16 address)
       break;
   }
 
-  if(!(address & 0x1000))
-  {
-    // Because of the way we've set up accessing above, we can only
-    // get here when the addresses are from 0x800 - 0xFFF
-    int hotspot = ((address & 0x0F00) >> 8) - 8;
-    return myHotSpotPageAccess[hotspot].device->peek(address);
-  }
-
-  return 0;
+  // Because of the way we've set up accessing above, we can only
+  // get here when the addresses are from 0x800 - 0xFFF
+  int hotspot = ((address & 0x0F00) >> 8) - 8;
+  return myHotSpotPageAccess[hotspot].device->peek(address);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -119,13 +114,14 @@ bool Cartridge0840::poke(uInt16 address, uInt8 value)
       break;
   }
 
+  // Because of the way accessing is set up, we will may get here by
+  // doing a write to 0x800 - 0xFFF or cart; we ignore the cart write
   if(!(address & 0x1000))
   {
-    // Because of the way we've set up accessing above, we can only
-    // get here when the addresses are from 0x800 - 0xFFF
     int hotspot = ((address & 0x0F00) >> 8) - 8;
     myHotSpotPageAccess[hotspot].device->poke(address, value);
   }
+
   return false;
 }
 
@@ -135,18 +131,17 @@ bool Cartridge0840::bank(uInt16 bank)
   if(bankLocked()) return false;
 
   // Remember what bank we're in
-  myCurrentBank = bank;
-  uInt16 offset = myCurrentBank << 12;
+  myBankOffset = bank << 12;
 
   // Setup the page access methods for the current bank
   System::PageAccess access(this, System::PA_READ);
 
   // Map ROM image into the system
-  for(uInt32 address = 0x1000; address < 0x2000; address += (1 << System::PAGE_SHIFT))
+  for(uInt16 addr = 0x1000; addr < 0x2000; addr += System::PAGE_SIZE)
   {
-    access.directPeekBase = &myImage[offset + (address & 0x0FFF)];
-    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x0FFF)];
-    mySystem->setPageAccess(address >> System::PAGE_SHIFT, access);
+    access.directPeekBase = &myImage[myBankOffset + (addr & 0x0FFF)];
+    access.codeAccessBase = &myCodeAccessBase[myBankOffset + (addr & 0x0FFF)];
+    mySystem->setPageAccess(addr, access);
   }
   return myBankChanged = true;
 }
@@ -154,7 +149,7 @@ bool Cartridge0840::bank(uInt16 bank)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt16 Cartridge0840::getBank() const
 {
-  return myCurrentBank;
+  return myBankOffset >> 12;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -166,12 +161,12 @@ uInt16 Cartridge0840::bankCount() const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge0840::patch(uInt16 address, uInt8 value)
 {
-  myImage[(myCurrentBank << 12) + (address & 0x0fff)] = value;
+  myImage[myBankOffset + (address & 0x0fff)] = value;
   return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* Cartridge0840::getImage(int& size) const
+const uInt8* Cartridge0840::getImage(uInt32& size) const
 {
   size = 8192;
   return myImage;
@@ -183,7 +178,7 @@ bool Cartridge0840::save(Serializer& out) const
   try
   {
     out.putString(name());
-    out.putShort(myCurrentBank);
+    out.putShort(myBankOffset);
   }
   catch(...)
   {
@@ -202,7 +197,7 @@ bool Cartridge0840::load(Serializer& in)
     if(in.getString() != name())
       return false;
 
-    myCurrentBank = in.getShort();
+    myBankOffset = in.getShort();
   }
   catch(...)
   {
@@ -211,7 +206,7 @@ bool Cartridge0840::load(Serializer& in)
   }
 
   // Remember what bank we were in
-  bank(myCurrentBank);
+  bank(myBankOffset);
 
   return true;
 }
