@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -31,11 +31,8 @@ Cartridge2K::Cartridge2K(const BytePtr& image, uInt32 size,
   while(mySize < size)
     mySize <<= 1;
 
-  // The smallest addressable area by Stella is 64 bytes
-  // This should really be read from the System, but for now I'm going
-  // to cheat a little and hard-code it to 64 (aka 2^6)
-  if(mySize < 64)
-    mySize = 64;
+  // We can't use a size smaller than the minimum page size in Stella
+  mySize = std::max<uInt32>(mySize, System::PAGE_SIZE);
 
   // Initialize ROM with illegal 6502 opcode that causes a real 6502 to jam
   myImage = make_unique<uInt8[]>(mySize);
@@ -62,28 +59,17 @@ void Cartridge2K::install(System& system)
   mySystem = &system;
 
   // Map ROM image into the system
+  // Note that we don't need our own peek/poke methods, since the mapping
+  // takes care of the entire address space
   System::PageAccess access(this, System::PA_READ);
-  for(uInt32 address = 0x1000; address < 0x2000; address += (1 << System::PAGE_SHIFT))
+  for(uInt16 addr = 0x1000; addr < 0x2000; addr += System::PAGE_SIZE)
   {
-    access.directPeekBase = &myImage[address & myMask];
-    access.codeAccessBase = &myCodeAccessBase[address & myMask];
-    mySystem->setPageAccess(address >> System::PAGE_SHIFT, access);
+    access.directPeekBase = &myImage[addr & myMask];
+    access.codeAccessBase = &myCodeAccessBase[addr & myMask];
+    mySystem->setPageAccess(addr, access);
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8 Cartridge2K::peek(uInt16 address)
-{
-  return myImage[address & myMask];
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Cartridge2K::poke(uInt16, uInt8)
-{
-  // This is ROM so poking has no effect :-)
-  return false;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge2K::patch(uInt16 address, uInt8 value)
 {
@@ -92,7 +78,7 @@ bool Cartridge2K::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* Cartridge2K::getImage(int& size) const
+const uInt8* Cartridge2K::getImage(uInt32& size) const
 {
   size = mySize;
   return myImage.get();
