@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -34,12 +34,20 @@ Cartridge3F::Cartridge3F(const BytePtr& image, uInt32 size,
   createCodeAccessBase(mySize);
 
   // Remember startup bank
-  myStartBank = 0;
+  myStartBank = bankCount() - 1; // last bank
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge3F::reset()
 {
+  // define random startup banks
+  // Note: This works for all Tigervision ROMs except for one version of Polaris
+  //   (md5: 203049f4d8290bb4521cc4402415e737) which requires 3 as startup bank
+  //   (or non-randomized RAM). All other ROMs take care of other startup banks.
+  //   The problematic version is most likely an incorrect dump with wrong
+  //   startup vectors.
+  randomizeStartBank();
+
   // We'll map the startup bank into the first segment upon reset
   bank(myStartBank);
 }
@@ -51,20 +59,17 @@ void Cartridge3F::install(System& system)
 
   System::PageAccess access(this, System::PA_READWRITE);
 
-  // Set the page accessing methods for the hot spots (for 100% emulation
-  // we need to chain any accesses below 0x40 to the TIA. Our poke() method
-  // does this via mySystem->tiaPoke(...), at least until we come up with a
-  // cleaner way to do it).
-  for(uInt32 i = 0x00; i < 0x40; i += (1 << System::PAGE_SHIFT))
-    mySystem->setPageAccess(i >> System::PAGE_SHIFT, access);
+  // The hotspot ($3F) is in TIA address space, so we claim it here
+  for(uInt16 addr = 0x00; addr < 0x40; addr += System::PAGE_SIZE)
+    mySystem->setPageAccess(addr, access);
 
   // Setup the second segment to always point to the last ROM slice
   access.type = System::PA_READ;
-  for(uInt32 j = 0x1800; j < 0x2000; j += (1 << System::PAGE_SHIFT))
+  for(uInt16 addr = 0x1800; addr < 0x2000; addr += System::PAGE_SIZE)
   {
-    access.directPeekBase = &myImage[(mySize - 2048) + (j & 0x07FF)];
-    access.codeAccessBase = &myCodeAccessBase[(mySize - 2048) + (j & 0x07FF)];
-    mySystem->setPageAccess(j >> System::PAGE_SHIFT, access);
+    access.directPeekBase = &myImage[(mySize - 2048) + (addr & 0x07FF)];
+    access.codeAccessBase = &myCodeAccessBase[(mySize - 2048) + (addr & 0x07FF)];
+    mySystem->setPageAccess(addr, access);
   }
 
   // Install pages for startup bank into the first segment
@@ -77,13 +82,9 @@ uInt8 Cartridge3F::peek(uInt16 address)
   address &= 0x0FFF;
 
   if(address < 0x0800)
-  {
     return myImage[(address & 0x07FF) + (myCurrentBank << 11)];
-  }
   else
-  {
     return myImage[(address & 0x07FF) + mySize - 2048];
-  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -95,10 +96,7 @@ bool Cartridge3F::poke(uInt16 address, uInt8 value)
   if(address <= 0x003F)
     bank(value);
 
-  // Pass the poke through to the TIA. In a real Atari, both the cart and the
-  // TIA see the address lines, and both react accordingly. In Stella, each
-  // 64-byte chunk of address space is "owned" by only one device. If we
-  // don't chain the poke to the TIA, then the TIA can't see it...
+  // Handle TIA space that we claimed above
   mySystem->tia().poke(address, value);
 
   return false;
@@ -128,12 +126,11 @@ bool Cartridge3F::bank(uInt16 bank)
   System::PageAccess access(this, System::PA_READ);
 
   // Map ROM image into the system
-  for(uInt32 address = 0x1000; address < 0x1800;
-      address += (1 << System::PAGE_SHIFT))
+  for(uInt16 addr = 0x1000; addr < 0x1800; addr += System::PAGE_SIZE)
   {
-    access.directPeekBase = &myImage[offset + (address & 0x07FF)];
-    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x07FF)];
-    mySystem->setPageAccess(address >> System::PAGE_SHIFT, access);
+    access.directPeekBase = &myImage[offset + (addr & 0x07FF)];
+    access.codeAccessBase = &myCodeAccessBase[offset + (addr & 0x07FF)];
+    mySystem->setPageAccess(addr, access);
   }
   return myBankChanged = true;
 }
@@ -164,7 +161,7 @@ bool Cartridge3F::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* Cartridge3F::getImage(int& size) const
+const uInt8* Cartridge3F::getImage(uInt32& size) const
 {
   size = mySize;
   return myImage.get();
