@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -43,7 +43,7 @@
 CartridgeBUS::CartridgeBUS(const BytePtr& image, uInt32 size,
                            const Settings& settings)
   : Cartridge(settings),
-    mySystemCycles(0),
+    myAudioCycles(0),
     myARMCycles(0),
     myFractionalClocks(0.0)
 {
@@ -63,11 +63,12 @@ CartridgeBUS::CartridgeBUS(const BytePtr& image, uInt32 size,
   // Pointer to the display RAM
   myDisplayImage = myBUSRAM + DSRAM;
 
-#ifdef THUMB_SUPPORT
   // Create Thumbulator ARM emulator
-  myThumbEmulator = make_unique<Thumbulator>((uInt16*)myImage, (uInt16*)myBUSRAM,
-    settings.getBool("thumb.trapfatal"), Thumbulator::ConfigureFor::BUS, this);
-#endif
+  myThumbEmulator = make_unique<Thumbulator>(
+    reinterpret_cast<uInt16*>(myImage), reinterpret_cast<uInt16*>(myBUSRAM),
+    settings.getBool("thumb.trapfatal"), Thumbulator::ConfigureFor::BUS, this
+  );
+
   setInitialState();
 }
 
@@ -77,8 +78,7 @@ void CartridgeBUS::reset()
   initializeRAM(myBUSRAM+2048, 8192-2048);
 
   // Update cycles to the current system cycles
-  mySystemCycles = mySystem->cycles();
-  myARMCycles = mySystem->cycles();
+  myAudioCycles = myARMCycles = 0;
   myFractionalClocks = 0.0;
 
   setInitialState();
@@ -107,17 +107,7 @@ void CartridgeBUS::setInitialState()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeBUS::consoleChanged(ConsoleTiming timing)
 {
-#ifdef THUMB_SUPPORT
   myThumbEmulator->setConsoleTiming(timing);
-#endif
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeBUS::systemCyclesReset()
-{
-  // Adjust the cycle counter so that it reflects the new value
-  mySystemCycles -= mySystem->cycles();
-  myARMCycles -= mySystem->cycles();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -127,8 +117,8 @@ void CartridgeBUS::install(System& system)
 
   // Map all of the accesses to call peek and poke
   System::PageAccess access(this, System::PA_READ);
-  for(uInt32 i = 0x1000; i < 0x1040; i += (1 << System::PAGE_SHIFT))
-    mySystem->setPageAccess(i >> System::PAGE_SHIFT, access);
+  for(uInt16 addr = 0x1000; addr < 0x1040; addr += System::PAGE_SIZE)
+    mySystem->setPageAccess(addr, access);
 
   // Mirror all access in TIA and RIOT; by doing so we're taking responsibility
   // for that address space in peek and poke below.
@@ -143,20 +133,18 @@ void CartridgeBUS::install(System& system)
 inline void CartridgeBUS::updateMusicModeDataFetchers()
 {
   // Calculate the number of cycles since the last update
-  Int32 cycles = mySystem->cycles() - mySystemCycles;
-  mySystemCycles = mySystem->cycles();
+  uInt32 cycles = uInt32(mySystem->cycles() - myAudioCycles);
+  myAudioCycles = mySystem->cycles();
 
   // Calculate the number of BUS OSC clocks since the last update
   double clocks = ((20000.0 * cycles) / 1193191.66666667) + myFractionalClocks;
-  Int32 wholeClocks = Int32(clocks);
+  uInt32 wholeClocks = uInt32(clocks);
   myFractionalClocks = clocks - double(wholeClocks);
 
-  if(wholeClocks <= 0)
-    return;
-
   // Let's update counters and flags of the music mode data fetchers
-  for(int x = 0; x <= 2; ++x)
-    myMusicCounters[x] += myMusicFrequencies[x] * wholeClocks;
+  if(wholeClocks > 0)
+    for(int x = 0; x <= 2; ++x)
+      myMusicCounters[x] += myMusicFrequencies[x] * wholeClocks;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -164,13 +152,12 @@ inline void CartridgeBUS::callFunction(uInt8 value)
 {
   switch (value)
   {
-  #ifdef THUMB_SUPPORT
-      // Call user written ARM code (will most likely be C compiled for ARM)
+    // Call user written ARM code (will most likely be C compiled for ARM)
     case 254: // call with IRQ driven audio, no special handling needed at this
               // time for Stella as ARM code "runs in zero 6507 cycles".
     case 255: // call without IRQ driven audio
       try {
-        Int32 cycles = mySystem->cycles() - myARMCycles;
+        Int32 cycles = Int32(mySystem->cycles() - myARMCycles);
         myARMCycles = mySystem->cycles();
 
         myThumbEmulator->run(cycles);
@@ -186,7 +173,6 @@ inline void CartridgeBUS::callFunction(uInt8 value)
         }
       }
       break;
-  #endif
   }
 }
 
@@ -206,7 +192,7 @@ uInt8 CartridgeBUS::peek(uInt16 address)
   {
     address &= 0x0FFF;
 
-    uInt8 peekvalue = myProgramImage[(myCurrentBank << 12) + address];
+    uInt8 peekvalue = myProgramImage[myBankOffset + address];
 
     // In debugger/bank-locked mode, we ignore all hotspots and in general
     // anything that can change the internal state of the cart
@@ -234,8 +220,8 @@ uInt8 CartridgeBUS::peek(uInt16 address)
     // test for JMP FASTJUMP where FASTJUMP = $0000
     if (BUS_STUFF_ON
         && peekvalue == 0x4C
-        && myProgramImage[(myCurrentBank << 12) + address+1] == 0
-        && myProgramImage[(myCurrentBank << 12) + address+2] == 0)
+        && myProgramImage[myBankOffset + address+1] == 0
+        && myProgramImage[myBankOffset + address+2] == 0)
     {
       myFastJumpActive = 2; // return next two peeks from datastream 17
       myJMPoperandAddress = address + 1;
@@ -445,18 +431,16 @@ bool CartridgeBUS::bank(uInt16 bank)
   if(bankLocked()) return false;
 
   // Remember what bank we're in
-  myCurrentBank = bank;
-  uInt16 offset = myCurrentBank << 12;
+  myBankOffset = bank << 12;
 
   // Setup the page access methods for the current bank
   System::PageAccess access(this, System::PA_READ);
 
   // Map Program ROM image into the system
-  for(uInt32 address = 0x1040; address < 0x2000;
-      address += (1 << System::PAGE_SHIFT))
+  for(uInt16 addr = 0x1040; addr < 0x2000; addr += System::PAGE_SIZE)
   {
-    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x0FFF)];
-    mySystem->setPageAccess(address >> System::PAGE_SHIFT, access);
+    access.codeAccessBase = &myCodeAccessBase[myBankOffset + (addr & 0x0FFF)];
+    mySystem->setPageAccess(addr, access);
   }
   return myBankChanged = true;
 }
@@ -464,7 +448,7 @@ bool CartridgeBUS::bank(uInt16 bank)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt16 CartridgeBUS::getBank() const
 {
-  return myCurrentBank;
+  return myBankOffset >> 12;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -481,7 +465,7 @@ bool CartridgeBUS::patch(uInt16 address, uInt8 value)
   // For now, we ignore attempts to patch the BUS address space
   if(address >= 0x0040)
   {
-    myProgramImage[(myCurrentBank << 12) + (address & 0x0FFF)] = value;
+    myProgramImage[myBankOffset + (address & 0x0FFF)] = value;
     return myBankChanged = true;
   }
   else
@@ -489,7 +473,7 @@ bool CartridgeBUS::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* CartridgeBUS::getImage(int& size) const
+const uInt8* CartridgeBUS::getImage(uInt32& size) const
 {
   size = 32768;
   return myImage;
@@ -541,7 +525,6 @@ uInt32 CartridgeBUS::thumbCallback(uInt8 function, uInt32 value1, uInt32 value2)
       // _GetWavePtr - return the counter
     case 2:
       return myMusicCounters[value1];
-      break;
 
       // _SetWaveSize - set size of waveform buffer
     case 3:
@@ -560,7 +543,7 @@ bool CartridgeBUS::save(Serializer& out) const
     out.putString(name());
 
     // Indicates which bank is currently active
-    out.putShort(myCurrentBank);
+    out.putShort(myBankOffset);
 
     // Harmony RAM
     out.putByteArray(myBUSRAM, 8192);
@@ -571,9 +554,9 @@ bool CartridgeBUS::save(Serializer& out) const
     out.putShort(myJMPoperandAddress);
 
     // Save cycles and clocks
-    out.putInt(mySystemCycles);
-    out.putInt((uInt32)(myFractionalClocks * 100000000.0));
-    out.putInt(myARMCycles);
+    out.putLong(myAudioCycles);
+    out.putDouble(myFractionalClocks);
+    out.putLong(myARMCycles);
 
     // Audio info
     out.putIntArray(myMusicCounters, 3);
@@ -604,7 +587,7 @@ bool CartridgeBUS::load(Serializer& in)
       return false;
 
     // Indicates which bank is currently active
-    myCurrentBank = in.getShort();
+    myBankOffset = in.getShort();
 
     // Harmony RAM
     in.getByteArray(myBUSRAM, 8192);
@@ -615,9 +598,9 @@ bool CartridgeBUS::load(Serializer& in)
     myJMPoperandAddress = in.getShort();
 
     // Get system cycles and fractional clocks
-    mySystemCycles = (Int32)in.getInt();
-    myFractionalClocks = (double)in.getInt() / 100000000.0;
-    myARMCycles = (Int32)in.getInt();
+    myAudioCycles = in.getLong();
+    myFractionalClocks = in.getDouble();
+    myARMCycles = in.getLong();
 
     // Audio info
     in.getIntArray(myMusicCounters, 3);
@@ -637,7 +620,7 @@ bool CartridgeBUS::load(Serializer& in)
   }
 
   // Now, go to the current bank
-  bank(myCurrentBank);
+  bank(myBankOffset >> 12);
 
   return true;
 }
@@ -673,16 +656,6 @@ uInt32 CartridgeBUS::getDatastreamIncrement(uInt8 index) const
         (myBUSRAM[DSxINC + index*4 + 3] << 24) ;   // high byte
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeBUS::setDatastreamIncrement(uInt8 index, uInt32 value)
-{
-//  index &= 0x0f;
-  myBUSRAM[DSxINC + index*4 + 0] = value & 0xff;          // low byte
-  myBUSRAM[DSxINC + index*4 + 1] = (value >> 8) & 0xff;
-  myBUSRAM[DSxINC + index*4 + 2] = (value >> 16) & 0xff;
-  myBUSRAM[DSxINC + index*4 + 3] = (value >> 24) & 0xff;  // high byte
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt32 CartridgeBUS::getAddressMap(uInt8 index) const
 {
