@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -66,13 +66,12 @@ CartridgeCDF::CartridgeCDF(const BytePtr& image, uInt32 size,
 
   setVersion();
 
-#ifdef THUMB_SUPPORT
   // Create Thumbulator ARM emulator
   myThumbEmulator = make_unique<Thumbulator>(
-      (uInt16*)myImage, (uInt16*)myCDFRAM, settings.getBool("thumb.trapfatal"),
-      myVersion ? Thumbulator::ConfigureFor::CDF1 : Thumbulator::ConfigureFor::CDF,
-      this);
-#endif
+    reinterpret_cast<uInt16*>(myImage), reinterpret_cast<uInt16*>(myCDFRAM),
+    settings.getBool("thumb.trapfatal"), myVersion ?
+    Thumbulator::ConfigureFor::CDF1 : Thumbulator::ConfigureFor::CDF, this);
+
   setInitialState();
 }
 
@@ -81,9 +80,7 @@ void CartridgeCDF::reset()
 {
   initializeRAM(myCDFRAM+2048, 8192-2048);
 
-  // Update cycles to the current system cycles
-  myAudioCycles = mySystem->cycles();
-  myARMCycles = mySystem->cycles();
+  myAudioCycles = myARMCycles = 0;
   myFractionalClocks = 0.0;
 
   setInitialState();
@@ -114,17 +111,7 @@ void CartridgeCDF::setInitialState()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeCDF::consoleChanged(ConsoleTiming timing)
 {
-#ifdef THUMB_SUPPORT
   myThumbEmulator->setConsoleTiming(timing);
-#endif
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeCDF::systemCyclesReset()
-{
-  // Adjust the cycle counter so that it reflects the new value
-  myAudioCycles -= mySystem->cycles();
-  myARMCycles -= mySystem->cycles();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -134,8 +121,8 @@ void CartridgeCDF::install(System& system)
 
   // Map all of the accesses to call peek and poke
   System::PageAccess access(this, System::PA_READ);
-  for(uInt32 i = 0x1000; i < 0x1040; i += (1 << System::PAGE_SHIFT))
-    mySystem->setPageAccess(i >> System::PAGE_SHIFT, access);
+  for(uInt16 addr = 0x1000; addr < 0x1040; addr += System::PAGE_SIZE)
+    mySystem->setPageAccess(addr, access);
 
   // Install pages for the startup bank
   bank(myStartBank);
@@ -145,20 +132,18 @@ void CartridgeCDF::install(System& system)
 inline void CartridgeCDF::updateMusicModeDataFetchers()
 {
   // Calculate the number of cycles since the last update
-  Int32 cycles = mySystem->cycles() - myAudioCycles;
+  uInt32 cycles = uInt32(mySystem->cycles() - myAudioCycles);
   myAudioCycles = mySystem->cycles();
 
   // Calculate the number of CDF OSC clocks since the last update
   double clocks = ((20000.0 * cycles) / 1193191.66666667) + myFractionalClocks;
-  Int32 wholeClocks = Int32(clocks);
+  uInt32 wholeClocks = uInt32(clocks);
   myFractionalClocks = clocks - double(wholeClocks);
 
-  if(wholeClocks <= 0)
-    return;
-
   // Let's update counters and flags of the music mode data fetchers
-  for(int x = 0; x <= 2; ++x)
-    myMusicCounters[x] += myMusicFrequencies[x] * wholeClocks;
+  if(wholeClocks > 0)
+    for(int x = 0; x <= 2; ++x)
+      myMusicCounters[x] += myMusicFrequencies[x] * wholeClocks;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -166,13 +151,12 @@ inline void CartridgeCDF::callFunction(uInt8 value)
 {
   switch (value)
   {
-#ifdef THUMB_SUPPORT
     // Call user written ARM code (will most likely be C compiled for ARM)
     case 254: // call with IRQ driven audio, no special handling needed at this
               // time for Stella as ARM code "runs in zero 6507 cycles".
     case 255: // call without IRQ driven audio
       try {
-        Int32 cycles = mySystem->cycles() - myARMCycles;
+        Int32 cycles = Int32(mySystem->cycles() - myARMCycles);
         myARMCycles = mySystem->cycles();
 
         myThumbEmulator->run(cycles);
@@ -188,7 +172,6 @@ inline void CartridgeCDF::callFunction(uInt8 value)
         }
       }
       break;
-#endif
   }
 }
 
@@ -197,7 +180,7 @@ uInt8 CartridgeCDF::peek(uInt16 address)
 {
   address &= 0x0FFF;
 
-  uInt8 peekvalue = myProgramImage[(myCurrentBank << 12) + address];
+  uInt8 peekvalue = myProgramImage[myBankOffset + address];
 
   // In debugger/bank-locked mode, we ignore all hotspots and in general
   // anything that can change the internal state of the cart
@@ -225,8 +208,8 @@ uInt8 CartridgeCDF::peek(uInt16 address)
   // test for JMP FASTJUMP where FASTJUMP = $0000
   if (FAST_FETCH_ON
       && peekvalue == 0x4C
-      && myProgramImage[(myCurrentBank << 12) + address+1] == 0
-      && myProgramImage[(myCurrentBank << 12) + address+2] == 0)
+      && myProgramImage[myBankOffset + address+1] == 0
+      && myProgramImage[myBankOffset + address+2] == 0)
   {
     myFastJumpActive = 2; // return next two peeks from datastream 31
     myJMPoperandAddress = address + 1;
@@ -409,18 +392,16 @@ bool CartridgeCDF::bank(uInt16 bank)
   if(bankLocked()) return false;
 
   // Remember what bank we're in
-  myCurrentBank = bank;
-  uInt16 offset = myCurrentBank << 12;
+  myBankOffset = bank << 12;
 
   // Setup the page access methods for the current bank
   System::PageAccess access(this, System::PA_READ);
 
   // Map Program ROM image into the system
-  for(uInt32 address = 0x1040; address < 0x2000;
-      address += (1 << System::PAGE_SHIFT))
+  for(uInt16 addr = 0x1040; addr < 0x2000; addr += System::PAGE_SIZE)
   {
-    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x0FFF)];
-    mySystem->setPageAccess(address >> System::PAGE_SHIFT, access);
+    access.codeAccessBase = &myCodeAccessBase[myBankOffset + (addr & 0x0FFF)];
+    mySystem->setPageAccess(addr, access);
   }
   return myBankChanged = true;
 }
@@ -428,7 +409,7 @@ bool CartridgeCDF::bank(uInt16 bank)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt16 CartridgeCDF::getBank() const
 {
-  return myCurrentBank;
+  return myBankOffset >> 12;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -445,7 +426,7 @@ bool CartridgeCDF::patch(uInt16 address, uInt8 value)
   // For now, we ignore attempts to patch the CDF address space
   if(address >= 0x0040)
   {
-    myProgramImage[(myCurrentBank << 12) + (address & 0x0FFF)] = value;
+    myProgramImage[myBankOffset + (address & 0x0FFF)] = value;
     return myBankChanged = true;
   }
   else
@@ -453,7 +434,7 @@ bool CartridgeCDF::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* CartridgeCDF::getImage(int& size) const
+const uInt8* CartridgeCDF::getImage(uInt32& size) const
 {
   size = 32768;
   return myImage;
@@ -479,7 +460,6 @@ uInt32 CartridgeCDF::thumbCallback(uInt8 function, uInt32 value1, uInt32 value2)
       // _GetWavePtr - return the counter
     case 2:
       return myMusicCounters[value1];
-      break;
 
       // _SetWaveSize - set size of waveform buffer
     case 3:
@@ -499,7 +479,7 @@ bool CartridgeCDF::save(Serializer& out) const
     out.putString(name());
 
     // Indicates which bank is currently active
-    out.putShort(myCurrentBank);
+    out.putShort(myBankOffset);
 
     // Indicates current mode
     out.putByte(myMode);
@@ -520,9 +500,9 @@ bool CartridgeCDF::save(Serializer& out) const
     out.putByteArray(myMusicWaveformSize, 3);
 
     // Save cycles and clocks
-    out.putInt(myAudioCycles);
-    out.putInt((uInt32)(myFractionalClocks * 100000000.0));
-    out.putInt(myARMCycles);
+    out.putLong(myAudioCycles);
+    out.putDouble(myFractionalClocks);
+    out.putLong(myARMCycles);
   }
   catch(...)
   {
@@ -542,7 +522,7 @@ bool CartridgeCDF::load(Serializer& in)
       return false;
 
     // Indicates which bank is currently active
-    myCurrentBank = in.getShort();
+    myBankOffset = in.getShort();
 
     // Indicates current mode
     myMode = in.getByte();
@@ -563,9 +543,9 @@ bool CartridgeCDF::load(Serializer& in)
     in.getByteArray(myMusicWaveformSize, 3);
 
     // Get cycles and clocks
-    myAudioCycles = (Int32)in.getInt();
-    myFractionalClocks = (double)in.getInt() / 100000000.0;
-    myARMCycles = (Int32)in.getInt();
+    myAudioCycles = in.getLong();
+    myFractionalClocks = in.getDouble();
+    myARMCycles = in.getLong();
   }
   catch(...)
   {
@@ -574,7 +554,7 @@ bool CartridgeCDF::load(Serializer& in)
   }
 
   // Now, go to the current bank
-  bank(myCurrentBank);
+  bank(myBankOffset >> 12);
 
   return true;
 }
@@ -612,17 +592,6 @@ uInt32 CartridgeCDF::getDatastreamIncrement(uInt8 index) const
         (myCDFRAM[address + 3] << 24) ;   // high byte
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeCDF::setDatastreamIncrement(uInt8 index, uInt32 value)
-{
-  uInt16 address = DSxINC[myVersion] + index * 4;
-
-  myCDFRAM[address + 0] = value & 0xff;          // low byte
-  myCDFRAM[address + 1] = (value >> 8) & 0xff;
-  myCDFRAM[address + 2] = (value >> 16) & 0xff;
-  myCDFRAM[address + 3] = (value >> 24) & 0xff;  // high byte
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt32 CartridgeCDF::getWaveform(uInt8 index) const
 {
