@@ -8,13 +8,14 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
+#include "M6532.hxx"
 #include "System.hxx"
 #include "CartFE.hxx"
 
@@ -22,26 +23,21 @@
 CartridgeFE::CartridgeFE(const BytePtr& image, uInt32 size,
                          const Settings& settings)
   : Cartridge(settings),
-    myLastAddress1(0),
-    myLastAddress2(0),
-    myLastAddressChanged(false)
+    myBankOffset(0),
+    myLastAccessWasFE(false)
 {
   // Copy the ROM image into my buffer
   memcpy(myImage, image.get(), std::min(8192u, size));
-
-  // We use System::PageAccess.codeAccessBase, but don't allow its use
-  // through a pointer, since the address space of FE carts can change
-  // at the instruction level, and PageAccess is normally defined at an
-  // interval of 64 bytes
-  //
-  // Instead, access will be through the getAccessFlags and setAccessFlags
-  // methods below
   createCodeAccessBase(8192);
+
+  myStartBank = 0;  // Decathlon requires this, since there is no startup vector in bank 1
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeFE::reset()
 {
+  bank(myStartBank);
+  myLastAccessWasFE = false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -49,50 +45,73 @@ void CartridgeFE::install(System& system)
 {
   mySystem = &system;
 
-  // Map all of the accesses to call peek and poke
-  System::PageAccess access(this, System::PA_READ);
-  for(uInt32 i = 0x1000; i < 0x2000; i += (1 << System::PAGE_SHIFT))
-    mySystem->setPageAccess(i >> System::PAGE_SHIFT, access);
+  // The hotspot $01FE is in a mirror of zero-page RAM
+  // We need to claim access to it here, and deal with it in peek/poke below
+  System::PageAccess access(this, System::PA_READWRITE);
+  for(uInt16 addr = 0x180; addr < 0x200; addr += System::PAGE_SIZE)
+    mySystem->setPageAccess(addr, access);
+
+  // Map all of the cart accesses to call peek and poke
+  access.type = System::PA_READ;
+  for(uInt16 addr = 0x1000; addr < 0x2000; addr += System::PAGE_SIZE)
+    mySystem->setPageAccess(addr, access);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 CartridgeFE::peek(uInt16 address)
 {
-  // The bank is determined by A13 of the processor
-  // We keep track of the two most recent accesses to determine which bank
-  // we're in, and when the values actually changed
-  myLastAddress2 = myLastAddress1;
-  myLastAddress1 = address;
-  myLastAddressChanged = true;
+  uInt8 value = (address < 0x200) ? mySystem->m6532().peek(address) :
+      myImage[myBankOffset + (address & 0x0FFF)];
 
-  return myImage[(address & 0x0FFF) + (((address & 0x2000) == 0) ? 4096 : 0)];
+  // Check if we hit hotspot
+  checkBankSwitch(address, value);
+
+  return value;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeFE::poke(uInt16, uInt8)
+bool CartridgeFE::poke(uInt16 address, uInt8 value)
 {
+  if(address < 0x200)
+    mySystem->m6532().poke(address, value);
+
+  // Check if we hit hotspot
+  checkBankSwitch(address, value);
+
   return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8 CartridgeFE::getAccessFlags(uInt16 address) const
+void CartridgeFE::checkBankSwitch(uInt16 address, uInt8 value)
 {
-  return myCodeAccessBase[(address & 0x0FFF) +
-            (((address & 0x2000) == 0) ? 4096 : 0)];
+  if(bankLocked())
+    return;
+
+  // Did we detect $01FE on the last address bus access?
+  // If so, we bankswitch according to the upper 3 bits of the data bus
+  // NOTE: see the header file for the significance of 'value & 0x20'
+  if(myLastAccessWasFE)
+    bank((value & 0x20) ? 0 : 1);
+
+  // On the next cycle, we use the (then) current data bus value to decode
+  // the bank to use
+  myLastAccessWasFE = address == 0x01FE;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeFE::setAccessFlags(uInt16 address, uInt8 flags)
+bool CartridgeFE::bank(uInt16 bank)
 {
-  myCodeAccessBase[(address & 0x0FFF) +
-      (((address & 0x2000) == 0) ? 4096 : 0)] |= flags;
+  if(bankLocked())
+    return false;
+
+  myBankOffset = bank << 12;
+  return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt16 CartridgeFE::getBank() const
 {
-  // The current bank depends on the last address accessed
-  return ((myLastAddress1 & 0x2000) == 0) ? 1 : 0;
+  return myBankOffset >> 12;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -101,32 +120,15 @@ uInt16 CartridgeFE::bankCount() const
   return 2;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeFE::bankChanged()
-{
-  if(myLastAddressChanged)
-  {
-    // A bankswitch occurs when the addresses transition from state to another
-    myBankChanged = ((myLastAddress1 & 0x2000) == 0) !=
-                    ((myLastAddress2 & 0x2000) == 0);
-    myLastAddressChanged = false;
-  }
-  else
-    myBankChanged = false;
-
-  // In any event, let the base class know about it
-  return Cartridge::bankChanged();
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeFE::patch(uInt16 address, uInt8 value)
 {
-  myImage[(address & 0x0FFF) + (((address & 0x2000) == 0) ? 4096 : 0)] = value;
+  myImage[myBankOffset + (address & 0x0FFF)] = value;
   return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* CartridgeFE::getImage(int& size) const
+const uInt8* CartridgeFE::getImage(uInt32& size) const
 {
   size = 8192;
   return myImage;
@@ -138,8 +140,8 @@ bool CartridgeFE::save(Serializer& out) const
   try
   {
     out.putString(name());
-    out.putShort(myLastAddress1);
-    out.putShort(myLastAddress2);
+    out.putShort(myBankOffset);
+    out.putBool(myLastAccessWasFE);
   }
   catch(...)
   {
@@ -158,8 +160,8 @@ bool CartridgeFE::load(Serializer& in)
     if(in.getString() != name())
       return false;
 
-    myLastAddress1 = in.getShort();
-    myLastAddress2 = in.getShort();
+    myBankOffset = in.getShort();
+    myLastAccessWasFE = in.getBool();
   }
   catch(...)
   {
