@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -22,7 +22,7 @@
 CartridgeUA::CartridgeUA(const BytePtr& image, uInt32 size,
                          const Settings& settings)
   : Cartridge(settings),
-    myCurrentBank(0)
+    myBankOffset(0)
 {
   // Copy the ROM image into my buffer
   memcpy(myImage, image.get(), std::min(8192u, size));
@@ -35,6 +35,9 @@ CartridgeUA::CartridgeUA(const BytePtr& image, uInt32 size,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeUA::reset()
 {
+  // define startup bank
+  randomizeStartBank();
+
   // Upon reset we switch to the startup bank
   bank(myStartBank);
 }
@@ -46,12 +49,12 @@ void CartridgeUA::install(System& system)
 
   // Get the page accessing methods for the hot spots since they overlap
   // areas within the TIA we'll need to forward requests to the TIA
-  myHotSpotPageAccess = mySystem->getPageAccess(0x0220 >> System::PAGE_SHIFT);
+  myHotSpotPageAccess = mySystem->getPageAccess(0x0220);
 
   // Set the page accessing methods for the hot spots
   System::PageAccess access(this, System::PA_READ);
-  mySystem->setPageAccess(0x0220 >> System::PAGE_SHIFT, access);
-  mySystem->setPageAccess(0x0240 >> System::PAGE_SHIFT, access);
+  mySystem->setPageAccess(0x0220, access);
+  mySystem->setPageAccess(0x0240, access);
 
   // Install pages for the startup bank
   bank(myStartBank);
@@ -79,14 +82,9 @@ uInt8 CartridgeUA::peek(uInt16 address)
       break;
   }
 
-  if(!(address & 0x1000))
-  {
-    return myHotSpotPageAccess.device->peek(address);
-  }
-  else
-  {
-    return 0;
-  }
+  // Because of the way accessing is set up, we will only get here
+  // when doing a TIA read
+  return myHotSpotPageAccess.device->peek(address);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -111,10 +109,11 @@ bool CartridgeUA::poke(uInt16 address, uInt8 value)
       break;
   }
 
+  // Because of the way accessing is set up, we will may get here by
+  // doing a write to TIA or cart; we ignore the cart write
   if(!(address & 0x1000))
-  {
     myHotSpotPageAccess.device->poke(address, value);
-  }
+
   return false;
 }
 
@@ -124,19 +123,17 @@ bool CartridgeUA::bank(uInt16 bank)
   if(bankLocked()) return false;
 
   // Remember what bank we're in
-  myCurrentBank = bank;
-  uInt16 offset = myCurrentBank << 12;
+  myBankOffset = bank << 12;
 
   // Setup the page access methods for the current bank
   System::PageAccess access(this, System::PA_READ);
 
   // Map ROM image into the system
-  for(uInt32 address = 0x1000; address < 0x2000;
-      address += (1 << System::PAGE_SHIFT))
+  for(uInt16 addr = 0x1000; addr < 0x2000; addr += System::PAGE_SIZE)
   {
-    access.directPeekBase = &myImage[offset + (address & 0x0FFF)];
-    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x0FFF)];
-    mySystem->setPageAccess(address >> System::PAGE_SHIFT, access);
+    access.directPeekBase = &myImage[myBankOffset + (addr & 0x0FFF)];
+    access.codeAccessBase = &myCodeAccessBase[myBankOffset + (addr & 0x0FFF)];
+    mySystem->setPageAccess(addr, access);
   }
   return myBankChanged = true;
 }
@@ -144,7 +141,7 @@ bool CartridgeUA::bank(uInt16 bank)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt16 CartridgeUA::getBank() const
 {
-  return myCurrentBank;
+  return myBankOffset >> 12;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -156,12 +153,12 @@ uInt16 CartridgeUA::bankCount() const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeUA::patch(uInt16 address, uInt8 value)
 {
-  myImage[(myCurrentBank << 12) + (address & 0x0FFF)] = value;
+  myImage[myBankOffset + (address & 0x0FFF)] = value;
   return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* CartridgeUA::getImage(int& size) const
+const uInt8* CartridgeUA::getImage(uInt32& size) const
 {
   size = 8192;
   return myImage;
@@ -173,7 +170,7 @@ bool CartridgeUA::save(Serializer& out) const
   try
   {
     out.putString(name());
-    out.putShort(myCurrentBank);
+    out.putShort(myBankOffset);
   }
   catch(...)
   {
@@ -192,7 +189,7 @@ bool CartridgeUA::load(Serializer& in)
     if(in.getString() != name())
       return false;
 
-    myCurrentBank = in.getShort();
+    myBankOffset = in.getShort();
   }
   catch(...)
   {
@@ -201,7 +198,7 @@ bool CartridgeUA::load(Serializer& in)
   }
 
   // Remember what bank we were in
-  bank(myCurrentBank);
+  bank(myBankOffset >> 12);
 
   return true;
 }
