@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,10 +16,7 @@
 //============================================================================
 
 #include <cassert>
-#include <iostream>
-#include <sstream>
 #include <stdexcept>
-#include <fstream>
 
 #include "AtariVox.hxx"
 #include "Booster.hxx"
@@ -50,13 +47,17 @@
 #include "AtariMouse.hxx"
 #include "TrakBall.hxx"
 #include "FrameBuffer.hxx"
+#include "TIASurface.hxx"
 #include "OSystem.hxx"
 #include "Menu.hxx"
 #include "CommandMenu.hxx"
 #include "Serializable.hxx"
 #include "Version.hxx"
-#include "FrameManager.hxx"
+#include "TIAConstants.hxx"
 #include "FrameLayout.hxx"
+#include "frame-manager/FrameManager.hxx"
+#include "frame-manager/FrameLayoutDetector.hxx"
+#include "frame-manager/YStartDetector.hxx"
 
 #ifdef DEBUGGER_SUPPORT
   #include "Debugger.hxx"
@@ -68,6 +69,10 @@
 
 #include "Console.hxx"
 
+namespace {
+  constexpr uInt8 YSTART_EXTRA = 2;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Console::Console(OSystem& osystem, unique_ptr<Cartridge>& cart,
                  const Properties& props)
@@ -77,7 +82,8 @@ Console::Console(OSystem& osystem, unique_ptr<Cartridge>& cart,
     myCart(std::move(cart)),
     myDisplayFormat(""),  // Unknown TV format @ start
     myFramerate(0.0),     // Unknown framerate @ start
-    myCurrentFormat(0),   // Unknown format @ start
+    myCurrentFormat(0),   // Unknown format @ start,
+    myAutodetectedYstart(0),
     myUserPaletteDefined(false),
     myConsoleTiming(ConsoleTiming::ntsc)
 {
@@ -88,7 +94,10 @@ Console::Console(OSystem& osystem, unique_ptr<Cartridge>& cart,
   my6502 = make_unique<M6502>(myOSystem.settings());
   myRiot = make_unique<M6532>(*this, myOSystem.settings());
   myTIA  = make_unique<TIA>(*this, myOSystem.sound(), myOSystem.settings());
-  mySwitches = make_unique<Switches>(myEvent, myProperties);
+  myFrameManager = make_unique<FrameManager>();
+  mySwitches = make_unique<Switches>(myEvent, myProperties, myOSystem.settings());
+
+  myTIA->setFrameManager(myFrameManager.get());
 
   // Construct the system and components
   mySystem = make_unique<System>(osystem, *my6502, *myRiot, *myTIA, *myCart);
@@ -112,38 +121,25 @@ Console::Console(OSystem& osystem, unique_ptr<Cartridge>& cart,
   const string& md5 = myProperties.get(Cartridge_MD5);
   setControllers(md5);
 
+  // Mute audio and clear framebuffer while autodetection runs
+  myOSystem.sound().mute(1);
+  myOSystem.frameBuffer().clear();
+
   if(myDisplayFormat == "AUTO" || myOSystem.settings().getBool("rominfo"))
   {
-    // Run the TIA, looking for PAL scanline patterns
-    // We turn off the SuperCharger progress bars, otherwise the SC BIOS
-    // will take over 250 frames!
-    // The 'fastscbios' option must be changed before the system is reset
-    bool fastscbios = myOSystem.settings().getBool("fastscbios");
-    myOSystem.settings().setValue("fastscbios", true);
+    autodetectFrameLayout();
 
-    uInt8 initialGarbageFrames = FrameManager::initialGarbageFrames();
-    uInt8 linesPAL = 0;
-    uInt8 linesNTSC = 0;
-
-    mySystem->reset(true);  // autodetect in reset enabled
-    myTIA->autodetectLayout(true);
-    for(int i = 0; i < 60; ++i) {
-      if (i > initialGarbageFrames)
-        myTIA->frameLayout() == FrameLayout::pal ? linesPAL++ : linesNTSC++;
-
-      myTIA->update();
-    }
-
-    myDisplayFormat = linesPAL > linesNTSC  ? "PAL" : "NTSC";
     if(myProperties.get(Display_Format) == "AUTO")
     {
       autodetected = "*";
       myCurrentFormat = 0;
     }
-
-    // Don't forget to reset the SC progress bars again
-    myOSystem.settings().setValue("fastscbios", fastscbios);
   }
+
+  if (atoi(myProperties.get(Display_YStart).c_str()) == 0) {
+    autodetectYStart();
+  }
+
   myConsoleInfo.DisplayFormat = myDisplayFormat + autodetected;
 
   // Set up the correct properties used when toggling format
@@ -183,14 +179,7 @@ Console::Console(OSystem& osystem, unique_ptr<Cartridge>& cart,
     myConsoleTiming = ConsoleTiming::secam;
   }
 
-  // Bumper Bash always require all 4 directions
-  // Other ROMs can use it if the setting is enabled
-  // Hopefully this list should stay short
-  // If it starts to get too long, we should add a ROM properties entry
-  bool joyallow4 = md5 == "aa1c41f86ec44c0a44eb64c332ce08af" || // Bumper Bash
-                   md5 == "16ee443c990215f61f7dd1e55a0d2256" || // Bumper Bash (PAL)
-                   md5 == "1bf503c724001b09be79c515ecfcbd03" || // Bumper Bash (Unknown)
-                   myOSystem.settings().getBool("joyallow4");
+  bool joyallow4 = myOSystem.settings().getBool("joyallow4");
   myOSystem.eventHandler().allowAllDirections(joyallow4);
 
   // Reset the system to its power-on state
@@ -199,8 +188,9 @@ Console::Console(OSystem& osystem, unique_ptr<Cartridge>& cart,
   // Finally, add remaining info about the console
   myConsoleInfo.CartName   = myProperties.get(Cartridge_Name);
   myConsoleInfo.CartMD5    = myProperties.get(Cartridge_MD5);
-  myConsoleInfo.Control0   = myLeftControl->about();
-  myConsoleInfo.Control1   = myRightControl->about();
+  bool swappedPorts = properties().get(Console_SwapPorts) == "YES";
+  myConsoleInfo.Control0   = myLeftControl->about(swappedPorts);
+  myConsoleInfo.Control1   = myRightControl->about(swappedPorts);
   myConsoleInfo.BankSwitch = myCart->about();
 
   myCart->setRomName(myConsoleInfo.CartName);
@@ -217,6 +207,54 @@ Console::~Console()
   myRightControl->close();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::autodetectFrameLayout()
+{
+  // Run the TIA, looking for PAL scanline patterns
+  // We turn off the SuperCharger progress bars, otherwise the SC BIOS
+  // will take over 250 frames!
+  // The 'fastscbios' option must be changed before the system is reset
+  bool fastscbios = myOSystem.settings().getBool("fastscbios");
+  myOSystem.settings().setValue("fastscbios", true);
+
+  FrameLayoutDetector frameLayoutDetector;
+  myTIA->setFrameManager(&frameLayoutDetector);
+  mySystem->reset(true);
+
+  for(int i = 0; i < 60; ++i) myTIA->update();
+
+  myTIA->setFrameManager(myFrameManager.get());
+
+  myDisplayFormat = frameLayoutDetector.detectedLayout() == FrameLayout::pal ? "PAL" : "NTSC";
+
+  // Don't forget to reset the SC progress bars again
+  myOSystem.settings().setValue("fastscbios", fastscbios);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::autodetectYStart()
+{
+  // We turn off the SuperCharger progress bars, otherwise the SC BIOS
+  // will take over 250 frames!
+  // The 'fastscbios' option must be changed before the system is reset
+  bool fastscbios = myOSystem.settings().getBool("fastscbios");
+  myOSystem.settings().setValue("fastscbios", true);
+
+  YStartDetector ystartDetector;
+  ystartDetector.setLayout(myDisplayFormat == "PAL" ? FrameLayout::pal : FrameLayout::ntsc);
+  myTIA->setFrameManager(&ystartDetector);
+  mySystem->reset(true);
+
+  for (int i = 0; i < 80; i++) myTIA->update();
+
+  myTIA->setFrameManager(myFrameManager.get());
+
+  myAutodetectedYstart = ystartDetector.detectedYStart() - YSTART_EXTRA;
+
+  // Don't forget to reset the SC progress bars again
+  myOSystem.settings().setValue("fastscbios", fastscbios);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Console::save(Serializer& out) const
 {
@@ -330,10 +368,12 @@ void Console::toggleFormat(int direction)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Console::toggleColorLoss()
 {
-  bool colorloss = !myOSystem.settings().getBool("colorloss");
+  bool colorloss = !myTIA->colorLossEnabled();
   if(myTIA->enableColorLoss(colorloss))
   {
-    myOSystem.settings().setValue("colorloss", colorloss);
+    myOSystem.settings().setValue(
+      myOSystem.settings().getBool("dev.settings") ? "dev.colorloss" : "plr.colorloss", colorloss);
+
     string message = string("PAL color-loss ") +
                      (colorloss ? "enabled" : "disabled");
     myOSystem.frameBuffer().showMessage(message);
@@ -344,7 +384,7 @@ void Console::toggleColorLoss()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Console::toggleColorLoss(bool state)
+void Console::enableColorLoss(bool state)
 {
   myTIA->enableColorLoss(state);
 }
@@ -487,18 +527,20 @@ void Console::setProperties(const Properties& props)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 FBInitStatus Console::initializeVideo(bool full)
 {
-  FBInitStatus fbstatus = kSuccess;
+  FBInitStatus fbstatus = FBInitStatus::Success;
 
   if(full)
   {
+    bool devSettings = myOSystem.settings().getBool("dev.settings");
     const string& title = string("Stella ") + STELLA_VERSION +
                    ": \"" + myProperties.get(Cartridge_Name) + "\"";
     fbstatus = myOSystem.frameBuffer().createDisplay(title,
                  myTIA->width() << 1, myTIA->height());
-    if(fbstatus != kSuccess)
+    if(fbstatus != FBInitStatus::Success)
       return fbstatus;
 
-    myOSystem.frameBuffer().showFrameStats(myOSystem.settings().getBool("stats"));
+    myOSystem.frameBuffer().showFrameStats(
+      myOSystem.settings().getBool(devSettings ? "dev.stats" : "plr.stats"));
     generateColorLossPalette();
   }
   setPalette(myOSystem.settings().getString("palette"));
@@ -569,7 +611,7 @@ void Console::changeYStart(int direction)
 
   if(direction == +1)       // increase YStart
   {
-    if(ystart >= FrameManager::maxYStart)
+    if(ystart >= TIAConstants::maxYStart)
     {
       myOSystem.frameBuffer().showMessage("YStart at maximum");
       return;
@@ -578,11 +620,17 @@ void Console::changeYStart(int direction)
   }
   else if(direction == -1)  // decrease YStart
   {
-    if(ystart == FrameManager::minYStart-1)
+    if (ystart == TIAConstants::minYStart && myAutodetectedYstart == 0) {
+      myOSystem.frameBuffer().showMessage("Autodetected YStart not available");
+      return;
+    }
+
+    if(ystart == TIAConstants::minYStart-1 && myAutodetectedYstart > 0)
     {
       myOSystem.frameBuffer().showMessage("YStart at minimum");
       return;
     }
+
     ystart--;
   }
   else
@@ -590,16 +638,16 @@ void Console::changeYStart(int direction)
 
   ostringstream val;
   val << ystart;
-  if(ystart == FrameManager::minYStart-1)
+  if(ystart == TIAConstants::minYStart-1)
     myOSystem.frameBuffer().showMessage("YStart autodetected");
   else
   {
-    if(myTIA->ystartIsAuto(ystart))
+    if(myAutodetectedYstart > 0 && myAutodetectedYstart == ystart)
     {
       // We've reached the auto-detect value, so reset
       myOSystem.frameBuffer().showMessage("YStart " + val.str() + " (Auto)");
       val.str("");
-      val << FrameManager::minYStart-1;
+      val << TIAConstants::minYStart-1;
     }
     else
       myOSystem.frameBuffer().showMessage("YStart " + val.str());
@@ -619,7 +667,7 @@ void Console::changeHeight(int direction)
   if(direction == +1)       // increase Height
   {
     height++;
-    if(height > FrameManager::maxViewableHeight || height > dheight)
+    if(height > TIAConstants::maxViewableHeight || height > dheight)
     {
       myOSystem.frameBuffer().showMessage("Height at maximum");
       return;
@@ -628,7 +676,7 @@ void Console::changeHeight(int direction)
   else if(direction == -1)  // decrease Height
   {
     height--;
-    if(height < FrameManager::minViewableHeight) height = 0;
+    if(height < TIAConstants::minViewableHeight) height = 0;
   }
   else
     return;
@@ -648,12 +696,10 @@ void Console::setTIAProperties()
 {
   uInt32 ystart = atoi(myProperties.get(Display_YStart).c_str());
   if(ystart != 0)
-    ystart = BSPF::clamp(ystart, FrameManager::minYStart, FrameManager::maxYStart);
+    ystart = BSPF::clamp(ystart, TIAConstants::minYStart, TIAConstants::maxYStart);
   uInt32 height = atoi(myProperties.get(Display_Height).c_str());
   if(height != 0)
-    height = BSPF::clamp(height, FrameManager::minViewableHeight, FrameManager::maxViewableHeight);
-
-  myTIA->autodetectLayout(false);
+    height = BSPF::clamp(height, TIAConstants::minViewableHeight, TIAConstants::maxViewableHeight);
 
   if(myDisplayFormat == "NTSC" || myDisplayFormat == "PAL60" ||
      myDisplayFormat == "SECAM60")
@@ -675,20 +721,16 @@ void Console::setTIAProperties()
     myTIA->setLayout(FrameLayout::pal);
   }
 
-  myTIA->setYStart(ystart);
+  myTIA->setYStart(ystart != 0 ? ystart : myAutodetectedYstart);
   myTIA->setHeight(height);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Console::setControllers(const string& rommd5)
 {
-  // Setup the controllers based on properties
-  const string& left  = myProperties.get(Controller_Left);
-  const string& right = myProperties.get(Controller_Right);
-
-  // Check for CompuMate controllers; they are special in that a handler
-  // creates them for us, and also that they must be used in both ports
-  if(left == "COMPUMATE" || right == "COMPUMATE")
+  // Check for CompuMate scheme; it is special in that a handler creates both
+  // controllers for us, and associates them with the bankswitching class
+  if(myCart->detectedType() == "CM")
   {
     myCMHandler = make_shared<CompuMate>(*this, myEvent, *mySystem);
 
@@ -701,153 +743,114 @@ void Console::setControllers(const string& rommd5)
 
     myLeftControl  = std::move(myCMHandler->leftController());
     myRightControl = std::move(myCMHandler->rightController());
-    return;
-  }
-
-  unique_ptr<Controller> leftC  = std::move(myLeftControl),
-                         rightC = std::move(myRightControl);
-
-  // Also check if we should swap the paddles plugged into a jack
-  bool swapPaddles = myProperties.get(Controller_SwapPaddles) == "YES";
-
-  // Construct left controller
-  if(left == "JOYSTICK")
-  {
-    // Already created in c'tor
-    // We save some time by not looking at all the other types
-    if(!leftC)
-      leftC = make_unique<Joystick>(Controller::Left, myEvent, *mySystem);
-  }
-  else if(left == "BOOSTERGRIP")
-  {
-    leftC = make_unique<BoosterGrip>(Controller::Left, myEvent, *mySystem);
-  }
-  else if(left == "DRIVING")
-  {
-    leftC = make_unique<Driving>(Controller::Left, myEvent, *mySystem);
-  }
-  else if((left == "KEYBOARD") || (left == "KEYPAD"))
-  {
-    leftC = make_unique<Keyboard>(Controller::Left, myEvent, *mySystem);
-  }
-  else if(BSPF::startsWithIgnoreCase(left, "PADDLES"))
-  {
-    bool swapAxis = false, swapDir = false;
-    if(left == "PADDLES_IAXIS")
-      swapAxis = true;
-    else if(left == "PADDLES_IDIR")
-      swapDir = true;
-    else if(left == "PADDLES_IAXDR")
-      swapAxis = swapDir = true;
-    leftC = make_unique<Paddles>(Controller::Left, myEvent, *mySystem,
-                              swapPaddles, swapAxis, swapDir);
-  }
-  else if(left == "AMIGAMOUSE")
-  {
-    leftC = make_unique<AmigaMouse>(Controller::Left, myEvent, *mySystem);
-  }
-  else if(left == "ATARIMOUSE")
-  {
-    leftC = make_unique<AtariMouse>(Controller::Left, myEvent, *mySystem);
-  }
-  else if(left == "TRAKBALL")
-  {
-    leftC = make_unique<TrakBall>(Controller::Left, myEvent, *mySystem);
-  }
-  else if(left == "GENESIS")
-  {
-    leftC = make_unique<Genesis>(Controller::Left, myEvent, *mySystem);
-  }
-  else if(left == "MINDLINK")
-  {
-    leftC = make_unique<MindLink>(Controller::Left, myEvent, *mySystem);
-  }
-
-  // Construct right controller
-  if(right == "JOYSTICK")
-  {
-    // Already created in c'tor
-    // We save some time by not looking at all the other types
-    if(!rightC)
-      rightC = make_unique<Joystick>(Controller::Right, myEvent, *mySystem);
-  }
-  else if(right == "BOOSTERGRIP")
-  {
-    rightC = make_unique<BoosterGrip>(Controller::Right, myEvent, *mySystem);
-  }
-  else if(right == "DRIVING")
-  {
-    rightC = make_unique<Driving>(Controller::Right, myEvent, *mySystem);
-  }
-  else if((right == "KEYBOARD") || (right == "KEYPAD"))
-  {
-    rightC = make_unique<Keyboard>(Controller::Right, myEvent, *mySystem);
-  }
-  else if(BSPF::startsWithIgnoreCase(right, "PADDLES"))
-  {
-    bool swapAxis = false, swapDir = false;
-    if(right == "PADDLES_IAXIS")
-      swapAxis = true;
-    else if(right == "PADDLES_IDIR")
-      swapDir = true;
-    else if(right == "PADDLES_IAXDR")
-      swapAxis = swapDir = true;
-    rightC = make_unique<Paddles>(Controller::Right, myEvent, *mySystem,
-                               swapPaddles, swapAxis, swapDir);
-  }
-  else if(left == "AMIGAMOUSE")
-  {
-    rightC = make_unique<AmigaMouse>(Controller::Left, myEvent, *mySystem);
-  }
-  else if(left == "ATARIMOUSE")
-  {
-    rightC = make_unique<AtariMouse>(Controller::Left, myEvent, *mySystem);
-  }
-  else if(left == "TRAKBALL")
-  {
-    rightC = make_unique<TrakBall>(Controller::Left, myEvent, *mySystem);
-  }
-  else if(right == "ATARIVOX")
-  {
-    const string& nvramfile = myOSystem.nvramDir() + "atarivox_eeprom.dat";
-    rightC = make_unique<AtariVox>(Controller::Right, myEvent,
-                   *mySystem, myOSystem.serialPort(),
-                   myOSystem.settings().getString("avoxport"), nvramfile);
-  }
-  else if(right == "SAVEKEY")
-  {
-    const string& nvramfile = myOSystem.nvramDir() + "savekey_eeprom.dat";
-    rightC = make_unique<SaveKey>(Controller::Right, myEvent, *mySystem,
-                               nvramfile);
-  }
-  else if(right == "GENESIS")
-  {
-    rightC = make_unique<Genesis>(Controller::Right, myEvent, *mySystem);
-  }
-  else if(right == "KIDVID")
-  {
-    rightC = make_unique<KidVid>(Controller::Right, myEvent, *mySystem, rommd5);
-  }
-  else if(right == "MINDLINK")
-  {
-    rightC = make_unique<MindLink>(Controller::Right, myEvent, *mySystem);
-  }
-
-  // Swap the ports if necessary
-  if(myProperties.get(Console_SwapPorts) == "NO")
-  {
-    myLeftControl  = std::move(leftC);
-    myRightControl = std::move(rightC);
   }
   else
   {
-    myLeftControl  = std::move(rightC);
-    myRightControl = std::move(leftC);
+    // Setup the controllers based on properties
+    const string& left  = myProperties.get(Controller_Left);
+    const string& right = myProperties.get(Controller_Right);
+
+    unique_ptr<Controller> leftC = getControllerPort(rommd5, left, Controller::Left),
+      rightC = getControllerPort(rommd5, right, Controller::Right);
+
+    // Swap the ports if necessary
+    if(myProperties.get(Console_SwapPorts) == "NO")
+    {
+      myLeftControl = std::move(leftC);
+      myRightControl = std::move(rightC);
+    }
+    else
+    {
+      myLeftControl = std::move(rightC);
+      myRightControl = std::move(leftC);
+    }
   }
 
   myTIA->bindToControllers();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+unique_ptr<Controller> Console::getControllerPort(const string& rommd5,
+    const string& controllerName, Controller::Jack port)
+{
+  unique_ptr<Controller> controller = std::move(myLeftControl);
+
+  if(controllerName == "JOYSTICK")
+  {
+    // Already created in c'tor
+    // We save some time by not looking at all the other types
+    if(!controller)
+      controller = make_unique<Joystick>(port, myEvent, *mySystem);
+  }
+  else if(controllerName == "BOOSTERGRIP")
+  {
+    controller = make_unique<BoosterGrip>(port, myEvent, *mySystem);
+  }
+  else if(controllerName == "DRIVING")
+  {
+    controller = make_unique<Driving>(port, myEvent, *mySystem);
+  }
+  else if((controllerName == "KEYBOARD") || (controllerName == "KEYPAD"))
+  {
+    controller = make_unique<Keyboard>(port, myEvent, *mySystem);
+  }
+  else if(BSPF::startsWithIgnoreCase(controllerName, "PADDLES"))
+  {
+    // Also check if we should swap the paddles plugged into a jack
+    bool swapPaddles = myProperties.get(Controller_SwapPaddles) == "YES";
+    bool swapAxis = false, swapDir = false;
+    if(controllerName == "PADDLES_IAXIS")
+      swapAxis = true;
+    else if(controllerName == "PADDLES_IDIR")
+      swapDir = true;
+    else if(controllerName == "PADDLES_IAXDR")
+      swapAxis = swapDir = true;
+    controller = make_unique<Paddles>(port, myEvent, *mySystem,
+                                      swapPaddles, swapAxis, swapDir);
+  }
+  else if(controllerName == "AMIGAMOUSE")
+  {
+    controller = make_unique<AmigaMouse>(port, myEvent, *mySystem);
+  }
+  else if(controllerName == "ATARIMOUSE")
+  {
+    controller = make_unique<AtariMouse>(port, myEvent, *mySystem);
+  }
+  else if(controllerName == "TRAKBALL")
+  {
+    controller = make_unique<TrakBall>(port, myEvent, *mySystem);
+  }
+  else if(controllerName == "ATARIVOX")
+  {
+    const string& nvramfile = myOSystem.nvramDir() + "atarivox_eeprom.dat";
+    controller = make_unique<AtariVox>(port, myEvent,
+        *mySystem, myOSystem.serialPort(),
+        myOSystem.settings().getString("avoxport"), nvramfile);
+  }
+  else if(controllerName == "SAVEKEY")
+  {
+    const string& nvramfile = myOSystem.nvramDir() + "savekey_eeprom.dat";
+    controller = make_unique<SaveKey>(port, myEvent, *mySystem,
+                                      nvramfile);
+  }
+  else if(controllerName == "GENESIS")
+  {
+    controller = make_unique<Genesis>(port, myEvent, *mySystem);
+  }
+  else if(controllerName == "KIDVID")
+  {
+    controller = make_unique<KidVid>(port, myEvent, *mySystem, rommd5);
+  }
+  else if(controllerName == "MINDLINK")
+  {
+    controller = make_unique<MindLink>(port, myEvent, *mySystem);
+  }
+  else  // What else can we do?
+    controller = make_unique<Joystick>(port, myEvent, *mySystem);
+
+  return controller;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Console::loadUserPalette()
 {
@@ -911,7 +914,7 @@ void Console::generateColorLossPalette()
   uInt32* palette[9] = {
     &ourNTSCPalette[0],    &ourPALPalette[0],    &ourSECAMPalette[0],
     &ourNTSCPaletteZ26[0], &ourPALPaletteZ26[0], &ourSECAMPaletteZ26[0],
-    0, 0, 0
+    nullptr, nullptr, nullptr
   };
   if(myUserPaletteDefined)
   {
@@ -922,7 +925,7 @@ void Console::generateColorLossPalette()
 
   for(int i = 0; i < 9; ++i)
   {
-    if(palette[i] == 0)
+    if(palette[i] == nullptr)
       continue;
 
     // Fill the odd numbered palette entries with gray values (calculated
@@ -1006,11 +1009,11 @@ void Console::attachDebugger(Debugger& dbg)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Console::stateChanged(EventHandler::State state)
+void Console::stateChanged(EventHandlerState state)
 {
   // For now, only the CompuMate cares about state changes
   if(myCMHandler)
-    myCMHandler->enableKeyHandling(state == EventHandler::S_EMULATE);
+    myCMHandler->enableKeyHandling(state == EventHandlerState::EMULATION);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
