@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -27,7 +27,7 @@ Controller::Controller(Jack jack, const Event& event, const System& system,
     myEvent(event),
     mySystem(system),
     myType(type),
-    myOnAnalogPinUpdateCallback(0)
+    myOnAnalogPinUpdateCallback(nullptr)
 {
   myDigitalPinState[One]   =
   myDigitalPinState[Two]   =
