@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -26,8 +26,10 @@
 #include "DialogContainer.hxx"
 #include "Event.hxx"
 #include "FrameBuffer.hxx"
+#include "TIASurface.hxx"
 #include "FSNode.hxx"
 #include "Launcher.hxx"
+#include "TimeMachine.hxx"
 #include "Menu.hxx"
 #include "OSystem.hxx"
 #include "Joystick.hxx"
@@ -42,6 +44,7 @@
 #include "Switches.hxx"
 #include "M6532.hxx"
 #include "MouseControl.hxx"
+#include "PNGLibrary.hxx"
 #include "Version.hxx"
 
 #include "EventHandler.hxx"
@@ -58,11 +61,12 @@
 EventHandler::EventHandler(OSystem& osystem)
   : myOSystem(osystem),
     myOverlay(nullptr),
-    myState(S_NONE),
+    myState(EventHandlerState::NONE),
     myAllowAllDirectionsFlag(false),
     myFryingFlag(false),
     myUseCtrlKeyFlag(true),
     mySkipMouseMotion(true),
+    myIs7800(false),
     myAltKeyCounter(0),
     myContSnapshotInterval(0),
     myContSnapshotCounter(0)
@@ -81,6 +85,11 @@ EventHandler::EventHandler(OSystem& osystem)
   myJoyHandler = make_unique<JoystickHandler>(osystem);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+EventHandler::~EventHandler()
+{
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EventHandler::initialize()
 {
@@ -110,21 +119,18 @@ void EventHandler::initialize()
   // Default phosphor blend
   Properties::setDefault(Display_PPBlend,
                          myOSystem.settings().getString("tv.phosblend"));
+
+  // Toggle 7800 mode
+  set7800Mode();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EventHandler::reset(State state)
+void EventHandler::reset(EventHandlerState state)
 {
   setEventState(state);
   myOSystem.state().reset();
 
   setContinuousSnapshots(0);
-
-  // Reset events almost immediately after starting emulation mode
-  // We wait a little while, since 'hold' events may be present, and we want
-  // time for the ROM to process them
-  if(state == S_EMULATE)
-    SDL_AddTimer(500, resetEventsCallback, static_cast<void*>(this));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -191,6 +197,15 @@ void EventHandler::toggleSAPortOrder()
 #endif
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void EventHandler::set7800Mode()
+{
+  if(myOSystem.hasConsole())
+    myIs7800 = myOSystem.console().switches().toggle7800Mode(myOSystem.settings());
+  else
+    myIs7800 = false;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EventHandler::poll(uInt64 time)
 {
@@ -199,31 +214,24 @@ void EventHandler::poll(uInt64 time)
 
   // Update controllers and console switches, and in general all other things
   // related to emulation
-  if(myState == S_EMULATE)
+  if(myState == EventHandlerState::EMULATION)
   {
     myOSystem.console().riot().update();
 
-#if 0
     // Now check if the StateManager should be saving or loading state
-    // Per-frame cheats are disabled if the StateManager is active, since
-    // it would interfere with proper playback
-    if(myOSystem.state().isActive())
-    {
+    // (for rewind and/or movies
+    if(myOSystem.state().mode() != StateManager::Mode::Off)
       myOSystem.state().update();
-    }
-    else
-#endif
-    {
-    #ifdef CHEATCODE_SUPPORT
-      for(auto& cheat: myOSystem.cheat().perFrame())
-        cheat->evaluate();
-    #endif
 
-      // Handle continuous snapshots
-      if(myContSnapshotInterval > 0 &&
-        (++myContSnapshotCounter % myContSnapshotInterval == 0))
-        takeSnapshot(uInt32(time) >> 10);  // not quite milliseconds, but close enough
-    }
+  #ifdef CHEATCODE_SUPPORT
+    for(auto& cheat: myOSystem.cheat().perFrame())
+      cheat->evaluate();
+  #endif
+
+    // Handle continuous snapshots
+    if(myContSnapshotInterval > 0 &&
+      (++myContSnapshotCounter % myContSnapshotInterval == 0))
+      takeSnapshot(uInt32(time) >> 10);  // not quite milliseconds, but close enough
   }
   else if(myOverlay)
   {
@@ -266,7 +274,7 @@ void EventHandler::handleKeyEvent(StellaKey key, StellaMod mod, bool state)
 
   // An attempt to speed up event processing; we quickly check for
   // Control or Alt/Cmd combos first
-  if(kbdAlt(mod) && state)
+  if(StellaModTest::isAlt(mod) && state)
   {
 #ifdef BSPF_MAC_OSX
     // These keys work in all states
@@ -286,9 +294,36 @@ void EventHandler::handleKeyEvent(StellaKey key, StellaMod mod, bool state)
     {
       myOSystem.frameBuffer().toggleFullscreen();
     }
-    // These only work when in emulation mode
-    else if(myState == S_EMULATE)
+    // State rewinding must work in pause mode too
+    else if(myState == EventHandlerState::EMULATION || myState == EventHandlerState::PAUSE)
     {
+      switch(key)
+      {
+        case KBDK_LEFT:  // Alt-left(-shift) rewinds 1(10) states
+          enterTimeMachineMenuMode((StellaModTest::isShift(mod) && state) ? 10 : 1, false);
+          return;
+
+        case KBDK_RIGHT:  // Alt-right(-shift) unwinds 1(10) states
+          enterTimeMachineMenuMode((StellaModTest::isShift(mod) && state) ? 10 : 1, true);
+          return;
+
+        case KBDK_DOWN:  // Alt-down rewinds to start of list
+          enterTimeMachineMenuMode(1000, false);
+          return;
+
+        case KBDK_UP:  // Alt-up rewinds to end of list
+          enterTimeMachineMenuMode(1000, true);
+          return;
+
+        default:
+          handled = false;
+          break;
+      }
+    }
+    // These only work when in emulation mode
+    if(!handled && myState == EventHandlerState::EMULATION)
+    {
+      handled = true;
       switch(key)
       {
         case KBDK_EQUALS:
@@ -447,6 +482,10 @@ void EventHandler::handleKeyEvent(StellaKey key, StellaMod mod, bool state)
           myOSystem.frameBuffer().toggleFrameStats();
           break;
 
+        case KBDK_T:  // Alt-t toggles Time Machine
+          myOSystem.state().toggleTimeMachine();
+          break;
+
         case KBDK_S:
           if(myContSnapshotInterval == 0)
           {
@@ -454,12 +493,12 @@ void EventHandler::handleKeyEvent(StellaKey key, StellaMod mod, bool state)
             uInt32 interval = myOSystem.settings().getInt("ssinterval");
             if(mod & KBDM_SHIFT)
             {
-              buf << "Enabling shotshots every frame";
+              buf << "Enabling snapshots every frame";
               interval = 1;
             }
             else
             {
-              buf << "Enabling shotshots in " << interval << " second intervals";
+              buf << "Enabling snapshots in " << interval << " second intervals";
               interval *= uInt32(myOSystem.frameRate());
             }
             myOSystem.frameBuffer().showMessage(buf.str());
@@ -484,7 +523,7 @@ void EventHandler::handleKeyEvent(StellaKey key, StellaMod mod, bool state)
     else
       handled = false;
   }
-  else if(kbdControl(mod) && state && myUseCtrlKeyFlag)
+  else if(StellaModTest::isControl(mod) && state && myUseCtrlKeyFlag)
   {
     // These keys work in all states
     if(key == KBDK_Q)
@@ -492,7 +531,7 @@ void EventHandler::handleKeyEvent(StellaKey key, StellaMod mod, bool state)
       handleEvent(Event::Quit, 1);
     }
     // These only work when in emulation mode
-    else if(myState == S_EMULATE)
+    else if(myState == EventHandlerState::EMULATION)
     {
       switch(key)
       {
@@ -511,7 +550,11 @@ void EventHandler::handleKeyEvent(StellaKey key, StellaMod mod, bool state)
 
         case KBDK_G:  // Ctrl-g (un)grabs mouse
           if(!myOSystem.frameBuffer().fullScreen())
+          {
             myOSystem.frameBuffer().toggleGrabMouse();
+            myOSystem.frameBuffer().showMessage(myOSystem.frameBuffer().grabMouseEnabled()
+                                                ? "Grab mouse enabled" : "Grab mouse disabled");
+          }
           break;
 
         case KBDK_L:  // Ctrl-l toggles PAL color-loss effect
@@ -565,20 +608,30 @@ void EventHandler::handleKeyEvent(StellaKey key, StellaMod mod, bool state)
 
   // Handle keys which switch eventhandler state
   // Arrange the logic to take advantage of short-circuit evaluation
-  if(!(kbdControl(mod) || kbdShift(mod) || kbdAlt(mod)) &&
+  if(!(StellaModTest::isControl(mod) || StellaModTest::isShift(mod) || StellaModTest::isAlt(mod)) &&
       !state && eventStateChange(myKeyTable[key][kEmulationMode]))
     return;
 
   // Otherwise, let the event handler deal with it
   switch(myState)
   {
-    case S_EMULATE:
+    case EventHandlerState::EMULATION:
       handleEvent(myKeyTable[key][kEmulationMode], state);
       break;
-    case S_PAUSE:
-      if(myKeyTable[key][kEmulationMode] == Event::TakeSnapshot)
-        handleEvent(myKeyTable[key][kEmulationMode], state);
+
+    case EventHandlerState::PAUSE:
+      switch(myKeyTable[key][kEmulationMode])
+      {
+        case Event::TakeSnapshot:
+        case Event::DebuggerMode:
+          handleEvent(myKeyTable[key][kEmulationMode], state);
+          break;
+
+        default:
+          break;
+      }
       break;
+
     default:
       if(myOverlay)
         myOverlay->handleKeyEvent(key, mod, state);
@@ -587,10 +640,10 @@ void EventHandler::handleKeyEvent(StellaKey key, StellaMod mod, bool state)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EventHandler::handleMouseMotionEvent(int x, int y, int xrel, int yrel, int button)
+void EventHandler::handleMouseMotionEvent(int x, int y, int xrel, int yrel)
 {
   // Determine which mode we're in, then send the event to the appropriate place
-  if(myState == S_EMULATE)
+  if(myState == EventHandlerState::EMULATION)
   {
     if(!mySkipMouseMotion)
     {
@@ -600,35 +653,30 @@ void EventHandler::handleMouseMotionEvent(int x, int y, int xrel, int yrel, int
     mySkipMouseMotion = false;
   }
   else if(myOverlay)
-    myOverlay->handleMouseMotionEvent(x, y, button);
+    myOverlay->handleMouseMotionEvent(x, y);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EventHandler::handleMouseButtonEvent(MouseButton b, int x, int y)
+void EventHandler::handleMouseButtonEvent(MouseButton b, bool pressed,
+                                          int x, int y)
 {
   // Determine which mode we're in, then send the event to the appropriate place
-  if(myState == S_EMULATE)
+  if(myState == EventHandlerState::EMULATION)
   {
     switch(b)
     {
-      case EVENT_LBUTTONDOWN:
-        myEvent.set(Event::MouseButtonLeftValue, 1);
+      case MouseButton::LEFT:
+        myEvent.set(Event::MouseButtonLeftValue, int(pressed));
         break;
-      case EVENT_LBUTTONUP:
-        myEvent.set(Event::MouseButtonLeftValue, 0);
-        break;
-      case EVENT_RBUTTONDOWN:
-        myEvent.set(Event::MouseButtonRightValue, 1);
-        break;
-      case EVENT_RBUTTONUP:
-        myEvent.set(Event::MouseButtonRightValue, 0);
+      case MouseButton::RIGHT:
+        myEvent.set(Event::MouseButtonRightValue, int(pressed));
         break;
       default:
         return;
     }
   }
   else if(myOverlay)
-    myOverlay->handleMouseButtonEvent(b, x, y);
+    myOverlay->handleMouseButtonEvent(b, pressed, x, y);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -646,7 +694,7 @@ void EventHandler::handleJoyEvent(int stick, int button, uInt8 state)
         return;
 
       // Determine which mode we're in, then send the event to the appropriate place
-      if(myState == S_EMULATE)
+      if(myState == EventHandlerState::EMULATION)
         handleEvent(joy->btnTable[button][kEmulationMode], state);
       else if(myOverlay)
         myOverlay->handleJoyEvent(stick, button, state);
@@ -666,7 +714,7 @@ void EventHandler::handleJoyEvent(int stick, int button, uInt8 state)
       // The 'type-4' here refers to the fact that 'StellaJoystick::JT_2600DAPTOR_LEFT'
       // and 'StellaJoystick::JT_2600DAPTOR_RIGHT' are at index 4 and 5 in the JoyType
       // enum; subtracting four gives us Controller 0 and 1
-      if(myState == S_EMULATE)
+      if(myState == EventHandlerState::EMULATION)
       {
         switch(myOSystem.console().leftController().type())
         {
@@ -701,7 +749,7 @@ void EventHandler::handleJoyAxisEvent(int stick, int axis, int value)
   switch(joy->type)
   {
     case StellaJoystick::JT_REGULAR:
-      if(myState == S_EMULATE)
+      if(myState == EventHandlerState::EMULATION)
       {
         // Every axis event has two associated values, negative and positive
         Event::Type eventAxisNeg = joy->axisTable[axis][0][kEmulationMode];
@@ -807,31 +855,31 @@ void EventHandler::handleJoyHatEvent(int stick, int hat, int value)
   // Preprocess all hat events, converting to Stella JoyHat type
   // Generate multiple equivalent hat events representing combined direction
   // when we get a diagonal hat event
-  if(myState == S_EMULATE)
+  if(myState == EventHandlerState::EMULATION)
   {
-    handleEvent(joy->hatTable[hat][EVENT_HATUP][kEmulationMode],
+    handleEvent(joy->hatTable[hat][int(JoyHat::UP)][kEmulationMode],
                 value & EVENT_HATUP_M);
-    handleEvent(joy->hatTable[hat][EVENT_HATRIGHT][kEmulationMode],
+    handleEvent(joy->hatTable[hat][int(JoyHat::RIGHT)][kEmulationMode],
                 value & EVENT_HATRIGHT_M);
-    handleEvent(joy->hatTable[hat][EVENT_HATDOWN][kEmulationMode],
+    handleEvent(joy->hatTable[hat][int(JoyHat::DOWN)][kEmulationMode],
                 value & EVENT_HATDOWN_M);
-    handleEvent(joy->hatTable[hat][EVENT_HATLEFT][kEmulationMode],
+    handleEvent(joy->hatTable[hat][int(JoyHat::LEFT)][kEmulationMode],
                 value & EVENT_HATLEFT_M);
   }
   else if(myOverlay)
   {
     if(value == EVENT_HATCENTER_M)
-      myOverlay->handleJoyHatEvent(stick, hat, EVENT_HATCENTER);
+      myOverlay->handleJoyHatEvent(stick, hat, JoyHat::CENTER);
     else
     {
       if(value & EVENT_HATUP_M)
-        myOverlay->handleJoyHatEvent(stick, hat, EVENT_HATUP);
+        myOverlay->handleJoyHatEvent(stick, hat, JoyHat::UP);
       if(value & EVENT_HATRIGHT_M)
-        myOverlay->handleJoyHatEvent(stick, hat, EVENT_HATRIGHT);
+        myOverlay->handleJoyHatEvent(stick, hat, JoyHat::RIGHT);
       if(value & EVENT_HATDOWN_M)
-        myOverlay->handleJoyHatEvent(stick, hat, EVENT_HATDOWN);
+        myOverlay->handleJoyHatEvent(stick, hat, JoyHat::DOWN);
       if(value & EVENT_HATLEFT_M)
-        myOverlay->handleJoyHatEvent(stick, hat, EVENT_HATLEFT);
+        myOverlay->handleJoyHatEvent(stick, hat, JoyHat::LEFT);
     }
   }
 }
@@ -841,19 +889,19 @@ void EventHandler::handleSystemEvent(SystemEvent e, int, int)
 {
   switch(e)
   {
-    case EVENT_WINDOW_EXPOSED:
+    case SystemEvent::WINDOW_EXPOSED:
       myOSystem.frameBuffer().update();
       break;
 
-    case EVENT_WINDOW_FOCUS_GAINED:
+    case SystemEvent::WINDOW_FOCUS_GAINED:
       // Used to handle Alt-x key combos; sometimes the key associated with
       // Alt gets 'stuck'  and is passed to the core for processing
       if(myAltKeyCounter > 0)
         myAltKeyCounter = 2;
       break;
 #if 0
-    case EVENT_WINDOW_MINIMIZED:
-      if(myState == S_EMULATE) enterMenuMode(S_MENU);
+    case SystemEvent::WINDOW_MINIMIZED:
+      if(myState == EventHandlerState::EMULATION) enterMenuMode(EventHandlerState::OPTIONSMENU);
         break;
 #endif
     default:  // handle other events as testing requires
@@ -863,7 +911,7 @@ void EventHandler::handleSystemEvent(SystemEvent e, int, int)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EventHandler::handleEvent(Event::Type event, int state)
+void EventHandler::handleEvent(Event::Type event, Int32 state)
 {
   // Take care of special events that aren't part of the emulation core
   // or need to be preprocessed before passing them on
@@ -941,8 +989,8 @@ void EventHandler::handleEvent(Event::Type event, int state)
       return;
 
     case Event::LauncherMode:
-      if((myState == S_EMULATE || myState == S_CMDMENU ||
-          myState == S_DEBUGGER) && state)
+      if((myState == EventHandlerState::EMULATION || myState == EventHandlerState::CMDMENU ||
+          myState == EventHandlerState::DEBUGGER) && state)
       {
         // Go back to the launcher, or immediately quit
         if(myOSystem.settings().getBool("exitlauncher") ||
@@ -990,21 +1038,21 @@ void EventHandler::handleEvent(Event::Type event, int state)
     ////////////////////////////////////////////////////////////////////////
     // Events which relate to switches()
     case Event::ConsoleColor:
-      if(state)
+      if(state && !myIs7800)
       {
         myEvent.set(Event::ConsoleBlackWhite, 0);
         myOSystem.frameBuffer().showMessage("Color Mode");
       }
       break;
     case Event::ConsoleBlackWhite:
-      if(state)
+      if(state && !myIs7800)
       {
         myEvent.set(Event::ConsoleColor, 0);
         myOSystem.frameBuffer().showMessage("BW Mode");
       }
       break;
     case Event::ConsoleColorToggle:
-      if(state)
+      if(state && !myIs7800)
       {
         if(myOSystem.console().switches().tvColor())
         {
@@ -1100,6 +1148,56 @@ void EventHandler::handleEvent(Event::Type event, int state)
   myEvent.set(event, state);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void EventHandler::handleConsoleStartupEvents()
+{
+  bool update = false;
+  if(myOSystem.settings().getBool("holdreset"))
+  {
+    handleEvent(Event::ConsoleReset, 1);
+    update = true;
+  }
+  if(myOSystem.settings().getBool("holdselect"))
+  {
+    handleEvent(Event::ConsoleSelect, 1);
+    update = true;
+  }
+
+  const string& holdjoy0 = myOSystem.settings().getString("holdjoy0");
+  update = update || holdjoy0 != "";
+  if(BSPF::containsIgnoreCase(holdjoy0, "U"))
+    handleEvent(Event::JoystickZeroUp, 1);
+  if(BSPF::containsIgnoreCase(holdjoy0, "D"))
+    handleEvent(Event::JoystickZeroDown, 1);
+  if(BSPF::containsIgnoreCase(holdjoy0, "L"))
+    handleEvent(Event::JoystickZeroLeft, 1);
+  if(BSPF::containsIgnoreCase(holdjoy0, "R"))
+    handleEvent(Event::JoystickZeroRight, 1);
+  if(BSPF::containsIgnoreCase(holdjoy0, "F"))
+    handleEvent(Event::JoystickZeroFire, 1);
+
+  const string& holdjoy1 = myOSystem.settings().getString("holdjoy1");
+  update = update || holdjoy1 != "";
+  if(BSPF::containsIgnoreCase(holdjoy1, "U"))
+    handleEvent(Event::JoystickOneUp, 1);
+  if(BSPF::containsIgnoreCase(holdjoy1, "D"))
+    handleEvent(Event::JoystickOneDown, 1);
+  if(BSPF::containsIgnoreCase(holdjoy1, "L"))
+    handleEvent(Event::JoystickOneLeft, 1);
+  if(BSPF::containsIgnoreCase(holdjoy1, "R"))
+    handleEvent(Event::JoystickOneRight, 1);
+  if(BSPF::containsIgnoreCase(holdjoy1, "F"))
+    handleEvent(Event::JoystickOneFire, 1);
+
+  if(update)
+    myOSystem.console().riot().update();
+
+#ifdef DEBUGGER_SUPPORT
+  if(myOSystem.settings().getBool("debug"))
+    enterDebugMode();
+#endif
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool EventHandler::eventStateChange(Event::Type type)
 {
@@ -1108,34 +1206,44 @@ bool EventHandler::eventStateChange(Event::Type type)
   switch(type)
   {
     case Event::PauseMode:
-      if(myState == S_EMULATE)
-        setEventState(S_PAUSE);
-      else if(myState == S_PAUSE)
-        setEventState(S_EMULATE);
+      if(myState == EventHandlerState::EMULATION)
+        setEventState(EventHandlerState::PAUSE);
+      else if(myState == EventHandlerState::PAUSE)
+        setEventState(EventHandlerState::EMULATION);
       else
         handled = false;
       break;
 
-    case Event::MenuMode:
-      if(myState == S_EMULATE)
-        enterMenuMode(S_MENU);
+    case Event::OptionsMenuMode:
+      if(myState == EventHandlerState::EMULATION || myState == EventHandlerState::PAUSE)
+        enterMenuMode(EventHandlerState::OPTIONSMENU);
       else
         handled = false;
       break;
 
     case Event::CmdMenuMode:
-      if(myState == S_EMULATE)
-        enterMenuMode(S_CMDMENU);
-      else if(myState == S_CMDMENU)
+      if(myState == EventHandlerState::EMULATION || myState == EventHandlerState::PAUSE)
+        enterMenuMode(EventHandlerState::CMDMENU);
+      else if(myState == EventHandlerState::CMDMENU)
+        leaveMenuMode();
+      else
+        handled = false;
+      break;
+
+    case Event::TimeMachineMode:
+      if(myState == EventHandlerState::EMULATION || myState == EventHandlerState::PAUSE)
+        enterTimeMachineMenuMode(0, false);
+      else if(myState == EventHandlerState::TIMEMACHINE)
         leaveMenuMode();
       else
         handled = false;
       break;
 
     case Event::DebuggerMode:
-      if(myState == S_EMULATE)
+      if(myState == EventHandlerState::EMULATION || myState == EventHandlerState::PAUSE
+         || myState == EventHandlerState::TIMEMACHINE)
         enterDebugMode();
-      else if(myState == S_DEBUGGER)
+      else if(myState == EventHandlerState::DEBUGGER)
         leaveDebugMode();
       else
         handled = false;
@@ -1244,12 +1352,13 @@ void EventHandler::setActionMappings(EventMode mode)
           {
             buf.str("");
             buf << "J" << stick << "/H" << hat;
-            switch(dir)
+            switch(JoyHat(dir))
             {
-              case EVENT_HATUP:    buf << "/up";    break;
-              case EVENT_HATDOWN:  buf << "/down";  break;
-              case EVENT_HATLEFT:  buf << "/left";  break;
-              case EVENT_HATRIGHT: buf << "/right"; break;
+              case JoyHat::UP:    buf << "/up";    break;
+              case JoyHat::DOWN:  buf << "/down";  break;
+              case JoyHat::LEFT:  buf << "/left";  break;
+              case JoyHat::RIGHT: buf << "/right"; break;
+              case JoyHat::CENTER:                 break;
             }
             if(key == "")
               key = key + buf.str();
@@ -1404,8 +1513,7 @@ bool EventHandler::addJoyAxisMapping(Event::Type event, EventMode mode,
   const StellaJoystick* joy = myJoyHandler->joy(stick);
   if(joy)
   {
-    if(axis >= 0 && axis < joy->numAxes &&
-       event >= 0 && event < Event::LastType)
+    if(axis >= 0 && axis < joy->numAxes && event < Event::LastType)
     {
       // This confusing code is because each axis has two associated values,
       // but analog events only affect one of the axis.
@@ -1440,8 +1548,7 @@ bool EventHandler::addJoyButtonMapping(Event::Type event, EventMode mode,
   const StellaJoystick* joy = myJoyHandler->joy(stick);
   if(joy)
   {
-    if(button >= 0 && button < joy->numButtons &&
-       event >= 0 && event < Event::LastType)
+    if(button >= 0 && button < joy->numButtons && event < Event::LastType)
     {
       joy->btnTable[button][mode] = event;
       if(updateMenus)
@@ -1455,17 +1562,17 @@ bool EventHandler::addJoyButtonMapping(Event::Type event, EventMode mode,
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool EventHandler::addJoyHatMapping(Event::Type event, EventMode mode,
-                                    int stick, int hat, int value,
+                                    int stick, int hat, JoyHat value,
                                     bool updateMenus)
 {
 #ifdef JOYSTICK_SUPPORT
   const StellaJoystick* joy = myJoyHandler->joy(stick);
   if(joy)
   {
-    if(hat >= 0 && hat < joy->numHats &&
-       event >= 0 && event < Event::LastType && value != EVENT_HATCENTER)
+    if(hat >= 0 && hat < joy->numHats && event < Event::LastType &&
+       value != JoyHat::CENTER)
     {
-      joy->hatTable[hat][value][mode] = event;
+      joy->hatTable[hat][int(value)][mode] = event;
       if(updateMenus)
         setActionMappings(mode);
       return true;
@@ -1578,8 +1685,9 @@ void EventHandler::setDefaultKeymap(Event::Type event, EventMode mode)
       setDefaultKey( KBDK_F12,       Event::TakeSnapshot      );
       setDefaultKey( KBDK_BACKSPACE, Event::Fry               );
       setDefaultKey( KBDK_PAUSE,     Event::PauseMode         );
-      setDefaultKey( KBDK_TAB,       Event::MenuMode          );
+      setDefaultKey( KBDK_TAB,       Event::OptionsMenuMode   );
       setDefaultKey( KBDK_BACKSLASH, Event::CmdMenuMode       );
+      setDefaultKey( KBDK_T,         Event::TimeMachineMode   );
       setDefaultKey( KBDK_GRAVE,     Event::DebuggerMode      );
       setDefaultKey( KBDK_ESCAPE,    Event::LauncherMode      );
       break;
@@ -1887,7 +1995,7 @@ void EventHandler::takeSnapshot(uInt32 number)
   {
     // Make sure we have a 'clean' image, with no onscreen messages
     myOSystem.frameBuffer().enableMessages(false);
-    myOSystem.frameBuffer().tiaSurface().render();
+    myOSystem.frameBuffer().tiaSurface().reRender();
 
     string message = "Snapshot saved";
     try
@@ -1959,7 +2067,7 @@ void EventHandler::setContinuousSnapshots(uInt32 interval)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EventHandler::enterMenuMode(State state)
+void EventHandler::enterMenuMode(EventHandlerState state)
 {
   setEventState(state);
   myOverlay->reStack();
@@ -1969,7 +2077,7 @@ void EventHandler::enterMenuMode(State state)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EventHandler::leaveMenuMode()
 {
-  setEventState(S_EMULATE);
+  setEventState(EventHandlerState::EMULATION);
   myOSystem.sound().mute(false);
 }
 
@@ -1977,7 +2085,7 @@ void EventHandler::leaveMenuMode()
 bool EventHandler::enterDebugMode()
 {
 #ifdef DEBUGGER_SUPPORT
-  if(myState == S_DEBUGGER || !myOSystem.hasConsole())
+  if(myState == EventHandlerState::DEBUGGER || !myOSystem.hasConsole())
     return false;
 
   // Make sure debugger starts in a consistent state
@@ -1985,23 +2093,23 @@ bool EventHandler::enterDebugMode()
   // mode, since it takes care of locking the debugger state, which will
   // probably be modified below
   myOSystem.debugger().setStartState();
-  setEventState(S_DEBUGGER);
+  setEventState(EventHandlerState::DEBUGGER);
 
   FBInitStatus fbstatus = myOSystem.createFrameBuffer();
-  if(fbstatus != kSuccess)
+  if(fbstatus != FBInitStatus::Success)
   {
     myOSystem.debugger().setQuitState();
-    setEventState(S_EMULATE);
-    if(fbstatus == kFailTooLarge)
+    setEventState(EventHandlerState::EMULATION);
+    if(fbstatus == FBInitStatus::FailTooLarge)
       myOSystem.frameBuffer().showMessage("Debugger window too large for screen",
-                                           kBottomCenter, true);
+                                          MessagePosition::BottomCenter, true);
     return false;
   }
   myOverlay->reStack();
   myOSystem.sound().mute(true);
 #else
   myOSystem.frameBuffer().showMessage("Debugger support not included",
-                                       kBottomCenter, true);
+                                      MessagePosition::BottomCenter, true);
 #endif
 
   return true;
@@ -2012,20 +2120,33 @@ void EventHandler::leaveDebugMode()
 {
 #ifdef DEBUGGER_SUPPORT
   // paranoia: this should never happen:
-  if(myState != S_DEBUGGER)
+  if(myState != EventHandlerState::DEBUGGER)
     return;
 
   // Make sure debugger quits in a consistent state
   myOSystem.debugger().setQuitState();
 
-  setEventState(S_EMULATE);
+  setEventState(EventHandlerState::EMULATION);
   myOSystem.createFrameBuffer();
   myOSystem.sound().mute(false);
 #endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EventHandler::setEventState(State state)
+void EventHandler::enterTimeMachineMenuMode(uInt32 numWinds, bool unwind)
+{
+  // add one extra state if we are in Time Machine mode
+  // TODO: maybe remove this state if we leave the menu at this new state
+  myOSystem.state().addExtraState("enter Time Machine dialog"); // force new state
+  if(numWinds)
+    myOSystem.state().windStates(numWinds, unwind);
+
+  // TODO: display last wind message (numWinds != 0) in time machine dialog
+  enterMenuMode(EventHandlerState::TIMEMACHINE);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void EventHandler::setEventState(EventHandlerState state)
 {
   myState = state;
 
@@ -2037,7 +2158,7 @@ void EventHandler::setEventState(State state)
   // keyboard acts as one large joystick with many (single) buttons
   switch(myState)
   {
-    case S_EMULATE:
+    case EventHandlerState::EMULATION:
       myOverlay = nullptr;
       myOSystem.sound().mute(false);
       enableTextEvents(false);
@@ -2045,35 +2166,42 @@ void EventHandler::setEventState(State state)
         myUseCtrlKeyFlag = false;
       break;
 
-    case S_PAUSE:
+    case EventHandlerState::PAUSE:
       myOverlay = nullptr;
       myOSystem.sound().mute(true);
       enableTextEvents(false);
       break;
 
-    case S_MENU:
+    case EventHandlerState::OPTIONSMENU:
       myOverlay = &myOSystem.menu();
       enableTextEvents(true);
       break;
 
-    case S_CMDMENU:
+    case EventHandlerState::CMDMENU:
       myOverlay = &myOSystem.commandMenu();
       enableTextEvents(true);
       break;
 
-    case S_LAUNCHER:
+    case EventHandlerState::TIMEMACHINE:
+      myOSystem.timeMachine().requestResize();
+      myOverlay = &myOSystem.timeMachine();
+      enableTextEvents(true);
+      break;
+
+    case EventHandlerState::LAUNCHER:
       myOverlay = &myOSystem.launcher();
       enableTextEvents(true);
+      myEvent.clear();
       break;
 
-#ifdef DEBUGGER_SUPPORT
-    case S_DEBUGGER:
+    case EventHandlerState::DEBUGGER:
+  #ifdef DEBUGGER_SUPPORT
       myOverlay = &myOSystem.debugger();
       enableTextEvents(true);
+  #endif
       break;
-#endif
 
-    default:
+    case EventHandlerState::NONE:
       myOverlay = nullptr;
       break;
   }
@@ -2085,21 +2213,11 @@ void EventHandler::setEventState(State state)
   if(myOSystem.hasConsole())
     myOSystem.console().stateChanged(myState);
 
-  // Always clear any pending events when changing states
-  myEvent.clear();
-
   // Sometimes an extraneous mouse motion event is generated
   // after a state change, which should be supressed
   mySkipMouseMotion = true;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt32 EventHandler::resetEventsCallback(uInt32 interval, void* param)
-{
-  (static_cast<EventHandler*>(param))->myEvent.clear();
-  return 0;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 EventHandler::ActionList EventHandler::ourEmulActionList[kEmulActionListSize] = {
   { Event::ConsoleSelect,          "Select",                   "", true  },
@@ -2107,6 +2225,7 @@ EventHandler::ActionList EventHandler::ourEmulActionList[kEmulActionListSize] =
   { Event::ConsoleColor,           "Color TV",                 "", true  },
   { Event::ConsoleBlackWhite,      "Black & White TV",         "", true  },
   { Event::ConsoleColorToggle,     "Swap Color / B&W TV",      "", true  },
+  { Event::Console7800Pause,       "7800 Pause Key",           "", true  },
   { Event::ConsoleLeftDiffA,       "P0 Difficulty A",          "", true  },
   { Event::ConsoleLeftDiffB,       "P0 Difficulty B",          "", true  },
   { Event::ConsoleLeftDiffToggle,  "P0 Swap Difficulty",       "", true  },
@@ -2121,8 +2240,9 @@ EventHandler::ActionList EventHandler::ourEmulActionList[kEmulActionListSize] =
   { Event::VolumeDecrease,         "Decrease volume",          "", false },
   { Event::VolumeIncrease,         "Increase volume",          "", false },
   { Event::PauseMode,              "Pause",                    "", false },
-  { Event::MenuMode,               "Enter options menu mode",  "", false },
-  { Event::CmdMenuMode,            "Toggle command menu mode", "", false },
+  { Event::OptionsMenuMode,        "Enter options menu UI",    "", false },
+  { Event::CmdMenuMode,            "Toggle command menu UI",   "", false },
+  { Event::TimeMachineMode,        "Toggle time machine UI",   "", false },
   { Event::DebuggerMode,           "Toggle debugger mode",     "", false },
   { Event::LauncherMode,           "Enter ROM launcher",       "", false },
   { Event::Quit,                   "Quit",                     "", false },
