@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,13 +16,14 @@
 //============================================================================
 
 #include <cassert>
-#include <iostream>
-#include <bitset>
 
 #include "Console.hxx"
 #include "Settings.hxx"
 #include "Switches.hxx"
 #include "System.hxx"
+#ifdef DEBUGGER_SUPPORT
+  #include "CartDebug.hxx"
+#endif
 
 #include "M6532.hxx"
 
@@ -31,7 +32,8 @@ M6532::M6532(const Console& console, const Settings& settings)
   : myConsole(console),
     mySettings(settings),
     myTimer(0), mySubTimer(0), myDivider(1),
-    myTimerWrapped(false), myWrappedThisCycle(false), mySetTimerCycle(0), myLastCycle(0),
+    myTimerWrapped(false), myWrappedThisCycle(false),
+    mySetTimerCycle(0), myLastCycle(0),
     myDDRA(0), myDDRB(0), myOutA(0), myOutB(0),
     myInterruptFlag(false),
     myEdgeDetectPositive(false)
@@ -41,8 +43,23 @@ M6532::M6532(const Console& console, const Settings& settings)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void M6532::reset()
 {
+  static constexpr uInt8 RAM_7800[128] = {
+    0xA9, 0x00, 0xAA, 0x85, 0x01, 0x95, 0x03, 0xE8, 0xE0, 0x2A, 0xD0, 0xF9, 0x85, 0x02, 0xA9, 0x04,
+    0xEA, 0x30, 0x23, 0xA2, 0x04, 0xCA, 0x10, 0xFD, 0x9A, 0x8D, 0x10, 0x01, 0x20, 0xCB, 0x04, 0x20,
+    0xCB, 0x04, 0x85, 0x11, 0x85, 0x1B, 0x85, 0x1C, 0x85, 0x0F, 0xEA, 0x85, 0x02, 0xA9, 0x00, 0xEA,
+    0x30, 0x04, 0x24, 0x03, 0x30, 0x09, 0xA9, 0x02, 0x85, 0x09, 0x8D, 0x12, 0xF1, 0xD0, 0x1E, 0x24,
+    0x02, 0x30, 0x0C, 0xA9, 0x02, 0x85, 0x06, 0x8D, 0x18, 0xF1, 0x8D, 0x60, 0xF4, 0xD0, 0x0E, 0x85,
+    0x2C, 0xA9, 0x08, 0x85, 0x1B, 0x20, 0xCB, 0x04, 0xEA, 0x24, 0x02, 0x30, 0xD9, 0xA9, 0xFD, 0x85,
+    0x08, 0x6C, 0xFC, 0xFF, 0xEA, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
+    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
+  };
+
   // Initialize the 128 bytes of memory
-  if(mySettings.getBool("ramrandom"))
+  bool devSettings = mySettings.getBool("dev.settings");
+  if(mySettings.getString(devSettings ? "dev.console" : "plr.console") == "7800")
+    for(uInt32 t = 0; t < 128; ++t)
+      myRAM[t] = RAM_7800[t];
+  else if(mySettings.getBool(devSettings ? "dev.ramrandom" : "plr.ramrandom"))
     for(uInt32 t = 0; t < 128; ++t)
       myRAM[t] = mySystem->randGenerator().next();
   else
@@ -54,8 +71,7 @@ void M6532::reset()
   myTimerWrapped = false;
   myWrappedThisCycle = false;
 
-  mySetTimerCycle = 0;
-  myLastCycle = mySystem->cycles();
+  mySetTimerCycle = myLastCycle = 0;
 
   // Zero the I/O registers
   myDDRA = myDDRB = myOutA = myOutB = 0x00;
@@ -68,19 +84,14 @@ void M6532::reset()
 
   // Edge-detect set to negative (high to low)
   myEdgeDetectPositive = false;
-}
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void M6532::systemCyclesReset()
-{
-  // System cycles are being reset to zero so we need to adjust
-  // the cycle count we remembered when the timer was last set
-  myLastCycle -= mySystem->cycles();
-  mySetTimerCycle -= mySystem->cycles();
+  // Let the controllers know about the reset
+  myConsole.leftController().reset();
+  myConsole.rightController().reset();
 
-  // We should also inform any 'smart' controllers as well
-  myConsole.leftController().systemCyclesReset();
-  myConsole.rightController().systemCyclesReset();
+#ifdef DEBUGGER_SUPPORT
+  createAccessBases();
+#endif // DEBUGGER_SUPPORT
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -109,7 +120,7 @@ void M6532::update()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void M6532::updateEmulation()
 {
-  uInt32 cycles = mySystem->cycles() - myLastCycle;
+  uInt32 cycles = uInt32(mySystem->cycles() - myLastCycle);
   uInt32 subTimer = mySubTimer;
 
   // Guard against further state changes if the debugger alread forwarded emulation
@@ -159,10 +170,17 @@ void M6532::installDelegate(System& system, Device& device)
   // All accesses are to the given device
   System::PageAccess access(&device, System::PA_READWRITE);
 
-  // We're installing in a 2600 system
-  for(int address = 0; address < 8192; address += (1 << System::PAGE_SHIFT))
-    if((address & 0x1080) == 0x0080)
-      mySystem->setPageAccess(address >> System::PAGE_SHIFT, access);
+  // Map all peek/poke to mirrors of RIOT address space to this class
+  // That is, all mirrors of ZP RAM ($80 - $FF) and IO ($280 - $29F) in the
+  // lower 4K of the 2600 address space are mapped here
+  // The two types of addresses are differentiated in peek/poke as follows:
+  //    (addr & 0x0200) == 0x0200 is IO     (A9 is 1)
+  //    (addr & 0x0300) == 0x0100 is Stack  (A8 is 1, A9 is 0)
+  //    (addr & 0x0300) == 0x0000 is ZP RAM (A8 is 0, A9 is 0)
+  for (uInt16 addr = 0; addr < 0x1000; addr += System::PAGE_SIZE)
+    if ((addr & 0x0080) == 0x0080) {
+      mySystem->setPageAccess(addr, access);
+    }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -170,11 +188,10 @@ uInt8 M6532::peek(uInt16 addr)
 {
   updateEmulation();
 
-  // Access RAM directly.  Originally, accesses to RAM could bypass
-  // this method and its pages could be installed directly into the
-  // system.  However, certain cartridges (notably 4A50) can mirror
-  // the RAM address space, making it necessary to chain accesses.
-  if((addr & 0x1080) == 0x0080 && (addr & 0x0200) == 0x0000)
+  // A9 distinguishes I/O registers from ZP RAM
+  // A9 = 1 is read from I/O
+  // A9 = 0 is read from RAM
+  if((addr & 0x0200) == 0x0000)
     return myRAM[addr & 0x007f];
 
   switch(addr & 0x07)
@@ -239,11 +256,10 @@ bool M6532::poke(uInt16 addr, uInt8 value)
 {
   updateEmulation();
 
-  // Access RAM directly.  Originally, accesses to RAM could bypass
-  // this method and its pages could be installed directly into the
-  // system.  However, certain cartridges (notably 4A50) can mirror
-  // the RAM address space, making it necessary to chain accesses.
-  if((addr & 0x1080) == 0x0080 && (addr & 0x0200) == 0x0000)
+  // A9 distinguishes I/O registers from ZP RAM
+  // A9 = 1 is write to I/O
+  // A9 = 0 is write to RAM
+  if((addr & 0x0200) == 0x0000)
   {
     myRAM[addr & 0x007f] = value;
     return true;
@@ -361,8 +377,8 @@ bool M6532::save(Serializer& out) const
     out.putInt(myDivider);
     out.putBool(myTimerWrapped);
     out.putBool(myWrappedThisCycle);
-    out.putInt(myLastCycle);
-    out.putInt(mySetTimerCycle);
+    out.putLong(myLastCycle);
+    out.putLong(mySetTimerCycle);
 
     out.putByte(myDDRA);
     out.putByte(myDDRB);
@@ -397,8 +413,8 @@ bool M6532::load(Serializer& in)
     myDivider = in.getInt();
     myTimerWrapped = in.getBool();
     myWrappedThisCycle = in.getBool();
-    myLastCycle = in.getInt();
-    mySetTimerCycle = in.getInt();
+    myLastCycle = in.getLong();
+    mySetTimerCycle = in.getLong();
 
     myDDRA = in.getByte();
     myDDRB = in.getByte();
@@ -449,5 +465,51 @@ Int32 M6532::intimClocks()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt32 M6532::timerClocks() const
 {
-  return mySystem->cycles() - mySetTimerCycle;
+  return uInt32(mySystem->cycles() - mySetTimerCycle);
 }
+
+#ifdef DEBUGGER_SUPPORT
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void M6532::createAccessBases()
+{
+  myRAMAccessBase = make_unique<uInt8[]>(RAM_SIZE);
+  memset(myRAMAccessBase.get(), CartDebug::NONE, RAM_SIZE);
+  myStackAccessBase = make_unique<uInt8[]>(STACK_SIZE);
+  memset(myStackAccessBase.get(), CartDebug::NONE, STACK_SIZE);
+  myIOAccessBase = make_unique<uInt8[]>(IO_SIZE);
+  memset(myIOAccessBase.get(), CartDebug::NONE, IO_SIZE);
+
+  myZPAccessDelay = make_unique<uInt8[]>(RAM_SIZE);
+  memset(myZPAccessDelay.get(), ZP_DELAY, RAM_SIZE);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8 M6532::getAccessFlags(uInt16 address) const
+{
+  if (address & IO_BIT)
+    return myIOAccessBase[address & IO_MASK];
+  else if (address & STACK_BIT)
+    return myStackAccessBase[address & STACK_MASK];
+  else
+    return myRAMAccessBase[address & RAM_MASK];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void M6532::setAccessFlags(uInt16 address, uInt8 flags)
+{
+  // ignore none flag
+  if (flags != CartDebug::NONE) {
+    if (address & IO_BIT)
+      myIOAccessBase[address & IO_MASK] |= flags;
+    else {
+      // the first access, either by direct RAM or stack access is assumed as initialization
+      if (myZPAccessDelay[address & RAM_MASK])
+        myZPAccessDelay[address & RAM_MASK]--;
+      else if (address & STACK_BIT)
+        myStackAccessBase[address & STACK_MASK] |= flags;
+      else
+        myRAMAccessBase[address & RAM_MASK] |= flags;
+    }
+  }
+}
+#endif // DEBUGGER_SUPPORT
