@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,8 +16,6 @@
 //============================================================================
 
 #include <cassert>
-#include <sstream>
-#include <fstream>
 
 #include <ctime>
 #ifdef HAVE_GETTIMEOFDAY
@@ -41,12 +39,16 @@
 #include "MD5.hxx"
 #include "Cart.hxx"
 #include "CartDetector.hxx"
+#include "FrameBuffer.hxx"
+#include "TIASurface.hxx"
 #include "Settings.hxx"
 #include "PropsSet.hxx"
 #include "EventHandler.hxx"
 #include "Menu.hxx"
 #include "CommandMenu.hxx"
 #include "Launcher.hxx"
+#include "TimeMachine.hxx"
+#include "PNGLibrary.hxx"
 #include "Widget.hxx"
 #include "Console.hxx"
 #include "Random.hxx"
@@ -62,7 +64,7 @@ OSystem::OSystem()
     myQuitLoop(false)
 {
   // Calculate startup time
-  myMillisAtStart = uInt32(time(NULL) * 1000);
+  myMillisAtStart = uInt32(time(nullptr) * 1000);
 
   // Get built-in features
   #ifdef SOUND_SUPPORT
@@ -138,6 +140,7 @@ bool OSystem::create()
   // Create menu and launcher GUI objects
   myMenu = make_unique<Menu>(*this);
   myCommandMenu = make_unique<CommandMenu>(*this);
+  myTimeMachine = make_unique<TimeMachine>(*this);
   myLauncher = make_unique<Launcher>(*this);
   myStateManager = make_unique<StateManager>(*this);
 
@@ -190,8 +193,8 @@ void OSystem::setConfigPaths()
   string s;
 
   validatePath(myStateDir, "statedir", myBaseDir + "state");
-  validatePath(mySnapshotSaveDir, "snapsavedir", defaultSnapSaveDir());
-  validatePath(mySnapshotLoadDir, "snaploaddir", defaultSnapLoadDir());
+  validatePath(mySnapshotSaveDir, "snapsavedir", defaultSaveDir());
+  validatePath(mySnapshotLoadDir, "snaploaddir", defaultLoadDir());
   validatePath(myNVRamDir, "nvramdir", myBaseDir + "nvram");
   validatePath(myCfgDir, "cfgdir", myBaseDir + "cfg");
 
@@ -244,30 +247,31 @@ void OSystem::setFramerate(float framerate)
 FBInitStatus OSystem::createFrameBuffer()
 {
   // Re-initialize the framebuffer to current settings
-  FBInitStatus fbstatus = kFailComplete;
+  FBInitStatus fbstatus = FBInitStatus::FailComplete;
   switch(myEventHandler->state())
   {
-    case EventHandler::S_EMULATE:
-    case EventHandler::S_PAUSE:
-    case EventHandler::S_MENU:
-    case EventHandler::S_CMDMENU:
-      if((fbstatus = myConsole->initializeVideo()) != kSuccess)
+    case EventHandlerState::EMULATION:
+    case EventHandlerState::PAUSE:
+    case EventHandlerState::OPTIONSMENU:
+    case EventHandlerState::CMDMENU:
+    case EventHandlerState::TIMEMACHINE:
+      if((fbstatus = myConsole->initializeVideo()) != FBInitStatus::Success)
         return fbstatus;
-      break;  // S_EMULATE, S_PAUSE, S_MENU, S_CMDMENU
+      break;
 
-    case EventHandler::S_LAUNCHER:
-      if((fbstatus = myLauncher->initializeVideo()) != kSuccess)
+    case EventHandlerState::LAUNCHER:
+      if((fbstatus = myLauncher->initializeVideo()) != FBInitStatus::Success)
         return fbstatus;
-      break;  // S_LAUNCHER
+      break;
 
-#ifdef DEBUGGER_SUPPORT
-    case EventHandler::S_DEBUGGER:
-      if((fbstatus = myDebugger->initializeVideo()) != kSuccess)
+    case EventHandlerState::DEBUGGER:
+  #ifdef DEBUGGER_SUPPORT
+      if((fbstatus = myDebugger->initializeVideo()) != FBInitStatus::Success)
         return fbstatus;
-      break;  // S_DEBUGGER
-#endif
+  #endif
+      break;
 
-    default:  // Should never happen
+    case EventHandlerState::NONE:  // Should never happen
       logMessage("ERROR: Unknown emulation state in createFrameBuffer()", 0);
       break;
   }
@@ -330,12 +334,12 @@ string OSystem::createConsole(const FilesystemNode& rom, const string& md5sum,
   #ifdef CHEATCODE_SUPPORT
     myCheatManager->loadCheats(myRomMD5);
   #endif
-    myEventHandler->reset(EventHandler::S_EMULATE);
+    myEventHandler->reset(EventHandlerState::EMULATION);
     myEventHandler->setMouseControllerMode(mySettings->getString("usemouse"));
-    if(createFrameBuffer() != kSuccess)  // Takes care of initializeVideo()
+    if(createFrameBuffer() != FBInitStatus::Success)  // Takes care of initializeVideo()
     {
       logMessage("ERROR: Couldn't create framebuffer for console", 0);
-      myEventHandler->reset(EventHandler::S_LAUNCHER);
+      myEventHandler->reset(EventHandlerState::LAUNCHER);
       return "ERROR: Couldn't create framebuffer for console";
     }
     myConsole->initializeAudio();
@@ -361,38 +365,7 @@ string OSystem::createConsole(const FilesystemNode& rom, const string& md5sum,
 
     // Also check if certain virtual buttons should be held down
     // These must be checked each time a new console is being created
-    if(mySettings->getBool("holdreset"))
-      myEventHandler->handleEvent(Event::ConsoleReset, 1);
-    if(mySettings->getBool("holdselect"))
-      myEventHandler->handleEvent(Event::ConsoleSelect, 1);
-
-    const string& holdjoy0 = mySettings->getString("holdjoy0");
-    if(BSPF::containsIgnoreCase(holdjoy0, "U"))
-      myEventHandler->handleEvent(Event::JoystickZeroUp, 1);
-    if(BSPF::containsIgnoreCase(holdjoy0, "D"))
-      myEventHandler->handleEvent(Event::JoystickZeroDown, 1);
-    if(BSPF::containsIgnoreCase(holdjoy0, "L"))
-      myEventHandler->handleEvent(Event::JoystickZeroLeft, 1);
-    if(BSPF::containsIgnoreCase(holdjoy0, "R"))
-      myEventHandler->handleEvent(Event::JoystickZeroRight, 1);
-    if(BSPF::containsIgnoreCase(holdjoy0, "F"))
-      myEventHandler->handleEvent(Event::JoystickZeroFire, 1);
-
-    const string& holdjoy1 = mySettings->getString("holdjoy1");
-    if(BSPF::containsIgnoreCase(holdjoy1, "U"))
-      myEventHandler->handleEvent(Event::JoystickOneUp, 1);
-    if(BSPF::containsIgnoreCase(holdjoy1, "D"))
-      myEventHandler->handleEvent(Event::JoystickOneDown, 1);
-    if(BSPF::containsIgnoreCase(holdjoy1, "L"))
-      myEventHandler->handleEvent(Event::JoystickOneLeft, 1);
-    if(BSPF::containsIgnoreCase(holdjoy1, "R"))
-      myEventHandler->handleEvent(Event::JoystickOneRight, 1);
-    if(BSPF::containsIgnoreCase(holdjoy1, "F"))
-      myEventHandler->handleEvent(Event::JoystickOneFire, 1);
-  #ifdef DEBUGGER_SUPPORT
-    if(mySettings->getBool("debug"))
-      myEventHandler->enterDebugMode();
-  #endif
+    myEventHandler->handleConsoleStartupEvents();
   }
   return EmptyString;
 }
@@ -407,7 +380,7 @@ bool OSystem::reloadConsole()
 bool OSystem::hasConsole() const
 {
   return myConsole != nullptr &&
-         myEventHandler->state() != EventHandler::S_LAUNCHER;
+         myEventHandler->state() != EventHandlerState::LAUNCHER;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -421,8 +394,8 @@ bool OSystem::createLauncher(const string& startdir)
   mySettings->setValue("tmpromdir", startdir);
   bool status = false;
 
-  myEventHandler->reset(EventHandler::S_LAUNCHER);
-  if(createFrameBuffer() == kSuccess)
+  myEventHandler->reset(EventHandlerState::LAUNCHER);
+  if(createFrameBuffer() == FBInitStatus::Success)
   {
     myLauncher->reStack();
     myFrameBuffer->setCursorState();
@@ -627,7 +600,7 @@ uInt64 OSystem::getTicks() const
 #ifdef HAVE_GETTIMEOFDAY
   // Gettimeofday natively refers to the UNIX epoch (a set time in the past)
   timeval now;
-  gettimeofday(&now, 0);
+  gettimeofday(&now, nullptr);
 
   return uInt64(now.tv_sec) * 1000000 + now.tv_usec;
 #else
@@ -660,7 +633,7 @@ void OSystem::mainLoop()
       // for that and reset the timers when appropriate
       if((myTimingInfo.virt - myTimingInfo.current) > (myTimePerFrame << 1))
       {
-        myTimingInfo.start = myTimingInfo.current = myTimingInfo.virt = getTicks();
+        myTimingInfo.current = myTimingInfo.virt = getTicks();
       }
 
       if(myTimingInfo.current < myTimingInfo.virt)
