@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -21,14 +21,26 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 SaveKey::SaveKey(Jack jack, const Event& event, const System& system,
-                 const string& eepromfile)
-  : Controller(jack, event, system, Controller::SaveKey)
+                 const string& eepromfile, Type type)
+  : Controller(jack, event, system, type)
 {
   myEEPROM = make_unique<MT24LC256>(eepromfile, system);
 
   myDigitalPinState[One] = myDigitalPinState[Two] = true;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+SaveKey::SaveKey(Jack jack, const Event& event, const System& system,
+                 const string& eepromfile)
+  : SaveKey(jack, event, system, eepromfile, Controller::SaveKey)
+{
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+SaveKey::~SaveKey()
+{
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool SaveKey::read(DigitalPin pin)
 {
@@ -72,17 +84,32 @@ void SaveKey::write(DigitalPin pin, bool value)
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void SaveKey::reset()
+{
+  myEEPROM->systemReset();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void SaveKey::close()
 {
-  // Force the EEPROM object to cleanup
   myEEPROM.reset();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void SaveKey::systemCyclesReset()
+void SaveKey::eraseAll()
 {
-  // The EEPROM keeps track of cycle counts, and needs to know when the
-  // cycles are reset
-  myEEPROM->systemCyclesReset();
+  myEEPROM->eraseAll();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void SaveKey::eraseCurrent()
+{
+  myEEPROM->eraseCurrent();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool SaveKey::isPageUsed(const uInt32 page) const
+{
+  return myEEPROM->isPageUsed(page);
 }
