@@ -8,16 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <fstream>
-#include <sstream>
-
 #include "FSNode.hxx"
 #include "Serializer.hxx"
 
@@ -38,7 +35,7 @@ Serializer::Serializer(const string& filename, bool readonly)
       {
         myStream = std::move(str);
         myStream->exceptions( ios_base::failbit | ios_base::badbit | ios_base::eofbit );
-        reset();
+        rewind();
       }
     }
   }
@@ -59,7 +56,7 @@ Serializer::Serializer(const string& filename, bool readonly)
     {
       myStream = std::move(str);
       myStream->exceptions( ios_base::failbit | ios_base::badbit | ios_base::eofbit );
-      reset();
+      rewind();
     }
   }
 }
@@ -76,12 +73,12 @@ Serializer::Serializer()
   {
     myStream->exceptions( ios_base::failbit | ios_base::badbit | ios_base::eofbit );
     putBool(true);
-    reset();
+    rewind();
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Serializer::reset()
+void Serializer::rewind()
 {
   myStream->clear();
   myStream->seekg(ios_base::beg);
@@ -133,6 +130,15 @@ void Serializer::getIntArray(uInt32* array, uInt32 size) const
   myStream->read(reinterpret_cast<char*>(array), sizeof(uInt32)*size);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt64 Serializer::getLong() const
+{
+  uInt64 val = 0;
+  myStream->read(reinterpret_cast<char*>(&val), sizeof(uInt64));
+
+  return val;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 double Serializer::getDouble() const
 {
@@ -195,6 +201,12 @@ void Serializer::putIntArray(const uInt32* array, uInt32 size)
   myStream->write(reinterpret_cast<const char*>(array), sizeof(uInt32)*size);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Serializer::putLong(uInt64 value)
+{
+  myStream->write(reinterpret_cast<char*>(&value), sizeof(uInt64));
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Serializer::putDouble(double value)
 {
