@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,9 +16,6 @@
 //============================================================================
 
 #include <cassert>
-#include <sstream>
-#include <fstream>
-#include <algorithm>
 
 #include "bspf.hxx"
 
@@ -37,19 +34,19 @@ Settings::Settings(OSystem& osystem)
 {
   // Video-related options
   setInternal("video", "");
+  setInternal("framerate", "0");
   setInternal("vsync", "true");
   setInternal("fullscreen", "false");
   setInternal("center", "false");
   setInternal("palette", "standard");
-  setInternal("colorloss", "false");
   setInternal("timing", "sleep");
   setInternal("uimessages", "true");
 
   // TIA specific options
   setInternal("tia.zoom", "3");
   setInternal("tia.inter", "false");
-  setInternal("tia.aspectn", "90");
-  setInternal("tia.aspectp", "100");
+  setInternal("tia.aspectn", "91");
+  setInternal("tia.aspectp", "109");
   setInternal("tia.fsfill", "false");
   setInternal("tia.dbgcolors", "roygpb");
 
@@ -57,8 +54,6 @@ Settings::Settings(OSystem& osystem)
   setInternal("tv.filter", "0");
   setInternal("tv.phosphor", "byrom");
   setInternal("tv.phosblend", "50");
-  setInternal("tv.jitter", "true");
-  setInternal("tv.jitter_recovery", "10");
   setInternal("tv.scanlines", "25");
   setInternal("tv.scaninter", "true");
   // TV options when using 'custom' mode
@@ -133,11 +128,7 @@ Settings::Settings(OSystem& osystem)
   setInternal("autoslot", "false");
   setInternal("loglevel", "1");
   setInternal("logtoconsole", "0");
-  setInternal("tiadriven", "false");
-  setInternal("cpurandom", "");
-  setInternal("ramrandom", "true");
   setInternal("avoxport", "");
-  setInternal("stats", "false");
   setInternal("fastscbios", "true");
   setInternal("threads", "false");
   setExternal("romloadcount", "0");
@@ -145,18 +136,56 @@ Settings::Settings(OSystem& osystem)
 
 #ifdef DEBUGGER_SUPPORT
   // Debugger/disassembly options
+  setInternal("dbg.fontsize", "medium");
   setInternal("dbg.fontstyle", "0");
-  setInternal("dbg.uhex", "true");
+  setInternal("dbg.uhex", "false");
+  setInternal("dbg.ghostreadstrap", "true");
   setInternal("dis.resolve", "true");
   setInternal("dis.gfxformat", "2");
   setInternal("dis.showaddr", "true");
   setInternal("dis.relocate", "false");
 #endif
 
-#ifdef DTHUMB_SUPPORT
+  // player settings
+  setInternal("plr.stats", "false");
+  setInternal("plr.bankrandom", "false");
+  setInternal("plr.ramrandom", "false");
+  setInternal("plr.cpurandom", "");
+  setInternal("plr.colorloss", "false");
+  setInternal("plr.tv.jitter", "true");
+  setInternal("plr.tv.jitter_recovery", "10");
+  setInternal("plr.debugcolors", "false");
+  setInternal("plr.tiadriven", "false");
+  setInternal("plr.console", "2600"); // 7800
+  setInternal("plr.timemachine", false);
+  setInternal("plr.tm.size", 100);
+  setInternal("plr.tm.uncompressed", 30);
+  setInternal("plr.tm.interval", "30f"); // = 0.5 seconds
+  setInternal("plr.tm.horizon", "10m"); // = ~10 minutes
   // Thumb ARM emulation options
-  setInternal("thumb.trapfatal", "true");
-#endif
+  setInternal("plr.thumb.trapfatal", "false");
+  setInternal("plr.eepromaccess", "false");
+
+  // developer settings
+  setInternal("dev.settings", "false");
+  setInternal("dev.stats", "true");
+  setInternal("dev.bankrandom", "true");
+  setInternal("dev.ramrandom", "true");
+  setInternal("dev.cpurandom", "SAXYP");
+  setInternal("dev.colorloss", "true");
+  setInternal("dev.tv.jitter", "true");
+  setInternal("dev.tv.jitter_recovery", "2");
+  setInternal("dev.debugcolors", "false");
+  setInternal("dev.tiadriven", "true");
+  setInternal("dev.console", "2600"); // 7800
+  setInternal("dev.timemachine", true);
+  setInternal("dev.tm.size", 100);
+  setInternal("dev.tm.uncompressed", 60);
+  setInternal("dev.tm.interval", "1f"); // = 1 frame
+  setInternal("dev.tm.horizon", "10s"); // = ~10 seconds
+  // Thumb ARM emulation options
+  setInternal("dev.thumb.trapfatal", "true");
+  setInternal("dev.eepromaccess", "true");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -291,8 +320,43 @@ void Settings::validate()
   i = getInt("tv.filter");
   if(i < 0 || i > 5)  setInternal("tv.filter", "0");
 
-  i = getInt("tv.jitter_recovery");
-  if(i < 1 || i > 20)  setInternal("tv.jitter_recovery", "10");
+  i = getInt("dev.tv.jitter_recovery");
+  if(i < 1 || i > 20) setInternal("dev.tv.jitter_recovery", "2");
+
+  int size = getInt("dev.tm.size");
+  if(size < 20 || size > 1000)
+  {
+    setInternal("dev.tm.size", 20);
+    size = 20;
+  }
+
+  i = getInt("dev.tm.uncompressed");
+  if(i < 0 || i > size) setInternal("dev.tm.uncompressed", size);
+
+  /*i = getInt("dev.tm.interval");
+  if(i < 0 || i > 5) setInternal("dev.tm.interval", 0);
+
+  i = getInt("dev.tm.horizon");
+  if(i < 0 || i > 6) setInternal("dev.tm.horizon", 1);*/
+
+  i = getInt("plr.tv.jitter_recovery");
+  if(i < 1 || i > 20) setInternal("plr.tv.jitter_recovery", "10");
+
+  size = getInt("plr.tm.size");
+  if(size < 20 || size > 1000)
+  {
+    setInternal("plr.tm.size", 20);
+    size = 20;
+  }
+
+  i = getInt("plr.tm.uncompressed");
+  if(i < 0 || i > size) setInternal("plr.tm.uncompressed", size);
+
+  /*i = getInt("plr.tm.interval");
+  if(i < 0 || i > 5) setInternal("plr.tm.interval", 3);
+
+  i = getInt("plr.tm.horizon");
+  if(i < 0 || i > 6) setInternal("plr.tm.horizon", 5);*/
 
 #ifdef SOUND_SUPPORT
   i = getInt("volume");
@@ -334,6 +398,10 @@ void Settings::validate()
   if(s != "small" && s != "medium" && s != "large")
     setInternal("launcherfont", "medium");
 
+  s = getString("dbg.fontsize");
+  if(s != "small" && s != "medium" && s != "large")
+    setInternal("dbg.fontsize", "medium");
+
   i = getInt("romviewer");
   if(i < 0)       setInternal("romviewer", "0");
   else if(i > 2)  setInternal("romviewer", "2");
@@ -370,7 +438,6 @@ void Settings::usage() const
     << "  -palette      <standard|     Use the specified color palette\n"
     << "                 z26|\n"
     << "                 user>\n"
-    << "  -colorloss    <1|0>          Enable PAL color-loss effect\n"
     << "  -framerate    <number>       Display the given number of frames per second (0 to auto-calculate)\n"
     << "  -timing       <sleep|busy>   Use the given type of wait between frames\n"
     << "  -uimessages   <1|0>          Show onscreen UI messages for different events\n"
@@ -392,8 +459,6 @@ void Settings::usage() const
     << "  -tv.filter    <0-5>          Set TV effects off (0) or to specified mode (1-5)\n"
     << "  -tv.phosphor  <always|byrom> When to use phosphor mode\n"
     << "  -tv.phosblend <0-100>        Set default blend level in phosphor mode\n"
-    << "  -tv.jitter    <1|0>          Enable TV jitter effect\n"
-    << "  -tv.jitter_recovery <1-20>   Set recovery time for TV jitter effect\n"
     << "  -tv.scanlines <0-100>        Set scanline intensity to percentage (0 disables completely)\n"
     << "  -tv.scaninter <1|0>          Enable interpolated (smooth) scanlines\n"
     << "  -tv.contrast    <value>      Set TV effects custom contrast to value 1.0 - 1.0\n"
@@ -423,7 +488,6 @@ void Settings::usage() const
     << "  -saport       <lr|rl>        How to assign virtual ports to multiple Stelladaptor/2600-daptors\n"
     << "  -ctrlcombo    <1|0>          Use key combos involving the Control key (Control-Q for quit may be disabled!)\n"
     << "  -autoslot     <1|0>          Automatically switch to next save slot when state saving\n"
-    << "  -stats        <1|0>          Overlay console info during emulation\n"
     << "  -fastscbios   <1|0>          Disable Supercharger BIOS progress loading bars\n"
     << "  -threads      <1|0>          Whether to using multi-threading during emulation\n"
     << "  -snapsavedir  <path>         The directory to save snapshot files to\n"
@@ -457,9 +521,6 @@ void Settings::usage() const
     << "  -holdselect                  Start the emulator with the Game Select switch held down\n"
     << "  -holdjoy0     <U,D,L,R,F>    Start the emulator with the left joystick direction/fire button held down\n"
     << "  -holdjoy1     <U,D,L,R,F>    Start the emulator with the right joystick direction/fire button held down\n"
-    << "  -tiadriven    <1|0>          Drive unused TIA pins randomly on a read/peek\n"
-    << "  -cpurandom    <1|0>          Randomize the contents of CPU registers on reset\n"
-    << "  -ramrandom    <1|0>          Randomize the contents of RAM on reset\n"
     << "  -maxres       <WxH>          Used by developers to force the maximum size of the application window\n"
     << "  -help                        Show the text you're now reading\n"
   #ifdef DEBUGGER_SUPPORT
@@ -472,10 +533,14 @@ void Settings::usage() const
     << "   -dis.showaddr  <1|0>        Show opcode addresses in disassembler\n"
     << "   -dis.relocate  <1|0>        Relocate calls out of address range in disassembler\n"
     << endl
-    << "   -dbg.res       <WxH>        The resolution to use in debugger mode\n"
-    << "   -dbg.fontstyle <0-3>        Font style to use in debugger window (bold vs. normal)\n"
-    << "   -break         <address>    Set a breakpoint at 'address'\n"
-    << "   -debug                      Start in debugger mode\n"
+    << "   -dbg.res       <WxH>          The resolution to use in debugger mode\n"
+    << "   -dbg.fontsize  <small|medium| Font size to use in debugger window\n"
+    << "                  large>\n"
+    << "   -dbg.fontstyle <0-3>          Font style to use in debugger window (bold vs. normal)\n"
+    << "   -dbg.ghostreadstrap <1|0>     Debugger traps on 'ghost' reads\n"
+    << "   -dbg.uhex      <0|1>          lower-/uppercase HEX display\n"
+    << "   -break         <address>      Set a breakpoint at 'address'\n"
+    << "   -debug                        Start in debugger mode\n"
     << endl
     << "   -bs          <arg>          Sets the 'Cartridge.Type' (bankswitch) property\n"
     << "   -type        <arg>          Same as using -bs\n"
@@ -493,10 +558,39 @@ void Settings::usage() const
     << "   -height      <arg>          Sets the 'Display.Height' property\n"
     << "   -pp          <arg>          Sets the 'Display.Phosphor' property\n"
     << "   -ppblend     <arg>          Sets the 'Display.PPBlend' property\n"
+    << endl
   #endif
-  #ifdef DTHUMB_SUPPORT
-    << "   -thumb.trapfatal <1|0>      Determines whether errors in ARM emulation throw an exception\n"
-  #endif
+
+    << " Various development related parameters for player settings mode\n"
+    << endl
+    << "  -dev.settings     <1|0>          Select developer (1) or player (0) settings mode\n"
+    << endl
+    << "  -plr.stats        <1|0>          Overlay console info during emulation\n"
+    << "  -plr.console      <2600|7800>    Select console for B/W and Pause key handling and RAM initialization\n"
+    << "  -plr.bankrandom   <1|0>          Randomize the startup bank on reset\n"
+    << "  -plr.ramrandom    <1|0>          Randomize the contents of RAM on reset\n"
+    << "  -plr.cpurandom    <1|0>          Randomize the contents of CPU registers on reset\n"
+    << "  -plr.debugcolors  <1|0>          Enable debug colors\n"
+    << "  -plr.colorloss    <1|0>          Enable PAL color-loss effect\n"
+    << "  -plr.tv.jitter    <1|0>          Enable TV jitter effect\n"
+    << "  -plr.tv.jitter_recovery <1-20>   Set recovery time for TV jitter effect\n"
+    << "  -plr.tiadriven    <1|0>          Drive unused TIA pins randomly on a read/peek\n"
+    << "  -plr.thumb.trapfatal <1|0>       Determines whether errors in ARM emulation throw an exception\n"
+    << "  -plr.eepromaccess <1|0>          Enable messages for AtariVox/SaveKey access messages\n"
+    << endl
+    << " The same parameters but for developer settings mode\n"
+    << "  -dev.stats        <1|0>          Overlay console info during emulation\n"
+    << "  -dev.console      <2600|7800>    Select console for B/W and Pause key handling and RAM initialization\n"
+    << "  -dev.bankrandom   <1|0>          Randomize the startup bank on reset\n"
+    << "  -dev.ramrandom    <1|0>          Randomize the contents of RAM on reset\n"
+    << "  -dev.cpurandom    <1|0>          Randomize the contents of CPU registers on reset\n"
+    << "  -dev.debugcolors  <1|0>          Enable debug colors\n"
+    << "  -dev.colorloss    <1|0>          Enable PAL color-loss effect\n"
+    << "  -dev.tv.jitter    <1|0>          Enable TV jitter effect\n"
+    << "  -dev.tv.jitter_recovery <1-20>   Set recovery time for TV jitter effect\n"
+    << "  -dev.tiadriven    <1|0>          Drive unused TIA pins randomly on a read/peek\n"
+    << "  -dev.thumb.trapfatal <1|0>       Determines whether errors in ARM emulation throw an exception\n"
+    << "  -dev.eepromaccess <1|0>          Enable messages for AtariVox/SaveKey access messages\n"
     << endl << std::flush;
 }
 
@@ -555,7 +649,7 @@ void Settings::saveConfig()
       << ";  Format MUST be as follows:" << endl
       << ";    command = value" << endl
       << ";" << endl
-      << ";  Commmands are the same as those specified on the commandline," << endl
+      << ";  Commands are the same as those specified on the commandline," << endl
       << ";  without the '-' character." << endl
       << ";" << endl
       << ";  Values are the same as those allowed on the commandline." << endl
