@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -17,12 +17,15 @@
 
 #include "Event.hxx"
 #include "Props.hxx"
+#include "Settings.hxx"
 #include "Switches.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Switches::Switches(const Event& event, const Properties& properties)
+Switches::Switches(const Event& event, const Properties& properties,
+                   const Settings& settings)
   : myEvent(event),
-    mySwitches(0xFF)
+    mySwitches(0xFF),
+    myIs7800(false)
 {
   if(properties.get(Console_RightDifficulty) == "B")
   {
@@ -50,18 +53,34 @@ Switches::Switches(const Event& event, const Properties& properties)
   {
     mySwitches &= ~0x08;
   }
+
+  toggle7800Mode(settings);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Switches::update()
 {
-  if(myEvent.get(Event::ConsoleColor) != 0)
+  if(myIs7800)
   {
-    mySwitches |= 0x08;
+    if(myEvent.get(Event::Console7800Pause) != 0)
+    {
+      mySwitches &= ~0x08;
+    }
+    else
+    {
+      mySwitches |= 0x08;
+    }
   }
-  else if(myEvent.get(Event::ConsoleBlackWhite) != 0)
+  else
   {
-    mySwitches &= ~0x08;
+    if(myEvent.get(Event::ConsoleColor) != 0)
+    {
+      mySwitches |= 0x08;
+    }
+    else if(myEvent.get(Event::ConsoleBlackWhite) != 0)
+    {
+      mySwitches &= ~0x08;
+    }
   }
 
   if(myEvent.get(Event::ConsoleRightDiffA) != 0)
@@ -130,3 +149,12 @@ bool Switches::load(Serializer& in)
   }
   return true;
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Switches::toggle7800Mode(const Settings& settings)
+{
+  bool devSettings = settings.getBool("dev.settings");
+  myIs7800 = devSettings && (settings.getString("dev.console") == "7800");
+
+  return myIs7800;
+}
