@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,7 +16,7 @@
 //============================================================================
 
 #include "System.hxx"
-#include "TIATypes.hxx"
+#include "TIAConstants.hxx"
 #include "TIASnd.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -74,8 +74,8 @@ string TIASound::channels(uInt32 hardware, bool stereo)
     case Hardware1:       return "Hardware1";
     case Hardware2Mono:   return "Hardware2Mono";
     case Hardware2Stereo: return "Hardware2Stereo";
-    default:              return EmptyString;
   }
+  return EmptyString;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
