@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,14 +16,13 @@
 //============================================================================
 
 #include <cmath>
-#include <algorithm>
 
 #include "FrameBuffer.hxx"
+#include "FBSurface.hxx"
 #include "Settings.hxx"
 #include "OSystem.hxx"
 #include "Console.hxx"
 #include "TIA.hxx"
-
 #include "TIASurface.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -55,7 +54,7 @@ TIASurface::TIASurface(OSystem& system)
   // Base TIA surface for use in taking snapshots in 1x mode
   myBaseTiaSurface = myFB.allocateSurface(kTIAW*2, kTIAH);
 
-  memset(myRGBFramebuffer, 0, AtariNTSC::outWidth(kTIAW) * kTIAH);
+  memset(myRGBFramebuffer, 0, sizeof(myRGBFramebuffer));
 
   // Enable/disable threading in the NTSC TV effects renderer
   myNTSCFilter.enableThreading(myOSystem.settings().getBool("threads"));
@@ -111,20 +110,27 @@ void TIASurface::setPalette(const uInt32* tia_palette, const uInt32* rgb_palette
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 const FBSurface& TIASurface::baseSurface(GUI::Rect& rect) const
 {
-  uInt32 tiaw = myTIA->width(), width = tiaw*2, height = myTIA->height();
+  uInt32 tiaw = myTIA->width(), width = tiaw * 2, height = myTIA->height();
   rect.setBounds(0, 0, width, height);
 
+  // Get Blargg buffer and width
+  uInt32 *blarggBuf, blarggPitch;
+  myTiaSurface->basePtr(blarggBuf, blarggPitch);
+  double blarggXFactor = double(blarggPitch) / width;
+  bool useBlargg = ntscEnabled();
+
   // Fill the surface with pixels from the TIA, scaled 2x horizontally
   uInt32 *buf_ptr, pitch;
   myBaseTiaSurface->basePtr(buf_ptr, pitch);
 
   for(uInt32 y = 0; y < height; ++y)
   {
-    for(uInt32 x = 0; x < tiaw; ++x)
+    for(uInt32 x = 0; x < width; ++x)
     {
-      uInt32 pixel = myFB.tiaSurface().pixel(y*tiaw+x);
-      *buf_ptr++ = pixel;
-      *buf_ptr++ = pixel;
+      if (useBlargg)
+        *buf_ptr++ = blarggBuf[y * blarggPitch + uInt32(nearbyint(x * blarggXFactor))];
+      else
+        *buf_ptr++ = myPalette[*(myTIA->frameBuffer() + y * tiaw + x / 2)];
     }
   }
 
@@ -134,22 +140,7 @@ const FBSurface& TIASurface::baseSurface(GUI::Rect& rect) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt32 TIASurface::pixel(uInt32 idx, uInt8 shift)
 {
-  uInt8 c = *(myTIA->frameBuffer() + idx) | shift;
-
-  if(!myUsePhosphor)
-    return myPalette[c];
-  else
-  {
-    const uInt32 p = myRGBFramebuffer[idx];
-
-    // Mix current calculated frame with previous displayed frame
-    const uInt32 retVal = getRGBPhosphor(myPalette[c], p);
-
-    // Store back into displayed frame buffer (for next frame)
-    myRGBFramebuffer[idx] = retVal;
-
-    return retVal;
-  }
+  return myPalette[*(myTIA->frameBuffer() + idx) | shift];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -238,7 +229,7 @@ void TIASurface::enablePhosphor(bool enable, int blend)
 
   myTiaSurface->setDirty();
   mySLineSurface->setDirty();
-  memset(myRGBFramebuffer, 0, AtariNTSC::outWidth(kTIAW) * kTIAH * 4);
+  memset(myRGBFramebuffer, 0, sizeof(myRGBFramebuffer));
 
   // Precalculate the average colors for the 'phosphor' effect
   if(myUsePhosphor)
@@ -290,7 +281,7 @@ void TIASurface::enableNTSC(bool enable)
 
   myTiaSurface->setDirty();
   mySLineSurface->setDirty();
-  memset(myRGBFramebuffer, 0, AtariNTSC::outWidth(kTIAW) * kTIAH * 4);
+  memset(myRGBFramebuffer, 0, sizeof(myRGBFramebuffer));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -394,3 +385,49 @@ void TIASurface::render()
     mySLineSurface->render();
   }
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIASurface::reRender()
+{
+  uInt32 width = myTIA->width();
+  uInt32 height = myTIA->height();
+  uInt32 pos = 0;
+  uInt32 *outPtr, outPitch;
+
+  myTiaSurface->basePtr(outPtr, outPitch);
+
+  switch (myFilter)
+  {
+    // for non-phosphor modes, render the frame again
+    case Filter::Normal:
+    case Filter::BlarggNormal:
+      render();
+      break;
+    // for phosphor modes, copy the phosphor framebuffer
+    case Filter::Phosphor:
+      for (uInt32 y = height; y; --y)
+      {
+        memcpy(outPtr, myRGBFramebuffer + pos, width);
+        outPtr += outPitch;
+        pos += width;
+      }
+      break;
+    case Filter::BlarggPhosphor:
+      memcpy(outPtr, myRGBFramebuffer, height * outPitch << 2);
+      break;
+  }
+
+  if (myUsePhosphor)
+  {
+    // Draw TIA image
+    myTiaSurface->setDirty();
+    myTiaSurface->render();
+
+    // Draw overlaying scanlines
+    if (myScanlinesEnabled)
+    {
+      mySLineSurface->setDirty();
+      mySLineSurface->render();
+    }
+  }
+}
