@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -38,8 +38,9 @@ DrawCounterDecodes& DrawCounterDecodes::DrawCounterDecodes::get()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DrawCounterDecodes::DrawCounterDecodes()
 {
-  for (uInt8 *decodes : {myDecodes0, myDecodes1, myDecodes2, myDecodes3,
-                         myDecodes4, myDecodes6})
+  uInt8 *decodeTables[] = {myDecodes0, myDecodes1, myDecodes2, myDecodes3, myDecodes4, myDecodes6};
+
+  for (uInt8 *decodes : decodeTables)
   {
     memset(decodes, 0, 160);
     decodes[156] = 1;
