@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -21,6 +21,8 @@
 #include "Control.hxx"
 #include "Paddles.hxx"
 #include "DelayQueueIteratorImpl.hxx"
+#include "TIAConstants.hxx"
+#include "frame-manager/FrameManager.hxx"
 
 #ifdef DEBUGGER_SUPPORT
   #include "CartDebug.hxx"
@@ -67,6 +69,7 @@ TIA::TIA(Console& console, Sound& sound, Settings& settings)
   : myConsole(console),
     mySound(sound),
     mySettings(settings),
+    myFrameManager(nullptr),
     myPlayfield(~CollisionMask::playfield & 0x7FFF),
     myMissile0(~CollisionMask::missile0 & 0x7FFF),
     myMissile1(~CollisionMask::missile1 & 0x7FFF),
@@ -76,19 +79,8 @@ TIA::TIA(Console& console, Sound& sound, Settings& settings)
     mySpriteEnabledBits(0xFF),
     myCollisionsEnabledBits(0xFF)
 {
-  myFrameManager.setHandlers(
-    [this] () {
-      onFrameStart();
-    },
-    [this] () {
-      onFrameComplete();
-    },
-    [this] () {
-      onRenderingStart();
-    }
-  );
-
-  myTIAPinsDriven = mySettings.getBool("tiadriven");
+  bool devSettings = mySettings.getBool("dev.settings");
+  myTIAPinsDriven = mySettings.getBool(devSettings ? "dev.tiadriven" : "plr.tiadriven");
 
   myBackground.setTIA(this);
   myPlayfield.setTIA(this);
@@ -98,12 +90,42 @@ TIA::TIA(Console& console, Sound& sound, Settings& settings)
   myMissile1.setTIA(this);
   myBall.setTIA(this);
 
-  myFrameManager.enableJitter(mySettings.getBool("tv.jitter"));
-  myFrameManager.setJitterFactor(mySettings.getInt("tv.jitter_recovery"));
+  myEnableJitter = mySettings.getBool(devSettings ? "dev.tv.jitter" : "plr.tv.jitter");
+  myJitterFactor = mySettings.getInt(devSettings ? "dev.tv.jitter_recovery" : "plr.tv.jitter_recovery");
 
   reset();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::setFrameManager(AbstractFrameManager *frameManager)
+{
+  clearFrameManager();
+
+  myFrameManager = frameManager;
+
+  myFrameManager->setHandlers(
+    [this] () {
+      onFrameStart();
+    },
+    [this] () {
+      onFrameComplete();
+    }
+  );
+
+  myFrameManager->enableJitter(myEnableJitter);
+  myFrameManager->setJitterFactor(myJitterFactor);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::clearFrameManager()
+{
+  if (!myFrameManager) return;
+
+  myFrameManager->clearHandlers();
+
+  myFrameManager = nullptr;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::reset()
 {
@@ -143,31 +165,28 @@ void TIA::reset()
 
   mySound.reset();
   myDelayQueue.reset();
-  myFrameManager.reset();
+
+  if (myFrameManager) myFrameManager->reset();
+
+  myCyclesAtFrameStart = 0;
 
   frameReset();  // Recalculate the size of the display
 
   // Must be done last, after all other items have reset
-  enableFixedColors(false);
+  enableFixedColors(mySettings.getBool(mySettings.getBool("dev.settings") ? "dev.debugcolors" : "plr.debugcolors"));
   setFixedColorPalette(mySettings.getString("tia.dbgcolors"));
+
+#ifdef DEBUGGER_SUPPORT
+  createAccessBase();
+#endif // DEBUGGER_SUPPORT
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::frameReset()
 {
-  memset(myFramebuffer, 0, 160 * FrameManager::frameBufferHeight);
+  memset(myFramebuffer, 0, 160 * TIAConstants::frameBufferHeight);
   myAutoFrameEnabled = mySettings.getInt("framerate") <= 0;
-  enableColorLoss(mySettings.getBool("colorloss"));
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void TIA::systemCyclesReset()
-{
-  const uInt32 cycles = mySystem->cycles();
-
-  myLastCycle -= cycles;
-
-  mySound.adjustCycleCounter(-1 * cycles);
+  enableColorLoss(mySettings.getBool("dev.settings") ? "dev.colorloss" : "plr.colorloss");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -185,10 +204,12 @@ void TIA::installDelegate(System& system, Device& device)
   // All accesses are to the given device
   System::PageAccess access(&device, System::PA_READWRITE);
 
-  // We're installing in a 2600 system
-  for(uInt32 i = 0; i < 8192; i += (1 << System::PAGE_SHIFT))
-    if((i & 0x1080) == 0x0000)
-      mySystem->setPageAccess(i >> System::PAGE_SHIFT, access);
+  // Map all peek/poke to mirrors of TIA address space to this class
+  // That is, all mirrors of ($00 - $3F) in the lower 4K of the 2600
+  // address space are mapped here
+  for(uInt16 addr = 0; addr < 0x1000; addr += System::PAGE_SIZE)
+    if((addr & TIA_BIT) == 0x0000)
+      mySystem->setPageAccess(addr, access);
 
   mySystem->m6502().setOnHaltCallback(
     [this] () {
@@ -207,7 +228,7 @@ bool TIA::save(Serializer& out) const
     if(!mySound.save(out)) return false;
 
     if(!myDelayQueue.save(out))   return false;
-    if(!myFrameManager.save(out)) return false;
+    if(!myFrameManager->save(out)) return false;
 
     if(!myBackground.save(out)) return false;
     if(!myPlayfield.save(out))  return false;
@@ -243,7 +264,7 @@ bool TIA::save(Serializer& out) const
     out.putInt(int(myPriority));
 
     out.putByte(mySubClock);
-    out.putInt(myLastCycle);
+    out.putLong(myLastCycle);
 
     out.putByte(mySpriteEnabledBits);
     out.putByte(myCollisionsEnabledBits);
@@ -255,6 +276,8 @@ bool TIA::save(Serializer& out) const
     out.putBool(myAutoFrameEnabled);
 
     out.putByteArray(myShadowRegisters, 64);
+
+    out.putLong(myCyclesAtFrameStart);
   }
   catch(...)
   {
@@ -276,7 +299,7 @@ bool TIA::load(Serializer& in)
     if(!mySound.load(in)) return false;
 
     if(!myDelayQueue.load(in))   return false;
-    if(!myFrameManager.load(in)) return false;
+    if(!myFrameManager->load(in)) return false;
 
     if(!myBackground.load(in)) return false;
     if(!myPlayfield.load(in))  return false;
@@ -312,7 +335,7 @@ bool TIA::load(Serializer& in)
     myPriority = Priority(in.getInt());
 
     mySubClock = in.getByte();
-    myLastCycle = in.getInt();
+    myLastCycle = in.getLong();
 
     mySpriteEnabledBits = in.getByte();
     myCollisionsEnabledBits = in.getByte();
@@ -324,6 +347,8 @@ bool TIA::load(Serializer& in)
     myAutoFrameEnabled = in.getBool();
 
     in.getByteArray(myShadowRegisters, 64);
+
+    myCyclesAtFrameStart = in.getLong();
   }
   catch(...)
   {
@@ -339,15 +364,15 @@ void TIA::bindToControllers()
 {
   myConsole.leftController().setOnAnalogPinUpdateCallback(
     [this] (Controller::AnalogPin pin) {
-      TIA& tia = mySystem->tia();
+      updateEmulation();
 
       switch (pin) {
         case Controller::AnalogPin::Five:
-          tia.updatePaddle(1);
+          updatePaddle(1);
           break;
 
         case Controller::AnalogPin::Nine:
-          tia.updatePaddle(0);
+          updatePaddle(0);
           break;
       }
     }
@@ -355,15 +380,15 @@ void TIA::bindToControllers()
 
   myConsole.rightController().setOnAnalogPinUpdateCallback(
     [this] (Controller::AnalogPin pin) {
-      TIA& tia = mySystem->tia();
+      updateEmulation();
 
       switch (pin) {
         case Controller::AnalogPin::Five:
-          tia.updatePaddle(3);
+          updatePaddle(3);
           break;
 
         case Controller::AnalogPin::Nine:
-          tia.updatePaddle(2);
+          updatePaddle(2);
           break;
       }
     }
@@ -479,7 +504,7 @@ bool TIA::poke(uInt16 address, uInt8 value)
       break;
 
     case VSYNC:
-      myFrameManager.setVsync(value & 0x02);
+      myFrameManager->setVsync(value & 0x02);
       myShadowRegisters[address] = value;
       break;
 
@@ -750,7 +775,7 @@ bool TIA::saveDisplay(Serializer& out) const
 {
   try
   {
-    out.putByteArray(myFramebuffer, 160*FrameManager::frameBufferHeight);
+    out.putByteArray(myFramebuffer, 160*TIAConstants::frameBufferHeight);
   }
   catch(...)
   {
@@ -767,7 +792,7 @@ bool TIA::loadDisplay(Serializer& in)
   try
   {
     // Reset frame buffer pointer and data
-    in.getByteArray(myFramebuffer, 160*FrameManager::frameBufferHeight);
+    in.getByteArray(myFramebuffer, 160*TIAConstants::frameBufferHeight);
   }
   catch(...)
   {
@@ -787,13 +812,12 @@ void TIA::update()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIA::enableColorLoss(bool enabled)
 {
-  if (consoleTiming() != ConsoleTiming::pal)
-    return false;
+  bool allowColorLoss = consoleTiming() == ConsoleTiming::pal;
 
-  if(enabled)
+  if(allowColorLoss && enabled)
   {
     myColorLossEnabled = true;
-    myColorLossActive = myFrameManager.scanlinesLastFrame() & 0x1;
+    myColorLossActive = myFrameManager->scanlinesLastFrame() & 0x1;
   }
   else
   {
@@ -808,7 +832,7 @@ bool TIA::enableColorLoss(bool enabled)
     myBackground.applyColorLoss();
   }
 
-  return true;
+  return allowColorLoss;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -817,7 +841,7 @@ bool TIA::electronBeamPos(uInt32& x, uInt32& y) const
   uInt8 clocks = clocksThisLine();
 
   x = (clocks < 68) ? 0 : clocks - 68;
-  y = myFrameManager.getY();
+  y = myFrameManager->getY();
 
   return isRendering();
 }
@@ -903,7 +927,11 @@ bool TIA::toggleCollisions()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIA::enableFixedColors(bool enable)
 {
-  int layout = myFrameManager.layout() == FrameLayout::pal ? 1 : 0;
+  // This will be called during reset at a point where no frame manager
+  // instance is available, so we guard aginst this here.
+  int layout = 0;
+  if (myFrameManager) layout = myFrameManager->layout() == FrameLayout::pal ? 1 : 0;
+
   myMissile0.setDebugColor(myFixedColorPalette[layout][FixedObject::M0]);
   myMissile1.setDebugColor(myFixedColorPalette[layout][FixedObject::M1]);
   myPlayer0.setDebugColor(myFixedColorPalette[layout][FixedObject::P0]);
@@ -939,26 +967,32 @@ bool TIA::setFixedColorPalette(const string& colors)
       case 'r':
         myFixedColorPalette[0][i] = FixedColor::NTSC_RED;
         myFixedColorPalette[1][i] = FixedColor::PAL_RED;
+        myFixedColorNames[i] = "Red   ";
         break;
       case 'o':
         myFixedColorPalette[0][i] = FixedColor::NTSC_ORANGE;
         myFixedColorPalette[1][i] = FixedColor::PAL_ORANGE;
+        myFixedColorNames[i] = "Orange";
         break;
       case 'y':
         myFixedColorPalette[0][i] = FixedColor::NTSC_YELLOW;
         myFixedColorPalette[1][i] = FixedColor::PAL_YELLOW;
+        myFixedColorNames[i] = "Yellow";
         break;
       case 'g':
         myFixedColorPalette[0][i] = FixedColor::NTSC_GREEN;
         myFixedColorPalette[1][i] = FixedColor::PAL_GREEN;
+        myFixedColorNames[i] = "Green ";
         break;
       case 'b':
         myFixedColorPalette[0][i] = FixedColor::NTSC_BLUE;
         myFixedColorPalette[1][i] = FixedColor::PAL_BLUE;
+        myFixedColorNames[i] = "Blue  ";
         break;
       case 'p':
         myFixedColorPalette[0][i] = FixedColor::NTSC_PURPLE;
         myFixedColorPalette[1][i] = FixedColor::PAL_PURPLE;
+        myFixedColorNames[i] = "Purple";
         break;
     }
   }
@@ -978,7 +1012,7 @@ bool TIA::driveUnusedPinsRandom(uInt8 mode)
   if (mode == 0 || mode == 1)
   {
     myTIAPinsDriven = bool(mode);
-    mySettings.setValue("tiadriven", myTIAPinsDriven);
+    mySettings.setValue(mySettings.getBool("dev.settings") ? "dev.tiadriven" : "plr.tiadriven", myTIAPinsDriven);
   }
   return myTIAPinsDriven;
 }
@@ -988,22 +1022,32 @@ bool TIA::toggleJitter(uInt8 mode)
 {
   switch (mode) {
     case 0:
-      myFrameManager.enableJitter(false);
+      myEnableJitter = false;
       break;
 
     case 1:
-      myFrameManager.enableJitter(true);
+      myEnableJitter = true;
       break;
 
     case 2:
-      myFrameManager.enableJitter(!myFrameManager.jitterEnabled());
+      myEnableJitter = !myEnableJitter;
       break;
 
     default:
       throw runtime_error("invalid argument for toggleJitter");
   }
 
-  return myFrameManager.jitterEnabled();
+  if (myFrameManager) myFrameManager->enableJitter(myEnableJitter);
+
+  return myEnableJitter;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::setJitterRecoveryFactor(Int32 factor)
+{
+  myJitterFactor = factor;
+
+  if (myFrameManager) myFrameManager->setJitterFactor(myJitterFactor);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1019,8 +1063,7 @@ TIA& TIA::updateScanline()
 {
   // Update frame by one scanline at a time
   uInt32 line = scanlines();
-  while (line == scanlines() && mySystem->m6502().execute(1))
-    updateEmulation();
+  while (line == scanlines() && mySystem->m6502().execute(1));
 
   return *this;
 }
@@ -1029,8 +1072,7 @@ TIA& TIA::updateScanline()
 TIA& TIA::updateScanlineByStep()
 {
   // Update frame by one CPU instruction/color clock
-  if (mySystem->m6502().execute(1))
-    updateEmulation();
+  mySystem->m6502().execute(1);
 
   return *this;
 }
@@ -1040,8 +1082,7 @@ TIA& TIA::updateScanlineByTrace(int target)
 {
   uInt32 count = 100;  // only try up to 100 steps
   while (mySystem->m6502().getPC() != target && count-- &&
-         mySystem->m6502().execute(1))
-    updateEmulation();
+         mySystem->m6502().execute(1));
 
   return *this;
 }
@@ -1055,12 +1096,12 @@ uInt8 TIA::registerValue(uInt8 reg) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::updateEmulation()
 {
-  const uInt32 systemCycles = mySystem->cycles();
+  const uInt64 systemCycles = mySystem->cycles();
 
   if (mySubClock > 2)
     throw runtime_error("subclock exceeds range");
 
-  const uInt32 cyclesToRun = 3 * (systemCycles - myLastCycle) + mySubClock;
+  const uInt32 cyclesToRun = 3 * uInt32(systemCycles - myLastCycle) + mySubClock;
 
   mySubClock = 0;
   myLastCycle = systemCycles;
@@ -1078,9 +1119,9 @@ void TIA::onFrameStart()
   {
     // Only activate it when necessary, since changing colours in
     // the graphical object forces the TIA cached line to be flushed
-    if (myFrameManager.scanlineCountTransitioned())
+    if (myFrameManager->scanlineParityChanged())
     {
-      myColorLossActive = myFrameManager.scanlinesLastFrame() & 0x1;
+      myColorLossActive = myFrameManager->scanlinesLastFrame() & 0x1;
 
       myMissile0.applyColorLoss();
       myMissile1.applyColorLoss();
@@ -1093,29 +1134,23 @@ void TIA::onFrameStart()
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void TIA::onRenderingStart()
-{
-  myXAtRenderingStart = myHctr > 68 ? myHctr - 68 : 0;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::onFrameComplete()
 {
   mySystem->m6502().stop();
-  mySystem->resetCycles();
+  myCyclesAtFrameStart = mySystem->cycles();
 
   if (myXAtRenderingStart > 0)
     memset(myFramebuffer, 0, myXAtRenderingStart);
 
   // Blank out any extra lines not drawn this frame
-  const uInt32 missingScanlines = myFrameManager.missingScanlines();
+  const Int32 missingScanlines = myFrameManager->missingScanlines();
   if (missingScanlines > 0)
-    memset(myFramebuffer + 160 * myFrameManager.getY(), 0, missingScanlines * 160);
+    memset(myFramebuffer + 160 * myFrameManager->getY(), 0, missingScanlines * 160);
 
   // Recalculate framerate, attempting to auto-correct for scanline 'jumps'
   if(myAutoFrameEnabled)
-    myConsole.setFramerate(myFrameManager.frameRate());
+    myConsole.setFramerate(myFrameManager->frameRate());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1145,7 +1180,7 @@ void TIA::cycle(uInt32 colorClocks)
       else
         tickHframe();
 
-      if (myCollisionUpdateRequired) updateCollision();
+      if (myCollisionUpdateRequired && !myFrameManager->vblank()) updateCollision();
     }
 
     if (++myHctr >= 228)
@@ -1194,12 +1229,14 @@ void TIA::tickHblank()
       if (myExtendedHblank) myHstate = HState::frame;
       break;
   }
+
+  if (myExtendedHblank && myHctr > 67) myPlayfield.tick(myHctr - 68 - myHctrDelta);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::tickHframe()
 {
-  const uInt32 y = myFrameManager.getY();
+  const uInt32 y = myFrameManager->getY();
   const uInt32 x = myHctr - 68 - myHctrDelta;
 
   myCollisionUpdateRequired = true;
@@ -1211,7 +1248,7 @@ void TIA::tickHframe()
   myPlayer1.tick();
   myBall.tick();
 
-  if (myFrameManager.isRendering())
+  if (myFrameManager->isRendering())
     renderPixel(x, y);
 }
 
@@ -1221,8 +1258,8 @@ void TIA::applyRsync()
   const uInt32 x = myHctr > 68 ? myHctr - 68 : 0;
 
   myHctrDelta = 225 - myHctr;
-  if (myFrameManager.isRendering())
-    memset(myFramebuffer + myFrameManager.getY() * 160 + x, 0, 160 - x);
+  if (myFrameManager->isRendering())
+    memset(myFramebuffer + myFrameManager->getY() * 160 + x, 0, 160 - x);
 
   myHctr = 225;
 }
@@ -1234,6 +1271,7 @@ void TIA::nextLine()
     cloneLastLine();
   }
 
+  myPlayfield.tick(0);
   myHctr = 0;
 
   if (!myMovementInProgress && myLinesSinceChange < 2) myLinesSinceChange++;
@@ -1241,9 +1279,9 @@ void TIA::nextLine()
   myHstate = HState::blank;
   myHctrDelta = 0;
 
-  myFrameManager.nextLine();
+  myFrameManager->nextLine();
 
-  if (myFrameManager.isRendering() && myFrameManager.getY() == 0) flushLineCache();
+  if (myFrameManager->isRendering() && myFrameManager->getY() == 0) flushLineCache();
 
   mySystem->m6502().clearHaltRequest();
 }
@@ -1251,9 +1289,9 @@ void TIA::nextLine()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::cloneLastLine()
 {
-  const auto y = myFrameManager.getY();
+  const auto y = myFrameManager->getY();
 
-  if (!myFrameManager.isRendering() || y == 0) return;
+  if (!myFrameManager->isRendering() || y == 0) return;
 
   uInt8* buffer = myFramebuffer;
 
@@ -1280,7 +1318,7 @@ void TIA::renderPixel(uInt32 x, uInt32 y)
 
   uInt8 color = 0;
 
-  if (!myFrameManager.vblank())
+  if (!myFrameManager->vblank())
   {
     switch (myPriority)
     {
@@ -1353,8 +1391,8 @@ void TIA::flushLineCache()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::clearHmoveComb()
 {
-  if (myFrameManager.isRendering() && myHstate == HState::blank)
-    memset(myFramebuffer + myFrameManager.getY() * 160, myColorHBlank, 8);
+  if (myFrameManager->isRendering() && myHstate == HState::blank)
+    memset(myFramebuffer + myFrameManager->getY() * 160, myColorHBlank, 8);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1367,7 +1405,7 @@ void TIA::delayedWrite(uInt8 address, uInt8 value)
   {
     case VBLANK:
       flushLineCache();
-      myFrameManager.setVblank(value & 0x02);
+      myFrameManager->setVblank(value & 0x02);
       break;
 
     case HMOVE:
@@ -1409,11 +1447,12 @@ void TIA::delayedWrite(uInt8 address, uInt8 value)
       break;
 
     case HMCLR:
-      myMissile0.hmm(0);
-      myMissile1.hmm(0);
-      myPlayer0.hmp(0);
-      myPlayer1.hmp(0);
-      myBall.hmbl(0);
+      // We must update the shadow registers for each HM object too
+      myMissile0.hmm(0);  myShadowRegisters[HMM0] = 0;
+      myMissile1.hmm(0);  myShadowRegisters[HMM1] = 0;
+      myPlayer0.hmp(0);   myShadowRegisters[HMP0] = 0;
+      myPlayer1.hmp(0);   myShadowRegisters[HMP1] = 0;
+      myBall.hmbl(0);     myShadowRegisters[HMBL] = 0;
       break;
 
     case GRP0:
@@ -1577,3 +1616,126 @@ uInt8 TIA::collCXBLPF() const
 {
   return (myCollisionMask & CollisionMask::ball & CollisionMask::playfield) ? 0x80 : 0;
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollP0PF()
+{
+  myCollisionMask ^= (CollisionMask::player0 & CollisionMask::playfield);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollP0BL()
+{
+  myCollisionMask ^= (CollisionMask::player0 & CollisionMask::ball);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollP0M1()
+{
+  myCollisionMask ^= (CollisionMask::player0 & CollisionMask::missile1);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollP0M0()
+{
+  myCollisionMask ^= (CollisionMask::player0 & CollisionMask::missile0);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollP0P1()
+{
+  myCollisionMask ^= (CollisionMask::player0 & CollisionMask::player1);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollP1PF()
+{
+  myCollisionMask ^= (CollisionMask::player1 & CollisionMask::playfield);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollP1BL()
+{
+  myCollisionMask ^= (CollisionMask::player1 & CollisionMask::ball);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollP1M1()
+{
+  myCollisionMask ^= (CollisionMask::player1 & CollisionMask::missile1);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollP1M0()
+{
+  myCollisionMask ^= (CollisionMask::player1 & CollisionMask::missile0);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollM0PF()
+{
+  myCollisionMask ^= (CollisionMask::missile0 & CollisionMask::playfield);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollM0BL()
+{
+  myCollisionMask ^= (CollisionMask::missile0 & CollisionMask::ball);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollM0M1()
+{
+  myCollisionMask ^= (CollisionMask::missile0 & CollisionMask::missile1);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollM1PF()
+{
+  myCollisionMask ^= (CollisionMask::missile1 & CollisionMask::playfield);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollM1BL()
+{
+  myCollisionMask ^= (CollisionMask::missile1 & CollisionMask::ball);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::toggleCollBLPF()
+{
+  myCollisionMask ^= (CollisionMask::ball & CollisionMask::playfield);
+}
+
+#ifdef DEBUGGER_SUPPORT
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::createAccessBase()
+{
+  myAccessBase = make_unique<uInt8[]>(TIA_SIZE);
+  memset(myAccessBase.get(), CartDebug::NONE, TIA_SIZE);
+  myAccessDelay = make_unique<uInt8[]>(TIA_SIZE);
+  memset(myAccessDelay.get(), TIA_DELAY, TIA_SIZE);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8 TIA::getAccessFlags(uInt16 address) const
+{
+  return myAccessBase[address & TIA_MASK];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::setAccessFlags(uInt16 address, uInt8 flags)
+{
+  // ignore none flag
+  if (flags != CartDebug::NONE) {
+    if (flags == CartDebug::WRITE) {
+      // the first two write accesses are assumed as initialization
+      if (myAccessDelay[address & TIA_MASK])
+        myAccessDelay[address & TIA_MASK]--;
+      else
+        myAccessBase[address & TIA_MASK] |= flags;
+    } else
+      myAccessBase[address & TIA_READ_MASK] |= flags;
+  }
+}
+#endif // DEBUGGER_SUPPORT
