@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -19,7 +19,7 @@
 #include "OSystem.hxx"
 #include "Version.hxx"
 #include "Widget.hxx"
-
+#include "Font.hxx"
 #include "AboutDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -28,7 +28,7 @@ AboutDialog::AboutDialog(OSystem& osystem, DialogContainer& parent,
   : Dialog(osystem, parent),
     myPage(1),
     myNumPages(4),
-    myLinesPerPage(12)
+    myLinesPerPage(13)
 {
   const int lineHeight   = font.getLineHeight(),
             fontWidth    = font.getMaxCharWidth(),
@@ -40,39 +40,39 @@ AboutDialog::AboutDialog(OSystem& osystem, DialogContainer& parent,
 
   // Set real dimensions
   _w = 55 * fontWidth + 8;
-  _h = 14 * lineHeight + 20;
+  _h = 15 * lineHeight + 20;
 
   // Add Previous, Next and Close buttons
   xpos = 10;  ypos = _h - buttonHeight - 10;
   myPrevButton =
     new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
-                     "Previous", kPrevCmd);
+                     "Previous", GuiObject::kPrevCmd);
   myPrevButton->clearFlags(WIDGET_ENABLED);
   wid.push_back(myPrevButton);
 
   xpos += buttonWidth + 7;
   myNextButton =
     new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
-                     "Next", kNextCmd);
+                     "Next", GuiObject::kNextCmd);
   wid.push_back(myNextButton);
 
   xpos = _w - buttonWidth - 10;
   ButtonWidget* b =
     new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
-                     "Close", kCloseCmd);
+                     "Close", GuiObject::kCloseCmd);
   wid.push_back(b);
   addOKWidget(b);  addCancelWidget(b);
 
   xpos = 5;  ypos = 5;
   myTitle = new StaticTextWidget(this, font, xpos, ypos, _w - 10, fontHeight,
-                                 "", kTextAlignCenter);
+                                 "", TextAlign::Center);
   myTitle->setTextColor(kTextColorEm);
 
   xpos = 10;  ypos += lineHeight + 4;
   for(int i = 0; i < myLinesPerPage; i++)
   {
     myDesc.push_back(new StaticTextWidget(this, font, xpos, ypos, _w - 20,
-                                          fontHeight, "", kTextAlignLeft));
+                                          fontHeight, "", TextAlign::Left));
     myDescStr.push_back("");
     ypos += fontHeight;
   }
@@ -104,48 +104,53 @@ void AboutDialog::updateStrings(int page, int lines, string& title)
       ADD_ATEXT(string("\\C\\c2Features: ") + instance().features());
       ADD_ATEXT(string("\\C\\c2") + instance().buildInfo());
       ADD_ALINE();
-      ADD_ATEXT("\\CCopyright (C) 1995-2017 The Stella Team");
+      ADD_ATEXT("\\CCopyright (C) 1995-2018 The Stella Team");
       ADD_ATEXT("\\C(https://stella-emu.github.io)");
       ADD_ALINE();
       ADD_ATEXT("\\CStella is now DonationWare!");
       ADD_ATEXT("\\C(https://stella-emu.github.io/donations.html)");
       ADD_ALINE();
-      ADD_ATEXT("\\CStella is free software released under the GNU GPL");
-      ADD_ATEXT("\\CSee manual for further details");
+      ADD_ATEXT("\\CStella is free software released under the GNU GPL.");
+      ADD_ATEXT("\\CSee manual for further details.");
       break;
 
     case 2:
       title = "The Stella Team";
-      ADD_ATEXT("\\L\\c0""  Bradford W. Mott");
-      ADD_ATEXT("\\L\\c2""    Original author");
       ADD_ATEXT("\\L\\c0""  Stephen Anthony");
-      ADD_ATEXT("\\L\\c2""    Lead developer, Linux/MacOS X/Windows maintainer");
+      ADD_ATEXT("\\L\\c2""    Lead developer, current maintainer for the");
+      ADD_ATEXT("\\L\\c2""    Linux/OSX and Windows ports ");
+      ADD_ATEXT("\\L\\c0""  Christian Speckner");
+      ADD_ATEXT("\\L\\c2""    Emulation core development, TIA core");
       ADD_ATEXT("\\L\\c0""  Eckhard Stolberg");
       ADD_ATEXT("\\L\\c2""    Emulation core development");
+      ADD_ATEXT("\\L\\c0""  Thomas Jentzsch");
+      ADD_ATEXT("\\L\\c2""    Emulation core development, jack-of-all-trades");
       ADD_ATEXT("\\L\\c0""  Brian Watson");
       ADD_ATEXT("\\L\\c2""    Emulation core enhancement, debugger support");
+      ADD_ATEXT("\\L\\c0""  Bradford W. Mott");
+      ADD_ATEXT("\\L\\c2""    Original author of Stella");
       break;
 
     case 3:
       title = "Contributors";
       ADD_ATEXT("\\L\\c0""  See https://stella-emu.github.io/credits.html for");
-      ADD_ATEXT("\\L\\c0""  people that have contributed to Stella");
+      ADD_ATEXT("\\L\\c0""  people that have contributed to Stella.");
       ADD_ALINE();
-      ADD_ATEXT("\\L\\c0""  Thanks to the ScummVM project for the GUI code");
+      ADD_ATEXT("\\L\\c0""  Thanks to the ScummVM project for the GUI code.");
       ADD_ALINE();
       ADD_ATEXT("\\L\\c0""  Thanks to Ian Bogost and the Georgia Tech");
-      ADD_ATEXT("\\L\\c0""  Atari Team for the CRT Simulation effects");
+      ADD_ATEXT("\\L\\c0""  Atari Team for the CRT Simulation effects.");
       break;
 
     case 4:
       title = "Cast of thousands";
       ADD_ATEXT("\\L\\c0""Special thanks to AtariAge for introducing the");
-      ADD_ATEXT("\\L\\c0""Atari 2600 to a whole new generation");
+      ADD_ATEXT("\\L\\c0""Atari 2600 to a whole new generation.");
       ADD_ATEXT("\\L\\c2""  http://www.atariage.com");
       ADD_ALINE();
       ADD_ATEXT("\\L\\c0""Finally, a huge thanks to the original Atari 2600");
       ADD_ATEXT("\\L\\c0""VCS team for giving us the magic, and to the");
-      ADD_ATEXT("\\L\\c0""homebrew developers for keeping the magic alive");
+      ADD_ATEXT("\\L\\c0""homebrew developers for keeping the magic alive.");
       break;
   }
 
@@ -163,7 +168,7 @@ void AboutDialog::displayInfo()
   for(int i = 0; i < myLinesPerPage; i++)
   {
     const char* str = myDescStr[i].c_str();
-    TextAlignment align = kTextAlignCenter;
+    TextAlign align = TextAlign::Center;
     uInt32 color  = kTextColor;
 
     while (str[0] == '\\')
@@ -171,15 +176,15 @@ void AboutDialog::displayInfo()
       switch (str[1])
       {
         case 'C':
-          align = kTextAlignCenter;
+          align = TextAlign::Center;
           break;
 
         case 'L':
-          align = kTextAlignLeft;
+          align = TextAlign::Left;
           break;
 
         case 'R':
-          align = kTextAlignRight;
+          align = TextAlign::Right;
           break;
 
         case 'c':
@@ -229,7 +234,7 @@ void AboutDialog::handleCommand(CommandSender* sender, int cmd, int data, int id
 {
   switch(cmd)
   {
-    case kNextCmd:
+    case GuiObject::kNextCmd:
       myPage++;
       if(myPage >= myNumPages)
         myNextButton->clearFlags(WIDGET_ENABLED);
@@ -239,7 +244,7 @@ void AboutDialog::handleCommand(CommandSender* sender, int cmd, int data, int id
       displayInfo();
       break;
 
-    case kPrevCmd:
+    case GuiObject::kPrevCmd:
       myPage--;
       if(myPage <= myNumPages)
         myNextButton->setFlags(WIDGET_ENABLED);
