@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -18,8 +18,8 @@
 #include "Console.hxx"
 #include "TIA.hxx"
 #include "Switches.hxx"
-#include "DialogContainer.hxx"
 #include "Dialog.hxx"
+#include "Font.hxx"
 #include "EventHandler.hxx"
 #include "OSystem.hxx"
 #include "Widget.hxx"
