@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -19,11 +19,16 @@
 //============================================================================
 
 #include "OSystem.hxx"
+#include "EventHandler.hxx"
 #include "FrameBuffer.hxx"
+#include "FBSurface.hxx"
+#include "Font.hxx"
 #include "Menu.hxx"
 #include "Dialog.hxx"
 #include "Widget.hxx"
 #include "TabWidget.hxx"
+#include "Settings.hxx"
+#include "Console.hxx"
 #include "Vec.hxx"
 
 /*
@@ -46,7 +51,8 @@ Dialog::Dialog(OSystem& instance, DialogContainer& parent,
     _visible(false),
     _processCancel(false),
     _surface(nullptr),
-    _tabID(0)
+    _tabID(0),
+    _flags(WIDGET_ENABLED | WIDGET_BORDER | WIDGET_CLEARBG)
 {
 }
 
@@ -87,7 +93,7 @@ void Dialog::close(bool refresh)
 {
   if (_mouseWidget)
   {
-    _mouseWidget->handleMouseLeft(0);
+    _mouseWidget->handleMouseLeft();
     _mouseWidget = nullptr;
   }
 
@@ -104,9 +110,8 @@ void Dialog::center()
   if(_surface)
   {
     const GUI::Size& screen = instance().frameBuffer().screenSize();
-    uInt32 x = (screen.w - getWidth()) >> 1;
-    uInt32 y = (screen.h - getHeight()) >> 1;
-    _surface->setDstPos(x, y);
+    const GUI::Rect& dst = _surface->dstRect();
+    _surface->setDstPos((screen.w - dst.width()) >> 1, (screen.h - dst.height()) >> 1);
   }
 }
 
@@ -267,9 +272,17 @@ void Dialog::drawDialog()
 
   if(_dirty)
   {
-//    cerr << "Dialog::drawDialog(): w = " << _w << ", h = " << _h << " @ " << &s << endl << endl;
-    s.fillRect(_x, _y, _w, _h, kDlgColor);
-    s.box(_x, _y, _w, _h, kColor, kShadowColor);
+    if(_flags & WIDGET_CLEARBG)
+      //    cerr << "Dialog::drawDialog(): w = " << _w << ", h = " << _h << " @ " << &s << endl << endl;
+      s.fillRect(_x, _y, _w, _h, kDlgColor);
+    else
+      s.invalidate();
+    if(_flags & WIDGET_BORDER)
+#ifndef FLAT_UI
+      s.box(_x, _y, _w, _h, kColor, kShadowColor);
+#else
+      s.frameRect(_x, _y, _w, _h, kColor);
+#endif // !FLAT_UI
 
     // Make all child widget dirty
     Widget* w = _firstWidget;
@@ -319,23 +332,32 @@ void Dialog::handleText(char text)
 void Dialog::handleKeyDown(StellaKey key, StellaMod mod)
 {
   // Test for TAB character
-  // Shift-left/shift-right cursor selects next tab
+  // Control-Tab selects next tab
+  // Shift-Control-Tab selects previous tab
   // Tab sets next widget in current tab
   // Shift-Tab sets previous widget in current tab
   Event::Type e = Event::NoType;
 
   // Detect selection of previous and next tab headers and objects
-  if(instance().eventHandler().kbdShift(mod))
+  if(key == KBDK_TAB)
   {
-    if(key == KBDK_LEFT && cycleTab(-1))
-      return;
-    else if(key == KBDK_RIGHT && cycleTab(+1))
-      return;
-    else if(key == KBDK_TAB)
-      e = Event::UINavPrev;
+    if(StellaModTest::isControl(mod))
+    {
+      // tab header navigation
+      if(StellaModTest::isShift(mod) && cycleTab(-1))
+        return;
+      else if(!StellaModTest::isShift(mod) && cycleTab(+1))
+        return;
+    }
+    else
+    {
+      // object navigation
+      if(StellaModTest::isShift(mod))
+        e = Event::UINavPrev;
+      else
+        e = Event::UINavNext;
+    }
   }
-  else if(key == KBDK_TAB)
-    e = Event::UINavNext;
 
   // Check the keytable now, since we might get one of the above events,
   // which must always be processed before any widget sees it.
@@ -362,7 +384,7 @@ void Dialog::handleKeyUp(StellaKey key, StellaMod mod)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Dialog::handleMouseDown(int x, int y, int button, int clickCount)
+void Dialog::handleMouseDown(int x, int y, MouseButton b, int clickCount)
 {
   Widget* w = findWidget(x, y);
 
@@ -371,11 +393,11 @@ void Dialog::handleMouseDown(int x, int y, int button, int clickCount)
 
   if(w)
     w->handleMouseDown(x - (w->getAbsX() - _x), y - (w->getAbsY() - _y),
-                       button, clickCount);
+                       b, clickCount);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Dialog::handleMouseUp(int x, int y, int button, int clickCount)
+void Dialog::handleMouseUp(int x, int y, MouseButton b, int clickCount)
 {
   if(_focusedWidget)
   {
@@ -387,9 +409,9 @@ void Dialog::handleMouseUp(int x, int y, int button, int clickCount)
   Widget* w = _dragWidget;
   if(w)
     w->handleMouseUp(x - (w->getAbsX() - _x), y - (w->getAbsY() - _y),
-                     button, clickCount);
+                     b, clickCount);
 
-  _dragWidget = 0;
+  _dragWidget = nullptr;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -407,7 +429,7 @@ void Dialog::handleMouseWheel(int x, int y, int direction)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Dialog::handleMouseMoved(int x, int y, int button)
+void Dialog::handleMouseMoved(int x, int y)
 {
   Widget* w;
 
@@ -423,17 +445,17 @@ void Dialog::handleMouseMoved(int x, int y, int button)
     if(mouseInFocusedWidget && _mouseWidget != w)
     {
       if(_mouseWidget)
-        _mouseWidget->handleMouseLeft(button);
+        _mouseWidget->handleMouseLeft();
       _mouseWidget = w;
-      w->handleMouseEntered(button);
+      w->handleMouseEntered();
     }
     else if (!mouseInFocusedWidget && _mouseWidget == w)
     {
-      _mouseWidget = 0;
-      w->handleMouseLeft(button);
+      _mouseWidget = nullptr;
+      w->handleMouseLeft();
     }
 
-    w->handleMouseMoved(x - wx, y - wy, button);
+    w->handleMouseMoved(x - wx, y - wy);
   }
 
   // While a "drag" is in process (i.e. mouse is moved while a button is pressed),
@@ -446,24 +468,24 @@ void Dialog::handleMouseMoved(int x, int y, int button)
   if (_mouseWidget != w)
   {
     if (_mouseWidget)
-      _mouseWidget->handleMouseLeft(button);
+      _mouseWidget->handleMouseLeft();
     if (w)
-      w->handleMouseEntered(button);
+      w->handleMouseEntered();
     _mouseWidget = w;
   }
 
   if (w && (w->getFlags() & WIDGET_TRACK_MOUSE))
-    w->handleMouseMoved(x - (w->getAbsX() - _x), y - (w->getAbsY() - _y), button);
+    w->handleMouseMoved(x - (w->getAbsX() - _x), y - (w->getAbsY() - _y));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Dialog::handleMouseClicks(int x, int y, int button)
+bool Dialog::handleMouseClicks(int x, int y, MouseButton b)
 {
   Widget* w = findWidget(x, y);
 
   if(w)
     return w->handleMouseClicks(x - (w->getAbsX() - _x),
-                                y - (w->getAbsY() - _y), button);
+                                y - (w->getAbsY() - _y), b);
   else
     return false;
 }
@@ -511,7 +533,7 @@ void Dialog::handleJoyAxis(int stick, int axis, int value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Dialog::handleJoyHat(int stick, int hat, int value)
+bool Dialog::handleJoyHat(int stick, int hat, JoyHat value)
 {
   Event::Type e =
     instance().eventHandler().eventForJoyHat(stick, hat, value, kMenuMode);
@@ -620,12 +642,12 @@ void Dialog::handleCommand(CommandSender* sender, int cmd, int data, int id)
 {
   switch(cmd)
   {
-    case kTabChangedCmd:
+    case TabWidget::kTabChangedCmd:
       if(_visible)
         buildCurrentFocusList(id);
       break;
 
-    case kCloseCmd:
+    case GuiObject::kCloseCmd:
       close();
       break;
   }
@@ -643,60 +665,40 @@ Widget* Dialog::findWidget(int x, int y) const
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Dialog::addOKCancelBGroup(WidgetArray& wid, const GUI::Font& font,
-                               const string& okText, const string& cancelText)
+                               const string& okText, const string& cancelText,
+                               bool focusOKButton)
 {
-
   int buttonWidth = std::max(font.getStringWidth("Cancel"),
                     std::max(font.getStringWidth(okText),
                     font.getStringWidth(okText))) + 15;
   int buttonHeight = font.getLineHeight() + 4;
-  ButtonWidget* b;
+
 #ifndef BSPF_MAC_OSX
-  b = new ButtonWidget(this, font, _w - 2 * (buttonWidth + 7), _h - buttonHeight - 10,
-                       buttonWidth, buttonHeight,
-                       okText == "" ? "OK" : okText, kOKCmd);
-  wid.push_back(b);
-  addOKWidget(b);
-  b = new ButtonWidget(this, font, _w - (buttonWidth + 10), _h - buttonHeight - 10,
-                       buttonWidth, buttonHeight,
-                       cancelText == "" ? "Cancel" : cancelText, kCloseCmd);
-  wid.push_back(b);
-  addCancelWidget(b);
+  addOKWidget(new ButtonWidget(this, font, _w - 2 * (buttonWidth + 7),
+      _h - buttonHeight - 10, buttonWidth, buttonHeight, okText, GuiObject::kOKCmd));
+  addCancelWidget(new ButtonWidget(this, font, _w - (buttonWidth + 10),
+      _h - buttonHeight - 10, buttonWidth, buttonHeight, cancelText, GuiObject::kCloseCmd));
 #else
-  b = new ButtonWidget(this, font, _w - 2 * (buttonWidth + 7), _h - buttonHeight - 10,
-                       buttonWidth, buttonHeight,
-                       cancelText == "" ? "Cancel" : cancelText, kCloseCmd);
-  wid.push_back(b);
-  addCancelWidget(b);
-  b = new ButtonWidget(this, font, _w - (buttonWidth + 10), _h - buttonHeight - 10,
-                       buttonWidth, buttonHeight,
-                       okText == "" ? "OK" : okText, kOKCmd);
-  wid.push_back(b);
-  addOKWidget(b);
+  addCancelWidget(new ButtonWidget(this, font, _w - 2 * (buttonWidth + 7),
+      _h - buttonHeight - 10, buttonWidth, buttonHeight, cancelText, GuiObject::kCloseCmd));
+  addOKWidget(new ButtonWidget(this, font, _w - (buttonWidth + 10),
+      _h - buttonHeight - 10, buttonWidth, buttonHeight, okText, GuiObject::kOKCmd));
 #endif
-}
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Dialog::Focus::Focus(Widget* w)
-  : widget(w)
-{
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Dialog::Focus::~Focus()
-{
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Dialog::TabFocus::TabFocus(TabWidget* w)
-  : widget(w),
-    currentTab(0)
-{
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Dialog::TabFocus::~TabFocus()
-{
+  // Note that 'focusOKButton' only takes effect when there are no other UI
+  // elements in the dialog; otherwise, the first widget of the dialog is always
+  // automatically focused first
+  // Changing this behaviour would require a fairly major refactoring of the UI code
+  if(focusOKButton)
+  {
+    wid.push_back(_okWidget);
+    wid.push_back(_cancelWidget);
+  }
+  else
+  {
+    wid.push_back(_cancelWidget);
+    wid.push_back(_okWidget);
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -728,16 +730,25 @@ Widget* Dialog::TabFocus::getNewFocus()
 bool Dialog::getResizableBounds(uInt32& w, uInt32& h) const
 {
   const GUI::Rect& r = instance().frameBuffer().imageRect();
+
+  double scaleFactor = 0.8;
+  if(instance().hasConsole())
+  {
+    bool ntsc = instance().console().about().InitialFrameRate == "60";
+    uInt32 aspect = instance().settings().getInt(ntsc ?"tia.aspectn" : "tia.aspectp");
+    scaleFactor = aspect / 100.0;
+  }
+
   if(r.width() <= FrameBuffer::kFBMinW || r.height() <= FrameBuffer::kFBMinH)
   {
-    w = uInt32(0.8 * FrameBuffer::kTIAMinW) * 2;
+    w = uInt32(scaleFactor * FrameBuffer::kTIAMinW) * 2;
     h = FrameBuffer::kTIAMinH * 2;
     return false;
   }
   else
   {
-    w = std::max(uInt32(0.8 * r.width()), uInt32(FrameBuffer::kFBMinW));
-    h = std::max(uInt32(0.8 * r.height()), uInt32(FrameBuffer::kFBMinH));
+    w = std::max(uInt32(scaleFactor * r.width()), uInt32(FrameBuffer::kFBMinW));
+    h = std::max(uInt32(scaleFactor * r.height()), uInt32(FrameBuffer::kFBMinH));
     return true;
   }
 }
