@@ -8,21 +8,19 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <sstream>
-
 #include "OSystem.hxx"
-#include "FrameBuffer.hxx"
+#include "FBSurface.hxx"
 #include "Dialog.hxx"
+#include "Font.hxx"
 #include "EditTextWidget.hxx"
 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 EditTextWidget::EditTextWidget(GuiObject* boss, const GUI::Font& font,
                                int x, int y, int w, int h, const string& text)
@@ -43,7 +41,7 @@ void EditTextWidget::setText(const string& str, bool changed)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EditTextWidget::handleMouseDown(int x, int y, int button, int clickCount)
+void EditTextWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
 {
   if(!isEditable())
     return;
@@ -72,18 +70,28 @@ void EditTextWidget::drawWidget(bool hilite)
   // Highlight changes
   if(_changed)
     s.fillRect(_x, _y, _w, _h, kDbgChangedColor);
+  else if(!isEditable())
+#ifndef FLAT_UI
+    s.fillRect(_x, _y, _w, _h, kBGColorHi);
+#else
+    s.fillRect(_x, _y, _w, _h, kDlgColor);
+#endif
 
   // Draw a thin frame around us.
+#ifndef FLAT_UI
   s.hLine(_x, _y, _x + _w - 1, kColor);
   s.hLine(_x, _y + _h - 1, _x +_w - 1, kShadowColor);
   s.vLine(_x, _y, _y + _h - 1, kColor);
   s.vLine(_x + _w - 1, _y, _y + _h - 1, kShadowColor);
+#else
+  s.frameRect(_x, _y, _w, _h, kColor);
+#endif
 
   // Draw the text
   adjustOffset();
   s.drawString(_font, editString(), _x + 2, _y + 2, getEditRect().width(),
                !_changed ? _textcolor : uInt32(kDbgChangedTextColor),
-               kTextAlignLeft, -_editScrollOffset, false);
+               TextAlign::Left, -_editScrollOffset, false);
 
   // Draw the caret
   drawCaret();
