@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -18,6 +18,7 @@
 #include "BSType.hxx"
 #include "Console.hxx"
 #include "MouseControl.hxx"
+#include "SaveKey.hxx"
 #include "Dialog.hxx"
 #include "EditTextWidget.hxx"
 #include "Launcher.hxx"
@@ -26,8 +27,9 @@
 #include "Props.hxx"
 #include "PropsSet.hxx"
 #include "TabWidget.hxx"
-#include "FrameManager.hxx"
+#include "TIAConstants.hxx"
 #include "Widget.hxx"
+#include "Font.hxx"
 
 #include "GameInfoDialog.hxx"
 
@@ -47,70 +49,76 @@ GameInfoDialog::GameInfoDialog(
             buttonWidth  = font.getStringWidth("Defaults") + 20,
             buttonHeight = font.getLineHeight() + 4;
   const int vBorder = 4;
+  const int hBorder = 2;
+
+  const int hSpace = 10;
+  const int vGap = 4;
+
   int xpos, ypos, lwidth, fwidth, pwidth, tabID;
   WidgetArray wid;
   VariantList items, ports, ctrls;
+  StaticTextWidget* t;
 
   // Set real dimensions
   _w = 52 * fontWidth + 8;
-  _h = 12 * (lineHeight + 4) + 10;
+  _h = 9 * (lineHeight + vGap) + vBorder * 2 + buttonHeight + fontHeight + ifont.getLineHeight() + 20;
 
   // The tab widget
-  xpos = 2; ypos = vBorder;
-  myTab = new TabWidget(this, font, xpos, ypos, _w - 2*xpos,
-                        _h - buttonHeight - fontHeight - ifont.getLineHeight() - 20);
+  xpos = hBorder; ypos = vBorder;
+  myTab = new TabWidget(this, font, xpos, ypos, _w - 2 * hBorder,
+                        _h - (buttonHeight + fontHeight + ifont.getLineHeight() + 20));
   addTabWidget(myTab);
 
   // 1) Cartridge properties
   tabID = myTab->addTab("Cartridge");
 
-  xpos = 10;
+  xpos = hSpace;
   lwidth = font.getStringWidth("Manufacturer ");
-  fwidth = _w - xpos - lwidth - 10;
+  fwidth = _w - xpos - lwidth - hSpace - hBorder * 2;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Name", kTextAlignLeft);
-  myName = new EditTextWidget(myTab, font, xpos+lwidth, ypos,
-                              fwidth, fontHeight, "");
+                       "Name", TextAlign::Left);
+  myName = new EditTextWidget(myTab, font, xpos+lwidth, ypos-1,
+                              fwidth, lineHeight, "");
   wid.push_back(myName);
 
-  ypos += lineHeight + 3;
+  ypos += lineHeight + vGap;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "MD5", kTextAlignLeft);
-  myMD5 = new StaticTextWidget(myTab, font, xpos+lwidth, ypos,
-                               fwidth, fontHeight,
-                               "", kTextAlignLeft);
+                       "MD5", TextAlign::Left);
+  myMD5 = new EditTextWidget(myTab, font, xpos + lwidth, ypos-1,
+                     fwidth, lineHeight, "");
+  myMD5->setEditable(false);
 
-  ypos += lineHeight + 3;
+  ypos += lineHeight + vGap;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Manufacturer", kTextAlignLeft);
-  myManufacturer = new EditTextWidget(myTab, font, xpos+lwidth, ypos,
-                                      fwidth, fontHeight, "");
+                       "Manufacturer", TextAlign::Left);
+  myManufacturer = new EditTextWidget(myTab, font, xpos+lwidth, ypos-1,
+                                      fwidth, lineHeight, "");
   wid.push_back(myManufacturer);
 
-  ypos += lineHeight + 3;
+  ypos += lineHeight + vGap;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Model", kTextAlignLeft);
-  myModelNo = new EditTextWidget(myTab, font, xpos+lwidth, ypos,
-                                 fwidth, fontHeight, "");
+                       "Model", TextAlign::Left);
+  myModelNo = new EditTextWidget(myTab, font, xpos+lwidth, ypos-1,
+                                 fwidth, lineHeight, "");
   wid.push_back(myModelNo);
 
-  ypos += lineHeight + 3;
+  ypos += lineHeight + vGap;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Rarity", kTextAlignLeft);
-  myRarity = new EditTextWidget(myTab, font, xpos+lwidth, ypos,
-                                fwidth, fontHeight, "");
+                       "Rarity", TextAlign::Left);
+  myRarity = new EditTextWidget(myTab, font, xpos+lwidth, ypos-1,
+                                fwidth, lineHeight, "");
   wid.push_back(myRarity);
 
-  ypos += lineHeight + 3;
+  ypos += lineHeight + vGap;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Note", kTextAlignLeft);
-  myNote = new EditTextWidget(myTab, font, xpos+lwidth, ypos,
-                              fwidth, fontHeight, "");
+                       "Note", TextAlign::Left);
+  myNote = new EditTextWidget(myTab, font, xpos+lwidth, ypos-1,
+                              fwidth, lineHeight, "");
   wid.push_back(myNote);
 
-  ypos += lineHeight + 3;
+  ypos += lineHeight + vGap;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Sound", kTextAlignLeft);
+                       "Sound", TextAlign::Left);
   pwidth = font.getStringWidth("Stereo");
   items.clear();
   VarList::push_back(items, "Mono", "MONO");
@@ -119,9 +127,9 @@ GameInfoDialog::GameInfoDialog(
                             pwidth, lineHeight, items, "", 0, 0);
   wid.push_back(mySound);
 
-  ypos += lineHeight + 3;
+  ypos += lineHeight + vGap;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Type", kTextAlignLeft);
+                       "Type", TextAlign::Left);
   pwidth = font.getStringWidth("CM (SpectraVideo CompuMate)");
   items.clear();
   for(int i = 0; i < int(BSType::NumSchemes); ++i)
@@ -138,11 +146,11 @@ GameInfoDialog::GameInfoDialog(
   wid.clear();
   tabID = myTab->addTab("Console");
 
-  xpos = 10; ypos = vBorder;
+  xpos = hSpace; ypos = vBorder;
   lwidth = font.getStringWidth("Right Difficulty ");
   pwidth = font.getStringWidth("B & W");
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Left Difficulty", kTextAlignLeft);
+                       "Left Difficulty", TextAlign::Left);
   items.clear();
   VarList::push_back(items, "B", "B");
   VarList::push_back(items, "A", "A");
@@ -150,17 +158,17 @@ GameInfoDialog::GameInfoDialog(
                                pwidth, lineHeight, items, "", 0, 0);
   wid.push_back(myLeftDiff);
 
-  ypos += lineHeight + 5;
+  ypos += lineHeight + vGap;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Right Difficulty", kTextAlignLeft);
+                       "Right Difficulty", TextAlign::Left);
   // ... use same items as above
   myRightDiff = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
                                 pwidth, lineHeight, items, "", 0, 0);
   wid.push_back(myRightDiff);
 
-  ypos += lineHeight + 5;
+  ypos += lineHeight + vGap;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "TV Type", kTextAlignLeft);
+                       "TV Type", TextAlign::Left);
   items.clear();
   VarList::push_back(items, "Color", "COLOR");
   VarList::push_back(items, "B & W", "BW");
@@ -176,11 +184,10 @@ GameInfoDialog::GameInfoDialog(
   wid.clear();
   tabID = myTab->addTab("Controller");
 
-  xpos = 10; ypos = vBorder;
-  lwidth = font.getStringWidth("P0 Controller ");
+  ypos = vBorder;
   pwidth = font.getStringWidth("Paddles_IAxis");
-  new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "P0 Controller", kTextAlignLeft);
+  myP0Label = new StaticTextWidget(myTab, font, hSpace, ypos+1,
+      "P0 Controller ", TextAlign::Left);
   ctrls.clear();
   VarList::push_back(ctrls, "Joystick",      "JOYSTICK"     );
   VarList::push_back(ctrls, "Paddles",       "PADDLES"      );
@@ -196,66 +203,50 @@ GameInfoDialog::GameInfoDialog(
   VarList::push_back(ctrls, "AtariVox",      "ATARIVOX"     );
   VarList::push_back(ctrls, "SaveKey",       "SAVEKEY"      );
   VarList::push_back(ctrls, "Sega Genesis",  "GENESIS"      );
-  VarList::push_back(ctrls, "CompuMate",     "COMPUMATE"    );
 //  VarList::push_back(ctrls, "KidVid",        "KIDVID"      );
   VarList::push_back(ctrls, "MindLink",      "MINDLINK"     );
-  myP0Controller = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                                   pwidth, lineHeight, ctrls, "", 0, 0);
+
+  myP0Controller = new PopUpWidget(myTab, font, myP0Label->getRight(), myP0Label->getTop()-1,
+                                   pwidth, lineHeight, ctrls, "", 0, kLeftCChanged);
   wid.push_back(myP0Controller);
 
-  xpos += lwidth+myP0Controller->getWidth() + 4;
-  new StaticTextWidget(myTab, font, xpos, ypos+1, font.getStringWidth("in "),
-                       fontHeight, "in ", kTextAlignLeft);
-  xpos += font.getStringWidth("in ");
-  pwidth = font.getStringWidth("right port");
-  ports.clear();
-  VarList::push_back(ports, "left port", "L");
-  VarList::push_back(ports, "right port", "R");
-  myLeftPort = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
-                               ports, "", 0, kLeftCChanged);
-  wid.push_back(myLeftPort);
-
-  xpos = 10;  ypos += lineHeight + 5;
+  ypos += lineHeight + vGap;
   pwidth = font.getStringWidth("Paddles_IAxis");
-  new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "P1 Controller", kTextAlignLeft);
-  myP1Controller = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                                   pwidth, lineHeight, ctrls, "", 0, 0);
+  myP1Label = new StaticTextWidget(myTab, font, hSpace, ypos+1,
+      "P1 Controller ", TextAlign::Left);
+  myP1Controller = new PopUpWidget(myTab, font, myP1Label->getRight(), myP1Label->getTop()-1,
+                                   pwidth, lineHeight, ctrls, "", 0, kRightCChanged);
   wid.push_back(myP1Controller);
 
-  xpos += lwidth+myP1Controller->getWidth() + 4;
-  pwidth = font.getStringWidth("right port");
-  new StaticTextWidget(myTab, font, xpos, ypos+1, font.getStringWidth("in "),
-                       fontHeight, "in ", kTextAlignLeft);
-  xpos += font.getStringWidth("in ");
-  myRightPort = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
-                                ports, "", 0, kRightCChanged);
-  wid.push_back(myRightPort);
-
-  xpos = 10;  ypos += lineHeight + 5;
-  pwidth = font.getStringWidth("Yes");
-  new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Swap Paddles", kTextAlignLeft);
-  items.clear();
-  VarList::push_back(items, "Yes", "YES");
-  VarList::push_back(items, "No", "NO");
-  mySwapPaddles = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                                  pwidth, lineHeight, items, "", 0, 0);
+  //ypos += lineHeight + vGap;
+  mySwapPorts = new CheckboxWidget(myTab, font, myP0Controller->getRight() + fontWidth*5, myP0Controller->getTop()+1,
+                                   "Swap Ports");
+  wid.push_back(mySwapPorts);
+  //ypos += lineHeight + vGap;
+  mySwapPaddles = new CheckboxWidget(myTab, font, myP1Controller->getRight() + fontWidth*5, myP1Controller->getTop()+1,
+                                     "Swap Paddles");
   wid.push_back(mySwapPaddles);
 
-  ypos += lineHeight + 8;
-  lwidth = font.getStringWidth("Mouse axis mode ");
+  // EEPROM erase button for P0
+  ypos += lineHeight + vGap + 4;
+  myEraseEEPROMLabel = new StaticTextWidget(myTab, font, hSpace, ypos, "AtariVox/SaveKey ");
+  myEraseEEPROMButton = new ButtonWidget(myTab, font, myEraseEEPROMLabel->getRight(), ypos - 4,
+                                           "Erase EEPROM", kEEButtonPressed);
+  myEraseEEPROMInfo = new StaticTextWidget(myTab, ifont, myEraseEEPROMButton->getRight() + 4, myEraseEEPROMLabel->getTop() + 3,
+                                           "(for this game only)");
+
+  ypos += lineHeight + vGap * 4;
+  lwidth = font.getStringWidth("Mouse axis mode  ");
   pwidth = font.getStringWidth("Specific axis");
   items.clear();
   VarList::push_back(items, "Automatic", "AUTO");
   VarList::push_back(items, "Specific axis", "specific");
   myMouseControl =
-    new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
-                   "Mouse axis mode ", lwidth, kMCtrlChanged);
+    new PopUpWidget(myTab, font, hSpace, ypos, pwidth, lineHeight, items,
+                   "Mouse axis mode  ", lwidth, kMCtrlChanged);
   wid.push_back(myMouseControl);
 
   // Mouse controller specific axis
-  lwidth = font.getStringWidth("X-Axis is ");
   pwidth = font.getStringWidth("MindLink 0");
   items.clear();
   VarList::push_back(items, "None",       MouseControl::NoControl);
@@ -268,27 +259,29 @@ GameInfoDialog::GameInfoDialog(
   VarList::push_back(items, "MindLink 0", MouseControl::MindLink0);
   VarList::push_back(items, "MindLink 1", MouseControl::MindLink1);
 
-  xpos = 45;  ypos += lineHeight + 4;
+  xpos = hSpace + lwidth;
+  lwidth = font.getStringWidth("X-Axis is ");
+  xpos -= lwidth;
+  ypos += lineHeight + vGap;
   myMouseX = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
-               "X-Axis is ", lwidth);
+               "X-Axis is ");
   wid.push_back(myMouseX);
 
-  ypos += lineHeight + 4;
-  myMouseY = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
-               "Y-Axis is ", lwidth);
+  ypos += lineHeight + vGap;
+  myMouseY = new PopUpWidget(myTab, font, myMouseX->getLeft(), ypos, pwidth, lineHeight, items,
+               "Y-Axis is ");
   wid.push_back(myMouseY);
 
-  xpos = 10;  ypos += lineHeight + 8;
+  xpos = hSpace;  ypos += lineHeight + vGap;
   lwidth = font.getStringWidth("Mouse axis range ");
-  myMouseRange = new SliderWidget(myTab, font, xpos, ypos, 8*fontWidth, lineHeight,
+  myMouseRange = new SliderWidget(myTab, font, hSpace, ypos, 8*fontWidth, lineHeight,
                                   "Mouse axis range ", lwidth, kMRangeChanged);
   myMouseRange->setMinValue(1); myMouseRange->setMaxValue(100);
   wid.push_back(myMouseRange);
 
   myMouseRangeLabel = new StaticTextWidget(myTab, font,
-                            xpos + myMouseRange->getWidth() + 4, ypos+1,
-                            3*fontWidth, fontHeight, "", kTextAlignLeft);
-  myMouseRangeLabel->setFlags(WIDGET_CLEARBG);
+                            myMouseRange->getRight() + 4, myMouseRange->getTop()+1,
+                            "   ", TextAlign::Left);
 
   // Add items for tab 2
   addToFocusList(wid, myTab, tabID);
@@ -298,11 +291,9 @@ GameInfoDialog::GameInfoDialog(
   wid.clear();
   tabID = myTab->addTab("Display");
 
-  xpos = 10; ypos = vBorder;
-  lwidth = font.getStringWidth("Use Phosphor ");
+  ypos = vBorder;
   pwidth = font.getStringWidth("Auto-detect");
-  new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Format", kTextAlignLeft);
+  t = new StaticTextWidget(myTab, font, hSpace, ypos+1, "Format ", TextAlign::Left);
   items.clear();
   VarList::push_back(items, "Auto-detect", "AUTO");
   VarList::push_back(items, "NTSC",    "NTSC");
@@ -311,58 +302,46 @@ GameInfoDialog::GameInfoDialog(
   VarList::push_back(items, "NTSC50",  "NTSC50");
   VarList::push_back(items, "PAL60",   "PAL60");
   VarList::push_back(items, "SECAM60", "SECAM60");
-  myFormat = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
+  myFormat = new PopUpWidget(myTab, font, t->getRight(), ypos,
                              pwidth, lineHeight, items, "", 0, 0);
   wid.push_back(myFormat);
 
-  ypos += lineHeight + 5;
-  new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "YStart", kTextAlignLeft);
-
-  myYStart = new SliderWidget(myTab, font, xpos+lwidth, ypos, 8*fontWidth, lineHeight,
+  ypos += lineHeight + vGap;
+  t = new StaticTextWidget(myTab, font, hSpace, ypos+1, "YStart ", TextAlign::Left);
+  myYStart = new SliderWidget(myTab, font, t->getRight(), ypos, 8*fontWidth, lineHeight,
                               "", 0, kYStartChanged);
-  myYStart->setMinValue(FrameManager::minYStart-1);
-  myYStart->setMaxValue(FrameManager::maxYStart);
+  myYStart->setMinValue(TIAConstants::minYStart-1);
+  myYStart->setMaxValue(TIAConstants::maxYStart);
   wid.push_back(myYStart);
-  myYStartLabel = new StaticTextWidget(myTab, font, xpos+lwidth+myYStart->getWidth() + 4,
-                                       ypos+1, 5*fontWidth, fontHeight, "", kTextAlignLeft);
-  myYStartLabel->setFlags(WIDGET_CLEARBG);
+  myYStartLabel = new StaticTextWidget(myTab, font, myYStart->getRight() + 4,
+                                       ypos+1, 5*fontWidth, fontHeight, "", TextAlign::Left);
 
-  ypos += lineHeight + 5;
-  new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Height", kTextAlignLeft);
-  myHeight = new SliderWidget(myTab, font, xpos+lwidth, ypos, 8*fontWidth, lineHeight,
+  ypos += lineHeight + vGap;
+  t = new StaticTextWidget(myTab, font, hSpace, ypos+1, "Height ", TextAlign::Left);
+  myHeight = new SliderWidget(myTab, font, t->getRight(), ypos, 8*fontWidth, lineHeight,
                               "", 0, kHeightChanged);
-  myHeight->setMinValue(FrameManager::minViewableHeight-1);
-  myHeight->setMaxValue(FrameManager::maxViewableHeight);
+  myHeight->setMinValue(TIAConstants::minViewableHeight-1);
+  myHeight->setMaxValue(TIAConstants::maxViewableHeight);
   wid.push_back(myHeight);
-  myHeightLabel = new StaticTextWidget(myTab, font, xpos+lwidth+myHeight->getWidth() + 4,
-                                       ypos+1, 5*fontWidth, fontHeight, "", kTextAlignLeft);
-  myHeightLabel->setFlags(WIDGET_CLEARBG);
+  myHeightLabel = new StaticTextWidget(myTab, font, myHeight->getRight() + 4,
+                                       ypos+1, 5*fontWidth, fontHeight, "", TextAlign::Left);
 
-  ypos += lineHeight + 5;
-  pwidth = font.getStringWidth("Yes");
-  new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Use Phosphor", kTextAlignLeft);
-  items.clear();
-  VarList::push_back(items, "Yes", "YES");
-  VarList::push_back(items, "No", "NO");
-  myPhosphor = new PopUpWidget(myTab, font, xpos+lwidth, ypos, pwidth,
-                               lineHeight, items, "", 0, kPhosphorChanged);
+  // Phosphor
+  ypos += lineHeight + vGap*4;
+  myPhosphor = new CheckboxWidget(myTab, font, hSpace, ypos+1, "Use Phosphor", kPhosphorChanged);
   wid.push_back(myPhosphor);
 
-  myPPBlend = new SliderWidget(myTab, font, xpos + lwidth + myPhosphor->getWidth() + 10,
-                               ypos, 8*fontWidth, lineHeight, "Blend ",
+  myPPBlend = new SliderWidget(myTab, font,
+                               myPhosphor->getRight() + 16, myPhosphor->getTop()-2,
+                               8*fontWidth, lineHeight, "Blend ",
                                font.getStringWidth("Blend "),
                                kPPBlendChanged);
   myPPBlend->setMinValue(0); myPPBlend->setMaxValue(100);
   wid.push_back(myPPBlend);
 
   myPPBlendLabel = new StaticTextWidget(myTab, font,
-                                        xpos + lwidth + myPhosphor->getWidth() + 10 +
-                                        myPPBlend->getWidth() + 4, ypos+1,
-                                        5*fontWidth, fontHeight, "", kTextAlignLeft);
-  myPPBlendLabel->setFlags(WIDGET_CLEARBG);
+                                        myPPBlend->getRight() + 4, myPhosphor->getTop(),
+                                        5*fontWidth, fontHeight, "", TextAlign::Left);
 
   // Add items for tab 3
   addToFocusList(wid, myTab, tabID);
@@ -373,16 +352,16 @@ GameInfoDialog::GameInfoDialog(
 
   // Add message concerning usage
   lwidth = ifont.getStringWidth("(*) Changes to properties require a ROM reload");
-  new StaticTextWidget(this, ifont, 10, _h - buttonHeight - fontHeight - 20,
+  new StaticTextWidget(this, ifont, hSpace, _h - (buttonHeight + fontHeight + 20),
                        lwidth, fontHeight,
                        "(*) Changes to properties require a ROM reload",
-                       kTextAlignLeft);
+                       TextAlign::Left);
 
   // Add Defaults, OK and Cancel buttons
   wid.clear();
   ButtonWidget* b;
   b = new ButtonWidget(this, font, 10, _h - buttonHeight - 10,
-                       buttonWidth, buttonHeight, "Defaults", kDefaultsCmd);
+                       buttonWidth, buttonHeight, "Defaults", GuiObject::kDefaultsCmd);
   wid.push_back(b);
   addOKCancelBGroup(wid, font);
   addBGroupToFocusList(wid);
@@ -410,6 +389,8 @@ void GameInfoDialog::loadConfig()
       loadView();
     }
   }
+
+  updateControllerStates();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -420,7 +401,7 @@ void GameInfoDialog::loadView()
 
   // Cartridge properties
   myName->setText(myGameProperties.get(Cartridge_Name));
-  myMD5->setLabel(myGameProperties.get(Cartridge_MD5));
+  myMD5->setText(myGameProperties.get(Cartridge_MD5));
   myManufacturer->setText(myGameProperties.get(Cartridge_Manufacturer));
   myModelNo->setText(myGameProperties.get(Cartridge_ModelNo));
   myRarity->setText(myGameProperties.get(Cartridge_Rarity));
@@ -433,14 +414,11 @@ void GameInfoDialog::loadView()
   myRightDiff->setSelected(myGameProperties.get(Console_RightDifficulty), "B");
   myTVType->setSelected(myGameProperties.get(Console_TelevisionType), "COLOR");
 
-  const string& swap = myGameProperties.get(Console_SwapPorts);
-  myLeftPort->setSelected((swap == "NO" ? "L" : "R"), "L");
-  myRightPort->setSelected((swap == "NO" ? "R" : "L"), "R");
-
   // Controller properties
   myP0Controller->setSelected(myGameProperties.get(Controller_Left), "JOYSTICK");
   myP1Controller->setSelected(myGameProperties.get(Controller_Right), "JOYSTICK");
-  mySwapPaddles->setSelected(myGameProperties.get(Controller_SwapPaddles), "NO");
+  mySwapPorts->setState(myGameProperties.get(Console_SwapPorts) == "YES");
+  mySwapPaddles->setState(myGameProperties.get(Controller_SwapPaddles) == "YES");
 
   // MouseAxis property (potentially contains 'range' information)
   istringstream m_axis(myGameProperties.get(Controller_MouseAxis));
@@ -483,10 +461,10 @@ void GameInfoDialog::loadView()
   myHeight->setValue(atoi(height.c_str()));
   myHeightLabel->setLabel(height == "0" ? "Auto" : height);
 
-  const string& phos = myGameProperties.get(Display_Phosphor);
-  myPhosphor->setSelected(phos, "NO");
-  myPPBlend->setEnabled(phos != "NO");
-  myPPBlendLabel->setEnabled(phos != "NO");
+  bool usePhosphor = myGameProperties.get(Display_Phosphor) == "YES";
+  myPhosphor->setState(usePhosphor);
+  myPPBlend->setEnabled(usePhosphor);
+  myPPBlendLabel->setEnabled(usePhosphor);
 
   const string& blend = myGameProperties.get(Display_PPBlend);
   myPPBlend->setValue(atoi(blend.c_str()));
@@ -518,9 +496,8 @@ void GameInfoDialog::saveConfig()
   // Controller properties
   myGameProperties.set(Controller_Left, myP0Controller->getSelectedTag().toString());
   myGameProperties.set(Controller_Right, myP1Controller->getSelectedTag().toString());
-  myGameProperties.set(Console_SwapPorts,
-    myLeftPort->getSelectedTag().toString() == "L" ? "NO" : "YES");
-  myGameProperties.set(Controller_SwapPaddles, mySwapPaddles->getSelectedTag().toString());
+  myGameProperties.set(Console_SwapPorts, (mySwapPorts->isEnabled() && mySwapPorts->getState()) ? "YES" : "NO");
+  myGameProperties.set(Controller_SwapPaddles, (mySwapPaddles->isEnabled() && mySwapPaddles->getState()) ? "YES" : "NO");
 
   // MouseAxis property (potentially contains 'range' information)
   string mcontrol = myMouseControl->getSelectedTag().toString();
@@ -538,7 +515,8 @@ void GameInfoDialog::saveConfig()
                        myYStartLabel->getLabel());
   myGameProperties.set(Display_Height, myHeightLabel->getLabel() == "Auto" ? "0" :
                        myHeightLabel->getLabel());
-  myGameProperties.set(Display_Phosphor, myPhosphor->getSelectedTag().toString());
+  myGameProperties.set(Display_Phosphor, myPhosphor->getState() ? "YES" : "NO");
+
   myGameProperties.set(Display_PPBlend, myPPBlendLabel->getLabel() == "Auto" ? "0" :
                        myPPBlendLabel->getLabel());
 
@@ -565,48 +543,115 @@ void GameInfoDialog::setDefaults()
   myDefaultsSelected = true;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void GameInfoDialog::updateControllerStates()
+{
+  const string& contrP0 = myP0Controller->getSelectedTag().toString();
+  const string& contrP1 = myP1Controller->getSelectedTag().toString();
+  bool enableEEEraseButton = false;
+  bool enableSwapPaddles = false;
+  bool enableSwapPorts = false;
+
+  // Compumate bankswitching scheme doesn't allow to select controllers
+  bool enableSelectControl = myType->getSelectedTag() != "CM";
+
+  enableSwapPorts = enableSelectControl;
+  enableSwapPaddles = BSPF::startsWithIgnoreCase(contrP0, "PADDLES") ||
+    BSPF::startsWithIgnoreCase(contrP1, "PADDLES");
+
+  if(instance().hasConsole())
+  {
+    const Controller& lport = instance().console().leftController();
+    const Controller& rport = instance().console().rightController();
+
+    // we only enable the button if we have a valid previous and new controller.
+    enableEEEraseButton = ((lport.type() == Controller::SaveKey && contrP0 == "SAVEKEY")
+                           || (lport.type() == Controller::AtariVox && contrP0 == "ATARIVOX")
+                           || (rport.type() == Controller::SaveKey && contrP1 == "SAVEKEY")
+                           || (rport.type() == Controller::AtariVox && contrP1 == "ATARIVOX"));
+  }
+
+  myP0Label->setEnabled(enableSelectControl);
+  myP1Label->setEnabled(enableSelectControl);
+  myP0Controller->setEnabled(enableSelectControl);
+  myP1Controller->setEnabled(enableSelectControl);
+
+  mySwapPorts->setEnabled(enableSwapPorts);
+  mySwapPaddles->setEnabled(enableSwapPaddles);
+
+  myEraseEEPROMLabel->setEnabled(enableEEEraseButton);
+  myEraseEEPROMButton->setEnabled(enableEEEraseButton);
+  myEraseEEPROMInfo->setEnabled(enableEEEraseButton);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void GameInfoDialog::eraseEEPROM()
+{
+  Controller& lport = instance().console().leftController();
+  Controller& rport = instance().console().rightController();
+
+  if(lport.type() == Controller::SaveKey || lport.type() == Controller::AtariVox)
+  {
+    SaveKey& skey = static_cast<SaveKey&>(lport);
+    skey.eraseCurrent();
+  }
+
+  if(rport.type() == Controller::SaveKey || rport.type() == Controller::AtariVox)
+  {
+    SaveKey& skey = static_cast<SaveKey&>(rport);
+    skey.eraseCurrent();
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void GameInfoDialog::handleCommand(CommandSender* sender, int cmd,
                                    int data, int id)
 {
   switch (cmd)
   {
-    case kOKCmd:
+    case GuiObject::kOKCmd:
       saveConfig();
       close();
       break;
 
-    case kDefaultsCmd:
+    case GuiObject::kDefaultsCmd:
       setDefaults();
       break;
 
-    case kLeftCChanged:
-      myRightPort->setSelectedIndex(
-        myLeftPort->getSelected() == 1 ? 0 : 1);
+    case TabWidget::kTabChangedCmd:
+      if(data == 2)  // 'Controller' tab selected
+        updateControllerStates();
+
+      // The underlying dialog still needs access to this command
+      Dialog::handleCommand(sender, cmd, data, 0);
       break;
 
+    case kLeftCChanged:
     case kRightCChanged:
-      myLeftPort->setSelectedIndex(
-        myRightPort->getSelected() == 1 ? 0 : 1);
+      updateControllerStates();
+      break;
+
+    case kEEButtonPressed:
+      eraseEEPROM();
       break;
 
     case kPhosphorChanged:
     {
-      bool status = myPhosphor->getSelectedTag().toString() == "YES";
+      bool status = myPhosphor->getState();
       myPPBlend->setEnabled(status);
       myPPBlendLabel->setEnabled(status);
       break;
     }
 
     case kYStartChanged:
-      if(myYStart->getValue() == FrameManager::minYStart-1)
+      if(myYStart->getValue() == TIAConstants::minYStart-1)
         myYStartLabel->setLabel("Auto");
       else
         myYStartLabel->setValue(myYStart->getValue());
       break;
 
     case kHeightChanged:
-      if(myHeight->getValue() == FrameManager::minViewableHeight-1)
+      if(myHeight->getValue() == TIAConstants::minViewableHeight-1)
         myHeightLabel->setLabel("Auto");
       else
         myHeightLabel->setValue(myHeight->getValue());
@@ -625,7 +670,7 @@ void GameInfoDialog::handleCommand(CommandSender* sender, int cmd,
 
     case kMCtrlChanged:
     {
-      bool state = myMouseControl->getSelectedTag().toString() != "auto";
+      bool state = myMouseControl->getSelectedTag() != "AUTO";
       myMouseX->setEnabled(state);
       myMouseY->setEnabled(state);
       break;
