@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,11 +16,9 @@
 //============================================================================
 
 #include "bspf.hxx"
-
 #include "Dialog.hxx"
-#include "OSystem.hxx"
 #include "Widget.hxx"
-
+#include "Font.hxx"
 #include "HelpDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -30,6 +28,7 @@ HelpDialog::HelpDialog(OSystem& osystem, DialogContainer& parent,
     myPage(1),
     myNumPages(5)
 {
+  const string ELLIPSIS = "\x1d";
   const int lineHeight   = font.getLineHeight(),
             fontWidth    = font.getMaxCharWidth(),
             fontHeight   = font.getFontHeight(),
@@ -46,37 +45,37 @@ HelpDialog::HelpDialog(OSystem& osystem, DialogContainer& parent,
   xpos = 10;  ypos = _h - buttonHeight - 10;
   myPrevButton =
     new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
-                     "Previous", kPrevCmd);
+                     "Previous", GuiObject::kPrevCmd);
   myPrevButton->clearFlags(WIDGET_ENABLED);
   wid.push_back(myPrevButton);
 
   xpos += buttonWidth + 7;
   myNextButton =
     new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
-                     "Next", kNextCmd);
+                     "Next", GuiObject::kNextCmd);
   wid.push_back(myNextButton);
 
   xpos = _w - buttonWidth - 10;
   ButtonWidget* b =
     new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
-                     "Close", kCloseCmd);
+                     "Close", GuiObject::kCloseCmd);
   wid.push_back(b);
   addOKWidget(b);  addCancelWidget(b);
 
   xpos = 5;  ypos = 5;
   myTitle = new StaticTextWidget(this, font, xpos, ypos, _w - 10, fontHeight,
-                                 "", kTextAlignCenter);
+                                 "", TextAlign::Center);
 
-  int lwidth = 15 * fontWidth;
+  int lwidth = 12 * fontWidth;
   xpos += 5;  ypos += lineHeight + 4;
   for(uInt8 i = 0; i < kLINES_PER_PAGE; i++)
   {
     myKey[i] =
       new StaticTextWidget(this, font, xpos, ypos, lwidth,
-                           fontHeight, "", kTextAlignLeft);
+                           fontHeight, "", TextAlign::Left);
     myDesc[i] =
       new StaticTextWidget(this, font, xpos+lwidth, ypos, _w - xpos - lwidth - 5,
-                           fontHeight, "", kTextAlignLeft);
+                           fontHeight, "", TextAlign::Left);
     ypos += fontHeight;
   }
 
@@ -110,7 +109,7 @@ void HelpDialog::updateStrings(uInt8 page, uInt8 lines, string& title)
       ADD_BIND("Cmd Q",     "Quit emulation");
 #endif
       ADD_BIND("Escape",     "Exit current game");
-      ADD_BIND("Tab",        "Enter options menu");
+      ADD_BIND("Tab",        "Enter 'Options' menu");
       ADD_BIND("\\",         "Toggle command menu");
       ADD_BIND(ALT_" =",     "Increase window size");
       ADD_BIND(ALT_" -",     "Decrease window size");
@@ -124,8 +123,8 @@ void HelpDialog::updateStrings(uInt8 page, uInt8 lines, string& title)
       title = "Special commands";
       ADD_BIND("Ctrl g", "Grab mouse (keep in window)");
       ADD_BIND("Ctrl f", "Switch between NTSC/PAL/SECAM");
-      ADD_BIND("Ctrl s", "Save game properties");
-      ADD_BIND("",       "  to a new file");
+      ADD_BIND("Ctrl s", "Save game properties to a");
+      ADD_BIND("",       "  new file");
       ADD_LINE();
       ADD_BIND("Ctrl 0", "Toggle controller for Mouse");
       ADD_BIND("Ctrl 1", "Toggle Stelladaptor left/right");
@@ -147,21 +146,25 @@ void HelpDialog::updateStrings(uInt8 page, uInt8 lines, string& title)
 
     case 4:
       title = "Developer commands";
-      ADD_BIND("~",         "Enter/exit debugger");
+      ADD_BIND("`",         "Enter/exit debugger");
       ADD_LINE();
       ADD_BIND(ALT_" PgUp", "Increase Display.YStart");
       ADD_BIND(ALT_" PgDn", "Decrease Display.YStart");
       ADD_BIND("Ctrl PgUp", "Increase Display.Height");
       ADD_BIND("Ctrl PgDn", "Decrease Display.Height");
+      ADD_LINE();
+      ADD_BIND(ALT_" L", "Toggle frame stats");
+      ADD_BIND(ALT_" ,", "Toggle 'Debug Colors' mode");
+      ADD_BIND(ALT_" t", "Toggle 'Time Machine' mode");
       break;
 
     case 5:
       title = "All other commands";
       ADD_LINE();
-      ADD_BIND("Remapped Events", "");
+      ADD_BIND("Remapped Eve", "nts");
       ADD_TEXT("Most other commands can be");
-      ADD_TEXT("remapped.  Please consult the");
-      ADD_TEXT("'Input Settings' section for");
+      ADD_TEXT("remapped. Please consult the");
+      ADD_TEXT("'Input Settings" + ELLIPSIS + "' menu for");
       ADD_TEXT("more information.");
       break;
   }
@@ -192,7 +195,7 @@ void HelpDialog::handleCommand(CommandSender* sender, int cmd,
 {
   switch(cmd)
   {
-    case kNextCmd:
+    case GuiObject::kNextCmd:
       myPage++;
       if(myPage >= myNumPages)
         myNextButton->clearFlags(WIDGET_ENABLED);
@@ -202,7 +205,7 @@ void HelpDialog::handleCommand(CommandSender* sender, int cmd,
       displayInfo();
       break;
 
-    case kPrevCmd:
+    case GuiObject::kPrevCmd:
       myPage--;
       if(myPage <= myNumPages)
         myNextButton->setFlags(WIDGET_ENABLED);
