@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,25 +16,33 @@
 //============================================================================
 
 #include "bspf.hxx"
-
 #include "OSystem.hxx"
+#include "Console.hxx"
+#include "EventHandler.hxx"
 #include "Joystick.hxx"
 #include "Paddles.hxx"
 #include "PointingDevice.hxx"
+#include "SaveKey.hxx"
+#include "AtariVox.hxx"
 #include "Settings.hxx"
 #include "EventMappingWidget.hxx"
 #include "EditTextWidget.hxx"
+#include "JoystickDialog.hxx"
 #include "PopUpWidget.hxx"
 #include "TabWidget.hxx"
 #include "Widget.hxx"
-
+#include "Font.hxx"
+#include "MessageBox.hxx"
 #include "InputDialog.hxx"
 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 InputDialog::InputDialog(OSystem& osystem, DialogContainer& parent,
                          const GUI::Font& font, int max_w, int max_h)
-  : Dialog(osystem, parent)
+  : Dialog(osystem, parent),
+    myConfirmMsg(nullptr),
+    myMaxWidth(max_w),
+    myMaxHeight(max_h)
+
 {
   const int lineHeight   = font.getLineHeight(),
             fontWidth    = font.getMaxCharWidth(),
@@ -84,12 +92,17 @@ InputDialog::InputDialog(OSystem& osystem, DialogContainer& parent,
   WidgetArray wid;
   ButtonWidget* b;
   b = new ButtonWidget(this, font, 10, _h - buttonHeight - 10,
-                       buttonWidth, buttonHeight, "Defaults", kDefaultsCmd);
+                       buttonWidth, buttonHeight, "Defaults", GuiObject::kDefaultsCmd);
   wid.push_back(b);
   addOKCancelBGroup(wid, font);
   addBGroupToFocusList(wid);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+InputDialog::~InputDialog()
+{
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void InputDialog::addDevicePortTab(const GUI::Font& font)
 {
@@ -99,114 +112,102 @@ void InputDialog::addDevicePortTab(const GUI::Font& font)
   int xpos, ypos, lwidth, pwidth, tabID;
   WidgetArray wid;
   VariantList items;
+  const int vGap = 4;
+  const int hSpace = 8;
 
   // Devices/ports
   tabID = myTab->addTab("Devices & Ports");
 
   // Stelladaptor mappings
-  xpos = 5;  ypos = 5;
-  lwidth = font.getStringWidth("Use mouse as a controller ");
+  ypos = vGap+2;
+  lwidth = font.getStringWidth("Digital paddle sensitivity "); // was: "Use mouse as a controller "
   pwidth = font.getStringWidth("-UI, -Emulation");
 
   VarList::push_back(items, "Left / Right", "lr");
   VarList::push_back(items, "Right / Left", "rl");
-  mySAPort = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
+  mySAPort = new PopUpWidget(myTab, font, hSpace, ypos, pwidth, lineHeight, items,
                              "Stelladaptor port order ", lwidth);
   wid.push_back(mySAPort);
 
   // Use mouse as controller
-  ypos += lineHeight + 5;
+  ypos += lineHeight + vGap;
   items.clear();
   VarList::push_back(items, "Always", "always");
   VarList::push_back(items, "Analog devices", "analog");
   VarList::push_back(items, "Never", "never");
-  myMouseControl = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
+  myMouseControl = new PopUpWidget(myTab, font, hSpace, ypos, pwidth, lineHeight, items,
                              "Use mouse as a controller ", lwidth);
   wid.push_back(myMouseControl);
 
   // Mouse cursor state
-  ypos += lineHeight + 5;
+  ypos += lineHeight + vGap;
   items.clear();
   VarList::push_back(items, "-UI, -Emulation", "0");
   VarList::push_back(items, "-UI, +Emulation", "1");
   VarList::push_back(items, "+UI, -Emulation", "2");
   VarList::push_back(items, "+UI, +Emulation", "3");
-  myCursorState = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
+  myCursorState = new PopUpWidget(myTab, font, hSpace, ypos, pwidth, lineHeight, items,
                              "Mouse cursor visibility ", lwidth);
   wid.push_back(myCursorState);
 #ifndef WINDOWED_SUPPORT
   myCursorState->clearFlags(WIDGET_ENABLED);
 #endif
 
-  // Add AtariVox serial port
-  ypos += lineHeight + 5;
-  lwidth = font.getStringWidth("AVox serial port ");
-  int fwidth = _w - xpos - lwidth - 20;
-  new StaticTextWidget(myTab, font, xpos, ypos, lwidth, fontHeight,
-                       "AVox serial port ", kTextAlignLeft);
-  myAVoxPort = new EditTextWidget(myTab, font, xpos+lwidth, ypos,
-                                  fwidth, fontHeight, "");
-  wid.push_back(myAVoxPort);
-
   lwidth = font.getStringWidth("Digital paddle sensitivity ");
   pwidth = font.getMaxCharWidth() * 8;
 
   // Add joystick deadzone setting
-  ypos += lineHeight + 8;
-  myDeadzone = new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+  ypos += lineHeight + vGap*3;
+  myDeadzone = new SliderWidget(myTab, font, hSpace, ypos, pwidth, lineHeight,
                                 "Joystick deadzone size ", lwidth, kDeadzoneChanged);
   myDeadzone->setMinValue(0); myDeadzone->setMaxValue(29);
-  xpos += myDeadzone->getWidth() + 5;
+  xpos = hSpace + myDeadzone->getWidth() + 5;
   myDeadzoneLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 5*fontWidth,
-                                         lineHeight, "", kTextAlignLeft);
-  myDeadzoneLabel->setFlags(WIDGET_CLEARBG);
+                                         lineHeight, "", TextAlign::Left);
   wid.push_back(myDeadzone);
 
   // Add paddle speed (digital emulation)
-  xpos = 5;  ypos += lineHeight + 4;
-  myDPaddleSpeed = new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+  ypos += lineHeight + vGap;
+  myDPaddleSpeed = new SliderWidget(myTab, font, hSpace, ypos, pwidth, lineHeight,
                                     "Digital paddle sensitivity ",
                                     lwidth, kDPSpeedChanged);
   myDPaddleSpeed->setMinValue(1); myDPaddleSpeed->setMaxValue(20);
-  xpos += myDPaddleSpeed->getWidth() + 5;
+  xpos = hSpace + myDPaddleSpeed->getWidth() + 5;
   myDPaddleLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight,
-                                        "", kTextAlignLeft);
-  myDPaddleLabel->setFlags(WIDGET_CLEARBG);
+                                        "", TextAlign::Left);
   wid.push_back(myDPaddleSpeed);
 
   // Add paddle speed (mouse emulation)
-  xpos = 5;  ypos += lineHeight + 4;
-  myMPaddleSpeed = new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+  ypos += lineHeight + vGap;
+  myMPaddleSpeed = new SliderWidget(myTab, font, hSpace, ypos, pwidth, lineHeight,
                                     "Mouse paddle sensitivity ",
                                     lwidth, kMPSpeedChanged);
   myMPaddleSpeed->setMinValue(1); myMPaddleSpeed->setMaxValue(20);
-  xpos += myMPaddleSpeed->getWidth() + 5;
+  xpos = hSpace + myMPaddleSpeed->getWidth() + 5;
   myMPaddleLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight,
-                                        "", kTextAlignLeft);
-  myMPaddleSpeed->setFlags(WIDGET_CLEARBG);
+                                        "", TextAlign::Left);
   wid.push_back(myMPaddleSpeed);
 
   // Add trackball speed
-  xpos = 5;  ypos += lineHeight + 4;
-  myTrackBallSpeed = new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+  ypos += lineHeight + vGap;
+  myTrackBallSpeed = new SliderWidget(myTab, font, hSpace, ypos, pwidth, lineHeight,
                                       "Trackball sensitivity ",
                                       lwidth, kTBSpeedChanged);
   myTrackBallSpeed->setMinValue(1); myTrackBallSpeed->setMaxValue(20);
-  xpos += myTrackBallSpeed->getWidth() + 5;
+  xpos = hSpace + myTrackBallSpeed->getWidth() + 5;
   myTrackBallLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight,
-                                          "", kTextAlignLeft);
-  myTrackBallSpeed->setFlags(WIDGET_CLEARBG);
+                                          "", TextAlign::Left);
   wid.push_back(myTrackBallSpeed);
 
   // Add 'allow all 4 directions' for joystick
-  xpos = 10;  ypos += lineHeight + 12;
-  myAllowAll4 = new CheckboxWidget(myTab, font, xpos, ypos,
+  ypos += lineHeight + vGap*3;
+  myAllowAll4 = new CheckboxWidget(myTab, font, hSpace, ypos,
                   "Allow all 4 directions on joystick");
   wid.push_back(myAllowAll4);
 
   // Grab mouse (in windowed mode)
-  ypos += lineHeight + 4;
-  myGrabMouse = new CheckboxWidget(myTab, font, xpos, ypos,
+  ypos += lineHeight + vGap;
+  myGrabMouse = new CheckboxWidget(myTab, font, hSpace, ypos,
 	                "Grab mouse in emulation mode");
   wid.push_back(myGrabMouse);
 #ifndef WINDOWED_SUPPORT
@@ -214,18 +215,41 @@ void InputDialog::addDevicePortTab(const GUI::Font& font)
 #endif
 
   // Enable/disable control key-combos
-  ypos += lineHeight + 4;
-  myCtrlCombo = new CheckboxWidget(myTab, font, xpos, ypos,
+  ypos += lineHeight + vGap;
+  myCtrlCombo = new CheckboxWidget(myTab, font, hSpace, ypos,
 	                "Use Control key combos");
   wid.push_back(myCtrlCombo);
 
+  int fwidth;
+
+  // Add EEPROM erase (part 1/2)
+  ypos += vGap*4;
+  fwidth = font.getStringWidth("AtariVox/SaveKey");
+  lwidth = font.getStringWidth("AtariVox/SaveKey");
+  new StaticTextWidget(myTab, font, _w - 14 - (fwidth + lwidth) / 2, ypos,
+                       "AtariVox/SaveKey");
+
   // Show joystick database
-  xpos += 20;  ypos += lineHeight + 8;
-  myJoyDlgButton = new ButtonWidget(myTab, font, xpos, ypos,
-    font.getStringWidth("Show Joystick Database") + 20, font.getLineHeight() + 4,
-    "Show Joystick Database", kDBButtonPressed);
+  ypos += lineHeight;
+  myJoyDlgButton = new ButtonWidget(myTab, font, hSpace, ypos, 20,
+    "Joystick database" + ELLIPSIS, kDBButtonPressed);
   wid.push_back(myJoyDlgButton);
 
+  // Add EEPROM erase (part 1/2)
+  myEraseEEPROMButton = new ButtonWidget(myTab, font, _w - 14 - fwidth, ypos,
+                                         fwidth, lineHeight+4,
+                                        "Erase EEPROM", kEEButtonPressed);
+
+  // Add AtariVox serial port
+  ypos += lineHeight + vGap*2;
+  lwidth = font.getStringWidth("AVox serial port ");
+  fwidth = _w - 14 - hSpace - lwidth;
+  new StaticTextWidget(myTab, font, hSpace, ypos, "AVox serial port ");
+  myAVoxPort = new EditTextWidget(myTab, font, hSpace + lwidth, ypos,
+                                  fwidth, fontHeight, "");
+
+  wid.push_back(myAVoxPort);
+
   // Add items for virtual device ports
   addToFocusList(wid, myTab, tabID);
 }
@@ -260,6 +284,18 @@ void InputDialog::loadConfig()
   // AtariVox serial port
   myAVoxPort->setText(instance().settings().getString("avoxport"));
 
+  // EEPROM erase (only enable in emulation mode and for valid controllers)
+  if(instance().hasConsole())
+  {
+    Controller& lport = instance().console().leftController();
+    Controller& rport = instance().console().rightController();
+
+    myEraseEEPROMButton->setEnabled(lport.type() == Controller::SaveKey || lport.type() == Controller::AtariVox ||
+                                    rport.type() == Controller::SaveKey || rport.type() == Controller::AtariVox);
+  }
+  else
+    myEraseEEPROMButton->setEnabled(false);
+
   // Allow all 4 joystick directions
   myAllowAll4->setState(instance().settings().getBool("joyallow4"));
 
@@ -313,7 +349,7 @@ void InputDialog::saveConfig()
   const string& cursor = myCursorState->getSelectedTag().toString();
   instance().settings().setValue("cursor", cursor);
   instance().settings().setValue("grabmouse", myGrabMouse->getState());
-  instance().frameBuffer().setCursorState();
+  instance().frameBuffer().enableGrabMouse(myGrabMouse->getState());
 
   // Enable/disable control key-combos
   instance().settings().setValue("ctrlcombo", myCtrlCombo->getState());
@@ -414,7 +450,7 @@ void InputDialog::handleJoyAxis(int stick, int axis, int value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool InputDialog::handleJoyHat(int stick, int hat, int value)
+bool InputDialog::handleJoyHat(int stick, int hat, JoyHat value)
 {
   // Remap joystick hat in remap mode, otherwise pass to parent dialog
   if(myEmulEventMapper->remapMode())
@@ -425,23 +461,44 @@ bool InputDialog::handleJoyHat(int stick, int hat, int value)
     return Dialog::handleJoyHat(stick, hat, value);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void InputDialog::eraseEEPROM()
+{
+  // This method will only be callable if a console exists, so we don't
+  // need to check again here
+  Controller& lport = instance().console().leftController();
+  Controller& rport = instance().console().rightController();
+
+  if(lport.type() == Controller::SaveKey || lport.type() == Controller::AtariVox)
+  {
+    SaveKey& skey = static_cast<SaveKey&>(lport);
+    skey.eraseCurrent();
+  }
+
+  if(rport.type() == Controller::SaveKey || rport.type() == Controller::AtariVox)
+  {
+    SaveKey& skey = static_cast<SaveKey&>(rport);
+    skey.eraseCurrent();
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void InputDialog::handleCommand(CommandSender* sender, int cmd,
                                 int data, int id)
 {
   switch(cmd)
   {
-    case kOKCmd:
+    case GuiObject::kOKCmd:
       saveConfig();
       close();
       break;
 
-    case kCloseCmd:
+    case GuiObject::kCloseCmd:
       // Revert changes made to event mapping
       close();
       break;
 
-    case kDefaultsCmd:
+    case GuiObject::kDefaultsCmd:
       setDefaults();
       break;
 
@@ -468,6 +525,29 @@ void InputDialog::handleCommand(CommandSender* sender, int cmd,
       myJoyDialog->show();
       break;
 
+    case kEEButtonPressed:
+      if(!myConfirmMsg)
+      {
+        StringList msg;
+        msg.push_back("This operation cannot be undone.");
+        msg.push_back("All data stored on your AtariVox");
+        msg.push_back("or SaveKey will be erased!");
+        msg.push_back("");
+        msg.push_back("If you are sure you want to erase");
+        msg.push_back("the data, click 'OK', otherwise ");
+        msg.push_back("click 'Cancel'.");
+        myConfirmMsg = make_unique<GUI::MessageBox>
+          (this, instance().frameBuffer().font(), msg,
+           myMaxWidth, myMaxHeight, kConfirmEEEraseCmd,
+           "OK", "Cancel", false);
+      }
+      myConfirmMsg->show();
+      break;
+
+    case kConfirmEEEraseCmd:
+      eraseEEPROM();
+      break;
+
     default:
       Dialog::handleCommand(sender, cmd, data, 0);
   }
