@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,11 +16,12 @@
 //============================================================================
 
 #include "OSystem.hxx"
+#include "EventHandler.hxx"
 #include "Widget.hxx"
+#include "Font.hxx"
 #include "EditTextWidget.hxx"
 #include "StringListWidget.hxx"
 #include "Variant.hxx"
-
 #include "JoystickDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -46,7 +47,7 @@ JoystickDialog::JoystickDialog(GuiObject* boss, const GUI::Font& font,
   ypos = _h - buttonHeight - 10;
   StaticTextWidget* t = new StaticTextWidget(this, font, xpos, ypos,
       font.getStringWidth("Joystick ID "), font.getFontHeight(),
-      "Joystick ID ", kTextAlignLeft);
+      "Joystick ID ", TextAlign::Left);
   xpos += t->getWidth() + 4;
   myJoyText = new EditTextWidget(this, font, xpos, ypos-2,
       font.getStringWidth("Unplugged")+8, font.getLineHeight(), "");
@@ -55,7 +56,7 @@ JoystickDialog::JoystickDialog(GuiObject* boss, const GUI::Font& font,
   // Add buttons at bottom
   xpos = _w - buttonWidth - 10;
   myCloseBtn = new ButtonWidget(this, font, xpos, ypos,
-      buttonWidth, buttonHeight, "Close", kCloseCmd);
+      buttonWidth, buttonHeight, "Close", GuiObject::kCloseCmd);
   addOKWidget(myCloseBtn);  addCancelWidget(myCloseBtn);
 
   buttonWidth = font.getStringWidth("Remove") + 20;
@@ -95,7 +96,7 @@ void JoystickDialog::handleCommand(CommandSender* sender, int cmd, int data, int
 {
   switch(cmd)
   {
-    case kOKCmd:
+    case GuiObject::kOKCmd:
       close();
       break;
 
