@@ -8,17 +8,14 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <sstream>
-
 #include "bspf.hxx"
-
 #include "BrowserDialog.hxx"
 #include "ContextMenu.hxx"
 #include "DialogContainer.hxx"
@@ -32,16 +29,18 @@
 #include "LauncherFilterDialog.hxx"
 #include "MessageBox.hxx"
 #include "OSystem.hxx"
+#include "FrameBuffer.hxx"
+#include "EventHandler.hxx"
+#include "StellaKeys.hxx"
 #include "Props.hxx"
 #include "PropsSet.hxx"
 #include "RomInfoWidget.hxx"
 #include "Settings.hxx"
 #include "StringListWidget.hxx"
 #include "Widget.hxx"
-
+#include "Font.hxx"
 #include "LauncherDialog.hxx"
 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 LauncherDialog::LauncherDialog(OSystem& osystem, DialogContainer& parent,
                                int x, int y, int w, int h)
@@ -69,13 +68,13 @@ LauncherDialog::LauncherDialog(OSystem& osystem, DialogContainer& parent,
   lwidth = font.getStringWidth("Select an item from the list ...");
   xpos += 10;  ypos += 8;
   new StaticTextWidget(this, font, xpos, ypos, lwidth, fontHeight,
-                       "Select an item from the list ...", kTextAlignLeft);
+                       "Select an item from the list ...", TextAlign::Left);
 
   lwidth2 = font.getStringWidth("XXXX items found");
   xpos = _w - lwidth2 - 10;
   myRomCount = new StaticTextWidget(this, font, xpos, ypos,
                                     lwidth2, fontHeight,
-                                    "", kTextAlignRight);
+                                    "", TextAlign::Right);
 
   // Add filter that can narrow the results shown in the listing
   // It has to fit between both labels
@@ -119,7 +118,7 @@ LauncherDialog::LauncherDialog(OSystem& osystem, DialogContainer& parent,
   xpos += 5;  ypos += myList->getHeight() + 4;
   lwidth = font.getStringWidth("Dir ");
   myDirLabel = new StaticTextWidget(this, font, xpos, ypos+2, lwidth, fontHeight,
-                                    "Dir", kTextAlignLeft);
+                                    "Dir", TextAlign::Left);
   xpos += lwidth + 5;
   myDir = new EditTextWidget(this, font, xpos, ypos, _w - xpos - 10, lineHeight, "");
   myDir->setEditable(false, true);
@@ -136,8 +135,8 @@ LauncherDialog::LauncherDialog(OSystem& osystem, DialogContainer& parent,
                                       "Go Up", kPrevDirCmd);
   wid.push_back(myPrevDirButton);
     xpos += bwidth + 8;
-  myOptionsButton = new ButtonWidget(this, font, xpos, ypos, bwidth, bheight,
-                                     "Options", kOptionsCmd);
+    myOptionsButton = new ButtonWidget(this, font, xpos, ypos, bwidth, bheight,
+                                       "Options" + ELLIPSIS, kOptionsCmd);
   wid.push_back(myOptionsButton);
     xpos += bwidth + 8;
   myQuitButton = new ButtonWidget(this, font, xpos, ypos, bwidth, bheight,
@@ -149,7 +148,7 @@ LauncherDialog::LauncherDialog(OSystem& osystem, DialogContainer& parent,
   wid.push_back(myQuitButton);
     xpos += bwidth + 8;
   myOptionsButton = new ButtonWidget(this, font, xpos, ypos, bwidth, bheight,
-                                     "Options", kOptionsCmd);
+                                     "Options" + ELLIPSIS, kOptionsCmd);
   wid.push_back(myOptionsButton);
     xpos += bwidth + 8;
   myPrevDirButton = new ButtonWidget(this, font, xpos, ypos, bwidth, bheight,
@@ -163,8 +162,7 @@ LauncherDialog::LauncherDialog(OSystem& osystem, DialogContainer& parent,
   mySelectedItem = 0;  // Highlight 'Rom Listing'
 
   // Create an options dialog, similar to the in-game one
-  myOptions = make_unique<OptionsDialog>(osystem, parent, this,
-                  int(w * 0.8), int(h * 0.8), true);
+  myOptions = make_unique<OptionsDialog>(osystem, parent, this, w, h, OptionsDialog::launcher);
 
   // Create a game list, which contains all the information about a ROM that
   // the launcher needs
@@ -174,8 +172,8 @@ LauncherDialog::LauncherDialog(OSystem& osystem, DialogContainer& parent,
 
   // Create context menu for ROM list options
   VariantList l;
-  VarList::push_back(l, "Power-on options", "override");
-  VarList::push_back(l, "Filter listing", "filter");
+  VarList::push_back(l, "Power-on options" + ELLIPSIS, "override");
+  VarList::push_back(l, "Filter listing" + ELLIPSIS, "filter");
   VarList::push_back(l, "Reload listing", "reload");
   myMenu = make_unique<ContextMenu>(this, osystem.frameBuffer().font(), l);
 
@@ -420,7 +418,6 @@ bool LauncherDialog::matchPattern(const string& s, const string& pattern) const
       }
     }
   }
-  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -428,23 +425,23 @@ void LauncherDialog::handleKeyDown(StellaKey key, StellaMod mod)
 {
   // Grab the key before passing it to the actual dialog and check for
   // Control-R (reload ROM listing)
-  if(instance().eventHandler().kbdControl(mod) && key == KBDK_R)
+  if(StellaModTest::isControl(mod) && key == KBDK_R)
     updateListing();
   else
     Dialog::handleKeyDown(key, mod);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void LauncherDialog::handleMouseDown(int x, int y, int button, int clickCount)
+void LauncherDialog::handleMouseDown(int x, int y, MouseButton b, int clickCount)
 {
   // Grab right mouse button for context menu, send left to base class
-  if(button == 2)
+  if(b == MouseButton::RIGHT)
   {
     // Add menu at current x,y mouse location
     myMenu->show(x + getAbsX(), y + getAbsY());
   }
   else
-    Dialog::handleMouseDown(x, y, button, clickCount);
+    Dialog::handleMouseDown(x, y, b, clickCount);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -486,7 +483,7 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
           if(result == EmptyString)
             instance().settings().setValue("lastrom", myList->getSelectedString());
           else
-            instance().frameBuffer().showMessage(result, kMiddleCenter, true);
+            instance().frameBuffer().showMessage(result, MessagePosition::MiddleCenter, true);
         }
       }
       break;
