@@ -8,28 +8,25 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <fstream>
-
 #include "bspf.hxx"
-
 #include "Dialog.hxx"
-#include "DialogContainer.hxx"
 #include "FSNode.hxx"
 #include "GuiObject.hxx"
 #include "OSystem.hxx"
+#include "FrameBuffer.hxx"
 #include "Settings.hxx"
 #include "PopUpWidget.hxx"
 #include "StringListWidget.hxx"
 #include "StringParser.hxx"
 #include "Widget.hxx"
-
+#include "Font.hxx"
 #include "LoggerDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -79,7 +76,8 @@ LoggerDialog::LoggerDialog(OSystem& osystem, DialogContainer& parent,
   // Add Defaults, OK and Cancel buttons
   ButtonWidget* b;
   b = new ButtonWidget(this, font, 10, _h - buttonHeight - 10,
-                       buttonWidth, buttonHeight, "Save log to disk", kDefaultsCmd);
+                       buttonWidth, buttonHeight, "Save log to disk",
+                       GuiObject::kDefaultsCmd);
   wid.push_back(b);
   addOKCancelBGroup(wid, font);
 
@@ -123,12 +121,12 @@ void LoggerDialog::handleCommand(CommandSender* sender, int cmd,
 {
   switch(cmd)
   {
-    case kOKCmd:
+    case GuiObject::kOKCmd:
       saveConfig();
       close();
       break;
 
-    case kDefaultsCmd:
+    case GuiObject::kDefaultsCmd:
       saveLogFile();
       break;
 
