@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -25,6 +25,6 @@
 Menu::Menu(OSystem& osystem)
   : DialogContainer(osystem)
 {
-  myBaseDialog = new OptionsDialog(myOSystem, *this, 0,
-      FrameBuffer::kFBMinW, FrameBuffer::kFBMinH, false);
+  myBaseDialog = new OptionsDialog(myOSystem, *this, nullptr,
+      FrameBuffer::kFBMinW, FrameBuffer::kFBMinH, OptionsDialog::emulator);
 }
