@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -19,8 +19,8 @@
 #include "OSystem.hxx"
 #include "Version.hxx"
 #include "Widget.hxx"
+#include "Font.hxx"
 #include "StringParser.hxx"
-
 #include "MessageBox.hxx"
 
 namespace GUI {
@@ -28,7 +28,8 @@ namespace GUI {
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 MessageBox::MessageBox(GuiObject* boss, const GUI::Font& font,
                        const StringList& text, int max_w, int max_h, int cmd,
-                       const string& okText, const string& cancelText)
+                       const string& okText, const string& cancelText,
+                       bool focusOKButton)
   : Dialog(boss->instance(), boss->parent(), 0, 0, max_w, max_h),
     CommandSender(boss),
     myCmd(cmd)
@@ -36,24 +37,18 @@ MessageBox::MessageBox(GuiObject* boss, const GUI::Font& font,
   addText(font, text);
 
   WidgetArray wid;
-  addOKCancelBGroup(wid, font, okText, cancelText);
+  addOKCancelBGroup(wid, font, okText, cancelText, focusOKButton);
   addToFocusList(wid);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 MessageBox::MessageBox(GuiObject* boss, const GUI::Font& font,
                        const string& text, int max_w, int max_h, int cmd,
-                       const string& okText, const string& cancelText)
-  : Dialog(boss->instance(), boss->parent(), 0, 0, max_w, max_h),
-    CommandSender(boss),
-    myCmd(cmd)
+                       const string& okText, const string& cancelText,
+                       bool focusOKButton)
+  : MessageBox(boss, font, StringParser(text).stringList(), max_w, max_h,
+               cmd, okText, cancelText, focusOKButton)
 {
-  StringParser p(text);
-  addText(font, p.stringList());
-
-  WidgetArray wid;
-  addOKCancelBGroup(wid, font, okText, cancelText);
-  addToFocusList(wid);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -75,7 +70,7 @@ void MessageBox::addText(const GUI::Font& font, const StringList& text)
   for(const auto& s: text)
   {
     new StaticTextWidget(this, font, xpos, ypos, _w - 20,
-                         fontHeight, s, kTextAlignLeft);
+                         fontHeight, s, TextAlign::Left);
     ypos += fontHeight;
   }
 }
@@ -85,7 +80,7 @@ void MessageBox::handleCommand(CommandSender* sender, int cmd, int data, int id)
 {
   switch(cmd)
   {
-    case kOKCmd:
+    case GuiObject::kOKCmd:
     {
       close();
 
