@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,24 +16,37 @@
 //============================================================================
 
 #include "bspf.hxx"
-
 #include "FrameBuffer.hxx"
+#include "FBSurface.hxx"
+#include "Font.hxx"
 #include "ContextMenu.hxx"
 #include "DialogContainer.hxx"
-
 #include "PopUpWidget.hxx"
 
 // Little up/down arrow
+#ifndef FLAT_UI
 static uInt32 up_down_arrows[8] = {
-  0x00000000,
-  0x00001000,
-  0x00011100,
-  0x00111110,
-  0x00000000,
-  0x00111110,
-  0x00011100,
-  0x00001000,
+  0b00000000,
+  0b00001000,
+  0b00011100,
+  0b00111110,
+  0b00000000,
+  0b00111110,
+  0b00011100,
+  0b00001000,
 };
+#else
+static uInt32 down_arrow[8] = {
+  0b100000001,
+  0b110000011,
+  0b111000111,
+  0b011101110,
+  0b001111100,
+  0b000111000,
+  0b000010000,
+  0b000000000
+};
+#endif
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PopUpWidget::PopUpWidget(GuiObject* boss, const GUI::Font& font,
@@ -42,28 +55,83 @@ PopUpWidget::PopUpWidget(GuiObject* boss, const GUI::Font& font,
   : Widget(boss, font, x, y - 1, w, h + 2),
     CommandSender(boss),
     _label(label),
-    _labelWidth(labelWidth)
+    _labelWidth(labelWidth),
+    _changed(false)
 {
-  _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS;
+  _flags = WIDGET_ENABLED | WIDGET_RETAIN_FOCUS;
   _bgcolor = kDlgColor;
-  _bgcolorhi = kWidColor;
+  _bgcolorhi = kDlgColor;     // do not highlight the background
   _textcolor = kTextColor;
-  _textcolorhi = kTextColor;
+  _textcolorhi = kTextColor;  // do not highlight the label
 
   if(!_label.empty() && _labelWidth == 0)
     _labelWidth = _font.getStringWidth(_label);
 
+#ifndef FLAT_UI
   _w = w + _labelWidth + 15;
+#else
+  _w = w + _labelWidth + 23;
+#endif
 
   // vertically center the arrows and text
   myTextY   = (_h - _font.getFontHeight()) / 2;
   myArrowsY = (_h - 8) / 2;
 
-  myMenu = make_unique<ContextMenu>(this, font, list, cmd);
+  myMenu = make_unique<ContextMenu>(this, font, list, cmd, w);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PopUpWidget::handleMouseDown(int x, int y, int button, int clickCount)
+void PopUpWidget::addItems(const VariantList& items)
+{
+  myMenu->addItems(items);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PopUpWidget::setSelected(const Variant& tag, const Variant& def)
+{
+  myMenu->setSelected(tag, def);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PopUpWidget::setSelectedIndex(int idx, bool changed)
+{
+  _changed = changed;
+  myMenu->setSelectedIndex(idx);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PopUpWidget::setSelectedMax(bool changed)
+{
+  _changed = changed;
+  myMenu->setSelectedMax();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PopUpWidget::clearSelection()
+{
+  myMenu->clearSelection();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int PopUpWidget::getSelected() const
+{
+  return myMenu->getSelected();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+const string& PopUpWidget::getSelectedName() const
+{
+  return myMenu->getSelectedName();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+const Variant& PopUpWidget::getSelectedTag() const
+{
+  return myMenu->getSelectedTag();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PopUpWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
 {
   if(isEnabled() && !myMenu->isVisible())
   {
@@ -89,6 +157,22 @@ void PopUpWidget::handleMouseWheel(int x, int y, int direction)
   }
 }
 
+#ifdef FLAT_UI
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PopUpWidget::handleMouseEntered()
+{
+  setFlags(WIDGET_HILITED);
+  setDirty();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PopUpWidget::handleMouseLeft()
+{
+  clearFlags(WIDGET_HILITED);
+  setDirty();
+}
+#endif
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool PopUpWidget::handleEvent(Event::Type e)
 {
@@ -98,7 +182,7 @@ bool PopUpWidget::handleEvent(Event::Type e)
   switch(e)
   {
     case Event::UISelect:
-      handleMouseDown(0, 0, 1, 0);
+      handleMouseDown(0, 0, MouseButton::LEFT, 0);
       return true;
     case Event::UIUp:
     case Event::UILeft:
@@ -137,10 +221,11 @@ void PopUpWidget::drawWidget(bool hilite)
   int x = _x + _labelWidth;
   int w = _w - _labelWidth;
 
+#ifndef FLAT_UI
   // Draw the label, if any
-  if (_labelWidth > 0)
+  if(_labelWidth > 0)
     s.drawString(_font, _label, _x, _y + myTextY, _labelWidth,
-                 isEnabled() ? _textcolor : uInt32(kColor), kTextAlignRight);
+                 isEnabled() ? _textcolor : uInt32(kColor), TextAlign::Right);
 
   // Draw a thin frame around us.
   s.hLine(x, _y, x + w - 1, kColor);
@@ -149,16 +234,32 @@ void PopUpWidget::drawWidget(bool hilite)
   s.vLine(x + w - 1, _y, _y +_h - 1, kShadowColor);
 
   // Fill the background
-  s.fillRect(x + 1, _y + 1, w - 2, _h - 2, kWidColor);
-
+  s.fillRect(x + 1, _y + 1, w - 2, _h - 2, _changed ? kDbgChangedColor : kWidColor);
   // Draw an arrow pointing down at the right end to signal this is a dropdown/popup
-  s.drawBitmap(up_down_arrows, x+w - 10, _y + myArrowsY,
+  s.drawBitmap(up_down_arrows, x + w - 10, _y + myArrowsY,
                !isEnabled() ? kColor : hilite ? kTextColorHi : kTextColor);
+#else
+  // Draw the label, if any
+  if(_labelWidth > 0)
+    s.drawString(_font, _label, _x, _y + myTextY, _labelWidth,
+                 isEnabled() ? _textcolor : uInt32(kColor), TextAlign::Left);
+
+  // Draw a thin frame around us.
+  s.frameRect(x, _y, w, _h, kColor);
+  s.frameRect(x + w - 16, _y + 1, 15, _h - 2, isEnabled() && hilite ? kTextColorHi : kBGColorLo);
+
+  // Fill the background
+  s.fillRect(x + 1, _y + 1, w - 17, _h - 2, _changed ? kDbgChangedColor : kWidColor);
+  s.fillRect(x + w - 15, _y + 2, 13, _h - 4, isEnabled() && hilite ? kWidColor : kBGColorHi);
+  // Draw an arrow pointing down at the right end to signal this is a dropdown/popup
+  s.drawBitmap(down_arrow, x + w - 13, _y + myArrowsY + 1,
+               !isEnabled() ? kColor : kTextColor, 9u, 8u);
+#endif
 
   // Draw the selected entry, if any
   const string& name = myMenu->getSelectedName();
-  TextAlignment align = (_font.getStringWidth(name) > w-6) ?
-                         kTextAlignRight : kTextAlignLeft;
+  TextAlign align = (_font.getStringWidth(name) > w-6) ?
+                     TextAlign::Right : TextAlign::Left;
   s.drawString(_font, name, x+2, _y+myTextY, w-6,
                !isEnabled() ? kColor : kTextColor, align);
 }
