@@ -8,21 +8,22 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
+#include "bspf.hxx"
 #include "OSystem.hxx"
+#include "FrameBuffer.hxx"
 #include "Widget.hxx"
 #include "Dialog.hxx"
+#include "Font.hxx"
 #include "DialogContainer.hxx"
 #include "ProgressDialog.hxx"
 
-#include "bspf.hxx"
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ProgressDialog::ProgressDialog(GuiObject* boss, const GUI::Font& font,
                                const string& message)
@@ -45,7 +46,7 @@ ProgressDialog::ProgressDialog(GuiObject* boss, const GUI::Font& font,
 
   xpos = fontWidth; ypos = lineHeight;
   myMessage = new StaticTextWidget(this, font, xpos, ypos, lwidth, fontHeight,
-                                   message, kTextAlignCenter);
+                                   message, TextAlign::Center);
   myMessage->setTextColor(kTextColorEm);
 
   xpos = fontWidth; ypos += 2 * lineHeight;
