@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,7 +16,6 @@
 //============================================================================
 
 #include "bspf.hxx"
-
 #include "Launcher.hxx"
 #include "LauncherFilterDialog.hxx"
 #include "BrowserDialog.hxx"
@@ -24,7 +23,9 @@
 #include "EditTextWidget.hxx"
 #include "ProgressDialog.hxx"
 #include "FSNode.hxx"
+#include "Font.hxx"
 #include "MessageBox.hxx"
+#include "FrameBuffer.hxx"
 #include "MD5.hxx"
 #include "Props.hxx"
 #include "PropsSet.hxx"
@@ -39,49 +40,50 @@ RomAuditDialog::RomAuditDialog(OSystem& osystem, DialogContainer& parent,
     myMaxWidth(max_w),
     myMaxHeight(max_h)
 {
-  const int vBorder = 8;
+  const int vBorder = 10;
+  const int hBorder = 10;
 
   const int lineHeight   = font.getLineHeight(),
             fontWidth    = font.getMaxCharWidth(),
             fontHeight   = font.getFontHeight(),
-            buttonWidth  = font.getStringWidth("Audit path") + 20,
+            buttonWidth  = font.getStringWidth("Audit path" + ELLIPSIS) + 20,
             buttonHeight = font.getLineHeight() + 4,
             lwidth = font.getStringWidth("ROMs without properties (skipped) ");
-  int xpos = vBorder, ypos = vBorder;
+  int xpos, ypos = vBorder;
   WidgetArray wid;
 
   // Set real dimensions
-  _w = 44 * fontWidth + 10;
+  _w = 54 * fontWidth + 10;
   _h = 7 * (lineHeight + 4) + 10;
 
   // Audit path
   ButtonWidget* romButton =
-    new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
-                     "Audit path", kChooseAuditDirCmd);
+    new ButtonWidget(this, font, hBorder, ypos, buttonWidth, buttonHeight,
+                     "Audit path" + ELLIPSIS, kChooseAuditDirCmd);
   wid.push_back(romButton);
-  xpos += buttonWidth + 10;
+  xpos = hBorder + buttonWidth + 10;
   myRomPath = new EditTextWidget(this, font, xpos, ypos + 2,
-                                 _w - xpos - 10, lineHeight, "");
+                                 _w - xpos - hBorder, lineHeight, "");
   wid.push_back(myRomPath);
 
   // Show results of ROM audit
-  xpos = vBorder + 10;  ypos += buttonHeight + 10;
-  new StaticTextWidget(this, font, xpos, ypos, lwidth, fontHeight,
-                       "ROMs with properties (renamed) ", kTextAlignLeft);
-  myResults1 = new EditTextWidget(this, font, xpos + lwidth, ypos - 2,
-                                  _w - xpos - lwidth - 10, lineHeight, "");
+  ypos += buttonHeight + 16;
+  new StaticTextWidget(this, font, hBorder, ypos, lwidth, fontHeight,
+                       "ROMs with properties (renamed) ", TextAlign::Left);
+  myResults1 = new EditTextWidget(this, font, hBorder + lwidth, ypos - 2,
+                                  _w - hBorder*2 - lwidth, lineHeight, "");
   myResults1->setEditable(false, true);
   ypos += buttonHeight;
-  new StaticTextWidget(this, font, xpos, ypos, lwidth, fontHeight,
-                       "ROMs without properties (skipped) ", kTextAlignLeft);
-  myResults2 = new EditTextWidget(this, font, xpos + lwidth, ypos - 2,
-                                  _w - xpos - lwidth - 10, lineHeight, "");
+  new StaticTextWidget(this, font, hBorder, ypos, lwidth, fontHeight,
+                       "ROMs without properties (skipped) ", TextAlign::Left);
+  myResults2 = new EditTextWidget(this, font, hBorder + lwidth, ypos - 2,
+                                  _w - hBorder*2 - lwidth, lineHeight, "");
   myResults2->setEditable(false, true);
 
   ypos += buttonHeight + 8;
-  new StaticTextWidget(this, font, xpos, ypos, _w - 20, fontHeight,
-                       "(*) WARNING: operation cannot be undone",
-                       kTextAlignLeft);
+  new StaticTextWidget(this, font, hBorder, ypos, _w - 20, fontHeight,
+                       "(*) WARNING: operation cannot be undone!",
+                       TextAlign::Left);
 
   // Add OK and Cancel buttons
   addOKCancelBGroup(wid, font, "Audit", "Done");
@@ -91,6 +93,11 @@ RomAuditDialog::RomAuditDialog(OSystem& osystem, DialogContainer& parent,
   myBrowser = make_unique<BrowserDialog>(this, font, myMaxWidth, myMaxHeight);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+RomAuditDialog::~RomAuditDialog()
+{
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomAuditDialog::loadConfig()
 {
@@ -168,7 +175,7 @@ void RomAuditDialog::handleCommand(CommandSender* sender, int cmd,
 {
   switch (cmd)
   {
-    case kOKCmd:
+    case GuiObject::kOKCmd:
       if(!myConfirmMsg)
       {
         StringList msg;
@@ -181,7 +188,8 @@ void RomAuditDialog::handleCommand(CommandSender* sender, int cmd,
         msg.push_back("audit, click 'OK', otherwise click 'Cancel'.");
         myConfirmMsg = make_unique<GUI::MessageBox>
                           (this, instance().frameBuffer().font(), msg,
-                          myMaxWidth, myMaxHeight, kConfirmAuditCmd);
+                          myMaxWidth, myMaxHeight, kConfirmAuditCmd,
+                           "OK", "Cancel", false);
       }
       myConfirmMsg->show();
       break;
