@@ -8,18 +8,25 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
+#include "EventHandler.hxx"
 #include "FrameBuffer.hxx"
+#include "Dialog.hxx"
+#include "FBSurface.hxx"
+#include "Font.hxx"
 #include "OSystem.hxx"
 #include "Settings.hxx"
+#include "Props.hxx"
+#include "PNGLibrary.hxx"
+#include "Rect.hxx"
 #include "Widget.hxx"
-
+#include "TIAConstants.hxx"
 #include "RomInfoWidget.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -28,8 +35,8 @@ RomInfoWidget::RomInfoWidget(GuiObject* boss, const GUI::Font& font,
   : Widget(boss, font, x, y, w, h),
     mySurfaceIsValid(false),
     myHaveProperties(false),
-    myAvail(w > 400 ? GUI::Size(640, FrameManager::maxViewableHeight*2) :
-                      GUI::Size(320, FrameManager::maxViewableHeight))
+    myAvail(w > 400 ? GUI::Size(640, TIAConstants::maxViewableHeight*2) :
+                      GUI::Size(320, TIAConstants::maxViewableHeight))
 {
   _flags = WIDGET_ENABLED;
   _bgcolor = _bgcolorhi = kWidColor;
@@ -52,7 +59,7 @@ void RomInfoWidget::setProperties(const Properties& props)
   myProperties = props;
 
   // Decide whether the information should be shown immediately
-  if(instance().eventHandler().state() == EventHandler::S_LAUNCHER)
+  if(instance().eventHandler().state() == EventHandlerState::LAUNCHER)
     parseProperties();
 }
 
@@ -64,7 +71,7 @@ void RomInfoWidget::clearProperties()
     mySurface->setVisible(mySurfaceIsValid);
 
   // Decide whether the information should be shown immediately
-  if(instance().eventHandler().state() == EventHandler::S_LAUNCHER)
+  if(instance().eventHandler().state() == EventHandlerState::LAUNCHER)
     setDirty();
 }
 
@@ -76,7 +83,7 @@ void RomInfoWidget::parseProperties()
   // only draw certain parts of it
   if(mySurface == nullptr)
   {
-    mySurface = instance().frameBuffer().allocateSurface(320*2, FrameManager::maxViewableHeight*2);
+    mySurface = instance().frameBuffer().allocateSurface(320*2, TIAConstants::maxViewableHeight*2);
     mySurface->attributes().smoothing = true;
     mySurface->applyAttributes();
 
@@ -116,8 +123,10 @@ void RomInfoWidget::parseProperties()
   myRomInfo.push_back("Model:  " + myProperties.get(Cartridge_ModelNo));
   myRomInfo.push_back("Rarity:  " + myProperties.get(Cartridge_Rarity));
   myRomInfo.push_back("Note:  " + myProperties.get(Cartridge_Note));
-  myRomInfo.push_back("Controllers:  " + myProperties.get(Controller_Left) +
-                      " (left), " + myProperties.get(Controller_Right) + " (right)");
+  bool swappedPorts = myProperties.get(Console_SwapPorts) == "YES";
+  myRomInfo.push_back("Controllers:  " + (!swappedPorts
+    ? myProperties.get(Controller_Left) + " (left), " + myProperties.get(Controller_Right) + " (right)"
+    : myProperties.get(Controller_Right) + " (left), " + myProperties.get(Controller_Left) + " (right)"));
 #if 0
   myRomInfo.push_back("YStart/Height:  " + myProperties.get(Display_YStart) +
                       "    " + myProperties.get(Display_Height));
