@@ -8,29 +8,28 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <sstream>
-
 #include "bspf.hxx"
-
 #include "Dialog.hxx"
 #include "OSystem.hxx"
+#include "FrameBuffer.hxx"
+#include "FBSurface.hxx"
 #include "ListWidget.hxx"
 #include "PopUpWidget.hxx"
 #include "ScrollBarWidget.hxx"
 #include "Settings.hxx"
 #include "TabWidget.hxx"
 #include "Widget.hxx"
+#include "Font.hxx"
 #ifdef DEBUGGER_SUPPORT
   #include "DebuggerDialog.hxx"
 #endif
-
 #include "UIDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -77,8 +76,7 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
   myLauncherWidthLabel =
       new StaticTextWidget(myTab, font,
                            xpos + myLauncherWidthSlider->getWidth() + 4,
-                           ypos + 1, 4*fontWidth, fontHeight, "", kTextAlignLeft);
-  myLauncherWidthLabel->setFlags(WIDGET_CLEARBG);
+                           ypos + 1, 4*fontWidth, fontHeight, "", TextAlign::Left);
   ypos += lineHeight + 4;
 
   myLauncherHeightSlider = new SliderWidget(myTab, font, xpos, ypos, pwidth,
@@ -91,8 +89,7 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
   myLauncherHeightLabel =
       new StaticTextWidget(myTab, font,
                            xpos + myLauncherHeightSlider->getWidth() + 4,
-                           ypos + 1, 4*fontWidth, fontHeight, "", kTextAlignLeft);
-  myLauncherHeightLabel->setFlags(WIDGET_CLEARBG);
+                           ypos + 1, 4*fontWidth, fontHeight, "", TextAlign::Left);
   ypos += lineHeight + 4;
 
   // Launcher font
@@ -134,100 +131,11 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
   lwidth = ifont.getStringWidth("(*) Changes require application restart");
   new StaticTextWidget(myTab, ifont, xpos, ypos, std::min(lwidth, _w-20), fontHeight,
                        "(*) Changes require application restart",
-                       kTextAlignLeft);
+                       TextAlign::Left);
 
   // Add items for tab 0
   addToFocusList(wid, myTab, tabID);
 
-  //////////////////////////////////////////////////////////
-  // 2) Debugger options
-  wid.clear();
-  tabID = myTab->addTab(" Debugger ");
-#ifdef DEBUGGER_SUPPORT
-  lwidth = font.getStringWidth("Debugger Height ");
-  xpos = ypos = vBorder;
-
-  // Debugger width and height
-  myDebuggerWidthSlider = new SliderWidget(myTab, font, xpos, ypos, pwidth,
-                                           lineHeight, "Debugger Width ",
-                                           lwidth, kDWidthChanged);
-  myDebuggerWidthSlider->setMinValue(DebuggerDialog::kSmallFontMinW);
-  myDebuggerWidthSlider->setMaxValue(ds.w);
-  myDebuggerWidthSlider->setStepValue(10);
-  wid.push_back(myDebuggerWidthSlider);
-  myDebuggerWidthLabel =
-      new StaticTextWidget(myTab, font,
-                           xpos + myDebuggerWidthSlider->getWidth() + 4,
-                           ypos + 1, 4*fontWidth, fontHeight, "", kTextAlignLeft);
-  myDebuggerWidthLabel->setFlags(WIDGET_CLEARBG);
-  ypos += lineHeight + 4;
-
-  myDebuggerHeightSlider = new SliderWidget(myTab, font, xpos, ypos, pwidth,
-                                            lineHeight, "Debugger Height ",
-                                            lwidth, kDHeightChanged);
-  myDebuggerHeightSlider->setMinValue(DebuggerDialog::kSmallFontMinH);
-  myDebuggerHeightSlider->setMaxValue(ds.h);
-  myDebuggerHeightSlider->setStepValue(10);
-  wid.push_back(myDebuggerHeightSlider);
-  myDebuggerHeightLabel =
-      new StaticTextWidget(myTab, font,
-                           xpos + myDebuggerHeightSlider->getWidth() + 4,
-                           ypos + 1, 4*fontWidth, fontHeight, "", kTextAlignLeft);
-  myDebuggerHeightLabel->setFlags(WIDGET_CLEARBG);
-
-  // Add minimum window size buttons for different fonts
-  int fbwidth = font.getStringWidth("Set window size for medium font") + 20;
-  xpos = (_w - fbwidth - 2*vBorder)/2;  ypos += 2*lineHeight + 4;
-  b = new ButtonWidget(myTab, font, xpos, ypos, fbwidth, buttonHeight,
-      "Set window size for small font", kDSmallSize);
-  wid.push_back(b);
-  ypos += b->getHeight() + 4;
-  b = new ButtonWidget(myTab, font, xpos, ypos, fbwidth, buttonHeight,
-      "Set window size for medium font", kDMediumSize);
-  wid.push_back(b);
-  ypos += b->getHeight() + 4;
-  b = new ButtonWidget(myTab, font, xpos, ypos, fbwidth, buttonHeight,
-      "Set window size for large font", kDLargeSize);
-  wid.push_back(b);
-  ypos += b->getHeight() + 12;
-
-  // Font style (bold label vs. text, etc)
-  lwidth = font.getStringWidth("Font Style ");
-  pwidth = font.getStringWidth("Bold non-labels only");
-  xpos = vBorder;
-  items.clear();
-  VarList::push_back(items, "All Normal font", "0");
-  VarList::push_back(items, "Bold labels only", "1");
-  VarList::push_back(items, "Bold non-labels only", "2");
-  VarList::push_back(items, "All Bold font", "3");
-  myDebuggerFontStyle =
-    new PopUpWidget(myTab, font, xpos, ypos+1, pwidth, lineHeight, items,
-                    "Font Style ", lwidth);
-  wid.push_back(myDebuggerFontStyle);
-
-  // Debugger is only realistically available in windowed modes 800x600 or greater
-  // (and when it's actually been compiled into the app)
-  bool debuggerAvailable =
-#if defined(DEBUGGER_SUPPORT) && defined(WINDOWED_SUPPORT)
-    (ds.w >= 800 && ds.h >= 600);  // TODO - maybe this logic can disappear?
-#else
-  false;
-#endif
-  if(!debuggerAvailable)
-  {
-    myDebuggerWidthSlider->clearFlags(WIDGET_ENABLED);
-    myDebuggerWidthLabel->clearFlags(WIDGET_ENABLED);
-    myDebuggerHeightSlider->clearFlags(WIDGET_ENABLED);
-    myDebuggerHeightLabel->clearFlags(WIDGET_ENABLED);
-  }
-
-  // Add items for tab 1
-  addToFocusList(wid, myTab, tabID);
-#else
-  new StaticTextWidget(myTab, font, 0, 20, _w-20, fontHeight,
-                       "Debugger support not included", kTextAlignCenter);
-#endif
-
   //////////////////////////////////////////////////////////
   // 3) Misc. options
   wid.clear();
@@ -241,6 +149,7 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
   items.clear();
   VarList::push_back(items, "Standard", "standard");
   VarList::push_back(items, "Classic", "classic");
+  VarList::push_back(items, "Light", "light");
   myPalettePopup = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
                                    items, "Interface Palette (*) ", lwidth);
   wid.push_back(myPalettePopup);
@@ -258,7 +167,7 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
   VarList::push_back(items, "900 ms", "900");
   VarList::push_back(items, "1 second", "1000");
   myListDelayPopup = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
-                                     items, "List quick delay (*) ", lwidth);
+                                     items, "List quick delay ", lwidth);
   wid.push_back(myListDelayPopup);
   ypos += lineHeight + 4;
 
@@ -284,7 +193,7 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
   lwidth = ifont.getStringWidth("(*) Requires application restart");
   new StaticTextWidget(myTab, ifont, xpos, ypos, std::min(lwidth, _w-20), fontHeight,
                        "(*) Requires application restart",
-                       kTextAlignLeft);
+                       TextAlign::Left);
 
   // Add items for tab 2
   addToFocusList(wid, myTab, tabID);
@@ -295,7 +204,7 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
   // Add Defaults, OK and Cancel buttons
   wid.clear();
   b = new ButtonWidget(this, font, 10, _h - buttonHeight - 10,
-                       buttonWidth, buttonHeight, "Defaults", kDefaultsCmd);
+                       buttonWidth, buttonHeight, "Defaults", GuiObject::kDefaultsCmd);
   wid.push_back(b);
   addOKCancelBGroup(wid, font);
   addBGroupToFocusList(wid);
@@ -330,25 +239,6 @@ void UIDialog::loadConfig()
   bool exitlauncher = instance().settings().getBool("exitlauncher");
   myLauncherExitPopup->setSelected(exitlauncher ? "1" : "0", "0");
 
-#ifdef DEBUGGER_SUPPORT
-  // Debugger size
-  const GUI::Size& ds = instance().settings().getSize("dbg.res");
-  w = ds.w, h = ds.h;
-  w = std::max(w, uInt32(DebuggerDialog::kSmallFontMinW));
-  h = std::max(h, uInt32(DebuggerDialog::kSmallFontMinH));
-  w = std::min(w, ds.w);
-  h = std::min(h, ds.h);
-
-  myDebuggerWidthSlider->setValue(w);
-  myDebuggerWidthLabel->setValue(w);
-  myDebuggerHeightSlider->setValue(h);
-  myDebuggerHeightLabel->setValue(h);
-
-  // Debugger font style
-  int style = instance().settings().getInt("dbg.fontstyle");
-  myDebuggerFontStyle->setSelected(style, "0");
-#endif
-
   // UI palette
   const string& pal = instance().settings().getString("uipalette");
   myPalettePopup->setSelected(pal, "standard");
@@ -384,15 +274,6 @@ void UIDialog::saveConfig()
   instance().settings().setValue("exitlauncher",
     myLauncherExitPopup->getSelectedTag().toString());
 
-  // Debugger size
-  instance().settings().setValue("dbg.res",
-    GUI::Size(myDebuggerWidthSlider->getValue(),
-              myDebuggerHeightSlider->getValue()));
-
-  // Debugger font style
-  instance().settings().setValue("dbg.fontstyle",
-    myDebuggerFontStyle->getSelectedTag().toString());
-
   // UI palette
   instance().settings().setValue("uipalette",
     myPalettePopup->getSelectedTag().toString());
@@ -427,21 +308,7 @@ void UIDialog::setDefaults()
       break;
     }
 
-    case 1:  // Debugger options
-    {
-#ifdef DEBUGGER_SUPPORT
-      uInt32 w = std::min(instance().frameBuffer().desktopSize().w, uInt32(DebuggerDialog::kMediumFontMinW));
-      uInt32 h = std::min(instance().frameBuffer().desktopSize().h, uInt32(DebuggerDialog::kMediumFontMinH));
-      myDebuggerWidthSlider->setValue(w);
-      myDebuggerWidthLabel->setValue(w);
-      myDebuggerHeightSlider->setValue(h);
-      myDebuggerHeightLabel->setValue(h);
-      myDebuggerFontStyle->setSelected("0");
-#endif
-      break;
-    }
-
-    case 2:  // Misc. options
+    case 1:  // Misc. options
       myPalettePopup->setSelected("standard");
       myListDelayPopup->setSelected("300");
       myWheelLinesPopup->setSelected("4");
@@ -467,43 +334,12 @@ void UIDialog::handleCommand(CommandSender* sender, int cmd, int data, int id)
       myLauncherHeightLabel->setValue(myLauncherHeightSlider->getValue());
       break;
 
-#ifdef DEBUGGER_SUPPORT
-    case kDWidthChanged:
-      myDebuggerWidthLabel->setValue(myDebuggerWidthSlider->getValue());
-      break;
-
-    case kDHeightChanged:
-      myDebuggerHeightLabel->setValue(myDebuggerHeightSlider->getValue());
-      break;
-
-    case kDSmallSize:
-      myDebuggerWidthSlider->setValue(DebuggerDialog::kSmallFontMinW);
-      myDebuggerWidthLabel->setValue(DebuggerDialog::kSmallFontMinW);
-      myDebuggerHeightSlider->setValue(DebuggerDialog::kSmallFontMinH);
-      myDebuggerHeightLabel->setValue(DebuggerDialog::kSmallFontMinH);
-      break;
-
-    case kDMediumSize:
-      myDebuggerWidthSlider->setValue(DebuggerDialog::kMediumFontMinW);
-      myDebuggerWidthLabel->setValue(DebuggerDialog::kMediumFontMinW);
-      myDebuggerHeightSlider->setValue(DebuggerDialog::kMediumFontMinH);
-      myDebuggerHeightLabel->setValue(DebuggerDialog::kMediumFontMinH);
-      break;
-
-    case kDLargeSize:
-      myDebuggerWidthSlider->setValue(DebuggerDialog::kLargeFontMinW);
-      myDebuggerWidthLabel->setValue(DebuggerDialog::kLargeFontMinW);
-      myDebuggerHeightSlider->setValue(DebuggerDialog::kLargeFontMinH);
-      myDebuggerHeightLabel->setValue(DebuggerDialog::kLargeFontMinH);
-      break;
-#endif
-
-    case kOKCmd:
+    case GuiObject::kOKCmd:
       saveConfig();
       close();
       break;
 
-    case kDefaultsCmd:
+    case GuiObject::kDefaultsCmd:
       setDefaults();
       break;
 
