@@ -8,38 +8,38 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <sstream>
-
 #include "bspf.hxx"
-
 #include "Control.hxx"
 #include "Dialog.hxx"
 #include "Menu.hxx"
 #include "OSystem.hxx"
-#include "ColorWidget.hxx"
 #include "EditTextWidget.hxx"
 #include "PopUpWidget.hxx"
 #include "Console.hxx"
 #include "TIA.hxx"
 #include "Settings.hxx"
 #include "Widget.hxx"
+#include "Font.hxx"
 #include "TabWidget.hxx"
 #include "NTSCFilter.hxx"
-
+#include "TIASurface.hxx"
 #include "VideoDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
-                         const GUI::Font& font, int max_w, int max_h, bool isGlobal)
+                         const GUI::Font& font, int max_w, int max_h)
   : Dialog(osystem, parent)
 {
+  const int VGAP = 4;
+  const int VBORDER = 8;
+  const int HBORDER = 8;
   const int lineHeight   = font.getLineHeight(),
             fontWidth    = font.getMaxCharWidth(),
             fontHeight   = font.getFontHeight(),
@@ -47,19 +47,22 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
             buttonHeight = font.getLineHeight() + 4;
   int xpos, ypos, tabID;
   int lwidth = font.getStringWidth("NTSC Aspect "),
-      pwidth = font.getStringWidth("XXXXxXXXX");
+    pwidth = font.getStringWidth("XXXXxXXXX"),
+    swidth = 69;
+
   WidgetArray wid;
   VariantList items;
 
   // Set real dimensions
   _w = std::min(52 * fontWidth + 10, max_w);
-  _h = std::min(16 * (lineHeight + 4) + 14, max_h);
+  _h = std::min(16 * (lineHeight + VGAP) + 14, max_h);
 
   // The tab widget
-  xpos = ypos = 5;
+  xpos = 2;  ypos = 4;
   myTab = new TabWidget(this, font, xpos, ypos, _w - 2*xpos, _h - buttonHeight - 20);
   addTabWidget(myTab);
 
+  xpos = HBORDER;  ypos = VBORDER;
   //////////////////////////////////////////////////////////
   // 1) General options
   tabID = myTab->addTab(" General ");
@@ -69,13 +72,13 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
                                instance().frameBuffer().supportedRenderers(),
                                "Renderer ", lwidth);
   wid.push_back(myRenderer);
-  ypos += lineHeight + 4;
+  ypos += lineHeight + VGAP;
 
   // TIA filters (will be dynamically filled later)
   myTIAZoom = new PopUpWidget(myTab, font, xpos, ypos, pwidth,
                               lineHeight, items, "TIA Zoom ", lwidth);
   wid.push_back(myTIAZoom);
-  ypos += lineHeight + 4;
+  ypos += lineHeight + VGAP;
 
   // TIA Palette
   items.clear();
@@ -85,7 +88,7 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   myTIAPalette = new PopUpWidget(myTab, font, xpos, ypos, pwidth,
                                  lineHeight, items, "TIA Palette ", lwidth);
   wid.push_back(myTIAPalette);
-  ypos += lineHeight + 4;
+  ypos += lineHeight + VGAP;
 
   // TIA interpolation
   items.clear();
@@ -94,7 +97,7 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   myTIAInterpolate = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
                                      items, "TIA Inter ", lwidth);
   wid.push_back(myTIAInterpolate);
-  ypos += lineHeight + 4;
+  ypos += lineHeight + VGAP;
 
   // Timing to use between frames
   items.clear();
@@ -103,92 +106,82 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   myFrameTiming = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
                                   items, "Timing (*) ", lwidth);
   wid.push_back(myFrameTiming);
-  ypos += lineHeight + 4;
+  ypos += lineHeight + VGAP;
 
   // Aspect ratio (NTSC mode)
   myNAspectRatio =
-    new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+    new SliderWidget(myTab, font, xpos, ypos-1, pwidth, lineHeight,
                      "NTSC Aspect ", lwidth, kNAspectRatioChanged);
   myNAspectRatio->setMinValue(80); myNAspectRatio->setMaxValue(120);
   wid.push_back(myNAspectRatio);
   myNAspectRatioLabel =
     new StaticTextWidget(myTab, font, xpos + myNAspectRatio->getWidth() + 4,
-                         ypos + 1, fontWidth * 3, fontHeight, "", kTextAlignLeft);
-  myNAspectRatioLabel->setFlags(WIDGET_CLEARBG);
-  ypos += lineHeight + 4;
+                         ypos + 1, fontWidth * 3, fontHeight, "", TextAlign::Left);
+  ypos += lineHeight + VGAP;
 
   // Aspect ratio (PAL mode)
   myPAspectRatio =
-    new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+    new SliderWidget(myTab, font, xpos, ypos-1, pwidth, lineHeight,
                      "PAL Aspect ", lwidth, kPAspectRatioChanged);
   myPAspectRatio->setMinValue(80); myPAspectRatio->setMaxValue(120);
   wid.push_back(myPAspectRatio);
   myPAspectRatioLabel =
     new StaticTextWidget(myTab, font, xpos + myPAspectRatio->getWidth() + 4,
-                         ypos + 1, fontWidth * 3, fontHeight, "", kTextAlignLeft);
-  myPAspectRatioLabel->setFlags(WIDGET_CLEARBG);
-  ypos += lineHeight + 4;
+                         ypos + 1, fontWidth * 3, fontHeight, "", TextAlign::Left);
+  ypos += lineHeight + VGAP;
 
   // Framerate
   myFrameRate =
-    new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+    new SliderWidget(myTab, font, xpos, ypos-1, pwidth, lineHeight,
                      "Framerate ", lwidth, kFrameRateChanged);
-  myFrameRate->setMinValue(0); myFrameRate->setMaxValue(300);
+  myFrameRate->setMinValue(0); myFrameRate->setMaxValue(900);
   myFrameRate->setStepValue(10);
   wid.push_back(myFrameRate);
   myFrameRateLabel =
     new StaticTextWidget(myTab, font, xpos + myFrameRate->getWidth() + 4,
-                         ypos + 1, fontWidth * 4, fontHeight, "", kTextAlignLeft);
-  myFrameRateLabel->setFlags(WIDGET_CLEARBG);
+                         ypos + 1, fontWidth * 4, fontHeight, "", TextAlign::Left);
 
   // Add message concerning usage
   const GUI::Font& infofont = instance().frameBuffer().infoFont();
   ypos = myTab->getHeight() - 5 - fontHeight - infofont.getFontHeight() - 10;
   new StaticTextWidget(myTab, infofont, 10, ypos,
         font.getStringWidth("(*) Requires application restart"), fontHeight,
-        "(*) Requires application restart", kTextAlignLeft);
+        "(*) Requires application restart", TextAlign::Left);
 
   // Move over to the next column
   xpos += myNAspectRatio->getWidth() + myNAspectRatioLabel->getWidth() + 30;
-  ypos = 10;
+  ypos = VBORDER;
 
   // Fullscreen
   myFullscreen = new CheckboxWidget(myTab, font, xpos, ypos, "Fullscreen");
   wid.push_back(myFullscreen);
-  ypos += lineHeight + 4;
+  ypos += lineHeight + VGAP;
 
   // FS stretch
   myUseStretch = new CheckboxWidget(myTab, font, xpos, ypos, "Fullscreen Fill");
   wid.push_back(myUseStretch);
-  ypos += lineHeight + 4;
+  ypos += lineHeight + VGAP;
 
   // Use sync to vblank
   myUseVSync = new CheckboxWidget(myTab, font, xpos, ypos, "VSync");
   wid.push_back(myUseVSync);
-  ypos += lineHeight + 4;
-
-  ypos += lineHeight;
-
-  // PAL color-loss effect
-  myColorLoss = new CheckboxWidget(myTab, font, xpos, ypos, "PAL color-loss");
-  wid.push_back(myColorLoss);
-  ypos += lineHeight + 4;
+  ypos += (lineHeight + VGAP) * 2;
 
   // Skip progress load bars for SuperCharger ROMs
   // Doesn't really belong here, but I couldn't find a better place for it
   myFastSCBios = new CheckboxWidget(myTab, font, xpos, ypos, "Fast SC/AR BIOS");
   wid.push_back(myFastSCBios);
-  ypos += lineHeight + 4;
+  ypos += lineHeight + VGAP;
 
   // Show UI messages onscreen
   myUIMessages = new CheckboxWidget(myTab, font, xpos, ypos, "Show UI messages");
   wid.push_back(myUIMessages);
-  ypos += lineHeight + 4;
+  ypos += lineHeight + VGAP;
 
   // Center window (in windowed mode)
   myCenter = new CheckboxWidget(myTab, font, xpos, ypos, "Center window");
   wid.push_back(myCenter);
-  ypos += lineHeight + 4;
+  ypos += lineHeight + VGAP;
 
   // Use multi-threading
   myUseThreads = new CheckboxWidget(myTab, font, xpos, ypos, "Use multi-threading");
@@ -201,7 +194,8 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   // 2) TV effects options
   wid.clear();
   tabID = myTab->addTab(" TV Effects ");
-  xpos = ypos = 8;
+  xpos = HBORDER;
+  ypos = VBORDER;
 
   // TV Mode
   items.clear();
@@ -212,30 +206,30 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   VarList::push_back(items, "Bad adjust", NTSCFilter::PRESET_BAD);
   VarList::push_back(items, "Custom", NTSCFilter::PRESET_CUSTOM);
   lwidth = font.getStringWidth("TV Mode ");
-  pwidth = font.getStringWidth("Bad adjust"),
+  pwidth = font.getStringWidth("Bad adjust");
   myTVMode =
     new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
                     items, "TV Mode ", lwidth, kTVModeChanged);
   wid.push_back(myTVMode);
-  ypos += lineHeight + 4;
+  ypos += lineHeight + VGAP;
 
   // Custom adjustables (using macro voodoo)
-  xpos += 8; ypos += 4;
+  xpos += 8+1; ypos += 0;
   pwidth = lwidth;
   lwidth = font.getStringWidth("Saturation ");
 
 #define CREATE_CUSTOM_SLIDERS(obj, desc)                                 \
   myTV ## obj =                                                          \
-    new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,        \
+    new SliderWidget(myTab, font, xpos, ypos-1, pwidth, lineHeight,      \
                          desc, lwidth, kTV ## obj ##Changed);            \
   myTV ## obj->setMinValue(0); myTV ## obj->setMaxValue(100);            \
   wid.push_back(myTV ## obj);                                            \
   myTV ## obj ## Label =                                                 \
     new StaticTextWidget(myTab, font, xpos+myTV ## obj->getWidth()+4,    \
-                    ypos+1, fontWidth*3, fontHeight, "", kTextAlignLeft);\
-  myTV ## obj->setFlags(WIDGET_CLEARBG);                                 \
-  ypos += lineHeight + 4
+                    ypos+1, fontWidth*3, fontHeight, "", TextAlign::Left);\
+  ypos += lineHeight + VGAP;
 
+  pwidth = swidth;
   CREATE_CUSTOM_SLIDERS(Contrast, "Contrast ");
   CREATE_CUSTOM_SLIDERS(Bright, "Brightness ");
   CREATE_CUSTOM_SLIDERS(Hue, "Hue ");
@@ -247,8 +241,8 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   CREATE_CUSTOM_SLIDERS(Fringe, "Fringing ");
   CREATE_CUSTOM_SLIDERS(Bleed, "Bleeding ");
 
-  xpos += myTVContrast->getWidth() + myTVContrastLabel->getWidth() + 20;
-  ypos = 8;
+  xpos += myTVContrast->getWidth() + myTVContrastLabel->getWidth() + 36;
+  ypos = VBORDER;
 
   lwidth = font.getStringWidth("Intensity ");
   pwidth = font.getMaxCharWidth() * 6;
@@ -260,50 +254,40 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   myTVPhosphor = new PopUpWidget(myTab, font, xpos, ypos,
       font.getStringWidth("Per-ROM"), lineHeight, items,
       "TV Phosphor ", font.getStringWidth("TV Phosphor "));
-  ypos += lineHeight + 4;
+  ypos += lineHeight + VGAP;
 
   // TV Phosphor default level
-  xpos += 20;
-  CREATE_CUSTOM_SLIDERS(PhosLevel, "Default ");
-  ypos += 4;
-
-  // TV jitter effect
-  xpos -= 20;
-  myTVJitter = new CheckboxWidget(myTab, font, xpos, ypos,
-                                  "Jitter/Roll Effect", kTVJitterChanged);
-  wid.push_back(myTVJitter);
-  xpos += 20;
-  ypos += lineHeight;
-  CREATE_CUSTOM_SLIDERS(JitterRec, "Recovery ");
-  myTVJitterRec->setMinValue(1); myTVJitterRec->setMaxValue(20);
-  ypos += 4;
+  xpos += 8-8+16;
+  pwidth = swidth;
+  CREATE_CUSTOM_SLIDERS(PhosLevel, "Default   ");
+  ypos += 6;
 
   // Scanline intensity and interpolation
-  xpos -= 20;
+  xpos -= 8+8;
   myTVScanLabel =
     new StaticTextWidget(myTab, font, xpos, ypos, font.getStringWidth("Scanline settings"),
-                         fontHeight, "Scanline settings", kTextAlignLeft);
+                         fontHeight, "Scanline settings", TextAlign::Left);
   ypos += lineHeight;
 
-  xpos += 20;
+  xpos += 8+8;
   CREATE_CUSTOM_SLIDERS(ScanIntense, "Intensity ");
 
   myTVScanInterpolate = new CheckboxWidget(myTab, font, xpos, ypos,
                                            "Interpolation");
   wid.push_back(myTVScanInterpolate);
-  ypos += lineHeight + 4;
+  ypos += lineHeight + 6;
 
   // Adjustable presets
-  xpos -= 20;
+  xpos -= 8+8;
   int cloneWidth = font.getStringWidth("Clone Bad Adjust") + 20;
 #define CREATE_CLONE_BUTTON(obj, desc)                                 \
   myClone ## obj =                                                     \
     new ButtonWidget(myTab, font, xpos, ypos, cloneWidth, buttonHeight,\
                      desc, kClone ## obj ##Cmd);                       \
   wid.push_back(myClone ## obj);                                       \
-  ypos += lineHeight + 10
+  ypos += lineHeight + 4 + VGAP
 
-  ypos += 4;
+  ypos += VGAP;
   CREATE_CLONE_BUTTON(Composite, "Clone Composite");
   CREATE_CLONE_BUTTON(Svideo, "Clone S-Video");
   CREATE_CLONE_BUTTON(RGB, "Clone RGB");
@@ -313,57 +297,6 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   // Add items for tab 2
   addToFocusList(wid, myTab, tabID);
 
-  //////////////////////////////////////////////////////////
-  // 3) TIA debug colours
-  wid.clear();
-  tabID = myTab->addTab(" Debug Colors ");
-  xpos = ypos = 8;
-
-  items.clear();
-  VarList::push_back(items, "Red", "r");
-  VarList::push_back(items, "Orange", "o");
-  VarList::push_back(items, "Yellow", "y");
-  VarList::push_back(items, "Green", "g");
-  VarList::push_back(items, "Blue", "b");
-  VarList::push_back(items, "Purple", "p");
-
-  static constexpr int dbg_cmds[6] = {
-    kP0ColourChangedCmd,  kM0ColourChangedCmd,  kP1ColourChangedCmd,
-    kM1ColourChangedCmd,  kPFColourChangedCmd,  kBLColourChangedCmd
-  };
-
-  auto createDebugColourWidgets = [&](int idx, const string& desc) {
-    int x = xpos;
-    myDbgColour[idx] = new PopUpWidget(myTab, font, x, ypos,
-        pwidth, lineHeight, items, desc, lwidth, dbg_cmds[idx]);
-    wid.push_back(myDbgColour[idx]);
-    x += myDbgColour[idx]->getWidth() + 10;
-    myDbgColourSwatch[idx] = new ColorWidget(myTab, font, x, ypos,
-        uInt32(2*lineHeight), lineHeight);
-    ypos += lineHeight + 8;
-    if(isGlobal)
-    {
-      myDbgColour[idx]->clearFlags(WIDGET_ENABLED);
-      myDbgColourSwatch[idx]->clearFlags(WIDGET_ENABLED);
-    }
-  };
-
-  createDebugColourWidgets(0, "Player 0 ");
-  createDebugColourWidgets(1, "Missile 0 ");
-  createDebugColourWidgets(2, "Player 1 ");
-  createDebugColourWidgets(3, "Missile 1 ");
-  createDebugColourWidgets(4, "Playfield ");
-  createDebugColourWidgets(5, "Ball ");
-
-  // Add message concerning usage
-  ypos = myTab->getHeight() - 5 - fontHeight - infofont.getFontHeight() - 10;
-  new StaticTextWidget(myTab, infofont, 10, ypos,
-        font.getStringWidth("(*) Colors must be different for each object"), fontHeight,
-        "(*) Colors must be different for each object", kTextAlignLeft);
-
-  // Add items for tab 2
-  addToFocusList(wid, myTab, tabID);
-
   // Activate the first tab
   myTab->setActiveTab(0);
 
@@ -371,14 +304,14 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   wid.clear();
   ButtonWidget* b;
   b = new ButtonWidget(this, font, 10, _h - buttonHeight - 10,
-                       buttonWidth, buttonHeight, "Defaults", kDefaultsCmd);
+                       buttonWidth, buttonHeight, "Defaults", GuiObject::kDefaultsCmd);
   wid.push_back(b);
   addOKCancelBGroup(wid, font);
   addBGroupToFocusList(wid);
 
   // Disable certain functions when we know they aren't present
 #ifndef WINDOWED_SUPPORT
-  myFullscreenCheckbox->clearFlags(WIDGET_ENABLED);
+  myFullscreen->clearFlags(WIDGET_ENABLED);
   myCenter->clearFlags(WIDGET_ENABLED);
 #endif
 }
@@ -430,9 +363,6 @@ void VideoDialog::loadConfig()
   // Use sync to vertical blank
   myUseVSync->setState(instance().settings().getBool("vsync"));
 
-  // PAL color-loss effect
-  myColorLoss->setState(instance().settings().getBool("colorloss"));
-
   // Show UI messages
   myUIMessages->setState(instance().settings().getBool("uimessages"));
 
@@ -462,19 +392,11 @@ void VideoDialog::loadConfig()
   myTVPhosLevel->setValue(instance().settings().getInt("tv.phosblend"));
   myTVPhosLevelLabel->setLabel(instance().settings().getString("tv.phosblend"));
 
-  // TV jitter
-  myTVJitterRec->setValue(instance().settings().getInt("tv.jitter_recovery"));
-  myTVJitterRecLabel->setLabel(instance().settings().getString("tv.jitter_recovery"));
-  handleTVJitterChange(instance().settings().getBool("tv.jitter"));
-
   // TV scanline intensity and interpolation
   myTVScanIntense->setValue(instance().settings().getInt("tv.scanlines"));
   myTVScanIntenseLabel->setLabel(instance().settings().getString("tv.scanlines"));
   myTVScanInterpolate->setState(instance().settings().getBool("tv.scaninter"));
 
-  // Debug colours
-  handleDebugColours(instance().settings().getString("tia.dbgcolors"));
-
   myTab->loadConfig();
 }
 
@@ -506,23 +428,18 @@ void VideoDialog::saveConfig()
   instance().settings().setValue("tia.aspectp", myPAspectRatioLabel->getLabel());
 
   // Framerate
-  int i = myFrameRate->getValue();
-  instance().settings().setValue("framerate", i);
+  int f = myFrameRate->getValue();
+  instance().settings().setValue("framerate", f);
   if(instance().hasConsole())
   {
     // Make sure auto-frame calculation is only enabled when necessary
-    instance().console().tia().enableAutoFrame(i <= 0);
-    instance().console().setFramerate(float(i));
+    instance().console().tia().enableAutoFrame(f <= 0);
+    instance().console().setFramerate(float(f));
   }
 
   // Fullscreen
   instance().settings().setValue("fullscreen", myFullscreen->getState());
 
-  // PAL color-loss effect
-  instance().settings().setValue("colorloss", myColorLoss->getState());
-  if(instance().hasConsole())
-    instance().console().toggleColorLoss(myColorLoss->getState());
-
   // Fullscreen stretch setting
   instance().settings().setValue("tia.fsfill", myUseStretch->getState());
 
@@ -569,27 +486,10 @@ void VideoDialog::saveConfig()
   instance().settings().setValue("tv.phosblend", myTVPhosLevelLabel->getLabel());
   Properties::setDefault(Display_PPBlend, myTVPhosLevelLabel->getLabel());
 
-  // TV jitter
-  instance().settings().setValue("tv.jitter", myTVJitter->getState());
-  instance().settings().setValue("tv.jitter_recovery", myTVJitterRecLabel->getLabel());
-  if(instance().hasConsole())
-  {
-    instance().console().tia().toggleJitter(myTVJitter->getState() ? 1 : 0);
-    instance().console().tia().setJitterRecoveryFactor(myTVJitterRec->getValue());
-  }
-
   // TV scanline intensity and interpolation
   instance().settings().setValue("tv.scanlines", myTVScanIntenseLabel->getLabel());
   instance().settings().setValue("tv.scaninter", myTVScanInterpolate->getState());
 
-  // Debug colours
-  string dbgcolors;
-  for(int i = 0; i < 6; ++i)
-    dbgcolors += myDbgColour[i]->getSelectedTag().toString();
-  if(instance().hasConsole() &&
-     instance().console().tia().setFixedColorPalette(dbgcolors))
-    instance().settings().setValue("tia.dbgcolors", dbgcolors);
-
   // Finally, issue a complete framebuffer re-initialization
   instance().createFrameBuffer();
 }
@@ -607,16 +507,15 @@ void VideoDialog::setDefaults()
       myFrameTiming->setSelected("sleep", "");
       myTIAInterpolate->setSelected("nearest", "");
       myNAspectRatio->setValue(90);
-      myNAspectRatioLabel->setLabel("90");
+      myNAspectRatioLabel->setLabel("91");
       myPAspectRatio->setValue(100);
-      myPAspectRatioLabel->setLabel("100");
+      myPAspectRatioLabel->setLabel("109");
       myFrameRate->setValue(0);
       myFrameRateLabel->setLabel("Auto");
 
       myFullscreen->setState(false);
       myUseStretch->setState(true);
       myUseVSync->setState(true);
-      myColorLoss->setState(false);
       myUIMessages->setState(true);
       myCenter->setState(false);
       myFastSCBios->setState(true);
@@ -635,11 +534,6 @@ void VideoDialog::setDefaults()
       myTVPhosLevel->setValue(50);
       myTVPhosLevelLabel->setLabel("50");
 
-      // TV jitter
-      myTVJitterRec->setValue(10);
-      myTVJitterRecLabel->setLabel("10");
-      handleTVJitterChange(true);
-
       // TV scanline intensity and interpolation
       myTVScanIntense->setValue(25);
       myTVScanIntenseLabel->setLabel("25");
@@ -650,12 +544,6 @@ void VideoDialog::setDefaults()
       loadTVAdjustables(NTSCFilter::PRESET_CUSTOM);
       break;
     }
-
-    case 2:  // Debug colours
-    {
-      handleDebugColours("roygpb");
-      break;
-    }
   }
 
   _dirty = true;
@@ -701,59 +589,6 @@ void VideoDialog::handleTVModeChange(NTSCFilter::Preset preset)
   _dirty = true;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void VideoDialog::handleTVJitterChange(bool enable)
-{
-  myTVJitter->setState(enable);
-  myTVJitterRec->setEnabled(enable);
-  myTVJitterRecLabel->setEnabled(enable);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void VideoDialog::handleDebugColours(int idx, int color)
-{
-  if(!instance().hasConsole() || idx < 0 || idx > 5)
-    return;
-
-  static constexpr int dbg_color[2][6] = {
-    { TIA::FixedColor::NTSC_RED,
-      TIA::FixedColor::NTSC_ORANGE,
-      TIA::FixedColor::NTSC_YELLOW,
-      TIA::FixedColor::NTSC_GREEN,
-      TIA::FixedColor::NTSC_BLUE,
-      TIA::FixedColor::NTSC_PURPLE
-    },
-    { TIA::FixedColor::PAL_RED,
-      TIA::FixedColor::PAL_ORANGE,
-      TIA::FixedColor::PAL_YELLOW,
-      TIA::FixedColor::PAL_GREEN,
-      TIA::FixedColor::PAL_BLUE,
-      TIA::FixedColor::PAL_PURPLE
-    }
-  };
-  int mode = instance().console().tia().frameLayout() == FrameLayout::ntsc ? 0 : 1;
-  myDbgColourSwatch[idx]->setColor(dbg_color[mode][color]);
-  myDbgColour[idx]->setSelectedIndex(color);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void VideoDialog::handleDebugColours(const string& colors)
-{
-  for(int i = 0; i < 6; ++i)
-  {
-    switch(colors[i])
-    {
-      case 'r':  handleDebugColours(i, 0);  break;
-      case 'o':  handleDebugColours(i, 1);  break;
-      case 'y':  handleDebugColours(i, 2);  break;
-      case 'g':  handleDebugColours(i, 3);  break;
-      case 'b':  handleDebugColours(i, 4);  break;
-      case 'p':  handleDebugColours(i, 5);  break;
-      default:                              break;
-    }
-  }
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void VideoDialog::loadTVAdjustables(NTSCFilter::Preset preset)
 {
@@ -788,12 +623,12 @@ void VideoDialog::handleCommand(CommandSender* sender, int cmd,
 {
   switch(cmd)
   {
-    case kOKCmd:
+    case GuiObject::kOKCmd:
       saveConfig();
       close();
       break;
 
-    case kDefaultsCmd:
+    case GuiObject::kDefaultsCmd:
       setDefaults();
       break;
 
@@ -837,10 +672,6 @@ void VideoDialog::handleCommand(CommandSender* sender, int cmd,
       break;
     case kTVPhosLevelChanged:  myTVPhosLevelLabel->setValue(myTVPhosLevel->getValue());
       break;
-    case kTVJitterChanged:  handleTVJitterChange(myTVJitter->getState());
-      break;
-    case kTVJitterRecChanged:  myTVJitterRecLabel->setValue(myTVJitterRec->getValue());
-      break;
     case kTVScanIntenseChanged:  myTVScanIntenseLabel->setValue(myTVScanIntense->getValue());
       break;
     case kCloneCompositeCmd: loadTVAdjustables(NTSCFilter::PRESET_COMPOSITE);
@@ -854,25 +685,6 @@ void VideoDialog::handleCommand(CommandSender* sender, int cmd,
     case kCloneCustomCmd: loadTVAdjustables(NTSCFilter::PRESET_CUSTOM);
       break;
 
-    case kP0ColourChangedCmd:
-      handleDebugColours(0, myDbgColour[0]->getSelected());
-      break;
-    case kM0ColourChangedCmd:
-      handleDebugColours(1, myDbgColour[1]->getSelected());
-      break;
-    case kP1ColourChangedCmd:
-      handleDebugColours(2, myDbgColour[2]->getSelected());
-      break;
-    case kM1ColourChangedCmd:
-      handleDebugColours(3, myDbgColour[3]->getSelected());
-      break;
-    case kPFColourChangedCmd:
-      handleDebugColours(4, myDbgColour[4]->getSelected());
-      break;
-    case kBLColourChangedCmd:
-      handleDebugColours(5, myDbgColour[5]->getSelected());
-      break;
-
     default:
       Dialog::handleCommand(sender, cmd, data, 0);
       break;
