@@ -26,7 +26,10 @@ echo "Copying documentation ..."
 ditto ../../Announce.txt ../../Changes.txt ../../Copyright.txt ../../License.txt ../../Readme.txt ../../Todo.txt "${DISK}"
 
 echo "Copying application ..."
-cp -r build/Deployment/Stella.app "${DISK}"
+cp -r DerivedData/Build/Products/Release/Stella.app "${DISK}"
+
+echo "Updating modification times ..."
+touch "${DISK}"/Stella.app "${DISK}"/*.txt
 
 echo "Ejecting ${DMG} ..."
 hdiutil eject "${DISK}"
