@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -32,21 +32,21 @@
 OSystemMACOSX::OSystemMACOSX()
   : OSystem()
 {
-  setBaseDir("~/Library/Application Support/Stella");
+  setBaseDir("~/Library/Application Support/Stella/");
 
   // This will be overridden, as OSX uses plist files for settings
   setConfigFile("~/Library/Application Support/Stella/stellarc");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string OSystemMACOSX::defaultSnapSaveDir()
+string OSystemMACOSX::defaultSaveDir() const
 {
-  FilesystemNode desktop("~/Desktop");
-  return desktop.isDirectory() ? desktop.getShortPath() : "~";
+  FilesystemNode desktop("~/Desktop/");
+  return desktop.isDirectory() ? desktop.getShortPath() : "~/";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string OSystemMACOSX::defaultSnapLoadDir()
+string OSystemMACOSX::defaultLoadDir() const
 {
-  return defaultSnapSaveDir();
+  return defaultSaveDir();
 }
