@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2017 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2018 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -298,22 +298,16 @@ bool FilesystemNodeWINDOWS::rename(const string& newfile)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 AbstractFSNode* FilesystemNodeWINDOWS::getParent() const
 {
-  if(!_isValid || _isPseudoRoot)
+  if(_isPseudoRoot)
     return nullptr;
 
-  FilesystemNodeWINDOWS* p = new FilesystemNodeWINDOWS();
   if(_path.size() > 3)
   {
     const char* start = _path.c_str();
     const char* end = lastPathComponent(_path);
 
-    p->_path = string(start, end - start);
-    p->_isValid = true;
-    p->_isDirectory = true;
-    p->_isFile = false;
-    p->_displayName = lastPathComponent(p->_path);
-    p->_isPseudoRoot = false;
+    return new FilesystemNodeWINDOWS(string(start, size_t(end - start)));
   }
-
-  return p;
+  else
+    return new FilesystemNodeWINDOWS();
 }
